@@ -32,9 +32,18 @@
 #include <windows.h>
 #include "msg.h"
 
-#define expect(expected, got) ok(expected == got, "Expected %d, got %d\n", expected, got);
-#define expect_hex(expected, got) ok(expected == got, "Expected %x, got %x\n", expected, got);
-#define expect_d(expected, got) ok(abs((expected) - (got)) <= 2, "Expected %d, got %d\n", expected, got);
+#define expect(expected,got) expect_(__LINE__, expected, got)
+static inline void expect_(unsigned line, DWORD expected, DWORD got)
+{
+    ok_(__FILE__, line)(expected == got, "Expected %ld, got %ld\n", expected, got);
+}
+
+#define expect_hex(expected,got) expect_hex_(__LINE__, expected, got)
+static inline void expect_hex_(unsigned line, DWORD expected, DWORD got)
+{
+    ok_(__FILE__, line)(expected == got, "Expected %lx, got %lx\n", expected, got);
+}
+#define expect_d(expected, got) ok(abs((expected) - (got)) <= 2, "Expected %ld, got %ld\n", expected, got);
 
 #define NUM_MSG_SEQUENCES   2
 #define PARENT_SEQ_INDEX    0
@@ -42,9 +51,12 @@
 
 #define SEL_NOTIFY_TEST_ID  100
 
+static BOOL (WINAPI *pInitCommonControlsEx)(const INITCOMMONCONTROLSEX*);
+
 static struct msg_sequence *sequences[NUM_MSG_SEQUENCES];
 
 static HWND parent_wnd;
+static BOOL got_MCN_SELECT, got_MCN_SELCHANGE;
 
 static const struct message create_parent_window_seq[] = {
     { WM_GETMINMAXINFO, sent },
@@ -273,7 +285,7 @@ static void test_monthcal(void)
     limits = SendMessageA(hwnd, MCM_GETRANGE, 0, (LPARAM)st);
     ok(limits == 0 ||
        broken(limits == GDTR_MIN), /* comctl32 <= 4.70 */
-       "No limits should be set (%d)\n", limits);
+       "No limits should be set (%ld)\n", limits);
     if (limits == GDTR_MIN)
     {
         win_skip("comctl32 <= 4.70 is broken\n");
@@ -306,7 +318,7 @@ static void test_monthcal(void)
     expect(0, st[1].wMilliseconds);
 
     limits = SendMessageA(hwnd, MCM_GETRANGE, 0, 0);
-    ok(limits == 0, "got %u\n", limits);
+    ok(limits == 0, "got %lu\n", limits);
 
     GetSystemTime(&st[0]);
     st[1] = st[0];
@@ -456,7 +468,7 @@ static void test_monthcal(void)
 
     /* 0 limit flags */
     limits = SendMessageA(hwnd, MCM_GETRANGE, 0, (LPARAM)st1);
-    ok(limits == GDTR_MIN, "got 0x%08x\n", limits);
+    ok(limits == GDTR_MIN, "got 0x%08lx\n", limits);
 
     GetSystemTime(st);
     st[1] = st[0];
@@ -465,7 +477,7 @@ static void test_monthcal(void)
     ok(r, "got %d\n", r);
 
     limits = SendMessageA(hwnd, MCM_GETRANGE, 0, (LPARAM)st);
-    ok(limits == 0, "got 0x%08x\n", limits);
+    ok(limits == 0, "got 0x%08lx\n", limits);
     ok(st[0].wYear == 0 && st[1].wYear == 0, "got %u, %u\n", st[0].wYear, st[1].wYear);
 
     /* flags are 0, set min limit */
@@ -477,7 +489,7 @@ static void test_monthcal(void)
     ok(r, "got %d\n", r);
 
     limits = SendMessageA(hwnd, MCM_GETRANGE, 0, (LPARAM)st1);
-    ok(limits == GDTR_MIN, "got 0x%08x\n", limits);
+    ok(limits == GDTR_MIN, "got 0x%08lx\n", limits);
     ok(st1[1].wYear == 0, "got %u\n", st1[1].wYear);
 
     /* now set max limit, check flags */
@@ -485,7 +497,7 @@ static void test_monthcal(void)
     ok(r, "got %d\n", r);
 
     limits = SendMessageA(hwnd, MCM_GETRANGE, 0, (LPARAM)st1);
-    ok(limits == GDTR_MAX, "got 0x%08x\n", limits);
+    ok(limits == GDTR_MAX, "got 0x%08lx\n", limits);
     ok(st1[0].wYear == 0, "got %u\n", st1[0].wYear);
 
     DestroyWindow(hwnd);
@@ -542,6 +554,12 @@ static LRESULT WINAPI parent_wnd_proc(HWND hwnd, UINT message, WPARAM wParam, LP
 
             if(GetWindowLongPtrA(nmchg->nmhdr.hwndFrom, GWLP_ID) != SEL_NOTIFY_TEST_ID)
               break;
+
+            if (hdr->code == MCN_SELECT)
+                got_MCN_SELECT = TRUE;
+            else
+                got_MCN_SELCHANGE = TRUE;
+
             SendMessageA(nmchg->nmhdr.hwndFrom, is_multisel ? MCM_GETSELRANGE : MCM_GETCURSEL,
                     0, (LPARAM)st);
 
@@ -598,8 +616,6 @@ static HWND create_parent_window(void)
 {
     HWND hwnd;
 
-    InitCommonControls();
-
     /* flush message sequences, so we can check the new sequence by the end of function */
     flush_sequences(sequences, NUM_MSG_SEQUENCES);
 
@@ -636,7 +652,7 @@ static LRESULT WINAPI monthcal_subclass_proc(HWND hwnd, UINT message, WPARAM wPa
          message == WM_STYLECHANGED) && lParam)
     {
         STYLESTRUCT *style = (STYLESTRUCT*)lParam;
-        trace("\told style: 0x%08x, new style: 0x%08x\n", style->styleOld, style->styleNew);
+        trace("\told style: 0x%08lx, new style: 0x%08lx\n", style->styleOld, style->styleNew);
     }
 
     defwndproc_counter++;
@@ -911,7 +927,7 @@ static void test_firstDay(void)
     SetLastError(0xdeadbeef);
     ret = GetLocaleInfoA(lcid, LOCALE_ICALENDARTYPE, caltype, 3);
     if (ret == 0) {
-        skip("Must know local calendar type (%x)\n", GetLastError());
+        skip("Must know local calendar type (%lx)\n", GetLastError());
         return;
     } else if (atoi(caltype) != CAL_GREGORIAN) {
         skip("MonthCalendar Control only supports Gregorian calendar (type: %s)\n", caltype);
@@ -1216,11 +1232,11 @@ if (0)
             } else {
                 title_index++;
 
-                if (sizeof(title_hits) / sizeof(title_hits[0]) <= title_index)
+                if (ARRAY_SIZE(title_hits) <= title_index)
                     break;
 
                 todo_wine_if(title_hits[title_index].todo)
-                    ok(title_hits[title_index].ht == res, "Expected %x, got %x, pos %d\n",
+                    ok(title_hits[title_index].ht == res, "Expected %x, got %x, pos %ld\n",
                                                           title_hits[title_index].ht, res, x);
             }
             old_res = res;
@@ -1241,8 +1257,7 @@ if (0)
 
     todo_wine ok(month_count + year_count >= 1, "Not enough month and year items\n");
 
-    ok(r.right <= x && title_index + 1 == sizeof(title_hits) / sizeof(title_hits[0]),
-       "Wrong title layout\n");
+    ok(r.right <= x && title_index + 1 == ARRAY_SIZE(title_hits), "Wrong title layout\n");
 
     DestroyWindow(hwnd);
 }
@@ -1799,7 +1814,7 @@ static void test_hittest_v6(void)
     mchit.iOffset = -1;
     mchit.iCol = mchit.iRow = -1;
     mchit.uHit = 0;
-    mchit.rc.left = mchit.rc.right = mchit.rc.top = mchit.rc.bottom = -1;
+    SetRect(&mchit.rc, -1, -1, -1, -1);
     ret = SendMessageA(hwnd, MCM_HITTEST, 0, (LPARAM)&mchit);
     expect_hex(MCHT_CALENDARDATE, ret);
     expect_hex(MCHT_CALENDARDATE, mchit.uHit);
@@ -1816,7 +1831,7 @@ static void test_hittest_v6(void)
     mchit.iOffset = -1;
     mchit.iCol = mchit.iRow = -1;
     mchit.uHit = 0;
-    mchit.rc.left = mchit.rc.right = mchit.rc.top = mchit.rc.bottom = -1;
+    SetRect(&mchit.rc, -1, -1, -1, -1);
     ret = SendMessageA(hwnd, MCM_HITTEST, 0, (LPARAM)&mchit);
     expect_hex(MCHT_TITLE, ret);
     expect_hex(MCHT_TITLE, mchit.uHit);
@@ -1826,7 +1841,7 @@ static void test_hittest_v6(void)
     expect(0, mchit.rc.left);
     expect(0, mchit.rc.top);
     expect_d(r.right, mchit.rc.right);
-    ok(mchit.rc.bottom > 0, "got %d\n", mchit.rc.bottom);
+    ok(mchit.rc.bottom > 0, "got %ld\n", mchit.rc.bottom);
 
     /* between two calendars */
     MoveWindow(hwnd, 0, 0, r.right * 5/2, r.bottom, FALSE);
@@ -1835,7 +1850,7 @@ static void test_hittest_v6(void)
     mchit.iOffset = -2;
     mchit.iCol = mchit.iRow = -2;
     mchit.uHit = ~0;
-    mchit.rc.left = mchit.rc.right = mchit.rc.top = mchit.rc.bottom = -1;
+    SetRect(&mchit.rc, -1, -1, -1, -1);
     ret = SendMessageA(hwnd, MCM_HITTEST, 0, (LPARAM)&mchit);
     todo_wine expect_hex(MCHT_NOWHERE, ret);
     todo_wine expect_hex(MCHT_NOWHERE, mchit.uHit);
@@ -1892,7 +1907,7 @@ static void test_MCM_SIZERECTTOMIN(void)
     }
 
     ret = SendMessageA(hwnd, MCM_SIZERECTTOMIN, 0, 0);
-    ok(ret == 0, "got %d\n", ret);
+    ok(ret == 0, "got %ld\n", ret);
 
     SetRectEmpty(&r);
     ret = SendMessageA(hwnd, MCM_SIZERECTTOMIN, 0, (LPARAM)&r);
@@ -1902,18 +1917,18 @@ static void test_MCM_SIZERECTTOMIN(void)
         DestroyWindow(hwnd);
         return;
     }
-    ok(ret == 1, "got %d\n", ret);
-    ok(r.left == 0 && r.right > 0, "got %d, %d\n", r.left, r.right);
+    ok(ret == 1, "got %ld\n", ret);
+    ok(r.left == 0 && r.right > 0, "got %ld, %ld\n", r.left, r.right);
 
     r = r2;
     ret = SendMessageA(hwnd, MCM_SIZERECTTOMIN, 0, (LPARAM)&r);
-    ok(ret == 1, "got %d\n", ret);
+    ok(ret == 1, "got %ld\n", ret);
 
     r2.right = (r2.right - r2.left) * 3;
     r2.bottom = (r2.bottom - r2.top) * 3;
     r2.left = r2.top = 0;
     ret = SendMessageA(hwnd, MCM_SIZERECTTOMIN, 0, (LPARAM)&r2);
-    ok(ret == 1, "got %d\n", ret);
+    ok(ret == 1, "got %ld\n", ret);
 
     DestroyWindow(hwnd);
 }
@@ -1969,7 +1984,7 @@ static void test_daystate(void)
     /* try to switch on */
     SetWindowLongA(hwnd, GWL_STYLE, GetWindowLongA(hwnd, GWL_STYLE) | MCS_DAYSTATE);
     style = GetWindowLongA(hwnd, GWL_STYLE);
-    ok((style & MCS_DAYSTATE) == 0, "got 0x%08x\n", style);
+    ok((style & MCS_DAYSTATE) == 0, "got 0x%08lx\n", style);
 
     DestroyWindow(hwnd);
 
@@ -1991,7 +2006,7 @@ static void test_daystate(void)
     /* try to switch off */
     SetWindowLongA(hwnd, GWL_STYLE, GetWindowLongA(hwnd, GWL_STYLE) & ~MCS_DAYSTATE);
     style = GetWindowLongA(hwnd, GWL_STYLE);
-    ok((style & MCS_DAYSTATE) == MCS_DAYSTATE, "got 0x%08x\n", style);
+    ok((style & MCS_DAYSTATE) == MCS_DAYSTATE, "got 0x%08lx\n", style);
 
     DestroyWindow(hwnd);
 }
@@ -2016,7 +2031,7 @@ static void test_sel_notify(void)
     };
     int i;
 
-    for(i = 0; i < sizeof styles / sizeof styles[0]; i++)
+    for(i = 0; i < ARRAY_SIZE(styles); i++)
     {
         hwnd = create_monthcal_control(styles[i].val);
         SetWindowLongPtrA(hwnd, GWLP_ID, SEL_NOTIFY_TEST_ID);
@@ -2033,28 +2048,41 @@ static void test_sel_notify(void)
             mchit.pt.y++;
             SendMessageA(hwnd, MCM_HITTEST, 0, (LPARAM)&mchit);
         }
+        got_MCN_SELECT = got_MCN_SELCHANGE = FALSE;
         SendMessageA(hwnd, WM_LBUTTONDOWN, 0, MAKELPARAM(mchit.pt.x, mchit.pt.y));
+        if (styles[i].val & MCS_MULTISELECT)
+            ok(got_MCN_SELCHANGE, "%d: MCN_SELCHANGE should be sent\n", i);
+        else
+            ok(!got_MCN_SELCHANGE, "%d: MCN_SELCHANGE should not be sent\n", i);
+        ok(!got_MCN_SELECT, "%d: MCN_SELECT should not be sent\n", i);
+        got_MCN_SELECT = got_MCN_SELCHANGE = FALSE;
         SendMessageA(hwnd, WM_LBUTTONUP, 0, MAKELPARAM(mchit.pt.x, mchit.pt.y));
+        if (styles[i].val & MCS_MULTISELECT)
+            ok(!got_MCN_SELCHANGE, "%d: MCN_SELCHANGE should not be sent\n", i);
+        else
+            ok(got_MCN_SELCHANGE, "%d: MCN_SELCHANGE should be sent\n", i);
+        ok(got_MCN_SELECT, "%d: MCN_SELECT should be sent\n", i);
         DestroyWindow(hwnd);
     }
 }
 
+static void init_functions(void)
+{
+    HMODULE hComCtl32 = LoadLibraryA("comctl32.dll");
+
+#define X(f) p##f = (void*)GetProcAddress(hComCtl32, #f);
+    X(InitCommonControlsEx);
+#undef X
+}
+
 START_TEST(monthcal)
 {
-    BOOL (WINAPI *pInitCommonControlsEx)(const INITCOMMONCONTROLSEX*);
     INITCOMMONCONTROLSEX iccex;
-    HMODULE hComctl32;
-
     ULONG_PTR ctx_cookie;
     HANDLE hCtx;
 
-    hComctl32 = GetModuleHandleA("comctl32.dll");
-    pInitCommonControlsEx = (void*)GetProcAddress(hComctl32, "InitCommonControlsEx");
-    if (!pInitCommonControlsEx)
-    {
-        win_skip("InitCommonControlsEx() is missing. Skipping the tests\n");
-        return;
-    }
+    init_functions();
+
     iccex.dwSize = sizeof(iccex);
     iccex.dwICC  = ICC_DATE_CLASSES;
     pInitCommonControlsEx(&iccex);
