@@ -70,7 +70,7 @@ static ULONG WINAPI ShellBrowser_AddRef(
     ShellBrowser *This = impl_from_IShellBrowser(iface);
     LONG ref = InterlockedIncrement(&This->ref);
 
-    TRACE("(%p) ref=%d\n", This, ref);
+    TRACE("(%p) ref=%ld\n", This, ref);
 
     return ref;
 }
@@ -80,7 +80,7 @@ static ULONG WINAPI ShellBrowser_Release(IShellBrowser* iface)
     ShellBrowser *This = impl_from_IShellBrowser(iface);
     LONG ref = InterlockedDecrement(&This->ref);
 
-    TRACE("(%p) ref=%d\n", This, ref);
+    TRACE("(%p) ref=%ld\n", This, ref);
 
     if(!ref) {
         assert(!This->doc_host);
@@ -182,7 +182,7 @@ static HRESULT WINAPI ShellBrowser_GetViewStateStream(
         IStream **ppStrm)
 {
     ShellBrowser *This = impl_from_IShellBrowser(iface);
-    FIXME("%p %x %p\n", This, grfMode, ppStrm);
+    FIXME("%p %lx %p\n", This, grfMode, ppStrm);
     return E_NOTIMPL;
 }
 
@@ -313,7 +313,7 @@ static HRESULT WINAPI BrowserService_GetTitle(
         DWORD cchName)
 {
     ShellBrowser *This = impl_from_IBrowserService(iface);
-    FIXME("%p %p %p %d\n", This, psv, pszName, cchName);
+    FIXME("%p %p %p %ld\n", This, psv, pszName, cchName);
     return E_NOTIMPL;
 }
 
@@ -383,7 +383,7 @@ static HRESULT WINAPI BrowserService_DisplayParseError(
         LPCWSTR pwszPath)
 {
     ShellBrowser *This = impl_from_IBrowserService(iface);
-    FIXME("%p %x %s\n", This, hres, debugstr_w(pwszPath));
+    FIXME("%p %lx %s\n", This, hres, debugstr_w(pwszPath));
     return E_NOTIMPL;
 }
 
@@ -393,7 +393,7 @@ static HRESULT WINAPI BrowserService_NavigateToPidl(
         DWORD grfHLNF)
 {
     ShellBrowser *This = impl_from_IBrowserService(iface);
-    FIXME("%p %p %d\n", This, pidl, grfHLNF);
+    FIXME("%p %p %ld\n", This, pidl, grfHLNF);
     return E_NOTIMPL;
 }
 
@@ -448,7 +448,7 @@ static HRESULT WINAPI BrowserService_SetFlags(
         DWORD dwFlagMask)
 {
     ShellBrowser *This = impl_from_IBrowserService(iface);
-    FIXME("%p %x %x\n", This, dwFlags, dwFlagMask);
+    FIXME("%p %lx %lx\n", This, dwFlags, dwFlagMask);
     return E_NOTIMPL;
 }
 
@@ -501,7 +501,7 @@ static HRESULT WINAPI BrowserService_GetBrowserByIndex(
         IUnknown **ppunk)
 {
     ShellBrowser *This = impl_from_IBrowserService(iface);
-    FIXME("%p %x %p\n", This, dwID, ppunk);
+    FIXME("%p %lx %p\n", This, dwID, ppunk);
     return E_NOTIMPL;
 }
 
@@ -661,7 +661,7 @@ static HRESULT WINAPI DocObjectService_FireBeforeNavigate2(
     WCHAR file_path[MAX_PATH];
     DWORD file_path_len = ARRAY_SIZE(file_path);
 
-    TRACE("%p %p %s %x %s %p %d %s %d %p\n", This, pDispatch, debugstr_w(lpszUrl),
+    TRACE("%p %p %s %lx %s %p %ld %s %d %p\n", This, pDispatch, debugstr_w(lpszUrl),
             dwFlags, debugstr_w(lpszFrameName), pPostData, cbPostData,
             debugstr_w(lpszHeaders), fPlayNavSound, pfCancel);
 
@@ -747,7 +747,7 @@ static HRESULT WINAPI DocObjectService_FireNavigateComplete2(
     BSTR url;
     HRESULT hres;
 
-    TRACE("%p %p %x\n", This, pHTMLWindow2, dwFlags);
+    TRACE("%p %p %lx\n", This, pHTMLWindow2, dwFlags);
 
     update_navigation_commands(This->doc_host);
 
@@ -787,7 +787,6 @@ static HRESULT WINAPI DocObjectService_FireNavigateComplete2(
 
     SysFreeString(url);
 
-    This->doc_host->busy = VARIANT_FALSE;
     IShellBrowser_Release(&This->IShellBrowser_iface);
     return S_OK;
 }
@@ -821,7 +820,7 @@ static HRESULT WINAPI DocObjectService_FireDocumentComplete(
     BSTR url;
     HRESULT hres;
 
-    TRACE("%p %p %x\n", This, pHTMLWindow, dwFlags);
+    TRACE("%p %p %lx\n", This, pHTMLWindow, dwFlags);
 
     hres = IHTMLWindow2_QueryInterface(pHTMLWindow, &IID_IHTMLPrivateWindow, (void**)&priv_window);
     if(FAILED(hres))
@@ -851,8 +850,6 @@ static HRESULT WINAPI DocObjectService_FireDocumentComplete(
     TRACE("<<<\n");
 
     SysFreeString(url);
-    if(This->doc_host)
-        This->doc_host->busy = VARIANT_FALSE;
 
     IShellBrowser_Release(&This->IShellBrowser_iface);
     return S_OK;
@@ -1051,7 +1048,7 @@ static HRESULT WINAPI NewWindowManager_EvaluateNewWindow(INewWindowManager *ifac
         DWORD dwUserActionTime)
 {
     NewWindowManager *This = impl_from_INewWindowManager(iface);
-    FIXME("(%p)->(%s %s %s %s %x %x %d)\n", This, debugstr_w(pszUrl), debugstr_w(pszName), debugstr_w(pszUrlContext),
+    FIXME("(%p)->(%s %s %s %s %x %lx %ld)\n", This, debugstr_w(pszUrl), debugstr_w(pszName), debugstr_w(pszUrlContext),
           debugstr_w(pszFeatures), fReplace, dwFlags, dwUserActionTime);
 
     /*
