@@ -25,38 +25,62 @@
 
 WINE_DEFAULT_DEBUG_CHANNEL(jscript);
 
+typedef struct _function_vtbl_t function_vtbl_t;
+
 typedef struct {
     jsdisp_t dispex;
-    builtin_invoke_t value_proc;
-    const WCHAR *name;
+    const function_vtbl_t *vtbl;
     DWORD flags;
-    scope_chain_t *scope_chain;
-    bytecode_t *code;
-    function_code_t *func_code;
     DWORD length;
 } FunctionInstance;
 
+struct _function_vtbl_t {
+    HRESULT (*call)(script_ctx_t*,FunctionInstance*,jsval_t,unsigned,unsigned,jsval_t*,jsval_t*);
+    HRESULT (*toString)(FunctionInstance*,jsstr_t**);
+    function_code_t* (*get_code)(FunctionInstance*);
+    void (*destructor)(FunctionInstance*);
+};
+
+typedef struct {
+    FunctionInstance function;
+    scope_chain_t *scope_chain;
+    bytecode_t *code;
+    function_code_t *func_code;
+} InterpretedFunction;
+
+typedef struct {
+    FunctionInstance function;
+    builtin_invoke_t proc;
+    const WCHAR *name;
+} NativeFunction;
+
+typedef struct {
+    FunctionInstance function;
+    FunctionInstance *target;
+    IDispatch *this;
+    unsigned argc;
+    jsval_t args[1];
+} BindFunction;
+
 typedef struct {
     jsdisp_t jsdisp;
-    FunctionInstance *function;
+    InterpretedFunction *function;
     jsval_t *buf;
     call_frame_t *frame;
     unsigned argc;
 } ArgumentsInstance;
 
+static HRESULT create_bind_function(script_ctx_t*,FunctionInstance*,IDispatch*,unsigned,jsval_t*,jsdisp_t**r);
+
 static inline FunctionInstance *function_from_jsdisp(jsdisp_t *jsdisp)
 {
     return CONTAINING_RECORD(jsdisp, FunctionInstance, dispex);
 }
 
-static inline FunctionInstance *function_from_vdisp(vdisp_t *vdisp)
+static inline FunctionInstance *function_this(jsval_t vthis)
 {
-    return function_from_jsdisp(vdisp->u.jsdisp);
-}
-
-static inline FunctionInstance *function_this(vdisp_t *jsthis)
-{
-    return is_vclass(jsthis, JSCLASS_FUNCTION) ? function_from_vdisp(jsthis) : NULL;
+    jsdisp_t *jsdisp = is_object_instance(vthis) ? to_jsdisp(get_object(vthis)) : NULL;
+    return (jsdisp && is_class(jsdisp, JSCLASS_FUNCTION)) ? function_from_jsdisp(jsdisp) : NULL;
 }
 
 static inline ArgumentsInstance *arguments_from_jsdisp(jsdisp_t *jsdisp)
@@ -64,15 +88,7 @@ static inline ArgumentsInstance *arguments_from_jsdisp(jsdisp_t *jsdisp)
     return CONTAINING_RECORD(jsdisp, ArgumentsInstance, jsdisp);
 }
 
-static const WCHAR prototypeW[] = {'p','r','o','t','o','t', 'y', 'p','e',0};
-
-static const WCHAR lengthW[] = {'l','e','n','g','t','h',0};
-static const WCHAR toStringW[] = {'t','o','S','t','r','i','n','g',0};
-static const WCHAR applyW[] = {'a','p','p','l','y',0};
-static const WCHAR callW[] = {'c','a','l','l',0};
-static const WCHAR argumentsW[] = {'a','r','g','u','m','e','n','t','s',0};
-
-static HRESULT Arguments_value(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, unsigned argc, jsval_t *argv,
+static HRESULT Arguments_value(script_ctx_t *ctx, jsval_t vthis, WORD flags, unsigned argc, jsval_t *argv,
         jsval_t *r)
 {
     FIXME("\n");
@@ -92,7 +108,7 @@ static void Arguments_destructor(jsdisp_t *jsdisp)
         heap_free(arguments->buf);
     }
 
-    jsdisp_release(&arguments->function->dispex);
+    jsdisp_release(&arguments->function->function.dispex);
     heap_free(arguments);
 }
 
@@ -122,7 +138,8 @@ static HRESULT Arguments_idx_get(jsdisp_t *jsdisp, unsigned idx, jsval_t *r)
         return jsval_copy(*ref, r);
 
     /* FIXME: Accessing by name won't work for duplicated argument names */
-    return jsdisp_propget_name(arguments->frame->base_scope->jsobj, arguments->function->func_code->params[idx], r);
+    return jsdisp_propget_name(arguments->frame->base_scope->jsobj,
+                               arguments->function->func_code->params[idx], r);
 }
 
 static HRESULT Arguments_idx_put(jsdisp_t *jsdisp, unsigned idx, jsval_t val)
@@ -145,12 +162,13 @@ static HRESULT Arguments_idx_put(jsdisp_t *jsdisp, unsigned idx, jsval_t val)
     }
 
     /* FIXME: Accessing by name won't work for duplicated argument names */
-    return jsdisp_propput_name(arguments->frame->base_scope->jsobj, arguments->function->func_code->params[idx], val);
+    return jsdisp_propput_name(arguments->frame->base_scope->jsobj,
+                               arguments->function->func_code->params[idx], val);
 }
 
 static const builtin_info_t Arguments_info = {
     JSCLASS_ARGUMENTS,
-    {NULL, Arguments_value, 0},
+    Arguments_value,
     0, NULL,
     Arguments_destructor,
     NULL,
@@ -164,8 +182,6 @@ HRESULT setup_arguments_object(script_ctx_t *ctx, call_frame_t *frame)
     ArgumentsInstance *args;
     HRESULT hres;
 
-    static const WCHAR caleeW[] = {'c','a','l','l','e','e',0};
-
     args = heap_alloc_zero(sizeof(*args));
     if(!args)
         return E_OUTOFMEMORY;
@@ -176,17 +192,17 @@ HRESULT setup_arguments_object(script_ctx_t *ctx, call_frame_t *frame)
         return hres;
     }
 
-    args->function = function_from_jsdisp(jsdisp_addref(frame->function_instance));
+    args->function = (InterpretedFunction*)function_from_jsdisp(jsdisp_addref(frame->function_instance));
     args->argc = frame->argc;
     args->frame = frame;
 
-    hres = jsdisp_define_data_property(&args->jsdisp, lengthW, PROPF_WRITABLE | PROPF_CONFIGURABLE,
+    hres = jsdisp_define_data_property(&args->jsdisp, L"length", PROPF_WRITABLE | PROPF_CONFIGURABLE,
                                        jsval_number(args->argc));
     if(SUCCEEDED(hres))
-        hres = jsdisp_define_data_property(&args->jsdisp, caleeW, PROPF_WRITABLE | PROPF_CONFIGURABLE,
-                                           jsval_obj(&args->function->dispex));
+        hres = jsdisp_define_data_property(&args->jsdisp, L"callee", PROPF_WRITABLE | PROPF_CONFIGURABLE,
+                                           jsval_obj(&args->function->function.dispex));
     if(SUCCEEDED(hres))
-        hres = jsdisp_propput(frame->base_scope->jsobj, argumentsW, PROPF_WRITABLE, jsval_obj(&args->jsdisp));
+        hres = jsdisp_propput(frame->base_scope->jsobj, L"arguments", PROPF_WRITABLE, TRUE, jsval_obj(&args->jsdisp));
     if(FAILED(hres)) {
         jsdisp_release(&args->jsdisp);
         return hres;
@@ -205,7 +221,7 @@ void detach_arguments_object(jsdisp_t *args_disp)
 
     /* Reset arguments value to cut the reference cycle. Note that since all activation contexts have
      * their own arguments property, it's impossible to use prototype's one during name lookup */
-    jsdisp_propput_name(frame->base_scope->jsobj, argumentsW, jsval_undefined());
+    jsdisp_propput_name(frame->base_scope->jsobj, L"arguments", jsval_undefined());
     arguments->frame = NULL;
 
     /* Don't bother coppying arguments if call frame holds the last reference. */
@@ -231,99 +247,8 @@ void detach_arguments_object(jsdisp_t *args_disp)
     jsdisp_release(frame->arguments_obj);
 }
 
-static HRESULT invoke_source(script_ctx_t *ctx, FunctionInstance *function, IDispatch *this_obj, unsigned argc, jsval_t *argv,
-        BOOL is_constructor, BOOL caller_execs_source, jsval_t *r)
-{
-    jsdisp_t *var_disp;
-    DWORD exec_flags = 0;
-    HRESULT hres;
-
-    if(ctx->state == SCRIPTSTATE_UNINITIALIZED || ctx->state == SCRIPTSTATE_CLOSED) {
-        WARN("Script engine state does not allow running code.\n");
-        return E_UNEXPECTED;
-    }
-
-    if(!function->func_code) {
-        FIXME("no source\n");
-        return E_FAIL;
-    }
-
-    hres = create_dispex(ctx, NULL, NULL, &var_disp);
-    if(FAILED(hres))
-        return hres;
-
-    if(caller_execs_source)
-        exec_flags |= EXEC_RETURN_TO_INTERP;
-    if(is_constructor)
-        exec_flags |= EXEC_CONSTRUCTOR;
-    hres = exec_source(ctx, exec_flags, function->code, function->func_code, function->scope_chain, this_obj,
-            &function->dispex, var_disp, argc, argv, r);
-
-    jsdisp_release(var_disp);
-    return hres;
-}
-
-static HRESULT invoke_value_proc(script_ctx_t *ctx, FunctionInstance *function, IDispatch *this_disp, WORD flags,
-        unsigned argc, jsval_t *argv, jsval_t *r)
-{
-    vdisp_t vthis;
-    HRESULT hres;
-
-    if(this_disp)
-        set_disp(&vthis, this_disp);
-    else if(ctx->host_global)
-        set_disp(&vthis, ctx->host_global);
-    else
-        set_jsdisp(&vthis, ctx->global);
-
-    hres = function->value_proc(ctx, &vthis, flags, argc, argv, r);
-
-    vdisp_release(&vthis);
-    return hres;
-}
-
-static HRESULT call_function(script_ctx_t *ctx, FunctionInstance *function, IDispatch *this_obj,
-        unsigned argc, jsval_t *argv, BOOL caller_execs_source, jsval_t *r)
-{
-    if(function->value_proc)
-        return invoke_value_proc(ctx, function, this_obj, DISPATCH_METHOD, argc, argv, r);
-
-    return invoke_source(ctx, function, this_obj, argc, argv, FALSE, caller_execs_source, r);
-}
-
-static HRESULT function_to_string(FunctionInstance *function, jsstr_t **ret)
-{
-    jsstr_t *str;
-
-    static const WCHAR native_prefixW[] = {'\n','f','u','n','c','t','i','o','n',' '};
-    static const WCHAR native_suffixW[] =
-        {'(',')',' ','{','\n',' ',' ',' ',' ','[','n','a','t','i','v','e',' ','c','o','d','e',']','\n','}','\n'};
-
-    if(function->value_proc) {
-        DWORD name_len;
-        WCHAR *ptr;
-
-        name_len = strlenW(function->name);
-        str = jsstr_alloc_buf(ARRAY_SIZE(native_prefixW) + ARRAY_SIZE(native_suffixW) + name_len, &ptr);
-        if(!str)
-            return E_OUTOFMEMORY;
-
-        memcpy(ptr, native_prefixW, sizeof(native_prefixW));
-        memcpy(ptr += ARRAY_SIZE(native_prefixW), function->name, name_len*sizeof(WCHAR));
-        memcpy(ptr + name_len, native_suffixW, sizeof(native_suffixW));
-    }else {
-        str = jsstr_alloc_len(function->func_code->source, function->func_code->source_len);
-        if(!str)
-            return E_OUTOFMEMORY;
-    }
-
-    *ret = str;
-    return S_OK;
-}
-
 HRESULT Function_invoke(jsdisp_t *func_this, IDispatch *jsthis, WORD flags, unsigned argc, jsval_t *argv, jsval_t *r)
 {
-    const BOOL caller_execs_source = (flags & DISPATCH_JSCRIPT_CALLEREXECSSOURCE) != 0;
     FunctionInstance *function;
 
     TRACE("func %p this %p\n", func_this, jsthis);
@@ -331,25 +256,7 @@ HRESULT Function_invoke(jsdisp_t *func_this, IDispatch *jsthis, WORD flags, unsi
     assert(is_class(func_this, JSCLASS_FUNCTION));
     function = function_from_jsdisp(func_this);
 
-    flags &= ~DISPATCH_JSCRIPT_INTERNAL_MASK;
-    if(function->value_proc)
-        return invoke_value_proc(function->dispex.ctx, function, jsthis, flags, argc, argv, r);
-
-    if(flags == DISPATCH_CONSTRUCT) {
-        jsdisp_t *this_obj;
-        HRESULT hres;
-
-        hres = create_object(function->dispex.ctx, &function->dispex, &this_obj);
-        if(FAILED(hres))
-            return hres;
-
-        hres = invoke_source(function->dispex.ctx, function, to_disp(this_obj), argc, argv, TRUE, caller_execs_source, r);
-        jsdisp_release(this_obj);
-        return hres;
-    }
-
-    assert(flags == DISPATCH_METHOD);
-    return invoke_source(function->dispex.ctx, function, jsthis, argc, argv, FALSE, caller_execs_source, r);
+    return function->vtbl->call(function->dispex.ctx, function, jsthis ? jsval_disp(jsthis) : jsval_null(), flags, argc, argv, r);
 }
 
 static HRESULT Function_get_length(script_ctx_t *ctx, jsdisp_t *jsthis, jsval_t *r)
@@ -360,7 +267,7 @@ static HRESULT Function_get_length(script_ctx_t *ctx, jsdisp_t *jsthis, jsval_t
     return S_OK;
 }
 
-static HRESULT Function_toString(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, unsigned argc, jsval_t *argv,
+static HRESULT Function_toString(script_ctx_t *ctx, jsval_t vthis, WORD flags, unsigned argc, jsval_t *argv,
         jsval_t *r)
 {
     FunctionInstance *function;
@@ -369,10 +276,10 @@ static HRESULT Function_toString(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags,
 
     TRACE("\n");
 
-    if(!(function = function_this(jsthis)))
-        return throw_type_error(ctx, JS_E_FUNCTION_EXPECTED, NULL);
+    if(!(function = function_this(vthis)))
+        return JS_E_FUNCTION_EXPECTED;
 
-    hres = function_to_string(function, &str);
+    hres = function->vtbl->toString(function, &str);
     if(FAILED(hres))
         return hres;
 
@@ -386,10 +293,10 @@ static HRESULT Function_toString(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags,
 static HRESULT array_to_args(script_ctx_t *ctx, jsdisp_t *arg_array, unsigned *argc, jsval_t **ret)
 {
     jsval_t *argv, val;
-    DWORD length, i;
+    UINT32 length, i;
     HRESULT hres;
 
-    hres = jsdisp_propget_name(arg_array, lengthW, &val);
+    hres = jsdisp_propget_name(arg_array, L"length", &val);
     if(FAILED(hres))
         return hres;
 
@@ -419,24 +326,32 @@ static HRESULT array_to_args(script_ctx_t *ctx, jsdisp_t *arg_array, unsigned *a
     return S_OK;
 }
 
-static HRESULT Function_apply(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, unsigned argc, jsval_t *argv, jsval_t *r)
+static HRESULT Function_apply(script_ctx_t *ctx, jsval_t vthis, WORD flags, unsigned argc, jsval_t *argv, jsval_t *r)
 {
+    jsval_t this_val = jsval_undefined();
     FunctionInstance *function;
     jsval_t *args = NULL;
     unsigned i, cnt = 0;
-    IDispatch *this_obj = NULL;
     HRESULT hres = S_OK;
 
     TRACE("\n");
 
-    if(!(function = function_this(jsthis)) && (jsthis->flags & VDISP_JSDISP))
-        return throw_type_error(ctx, JS_E_FUNCTION_EXPECTED, NULL);
+    if(is_null_disp(vthis))
+        return JS_E_OBJECT_REQUIRED;
+    if(!is_object_instance(vthis) || (!(function = function_this(vthis)) && to_jsdisp(get_object(vthis))))
+        return JS_E_FUNCTION_EXPECTED;
 
     if(argc) {
-        if(!is_undefined(argv[0]) && !is_null(argv[0])) {
+        if(ctx->version < SCRIPTLANGUAGEVERSION_ES5 && !is_undefined(argv[0]) && !is_null(argv[0])) {
+            IDispatch *this_obj;
             hres = to_object(ctx, argv[0], &this_obj);
             if(FAILED(hres))
                 return hres;
+            this_val = jsval_disp(this_obj);
+        }else {
+            hres = jsval_copy(argv[0], &this_val);
+            if(FAILED(hres))
+                return hres;
         }
     }
 
@@ -463,10 +378,11 @@ static HRESULT Function_apply(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, un
 
     if(SUCCEEDED(hres)) {
         if(function) {
-            hres = call_function(ctx, function, this_obj, cnt, args, (flags & DISPATCH_JSCRIPT_CALLEREXECSSOURCE) != 0, r);
+            hres = function->vtbl->call(ctx, function, this_val, flags, cnt, args, r);
         }else {
             jsval_t res;
-            hres = disp_call_value(ctx, jsthis->u.disp, this_obj, DISPATCH_METHOD, cnt, args, &res);
+            hres = disp_call_value(ctx, get_object(vthis), is_object_instance(this_val) ? get_object(this_val) : NULL,
+                                   DISPATCH_METHOD, cnt, args, &res);
             if(SUCCEEDED(hres)) {
                 if(r)
                     *r = res;
@@ -476,70 +392,109 @@ static HRESULT Function_apply(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, un
         }
     }
 
-    if(this_obj)
-        IDispatch_Release(this_obj);
+    jsval_release(this_val);
     for(i=0; i < cnt; i++)
         jsval_release(args[i]);
     heap_free(args);
     return hres;
 }
 
-static HRESULT Function_call(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, unsigned argc, jsval_t *argv,
+static HRESULT Function_call(script_ctx_t *ctx, jsval_t vthis, WORD flags, unsigned argc, jsval_t *argv,
         jsval_t *r)
 {
+    jsval_t this_val = jsval_undefined();
     FunctionInstance *function;
-    IDispatch *this_obj = NULL;
     unsigned cnt = 0;
     HRESULT hres;
 
     TRACE("\n");
 
-    if(!(function = function_this(jsthis)))
-        return throw_type_error(ctx, JS_E_FUNCTION_EXPECTED, NULL);
+    if(is_null_disp(vthis))
+        return JS_E_OBJECT_REQUIRED;
+    if(!(function = function_this(vthis)))
+        return JS_E_FUNCTION_EXPECTED;
 
     if(argc) {
-        if(!is_undefined(argv[0]) && !is_null(argv[0])) {
+        if(ctx->version < SCRIPTLANGUAGEVERSION_ES5 && !is_undefined(argv[0]) && !is_null(argv[0])) {
+            IDispatch *this_obj;
             hres = to_object(ctx, argv[0], &this_obj);
             if(FAILED(hres))
                 return hres;
+            this_val = jsval_disp(this_obj);
+        }else {
+            hres = jsval_copy(argv[0], &this_val);
+            if(FAILED(hres))
+                return hres;
         }
-
         cnt = argc-1;
     }
 
-    hres = call_function(ctx, function, this_obj, cnt, argv+1, (flags & DISPATCH_JSCRIPT_CALLEREXECSSOURCE) != 0, r);
+    hres = function->vtbl->call(ctx, function, this_val, flags, cnt, argv + 1, r);
 
-    if(this_obj)
-        IDispatch_Release(this_obj);
+    jsval_release(this_val);
     return hres;
 }
 
-HRESULT Function_value(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, unsigned argc, jsval_t *argv,
+static HRESULT Function_bind(script_ctx_t *ctx, jsval_t vthis, WORD flags, unsigned argc, jsval_t *argv,
+        jsval_t *r)
+{
+    IDispatch *bound_this = NULL;
+    FunctionInstance *function;
+    jsdisp_t *new_function;
+    HRESULT hres;
+
+    TRACE("\n");
+
+    if(!(function = function_this(vthis)))
+        return JS_E_FUNCTION_EXPECTED;
+
+    if(argc < 1) {
+        FIXME("no this argument\n");
+        return E_NOTIMPL;
+    }
+
+    if(is_object_instance(argv[0])) {
+        bound_this = get_object(argv[0]);
+    }else if(!is_null(argv[0])) {
+        FIXME("%s is not an object instance\n", debugstr_jsval(argv[0]));
+        return E_NOTIMPL;
+    }
+
+    hres = create_bind_function(ctx, function, bound_this, argc - 1, argv + 1, &new_function);
+    if(FAILED(hres))
+        return hres;
+
+    if(r)
+        *r = jsval_obj(new_function);
+    else
+        jsdisp_release(new_function);
+    return S_OK;
+}
+
+HRESULT Function_value(script_ctx_t *ctx, jsval_t vthis, WORD flags, unsigned argc, jsval_t *argv,
         jsval_t *r)
 {
     FunctionInstance *function;
 
     TRACE("\n");
 
-    if(!is_vclass(jsthis, JSCLASS_FUNCTION)) {
+    if(!(function = function_this(vthis))) {
         ERR("dispex is not a function\n");
         return E_FAIL;
     }
 
-    function = function_from_jsdisp(jsthis->u.jsdisp);
-
-    assert(function->value_proc != NULL);
-    return invoke_value_proc(ctx, function, NULL, flags, argc, argv, r);
+    return function->vtbl->call(ctx, function, vthis, flags, argc, argv, r);
 }
 
 HRESULT Function_get_value(script_ctx_t *ctx, jsdisp_t *jsthis, jsval_t *r)
 {
+    FunctionInstance *function = function_from_jsdisp(jsthis);
     jsstr_t *str;
     HRESULT hres;
 
     TRACE("\n");
 
-    hres = function_to_string(function_from_jsdisp(jsthis), &str);
+    hres = function->vtbl->toString(function, &str);
     if(FAILED(hres))
         return hres;
 
@@ -571,28 +526,35 @@ static HRESULT Function_get_arguments(script_ctx_t *ctx, jsdisp_t *jsthis, jsval
     return S_OK;
 }
 
+function_code_t *Function_get_code(jsdisp_t *jsthis)
+{
+    FunctionInstance *function;
+
+    assert(is_class(jsthis, JSCLASS_FUNCTION));
+    function = function_from_jsdisp(jsthis);
+
+    return function->vtbl->get_code(function);
+}
+
 static void Function_destructor(jsdisp_t *dispex)
 {
-    FunctionInstance *This = function_from_jsdisp(dispex);
-
-    if(This->code)
-        release_bytecode(This->code);
-    if(This->scope_chain)
-        scope_release(This->scope_chain);
-    heap_free(This);
+    FunctionInstance *function = function_from_jsdisp(dispex);
+    function->vtbl->destructor(function);
+    heap_free(function);
 }
 
 static const builtin_prop_t Function_props[] = {
-    {applyW,                 Function_apply,                 PROPF_METHOD|2},
-    {argumentsW,             NULL, 0,                        Function_get_arguments},
-    {callW,                  Function_call,                  PROPF_METHOD|1},
-    {lengthW,                NULL, 0,                        Function_get_length},
-    {toStringW,              Function_toString,              PROPF_METHOD}
+    {L"apply",               Function_apply,                 PROPF_METHOD|2},
+    {L"arguments",           NULL, 0,                        Function_get_arguments},
+    {L"bind",                Function_bind,                  PROPF_METHOD|PROPF_ES5|1},
+    {L"call",                Function_call,                  PROPF_METHOD|1},
+    {L"length",              NULL, 0,                        Function_get_length},
+    {L"toString",            Function_toString,              PROPF_METHOD}
 };
 
 static const builtin_info_t Function_info = {
     JSCLASS_FUNCTION,
-    DEFAULT_FUNCTION_VALUE,
+    Function_value,
     ARRAY_SIZE(Function_props),
     Function_props,
     Function_destructor,
@@ -600,26 +562,26 @@ static const builtin_info_t Function_info = {
 };
 
 static const builtin_prop_t FunctionInst_props[] = {
-    {argumentsW,             NULL, 0,                        Function_get_arguments},
-    {lengthW,                NULL, 0,                        Function_get_length}
+    {L"arguments",           NULL, 0,                        Function_get_arguments},
+    {L"length",              NULL, 0,                        Function_get_length}
 };
 
 static const builtin_info_t FunctionInst_info = {
     JSCLASS_FUNCTION,
-    DEFAULT_FUNCTION_VALUE,
+    Function_value,
     ARRAY_SIZE(FunctionInst_props),
     FunctionInst_props,
     Function_destructor,
     NULL
 };
 
-static HRESULT create_function(script_ctx_t *ctx, const builtin_info_t *builtin_info, DWORD flags,
-        BOOL funcprot, jsdisp_t *prototype, FunctionInstance **ret)
+static HRESULT create_function(script_ctx_t *ctx, const builtin_info_t *builtin_info, const function_vtbl_t *vtbl, size_t size,
+        DWORD flags, BOOL funcprot, jsdisp_t *prototype, void **ret)
 {
     FunctionInstance *function;
     HRESULT hres;
 
-    function = heap_alloc_zero(sizeof(FunctionInstance));
+    function = heap_alloc_zero(size);
     if(!function)
         return E_OUTOFMEMORY;
 
@@ -634,6 +596,7 @@ static HRESULT create_function(script_ctx_t *ctx, const builtin_info_t *builtin_
         return hres;
     }
 
+    function->vtbl = vtbl;
     function->flags = flags;
     function->length = flags & PROPF_ARGMASK;
 
@@ -641,38 +604,85 @@ static HRESULT create_function(script_ctx_t *ctx, const builtin_info_t *builtin_
     return S_OK;
 }
 
+static HRESULT NativeFunction_call(script_ctx_t *ctx, FunctionInstance *func, jsval_t vthis, unsigned flags,
+        unsigned argc, jsval_t *argv, jsval_t *r)
+{
+    NativeFunction *function = (NativeFunction*)func;
+
+    return function->proc(ctx, vthis, flags & ~DISPATCH_JSCRIPT_INTERNAL_MASK, argc, argv, r);
+}
+
+static HRESULT NativeFunction_toString(FunctionInstance *func, jsstr_t **ret)
+{
+    NativeFunction *function = (NativeFunction*)func;
+    DWORD name_len;
+    jsstr_t *str;
+    WCHAR *ptr;
+
+    static const WCHAR native_prefixW[] = L"\nfunction ";
+    static const WCHAR native_suffixW[] = L"() {\n    [native code]\n}\n";
+
+    name_len = function->name ? lstrlenW(function->name) : 0;
+    str = jsstr_alloc_buf(ARRAY_SIZE(native_prefixW) + ARRAY_SIZE(native_suffixW) + name_len - 2, &ptr);
+    if(!str)
+        return E_OUTOFMEMORY;
+
+    memcpy(ptr, native_prefixW, sizeof(native_prefixW));
+    ptr += ARRAY_SIZE(native_prefixW) - 1;
+    memcpy(ptr, function->name, name_len*sizeof(WCHAR));
+    ptr += name_len;
+    memcpy(ptr, native_suffixW, sizeof(native_suffixW));
+
+    *ret = str;
+    return S_OK;
+}
+
+static function_code_t *NativeFunction_get_code(FunctionInstance *function)
+{
+    return NULL;
+}
+
+static void NativeFunction_destructor(FunctionInstance *function)
+{
+}
+
+static const function_vtbl_t NativeFunctionVtbl = {
+    NativeFunction_call,
+    NativeFunction_toString,
+    NativeFunction_get_code,
+    NativeFunction_destructor
+};
+
 HRESULT create_builtin_function(script_ctx_t *ctx, builtin_invoke_t value_proc, const WCHAR *name,
         const builtin_info_t *builtin_info, DWORD flags, jsdisp_t *prototype, jsdisp_t **ret)
 {
-    FunctionInstance *function;
+    NativeFunction *function;
     HRESULT hres;
 
-    hres = create_function(ctx, builtin_info, flags, FALSE, NULL, &function);
+    hres = create_function(ctx, builtin_info, &NativeFunctionVtbl, sizeof(NativeFunction), flags, FALSE, NULL, (void**)&function);
     if(FAILED(hres))
         return hres;
 
     if(builtin_info)
-        hres = jsdisp_define_data_property(&function->dispex, lengthW, 0,
-                                           jsval_number(function->length));
+        hres = jsdisp_define_data_property(&function->function.dispex, L"length", 0,
+                                           jsval_number(function->function.length));
     if(SUCCEEDED(hres))
-        hres = jsdisp_define_data_property(&function->dispex, prototypeW, 0, jsval_obj(prototype));
+        hres = jsdisp_define_data_property(&function->function.dispex, L"prototype", 0, prototype ? jsval_obj(prototype) : jsval_null());
     if(FAILED(hres)) {
-        jsdisp_release(&function->dispex);
+        jsdisp_release(&function->function.dispex);
         return hres;
     }
 
-    function->value_proc = value_proc;
+    function->proc = value_proc;
     function->name = name;
 
-    *ret = &function->dispex;
+    *ret = &function->function.dispex;
     return S_OK;
 }
 
 static HRESULT set_constructor_prop(script_ctx_t *ctx, jsdisp_t *constr, jsdisp_t *prot)
 {
-    static const WCHAR constructorW[] = {'c','o','n','s','t','r','u','c','t','o','r',0};
-
-    return jsdisp_define_data_property(prot, constructorW, PROPF_WRITABLE | PROPF_CONFIGURABLE,
+    return jsdisp_define_data_property(prot, L"constructor", PROPF_WRITABLE | PROPF_CONFIGURABLE,
                                        jsval_obj(constr));
 }
 
@@ -696,10 +706,82 @@ HRESULT create_builtin_constructor(script_ctx_t *ctx, builtin_invoke_t value_pro
     return S_OK;
 }
 
+static HRESULT InterpretedFunction_call(script_ctx_t *ctx, FunctionInstance *func, jsval_t vthis, unsigned flags,
+         unsigned argc, jsval_t *argv, jsval_t *r)
+{
+    InterpretedFunction *function = (InterpretedFunction*)func;
+    IDispatch *this_obj = NULL;
+    DWORD exec_flags = 0;
+    jsdisp_t *new_obj;
+    HRESULT hres;
+
+    TRACE("%p\n", function);
+
+    if(ctx->state == SCRIPTSTATE_UNINITIALIZED || ctx->state == SCRIPTSTATE_CLOSED) {
+        WARN("Script engine state does not allow running code.\n");
+        return E_UNEXPECTED;
+    }
+
+    if(flags & DISPATCH_CONSTRUCT) {
+        hres = create_object(ctx, &function->function.dispex, &new_obj);
+        if(FAILED(hres))
+            return hres;
+        this_obj = to_disp(new_obj);
+    }else if(is_object_instance(vthis)) {
+        this_obj = get_object(vthis);
+        IDispatch_AddRef(this_obj);
+    }else if(ctx->version >= SCRIPTLANGUAGEVERSION_ES5 && !is_undefined(vthis) && !is_null(vthis)) {
+        hres = to_object(ctx, vthis, &this_obj);
+        if(FAILED(hres))
+            return hres;
+    }
+
+    if(flags & DISPATCH_JSCRIPT_CALLEREXECSSOURCE)
+        exec_flags |= EXEC_RETURN_TO_INTERP;
+    if(flags & DISPATCH_CONSTRUCT)
+        exec_flags |= EXEC_CONSTRUCTOR;
+    hres = exec_source(ctx, exec_flags, function->code, function->func_code, function->scope_chain, this_obj,
+                       &function->function.dispex, argc, argv, r);
+    if(this_obj)
+        IDispatch_Release(this_obj);
+    return hres;
+}
+
+static HRESULT InterpretedFunction_toString(FunctionInstance *func, jsstr_t **ret)
+{
+    InterpretedFunction *function = (InterpretedFunction*)func;
+
+    *ret = jsstr_alloc_len(function->func_code->source, function->func_code->source_len);
+    return *ret ? S_OK : E_OUTOFMEMORY;
+}
+
+static function_code_t *InterpretedFunction_get_code(FunctionInstance *func)
+{
+    InterpretedFunction *function = (InterpretedFunction*)func;
+
+    return function->func_code;
+}
+
+static void InterpretedFunction_destructor(FunctionInstance *func)
+{
+    InterpretedFunction *function = (InterpretedFunction*)func;
+
+    release_bytecode(function->code);
+    if(function->scope_chain)
+        scope_release(function->scope_chain);
+}
+
+static const function_vtbl_t InterpretedFunctionVtbl = {
+    InterpretedFunction_call,
+    InterpretedFunction_toString,
+    InterpretedFunction_get_code,
+    InterpretedFunction_destructor
+};
+
 HRESULT create_source_function(script_ctx_t *ctx, bytecode_t *code, function_code_t *func_code,
         scope_chain_t *scope_chain, jsdisp_t **ret)
 {
-    FunctionInstance *function;
+    InterpretedFunction *function;
     jsdisp_t *prototype;
     HRESULT hres;
 
@@ -707,14 +789,15 @@ HRESULT create_source_function(script_ctx_t *ctx, bytecode_t *code, function_cod
     if(FAILED(hres))
         return hres;
 
-    hres = create_function(ctx, NULL, PROPF_CONSTR, FALSE, NULL, &function);
+    hres = create_function(ctx, NULL, &InterpretedFunctionVtbl, sizeof(InterpretedFunction), PROPF_CONSTR,
+                           FALSE, NULL, (void**)&function);
     if(SUCCEEDED(hres)) {
-        hres = jsdisp_define_data_property(&function->dispex, prototypeW, PROPF_WRITABLE,
+        hres = jsdisp_define_data_property(&function->function.dispex, L"prototype", PROPF_WRITABLE,
                                            jsval_obj(prototype));
         if(SUCCEEDED(hres))
-            hres = set_constructor_prop(ctx, &function->dispex, prototype);
+            hres = set_constructor_prop(ctx, &function->function.dispex, prototype);
         if(FAILED(hres))
-            jsdisp_release(&function->dispex);
+            jsdisp_release(&function->function.dispex);
     }
     jsdisp_release(prototype);
     if(FAILED(hres))
@@ -728,9 +811,101 @@ HRESULT create_source_function(script_ctx_t *ctx, bytecode_t *code, function_cod
     bytecode_addref(code);
     function->code = code;
     function->func_code = func_code;
-    function->length = function->func_code->param_cnt;
+    function->function.length = function->func_code->param_cnt;
 
-    *ret = &function->dispex;
+    *ret = &function->function.dispex;
+    return S_OK;
+}
+
+static HRESULT BindFunction_call(script_ctx_t *ctx, FunctionInstance *func, jsval_t vthis, unsigned flags,
+         unsigned argc, jsval_t *argv, jsval_t *r)
+{
+    BindFunction *function = (BindFunction*)func;
+    jsval_t *call_args = NULL;
+    unsigned call_argc;
+    HRESULT hres;
+
+    TRACE("%p\n", function);
+
+    call_argc = function->argc + argc;
+    if(call_argc) {
+        call_args = heap_alloc(call_argc * sizeof(*call_args));
+        if(!call_args)
+            return E_OUTOFMEMORY;
+
+        if(function->argc)
+            memcpy(call_args, function->args, function->argc * sizeof(*call_args));
+        if(argc)
+            memcpy(call_args + function->argc, argv, argc * sizeof(*call_args));
+    }
+
+    hres = function->target->vtbl->call(ctx, function->target, function->this ? jsval_disp(function->this) : jsval_null(),
+                                        flags, call_argc, call_args, r);
+
+    heap_free(call_args);
+    return hres;
+}
+
+static HRESULT BindFunction_toString(FunctionInstance *function, jsstr_t **ret)
+{
+    *ret = jsstr_alloc(L"\nfunction() {\n    [native code]\n}\n");
+    return *ret ? S_OK : E_OUTOFMEMORY;
+}
+
+static function_code_t *BindFunction_get_code(FunctionInstance *function)
+{
+    return NULL;
+}
+
+static void BindFunction_destructor(FunctionInstance *func)
+{
+    BindFunction *function = (BindFunction*)func;
+    unsigned i;
+
+    TRACE("%p\n", function);
+
+    for(i = 0; i < function->argc; i++)
+        jsval_release(function->args[i]);
+    jsdisp_release(&function->target->dispex);
+    if(function->this)
+        IDispatch_Release(function->this);
+}
+
+static const function_vtbl_t BindFunctionVtbl = {
+    BindFunction_call,
+    BindFunction_toString,
+    BindFunction_get_code,
+    BindFunction_destructor
+};
+
+static HRESULT create_bind_function(script_ctx_t *ctx, FunctionInstance *target, IDispatch *bound_this, unsigned argc,
+                                    jsval_t *argv, jsdisp_t **ret)
+{
+    BindFunction *function;
+    HRESULT hres;
+
+    hres = create_function(ctx, NULL, &BindFunctionVtbl, FIELD_OFFSET(BindFunction, args[argc]), PROPF_METHOD,
+                           FALSE, NULL, (void**)&function);
+    if(FAILED(hres))
+        return hres;
+
+    jsdisp_addref(&target->dispex);
+    function->target = target;
+
+    if(bound_this)
+        IDispatch_AddRef(function->this = bound_this);
+
+    for(function->argc = 0; function->argc < argc; function->argc++) {
+        hres = jsval_copy(argv[function->argc], function->args + function->argc);
+        if(FAILED(hres)) {
+            jsdisp_release(&function->function.dispex);
+            return hres;
+        }
+    }
+
+    function->function.length = target->length > argc ? target->length - argc : 0;
+
+    *ret = &function->function.dispex;
     return S_OK;
 }
 
@@ -744,9 +919,9 @@ static HRESULT construct_function(script_ctx_t *ctx, unsigned argc, jsval_t *arg
     int j = 0;
     HRESULT hres = S_OK;
 
-    static const WCHAR function_anonymousW[] = {'f','u','n','c','t','i','o','n',' ','a','n','o','n','y','m','o','u','s','('};
-    static const WCHAR function_beginW[] = {')',' ','{','\n'};
-    static const WCHAR function_endW[] = {'\n','}',0};
+    static const WCHAR function_anonymousW[] = L"function anonymous(";
+    static const WCHAR function_beginW[] = L") {\n";
+    static const WCHAR function_endW[] = L"\n}";
 
     if(argc) {
         params = heap_alloc(argc*sizeof(*params));
@@ -764,11 +939,11 @@ static HRESULT construct_function(script_ctx_t *ctx, unsigned argc, jsval_t *arg
     }
 
     if(SUCCEEDED(hres)) {
-        len += ARRAY_SIZE(function_anonymousW) + ARRAY_SIZE(function_beginW) + ARRAY_SIZE(function_endW);
+        len += ARRAY_SIZE(function_anonymousW) + ARRAY_SIZE(function_beginW) + ARRAY_SIZE(function_endW) - 2;
         str = heap_alloc(len*sizeof(WCHAR));
         if(str) {
             memcpy(str, function_anonymousW, sizeof(function_anonymousW));
-            ptr = str + ARRAY_SIZE(function_anonymousW);
+            ptr = str + ARRAY_SIZE(function_anonymousW) - 1;
             if(argc > 1) {
                 while(1) {
                     ptr += jsstr_flush(params[j], ptr);
@@ -779,7 +954,7 @@ static HRESULT construct_function(script_ctx_t *ctx, unsigned argc, jsval_t *arg
                 }
             }
             memcpy(ptr, function_beginW, sizeof(function_beginW));
-            ptr += ARRAY_SIZE(function_beginW);
+            ptr += ARRAY_SIZE(function_beginW) - 1;
             if(argc)
                 ptr += jsstr_flush(params[argc-1], ptr);
             memcpy(ptr, function_endW, sizeof(function_endW));
@@ -796,7 +971,8 @@ static HRESULT construct_function(script_ctx_t *ctx, unsigned argc, jsval_t *arg
     if(FAILED(hres))
         return hres;
 
-    hres = compile_script(ctx, str, NULL, NULL, FALSE, FALSE, &code);
+    hres = compile_script(ctx, str, 0, 0, NULL, NULL, FALSE, FALSE,
+                          ctx->call_ctx ? ctx->call_ctx->bytecode->named_item : NULL, &code);
     heap_free(str);
     if(FAILED(hres))
         return hres;
@@ -816,7 +992,7 @@ static HRESULT construct_function(script_ctx_t *ctx, unsigned argc, jsval_t *arg
     return S_OK;
 }
 
-static HRESULT FunctionConstr_value(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, unsigned argc, jsval_t *argv,
+static HRESULT FunctionConstr_value(script_ctx_t *ctx, jsval_t vthis, WORD flags, unsigned argc, jsval_t *argv,
         jsval_t *r)
 {
     HRESULT hres;
@@ -832,7 +1008,8 @@ static HRESULT FunctionConstr_value(script_ctx_t *ctx, vdisp_t *jsthis, WORD fla
         if(FAILED(hres))
             return hres;
 
-        *r = jsval_disp(ret);
+        if(r) *r = jsval_disp(ret);
+        else  IDispatch_Release(ret);
         break;
     }
     default:
@@ -843,7 +1020,7 @@ static HRESULT FunctionConstr_value(script_ctx_t *ctx, vdisp_t *jsthis, WORD fla
     return S_OK;
 }
 
-static HRESULT FunctionProt_value(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, unsigned argc, jsval_t *argv,
+static HRESULT FunctionProt_value(script_ctx_t *ctx, jsval_t vthis, WORD flags, unsigned argc, jsval_t *argv,
         jsval_t *r)
 {
     FIXME("\n");
@@ -852,32 +1029,32 @@ static HRESULT FunctionProt_value(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags
 
 HRESULT init_function_constr(script_ctx_t *ctx, jsdisp_t *object_prototype)
 {
-    FunctionInstance *prot, *constr;
+    NativeFunction *prot, *constr;
     HRESULT hres;
 
-    static const WCHAR FunctionW[] = {'F','u','n','c','t','i','o','n',0};
-
-    hres = create_function(ctx, &Function_info, PROPF_CONSTR, TRUE, object_prototype, &prot);
+    hres = create_function(ctx, &Function_info, &NativeFunctionVtbl, sizeof(NativeFunction), PROPF_CONSTR,
+                           TRUE, object_prototype, (void**)&prot);
     if(FAILED(hres))
         return hres;
 
-    prot->value_proc = FunctionProt_value;
-    prot->name = prototypeW;
+    prot->proc = FunctionProt_value;
+    prot->name = L"prototype";
 
-    hres = create_function(ctx, &FunctionInst_info, PROPF_CONSTR|1, TRUE, &prot->dispex, &constr);
+    hres = create_function(ctx, &FunctionInst_info, &NativeFunctionVtbl, sizeof(NativeFunction), PROPF_CONSTR|1,
+                           TRUE, &prot->function.dispex, (void**)&constr);
     if(SUCCEEDED(hres)) {
-        constr->value_proc = FunctionConstr_value;
-        constr->name = FunctionW;
-        hres = jsdisp_define_data_property(&constr->dispex, prototypeW, 0, jsval_obj(&prot->dispex));
+        constr->proc = FunctionConstr_value;
+        constr->name = L"Function";
+        hres = jsdisp_define_data_property(&constr->function.dispex, L"prototype", 0, jsval_obj(&prot->function.dispex));
         if(SUCCEEDED(hres))
-            hres = set_constructor_prop(ctx, &constr->dispex, &prot->dispex);
+            hres = set_constructor_prop(ctx, &constr->function.dispex, &prot->function.dispex);
         if(FAILED(hres))
-            jsdisp_release(&constr->dispex);
+            jsdisp_release(&constr->function.dispex);
     }
-    jsdisp_release(&prot->dispex);
+    jsdisp_release(&prot->function.dispex);
     if(FAILED(hres))
         return hres;
 
-    ctx->function_constr = &constr->dispex;
+    ctx->function_constr = &constr->function.dispex;
     return S_OK;
 }
