@@ -22,8 +22,6 @@
 #include <stdarg.h>
 
 #define COBJMACROS
-#include "wine/unicode.h"
-#include "wine/library.h"
 #include "windef.h"
 #include "winbase.h"
 #include "winuser.h"
@@ -32,6 +30,7 @@
 #include "ole2.h"
 #include "ocidl.h"
 #include "shellapi.h"
+#include "strongname.h"
 
 #include "initguid.h"
 #include "msxml2.h"
@@ -51,7 +50,13 @@
 WINE_DEFAULT_DEBUG_CHANNEL( mscoree );
 WINE_DECLARE_DEBUG_CHANNEL(winediag);
 
-static HINSTANCE MSCOREE_hInstance;
+struct print_handler_tls
+{
+    int length;
+    char buffer[1018];
+};
+
+static DWORD print_tls_index = TLS_OUT_OF_INDEXES;
 
 typedef HRESULT (*fnCreateInstance)(REFIID riid, LPVOID *ppObj);
 
@@ -81,7 +86,7 @@ static BOOL get_install_root(LPWSTR install_dir)
     if (RegOpenKeyExW(HKEY_LOCAL_MACHINE, dotnet_key, 0, KEY_READ, &key))
         return FALSE;
 
-    len = MAX_PATH;
+    len = MAX_PATH * sizeof(WCHAR);
     if (RegQueryValueExW(key, install_root, 0, NULL, (LPBYTE)install_dir, &len))
     {
         RegCloseKey(key);
@@ -128,7 +133,7 @@ static ULONG WINAPI mscorecf_AddRef(IClassFactory *iface )
     mscorecf *This = impl_from_IClassFactory(iface);
     ULONG ref = InterlockedIncrement(&This->ref);
 
-    TRACE("%p ref=%u\n", This, ref);
+    TRACE("%p ref=%lu\n", This, ref);
 
     return ref;
 }
@@ -138,7 +143,7 @@ static ULONG WINAPI mscorecf_Release(IClassFactory *iface )
     mscorecf *This = impl_from_IClassFactory(iface);
     ULONG ref = InterlockedDecrement(&This->ref);
 
-    TRACE("%p ref=%u\n", This, ref);
+    TRACE("%p ref=%lu\n", This, ref);
 
     if (ref == 0)
     {
@@ -171,7 +176,7 @@ static HRESULT WINAPI mscorecf_CreateInstance(IClassFactory *iface,LPUNKNOWN pOu
     }
     else
     {
-        WARN("Cannot create an instance object. 0x%08x\n", hr);
+        WARN("Cannot create an instance object. 0x%08lx\n", hr);
     }
     return hr;
 }
@@ -199,13 +204,13 @@ HRESULT WINAPI CorBindToRuntimeHost(LPCWSTR pwszVersion, LPCWSTR pwszBuildFlavor
     HRESULT ret;
     ICLRRuntimeInfo *info;
 
-    TRACE("(%s, %s, %s, %p, %d, %s, %s, %p)\n", debugstr_w(pwszVersion),
+    TRACE("(%s, %s, %s, %p, %ld, %s, %s, %p)\n", debugstr_w(pwszVersion),
           debugstr_w(pwszBuildFlavor), debugstr_w(pwszHostConfigFile), pReserved,
           startupFlags, debugstr_guid(rclsid), debugstr_guid(riid), ppv);
 
     *ppv = NULL;
 
-    ret = get_runtime_info(NULL, pwszVersion, pwszHostConfigFile, startupFlags, 0, TRUE, &info);
+    ret = get_runtime_info(NULL, pwszVersion, pwszHostConfigFile, NULL, startupFlags, 0, TRUE, &info);
 
     if (SUCCEEDED(ret))
     {
@@ -217,24 +222,74 @@ HRESULT WINAPI CorBindToRuntimeHost(LPCWSTR pwszVersion, LPCWSTR pwszBuildFlavor
     return ret;
 }
 
+void CDECL mono_print_handler_fn(const char *string, INT is_stdout)
+{
+    struct print_handler_tls *tls = TlsGetValue(print_tls_index);
+
+    if (!tls)
+    {
+        tls = HeapAlloc(GetProcessHeap(), 0, sizeof(*tls));
+        tls->length = 0;
+        TlsSetValue(print_tls_index, tls);
+    }
+
+    while (*string)
+    {
+        int remaining_buffer = sizeof(tls->buffer) - tls->length;
+        int length = strlen(string);
+        const char *newline = memchr(string, '\n', min(length, remaining_buffer));
+
+        if (newline)
+        {
+            length = newline - string + 1;
+            wine_dbg_printf("%.*s%.*s", tls->length, tls->buffer, length, string);
+            tls->length = 0;
+            string += length;
+        }
+        else if (length > remaining_buffer)
+        {
+            /* this would overflow Wine's debug buffer */
+            wine_dbg_printf("%.*s%.*s\n", tls->length, tls->buffer, remaining_buffer, string);
+            tls->length = 0;
+            string += remaining_buffer;
+        }
+        else
+        {
+            memcpy(tls->buffer + tls->length, string, length);
+            tls->length += length;
+            break;
+        }
+    }
+}
+
 BOOL WINAPI DllMain(HINSTANCE hinstDLL, DWORD fdwReason, LPVOID lpvReserved)
 {
-    TRACE("(%p, %d, %p)\n", hinstDLL, fdwReason, lpvReserved);
-
-    MSCOREE_hInstance = hinstDLL;
+    TRACE("(%p, %ld, %p)\n", hinstDLL, fdwReason, lpvReserved);
 
     switch (fdwReason)
     {
-    case DLL_WINE_PREATTACH:
-        return FALSE;  /* prefer native version */
     case DLL_PROCESS_ATTACH:
         runtimehost_init();
-        DisableThreadLibraryCalls(hinstDLL);
+
+        print_tls_index = TlsAlloc();
+
+        if (print_tls_index == TLS_OUT_OF_INDEXES)
+            return FALSE;
+
+        break;
+    case DLL_THREAD_DETACH:
+        if (print_tls_index != TLS_OUT_OF_INDEXES)
+            HeapFree(GetProcessHeap(), 0, TlsGetValue(print_tls_index));
         break;
     case DLL_PROCESS_DETACH:
         expect_no_runtimes();
         if (lpvReserved) break; /* process is terminating */
         runtimehost_uninit();
+        if (print_tls_index != TLS_OUT_OF_INDEXES)
+        {
+            HeapFree(GetProcessHeap(), 0, TlsGetValue(print_tls_index));
+            TlsFree(print_tls_index);
+        }
         break;
     }
     return TRUE;
@@ -242,7 +297,7 @@ BOOL WINAPI DllMain(HINSTANCE hinstDLL, DWORD fdwReason, LPVOID lpvReserved)
 
 __int32 WINAPI _CorExeMain2(PBYTE ptrMemory, DWORD cntMemory, LPWSTR imageName, LPWSTR loaderName, LPWSTR cmdLine)
 {
-    TRACE("(%p, %u, %s, %s, %s)\n", ptrMemory, cntMemory, debugstr_w(imageName), debugstr_w(loaderName), debugstr_w(cmdLine));
+    TRACE("(%p, %lu, %s, %s, %s)\n", ptrMemory, cntMemory, debugstr_w(imageName), debugstr_w(loaderName), debugstr_w(cmdLine));
     FIXME("Directly running .NET applications not supported.\n");
     return -1;
 }
@@ -269,12 +324,12 @@ HRESULT WINAPI GetCORSystemDirectory(LPWSTR pbuffer, DWORD cchBuffer, DWORD *dwL
     ICLRRuntimeInfo *info;
     HRESULT ret;
 
-    TRACE("(%p, %d, %p)!\n", pbuffer, cchBuffer, dwLength);
+    TRACE("(%p, %ld, %p)!\n", pbuffer, cchBuffer, dwLength);
 
     if (!dwLength || !pbuffer)
         return E_POINTER;
 
-    ret = get_runtime_info(NULL, NULL, NULL, 0, RUNTIME_INFO_UPGRADE_VERSION, TRUE, &info);
+    ret = get_runtime_info(NULL, NULL, NULL, NULL, 0, RUNTIME_INFO_UPGRADE_VERSION, TRUE, &info);
 
     if (SUCCEEDED(ret))
     {
@@ -292,12 +347,12 @@ HRESULT WINAPI GetCORVersion(LPWSTR pbuffer, DWORD cchBuffer, DWORD *dwLength)
     ICLRRuntimeInfo *info;
     HRESULT ret;
 
-    TRACE("(%p, %d, %p)!\n", pbuffer, cchBuffer, dwLength);
+    TRACE("(%p, %ld, %p)!\n", pbuffer, cchBuffer, dwLength);
 
     if (!dwLength || !pbuffer)
         return E_POINTER;
 
-    ret = get_runtime_info(NULL, NULL, NULL, 0, RUNTIME_INFO_UPGRADE_VERSION, TRUE, &info);
+    ret = get_runtime_info(NULL, NULL, NULL, NULL, 0, RUNTIME_INFO_UPGRADE_VERSION, TRUE, &info);
 
     if (SUCCEEDED(ret))
     {
@@ -320,6 +375,13 @@ HRESULT WINAPI CorIsLatestSvc(int *unk1, int *unk2)
     return S_OK;
 }
 
+HRESULT WINAPI CorGetSvc(void *unk)
+{
+    ERR_(winediag)("If this function is called, it is likely the result of a broken .NET installation\n");
+
+    return E_NOTIMPL;
+}
+
 HRESULT WINAPI GetRequestedRuntimeInfo(LPCWSTR pExe, LPCWSTR pwszVersion, LPCWSTR pConfigurationFile,
     DWORD startupFlags, DWORD runtimeInfoFlags, LPWSTR pDirectory, DWORD dwDirectory, DWORD *dwDirectoryLength,
     LPWSTR pVersion, DWORD cchBuffer, DWORD *dwlength)
@@ -328,7 +390,7 @@ HRESULT WINAPI GetRequestedRuntimeInfo(LPCWSTR pExe, LPCWSTR pwszVersion, LPCWST
     ICLRRuntimeInfo *info;
     DWORD length_dummy;
 
-    TRACE("(%s, %s, %s, 0x%08x, 0x%08x, %p, 0x%08x, %p, %p, 0x%08x, %p)\n", debugstr_w(pExe),
+    TRACE("(%s, %s, %s, 0x%08lx, 0x%08lx, %p, 0x%08lx, %p, %p, 0x%08lx, %p)\n", debugstr_w(pExe),
           debugstr_w(pwszVersion), debugstr_w(pConfigurationFile), startupFlags, runtimeInfoFlags, pDirectory,
           dwDirectory, dwDirectoryLength, pVersion, cchBuffer, dwlength);
 
@@ -336,7 +398,7 @@ HRESULT WINAPI GetRequestedRuntimeInfo(LPCWSTR pExe, LPCWSTR pwszVersion, LPCWST
 
     if (!dwlength) dwlength = &length_dummy;
 
-    ret = get_runtime_info(pExe, pwszVersion, pConfigurationFile, startupFlags, runtimeInfoFlags, TRUE, &info);
+    ret = get_runtime_info(pExe, pwszVersion, pConfigurationFile, NULL, startupFlags, runtimeInfoFlags, TRUE, &info);
 
     if (SUCCEEDED(ret))
     {
@@ -360,7 +422,7 @@ HRESULT WINAPI GetRequestedRuntimeInfo(LPCWSTR pExe, LPCWSTR pwszVersion, LPCWST
 
 HRESULT WINAPI GetRequestedRuntimeVersion(LPWSTR pExe, LPWSTR pVersion, DWORD cchBuffer, DWORD *dwlength)
 {
-    TRACE("(%s, %p, %d, %p)\n", debugstr_w(pExe), pVersion, cchBuffer, dwlength);
+    TRACE("(%s, %p, %ld, %p)\n", debugstr_w(pExe), pVersion, cchBuffer, dwlength);
 
     if(!dwlength)
         return E_POINTER;
@@ -376,7 +438,7 @@ HRESULT WINAPI GetRealProcAddress(LPCSTR procname, void **ppv)
 
 HRESULT WINAPI GetFileVersion(LPCWSTR szFilename, LPWSTR szBuffer, DWORD cchBuffer, DWORD *dwLength)
 {
-    TRACE("(%s, %p, %d, %p)\n", debugstr_w(szFilename), szBuffer, cchBuffer, dwLength);
+    TRACE("(%s, %p, %ld, %p)\n", debugstr_w(szFilename), szBuffer, cchBuffer, dwLength);
 
     if (!szFilename || !dwLength)
         return E_POINTER;
@@ -414,11 +476,11 @@ HRESULT WINAPI LoadLibraryShim( LPCWSTR szDllName, LPCWSTR szVersion, LPVOID pvR
             else
                 szVersion = default_version;
         }
-        strcatW(dll_filename, szVersion);
-        strcatW(dll_filename, slash);
+        lstrcatW(dll_filename, szVersion);
+        lstrcatW(dll_filename, slash);
     }
 
-    strcatW(dll_filename, szDllName);
+    lstrcatW(dll_filename, szDllName);
 
     *phModDll = LoadLibraryW(dll_filename);
 
@@ -433,7 +495,7 @@ HRESULT WINAPI LockClrVersion(FLockClrVersionCallback hostCallback, FLockClrVers
 
 HRESULT WINAPI CoInitializeCor(DWORD fFlags)
 {
-    FIXME("(0x%08x): stub\n", fFlags);
+    FIXME("(0x%08lx): stub\n", fFlags);
     return S_OK;
 }
 
@@ -445,7 +507,7 @@ HRESULT WINAPI GetAssemblyMDImport(LPCWSTR szFileName, REFIID riid, IUnknown **p
 
 HRESULT WINAPI GetVersionFromProcess(HANDLE hProcess, LPWSTR pVersion, DWORD cchBuffer, DWORD *dwLength)
 {
-    FIXME("(%p, %p, %d, %p): stub\n", hProcess, pVersion, cchBuffer, dwLength);
+    FIXME("(%p, %p, %ld, %p): stub\n", hProcess, pVersion, cchBuffer, dwLength);
     return E_NOTIMPL;
 }
 
@@ -456,7 +518,7 @@ HRESULT WINAPI LoadStringRCEx(LCID culture, UINT resId, LPWSTR pBuffer, int iBuf
         return E_INVALIDARG;
     pBuffer[0] = 0;
     if (resId) {
-        FIXME("(%d, %x, %p, %d, %d, %p): semi-stub\n", culture, resId, pBuffer, iBufLen, bQuiet, pBufLen);
+        FIXME("(%ld, %x, %p, %d, %d, %p): semi-stub\n", culture, resId, pBuffer, iBufLen, bQuiet, pBufLen);
         res = E_NOTIMPL;
     }
     else
@@ -477,12 +539,12 @@ HRESULT WINAPI CorBindToRuntimeEx(LPWSTR szVersion, LPWSTR szBuildFlavor, DWORD
     HRESULT ret;
     ICLRRuntimeInfo *info;
 
-    TRACE("%s %s %d %s %s %p\n", debugstr_w(szVersion), debugstr_w(szBuildFlavor), nflags, debugstr_guid( rslsid ),
+    TRACE("%s %s %ld %s %s %p\n", debugstr_w(szVersion), debugstr_w(szBuildFlavor), nflags, debugstr_guid( rslsid ),
           debugstr_guid( riid ), ppv);
 
     *ppv = NULL;
 
-    ret = get_runtime_info(NULL, szVersion, NULL, nflags, RUNTIME_INFO_UPGRADE_VERSION, TRUE, &info);
+    ret = get_runtime_info(NULL, szVersion, NULL, NULL, nflags, RUNTIME_INFO_UPGRADE_VERSION, TRUE, &info);
 
     if (SUCCEEDED(ret))
     {
@@ -503,7 +565,7 @@ HRESULT WINAPI CorBindToCurrentRuntime(LPCWSTR filename, REFCLSID rclsid, REFIID
 
     *ppv = NULL;
 
-    ret = get_runtime_info(NULL, NULL, filename, 0, RUNTIME_INFO_UPGRADE_VERSION, TRUE, &info);
+    ret = get_runtime_info(NULL, NULL, filename, NULL, 0, RUNTIME_INFO_UPGRADE_VERSION, TRUE, &info);
 
     if (SUCCEEDED(ret))
     {
@@ -526,7 +588,7 @@ STDAPI ClrCreateManagedInstance(LPCWSTR pTypeName, REFIID riid, void **ppObject)
     TRACE("(%s,%s,%p)\n", debugstr_w(pTypeName), debugstr_guid(riid), ppObject);
 
     /* FIXME: How to determine which runtime version to use? */
-    ret = get_runtime_info(NULL, NULL, NULL, 0, RUNTIME_INFO_UPGRADE_VERSION, TRUE, &info);
+    ret = get_runtime_info(NULL, NULL, NULL, NULL, 0, RUNTIME_INFO_UPGRADE_VERSION, TRUE, &info);
 
     if (SUCCEEDED(ret))
     {
@@ -550,16 +612,17 @@ STDAPI ClrCreateManagedInstance(LPCWSTR pTypeName, REFIID riid, void **ppObject)
     return ret;
 }
 
-BOOL WINAPI StrongNameSignatureVerification(LPCWSTR filename, DWORD inFlags, DWORD* pOutFlags)
+BOOLEAN WINAPI StrongNameSignatureVerification(LPCWSTR filename, DWORD inFlags, DWORD *pOutFlags)
 {
-    FIXME("(%s, 0x%X, %p): stub\n", debugstr_w(filename), inFlags, pOutFlags);
+    FIXME("(%s, 0x%lX, %p): stub\n", debugstr_w(filename), inFlags, pOutFlags);
     return FALSE;
 }
 
-BOOL WINAPI StrongNameSignatureVerificationEx(LPCWSTR filename, BOOL forceVerification, BOOL* pVerified)
+BOOLEAN WINAPI StrongNameSignatureVerificationEx(LPCWSTR filename, BOOLEAN forceVerification, BOOLEAN *pVerified)
 {
     FIXME("(%s, %u, %p): stub\n", debugstr_w(filename), forceVerification, pVerified);
-    return FALSE;
+    *pVerified = TRUE;
+    return TRUE;
 }
 
 HRESULT WINAPI CreateDebuggingInterfaceFromVersion(int nDebugVersion, LPCWSTR version, IUnknown **ppv)
@@ -578,7 +641,7 @@ HRESULT WINAPI CreateDebuggingInterfaceFromVersion(int nDebugVersion, LPCWSTR ve
 
     *ppv = NULL;
 
-    if(strcmpW(version, v2_0) != 0)
+    if(wcscmp(version, v2_0) != 0)
     {
         FIXME("Currently .NET Version '%s' not support.\n", debugstr_w(version));
         return E_INVALIDARG;
@@ -665,16 +728,22 @@ static void parse_msi_version_string(const char *version, int *parts)
     }
 }
 
-static BOOL install_wine_mono(void)
+static int compare_versions(const char *a, const char *b)
+{
+    int a_parts[3], b_parts[3], i;
+
+    parse_msi_version_string(a, a_parts);
+    parse_msi_version_string(b, b_parts);
+
+    for (i=0; i<3; i++)
+        if (a_parts[i] != b_parts[i])
+            return a_parts[i] - b_parts[i];
+
+    return 0;
+}
+
+static BOOL invoke_appwiz(void)
 {
-    BOOL is_wow64 = FALSE;
-    HMODULE hmsi;
-    UINT (WINAPI *pMsiEnumRelatedProductsA)(LPCSTR,DWORD,DWORD,LPSTR);
-    UINT (WINAPI *pMsiGetProductInfoA)(LPCSTR,LPCSTR,LPSTR,DWORD*);
-    char versionstringbuf[15];
-    char productcodebuf[39];
-    UINT res;
-    DWORD buffer_size;
     PROCESS_INFORMATION pi;
     STARTUPINFOW si;
     WCHAR app[MAX_PATH];
@@ -682,75 +751,10 @@ static BOOL install_wine_mono(void)
     LONG len;
     BOOL ret;
 
-    static const char* mono_version = "4.7.5";
-    static const char* mono_upgrade_code = "{DE624609-C6B5-486A-9274-EF0B854F6BC5}";
-
     static const WCHAR controlW[] = {'\\','c','o','n','t','r','o','l','.','e','x','e',0};
     static const WCHAR argsW[] =
         {' ','a','p','p','w','i','z','.','c','p','l',' ','i','n','s','t','a','l','l','_','m','o','n','o',0};
 
-    IsWow64Process(GetCurrentProcess(), &is_wow64);
-
-    if (is_wow64)
-    {
-        TRACE("not installing mono in wow64 process\n");
-        return TRUE;
-    }
-
-    hmsi = LoadLibraryA("msi");
-
-    if (!hmsi)
-    {
-        ERR("couldn't load msi.dll\n");
-        return FALSE;
-    }
-
-    pMsiEnumRelatedProductsA = (void*)GetProcAddress(hmsi, "MsiEnumRelatedProductsA");
-
-    res = pMsiEnumRelatedProductsA(mono_upgrade_code, 0, 0, productcodebuf);
-
-    if (res == ERROR_SUCCESS)
-    {
-        pMsiGetProductInfoA = (void*)GetProcAddress(hmsi, "MsiGetProductInfoA");
-
-        buffer_size = sizeof(versionstringbuf);
-
-        res = pMsiGetProductInfoA(productcodebuf, "VersionString", versionstringbuf, &buffer_size);
-    }
-    else if (res != ERROR_NO_MORE_ITEMS)
-    {
-        ERR("MsiEnumRelatedProducts failed, err=%u\n", res);
-    }
-
-    FreeLibrary(hmsi);
-
-    if (res == ERROR_SUCCESS)
-    {
-        int current_version[3], wanted_version[3], i;
-
-        TRACE("found installed version %s\n", versionstringbuf);
-
-        parse_msi_version_string(versionstringbuf, current_version);
-        parse_msi_version_string(mono_version, wanted_version);
-
-        for (i=0; i<3; i++)
-        {
-            if (current_version[i] < wanted_version[i])
-                break;
-            else if (current_version[i] > wanted_version[i])
-            {
-                TRACE("installed version is newer than %s, quitting\n", mono_version);
-                return TRUE;
-            }
-        }
-
-        if (i == 3)
-        {
-            TRACE("version %s is already installed, quitting\n", mono_version);
-            return TRUE;
-        }
-    }
-
     len = GetSystemDirectoryW(app, MAX_PATH - ARRAY_SIZE(controlW));
     memcpy(app+len, controlW, sizeof(controlW));
 
@@ -776,6 +780,163 @@ static BOOL install_wine_mono(void)
     return ret;
 }
 
+static BOOL get_support_msi(LPCWSTR mono_path, LPWSTR msi_path)
+{
+    static const WCHAR support_msi_relative[] = {'\\','s','u','p','p','o','r','t','\\','w','i','n','e','m','o','n','o','-','s','u','p','p','o','r','t','.','m','s','i',0};
+    UINT (WINAPI *pMsiOpenPackageW)(LPCWSTR,ULONG*);
+    UINT (WINAPI *pMsiGetProductPropertyA)(ULONG,LPCSTR,LPSTR,LPDWORD);
+    UINT (WINAPI *pMsiCloseHandle)(ULONG);
+    HMODULE hmsi = NULL;
+    char versionstringbuf[15];
+    UINT res;
+    DWORD buffer_size;
+    ULONG msiproduct;
+    BOOL ret=FALSE;
+
+    hmsi = GetModuleHandleA("msi");
+
+    lstrcpyW(msi_path, mono_path);
+    lstrcatW(msi_path, support_msi_relative);
+
+    pMsiOpenPackageW = (void*)GetProcAddress(hmsi, "MsiOpenPackageW");
+
+    res = pMsiOpenPackageW(msi_path, &msiproduct);
+
+    if (res == ERROR_SUCCESS)
+    {
+        buffer_size = sizeof(versionstringbuf);
+
+        pMsiGetProductPropertyA = (void*)GetProcAddress(hmsi, "MsiGetProductPropertyA");
+
+        res = pMsiGetProductPropertyA(msiproduct, "ProductVersion", versionstringbuf, &buffer_size);
+
+        pMsiCloseHandle = (void*)GetProcAddress(hmsi, "MsiCloseHandle");
+
+        pMsiCloseHandle(msiproduct);
+    }
+
+    if (res == ERROR_SUCCESS) {
+        TRACE("found support msi version %s at %s\n", versionstringbuf, debugstr_w(msi_path));
+
+        if (compare_versions(WINE_MONO_VERSION, versionstringbuf) <= 0)
+        {
+            ret = TRUE;
+        }
+    }
+
+    return ret;
+}
+
+static BOOL install_wine_mono(void)
+{
+    BOOL is_wow64 = FALSE;
+    HMODULE hmsi = NULL;
+    HRESULT initresult = E_FAIL;
+    UINT (WINAPI *pMsiEnumRelatedProductsA)(LPCSTR,DWORD,DWORD,LPSTR);
+    UINT (WINAPI *pMsiGetProductInfoA)(LPCSTR,LPCSTR,LPSTR,DWORD*);
+    UINT (WINAPI *pMsiInstallProductW)(LPCWSTR,LPCWSTR);
+    char versionstringbuf[15];
+    char productcodebuf[39];
+    UINT res;
+    DWORD buffer_size;
+    BOOL ret;
+    WCHAR mono_path[MAX_PATH];
+    WCHAR support_msi_path[MAX_PATH];
+
+    static const char* mono_upgrade_code = "{DE624609-C6B5-486A-9274-EF0B854F6BC5}";
+
+    IsWow64Process(GetCurrentProcess(), &is_wow64);
+
+    if (is_wow64)
+    {
+        TRACE("not installing mono in wow64 process\n");
+        return TRUE;
+    }
+
+    TRACE("searching for mono runtime\n");
+
+    if (!get_mono_path(mono_path, FALSE))
+    {
+        TRACE("mono runtime not found\n");
+        return invoke_appwiz();
+    }
+
+    TRACE("mono runtime is at %s\n", debugstr_w(mono_path));
+
+    hmsi = LoadLibraryA("msi");
+
+    if (!hmsi)
+    {
+        ERR("couldn't load msi.dll\n");
+        return FALSE;
+    }
+
+    pMsiEnumRelatedProductsA = (void*)GetProcAddress(hmsi, "MsiEnumRelatedProductsA");
+
+    res = pMsiEnumRelatedProductsA(mono_upgrade_code, 0, 0, productcodebuf);
+
+    if (res == ERROR_SUCCESS)
+    {
+        pMsiGetProductInfoA = (void*)GetProcAddress(hmsi, "MsiGetProductInfoA");
+
+        buffer_size = sizeof(versionstringbuf);
+
+        res = pMsiGetProductInfoA(productcodebuf, "VersionString", versionstringbuf, &buffer_size);
+    }
+    else if (res != ERROR_NO_MORE_ITEMS)
+    {
+        ERR("MsiEnumRelatedProducts failed, err=%u\n", res);
+    }
+
+    if (res == ERROR_SUCCESS)
+    {
+        TRACE("found installed support package %s\n", versionstringbuf);
+
+        if (compare_versions(WINE_MONO_VERSION, versionstringbuf) <= 0)
+        {
+            TRACE("support package is at least %s, quitting\n", WINE_MONO_VERSION);
+            ret = TRUE;
+            goto end;
+        }
+    }
+
+    initresult = CoInitialize(NULL);
+
+    ret = get_support_msi(mono_path, support_msi_path);
+    if (!ret)
+    {
+        /* Try looking outside c:\windows\mono */
+        ret = (get_mono_path(mono_path, TRUE) &&
+            get_support_msi(mono_path, support_msi_path));
+    }
+
+    if (ret)
+    {
+        TRACE("installing support msi\n");
+
+        pMsiInstallProductW = (void*)GetProcAddress(hmsi, "MsiInstallProductW");
+
+        res = pMsiInstallProductW(support_msi_path, NULL);
+
+        if (res == ERROR_SUCCESS)
+        {
+            ret = TRUE;
+            goto end;
+        }
+        else
+            ERR("MsiInstallProduct failed, err=%i\n", res);
+    }
+
+    ret = invoke_appwiz();
+
+end:
+    if (hmsi)
+        FreeLibrary(hmsi);
+    if (SUCCEEDED(initresult))
+        CoUninitialize();
+    return ret;
+}
+
 /* CROSSOVER HACK bug 10194 */
 void WINAPI wine_install_mono(HWND hwnd, HINSTANCE hinst, LPSTR cmdline, int cmdshow)
 {
@@ -786,17 +947,12 @@ HRESULT WINAPI DllRegisterServer(void)
 {
     install_wine_mono();
 
-    return __wine_register_resources( MSCOREE_hInstance );
+    return __wine_register_resources();
 }
 
 HRESULT WINAPI DllUnregisterServer(void)
 {
-    return __wine_unregister_resources( MSCOREE_hInstance );
-}
-
-HRESULT WINAPI DllCanUnloadNow(VOID)
-{
-    return S_FALSE;
+    return __wine_unregister_resources();
 }
 
 void WINAPI CoEEShutDownCOM(void)
