@@ -17,7 +17,6 @@
  */
 
 #include <stdarg.h>
-#include <assert.h>
 
 #define COBJMACROS
 #define NONAMELESSUNION
@@ -45,8 +44,10 @@ static CRITICAL_SECTION_DEBUG cs_dispex_static_data_dbg =
 };
 static CRITICAL_SECTION cs_dispex_static_data = { &cs_dispex_static_data_dbg, -1, 0, 0, 0, 0 };
 
-
-static const WCHAR objectW[] = {'[','o','b','j','e','c','t',']',0};
+typedef struct {
+    IID iid;
+    VARIANT default_value;
+} func_arg_info_t;
 
 typedef struct {
     DISPID id;
@@ -58,8 +59,10 @@ typedef struct {
     SHORT get_vtbl_off;
     SHORT func_disp_idx;
     USHORT argc;
+    USHORT default_value_cnt;
     VARTYPE prop_vt;
     VARTYPE *arg_types;
+    func_arg_info_t *arg_info;
 } func_info_t;
 
 struct dispex_data_t {
@@ -108,7 +111,7 @@ struct dispex_dynamic_data_t {
 
 #define FDEX_VERSION_MASK 0xf0000000
 
-static ITypeLib *typelib;
+static ITypeLib *typelib, *typelib_private;
 static ITypeInfo *typeinfos[LAST_tid];
 static struct list dispex_data_list = LIST_INIT(dispex_data_list);
 
@@ -116,24 +119,46 @@ static REFIID tid_ids[] = {
 #define XIID(iface) &IID_ ## iface,
 #define XDIID(iface) &DIID_ ## iface,
 TID_LIST
+    NULL,
+PRIVATE_TID_LIST
 #undef XIID
 #undef XDIID
 };
 
 static HRESULT load_typelib(void)
 {
+    WCHAR module_path[MAX_PATH + 3];
     HRESULT hres;
     ITypeLib *tl;
+    DWORD len;
 
     hres = LoadRegTypeLib(&LIBID_MSHTML, 4, 0, LOCALE_SYSTEM_DEFAULT, &tl);
     if(FAILED(hres)) {
-        ERR("LoadRegTypeLib failed: %08x\n", hres);
+        ERR("LoadRegTypeLib failed: %08lx\n", hres);
         return hres;
     }
 
     if(InterlockedCompareExchangePointer((void**)&typelib, tl, NULL))
         ITypeLib_Release(tl);
-    return hres;
+
+    len = GetModuleFileNameW(hInst, module_path, MAX_PATH + 1);
+    if (!len || len == MAX_PATH + 1)
+    {
+        ERR("Could not get module file name, len %lu.\n", len);
+        return E_FAIL;
+    }
+    lstrcatW(module_path, L"\\1");
+
+    hres = LoadTypeLibEx(module_path, REGKIND_NONE, &tl);
+    if(FAILED(hres)) {
+        ERR("LoadTypeLibEx failed for private typelib: %08lx\n", hres);
+        return hres;
+    }
+
+    if(InterlockedCompareExchangePointer((void**)&typelib_private, tl, NULL))
+        ITypeLib_Release(tl);
+
+    return S_OK;
 }
 
 static HRESULT get_typeinfo(tid_t tid, ITypeInfo **typeinfo)
@@ -148,9 +173,9 @@ static HRESULT get_typeinfo(tid_t tid, ITypeInfo **typeinfo)
     if(!typeinfos[tid]) {
         ITypeInfo *ti;
 
-        hres = ITypeLib_GetTypeInfoOfGuid(typelib, tid_ids[tid], &ti);
+        hres = ITypeLib_GetTypeInfoOfGuid(tid > LAST_public_tid ? typelib_private : typelib, tid_ids[tid], &ti);
         if(FAILED(hres)) {
-            ERR("GetTypeInfoOfGuid(%s) failed: %08x\n", debugstr_mshtml_guid(tid_ids[tid]), hres);
+            ERR("GetTypeInfoOfGuid(%s) failed: %08lx\n", debugstr_mshtml_guid(tid_ids[tid]), hres);
             return hres;
         }
 
@@ -165,14 +190,21 @@ static HRESULT get_typeinfo(tid_t tid, ITypeInfo **typeinfo)
 void release_typelib(void)
 {
     dispex_data_t *iter;
-    unsigned i;
+    unsigned i, j;
 
     while(!list_empty(&dispex_data_list)) {
         iter = LIST_ENTRY(list_head(&dispex_data_list), dispex_data_t, entry);
         list_remove(&iter->entry);
 
-        for(i=0; i < iter->func_cnt; i++)
+        for(i = 0; i < iter->func_cnt; i++) {
+            if(iter->funcs[i].default_value_cnt && iter->funcs[i].arg_info) {
+                for(j = 0; j < iter->funcs[i].argc; j++)
+                    VariantClear(&iter->funcs[i].arg_info[j].default_value);
+            }
+            heap_free(iter->funcs[i].arg_types);
+            heap_free(iter->funcs[i].arg_info);
             SysFreeString(iter->funcs[i].name);
+        }
 
         heap_free(iter->funcs);
         heap_free(iter->name_table);
@@ -187,6 +219,7 @@ void release_typelib(void)
             ITypeInfo_Release(typeinfos[i]);
 
     ITypeLib_Release(typelib);
+    ITypeLib_Release(typelib_private);
     DeleteCriticalSection(&cs_dispex_static_data);
 }
 
@@ -200,8 +233,10 @@ HRESULT get_class_typeinfo(const CLSID *clsid, ITypeInfo **typeinfo)
         return hres;
 
     hres = ITypeLib_GetTypeInfoOfGuid(typelib, clsid, typeinfo);
+    if (FAILED(hres))
+        hres = ITypeLib_GetTypeInfoOfGuid(typelib_private, clsid, typeinfo);
     if(FAILED(hres))
-        ERR("GetTypeInfoOfGuid failed: %08x\n", hres);
+        ERR("GetTypeInfoOfGuid failed: %08lx\n", hres);
     return hres;
 }
 
@@ -213,6 +248,7 @@ HRESULT get_class_typeinfo(const CLSID *clsid, ITypeInfo **typeinfo)
     CASE_VT(VT_UI4, UINT32, V_UI4);                     \
     CASE_VT(VT_R4, float, V_R4);                        \
     CASE_VT(VT_BSTR, BSTR, V_BSTR);                     \
+    CASE_VT(VT_DISPATCH, IDispatch*, V_DISPATCH);       \
     CASE_VT(VT_BOOL, VARIANT_BOOL, V_BOOL)
 
 /* List all types used by IDispatchEx-based properties */
@@ -221,7 +257,6 @@ HRESULT get_class_typeinfo(const CLSID *clsid, ITypeInfo **typeinfo)
     CASE_VT(VT_VARIANT, VARIANT, *);                    \
     CASE_VT(VT_PTR, void*, V_BYREF);                    \
     CASE_VT(VT_UNKNOWN, IUnknown*, V_UNKNOWN);          \
-    CASE_VT(VT_DISPATCH, IDispatch*, V_DISPATCH);       \
     CASE_VT(VT_UI8, ULONGLONG, V_UI8)
 
 static BOOL is_arg_type_supported(VARTYPE vt)
@@ -242,11 +277,13 @@ static void add_func_info(dispex_data_t *data, tid_t tid, const FUNCDESC *desc,
     HRESULT hres;
 
     hres = ITypeInfo_GetDocumentation(dti, desc->memid, &name, NULL, NULL, NULL);
-    if(FAILED(hres))
+    if(FAILED(hres)) {
+        WARN("GetDocumentation failed: %08lx\n", hres);
         return;
+    }
 
     for(info = data->funcs; info < data->funcs+data->func_cnt; info++) {
-        if(info->id == desc->memid || !strcmpW(info->name, name)) {
+        if(info->id == desc->memid || !wcscmp(info->name, name)) {
             if(info->tid != tid) {
                 SysFreeString(name);
                 return; /* Duplicated in other interface */
@@ -255,6 +292,8 @@ static void add_func_info(dispex_data_t *data, tid_t tid, const FUNCDESC *desc,
         }
     }
 
+    TRACE("adding %s...\n", debugstr_w(name));
+
     if(info == data->funcs+data->func_cnt) {
         if(data->func_cnt == data->func_size)
             data->funcs = heap_realloc_zero(data->funcs, (data->func_size <<= 1)*sizeof(func_info_t));
@@ -281,18 +320,27 @@ static void add_func_info(dispex_data_t *data, tid_t tid, const FUNCDESC *desc,
         assert(info->argc < MAX_ARGS);
         assert(desc->funckind == FUNC_DISPATCH);
 
-        info->arg_types = heap_alloc(sizeof(*info->arg_types) * info->argc);
+        info->arg_info = heap_alloc_zero(sizeof(*info->arg_info) * info->argc);
+        if(!info->arg_info)
+            return;
+
+        info->prop_vt = desc->elemdescFunc.tdesc.vt;
+        if(info->prop_vt != VT_VOID && info->prop_vt != VT_PTR && !is_arg_type_supported(info->prop_vt)) {
+            TRACE("%s: return type %d\n", debugstr_w(info->name), info->prop_vt);
+            return; /* Fallback to ITypeInfo::Invoke */
+        }
+
+        info->arg_types = heap_alloc(sizeof(*info->arg_types) * (info->argc + (info->prop_vt == VT_VOID ? 0 : 1)));
         if(!info->arg_types)
-            return; /* FIXME: real error instead of fallback */
+            return;
 
         for(i=0; i < info->argc; i++)
             info->arg_types[i] = desc->lprgelemdescParam[i].tdesc.vt;
 
-        info->prop_vt = desc->elemdescFunc.tdesc.vt;
-        if(info->prop_vt != VT_VOID && !is_arg_type_supported(info->prop_vt)) {
-            TRACE("%s: return type %d\n", debugstr_w(info->name), info->prop_vt);
-            return; /* Fallback to ITypeInfo::Invoke */
-        }
+        if(info->prop_vt == VT_PTR)
+            info->arg_types[info->argc] = VT_BYREF | VT_DISPATCH;
+        else if(info->prop_vt != VT_VOID)
+            info->arg_types[info->argc] = VT_BYREF | info->prop_vt;
 
         if(desc->cParamsOpt) {
             TRACE("%s: optional params\n", debugstr_w(info->name));
@@ -300,13 +348,44 @@ static void add_func_info(dispex_data_t *data, tid_t tid, const FUNCDESC *desc,
         }
 
         for(i=0; i < info->argc; i++) {
-            if(!is_arg_type_supported(info->arg_types[i])) {
+            TYPEDESC *tdesc = &desc->lprgelemdescParam[i].tdesc;
+            if(tdesc->vt == VT_PTR && tdesc->u.lptdesc->vt == VT_USERDEFINED) {
+                ITypeInfo *ref_type_info;
+                TYPEATTR *attr;
+
+                hres = ITypeInfo_GetRefTypeInfo(dti, tdesc->u.lptdesc->u.hreftype, &ref_type_info);
+                if(FAILED(hres)) {
+                    ERR("Could not get referenced type info: %08lx\n", hres);
+                    return;
+                }
+
+                hres = ITypeInfo_GetTypeAttr(ref_type_info, &attr);
+                if(SUCCEEDED(hres)) {
+                    assert(attr->typekind == TKIND_DISPATCH);
+                    info->arg_info[i].iid = attr->guid;
+                    ITypeInfo_ReleaseTypeAttr(ref_type_info, attr);
+                }else {
+                    ERR("GetTypeAttr failed: %08lx\n", hres);
+                }
+                ITypeInfo_Release(ref_type_info);
+                if(FAILED(hres))
+                    return;
+                info->arg_types[i] = VT_DISPATCH;
+            }else if(!is_arg_type_supported(info->arg_types[i])) {
+                TRACE("%s: unsupported arg type %s\n", debugstr_w(info->name), debugstr_vt(info->arg_types[i]));
                 return; /* Fallback to ITypeInfo for unsupported arg types */
             }
 
             if(desc->lprgelemdescParam[i].u.paramdesc.wParamFlags & PARAMFLAG_FHASDEFAULT) {
-                TRACE("%s param %d: default value\n", debugstr_w(info->name), i);
-                return; /* Fallback to ITypeInfo::Invoke */
+                hres = VariantCopy(&info->arg_info[i].default_value,
+                                   &desc->lprgelemdescParam[i].u.paramdesc.pparamdescex->varDefaultValue);
+                if(FAILED(hres)) {
+                    ERR("Could not copy default value: %08lx\n", hres);
+                    return;
+                }
+                TRACE("%s param %d: default value %s\n", debugstr_w(info->name),
+                      i, debugstr_variant(&info->arg_info[i].default_value));
+                info->default_value_cnt++;
             }
         }
 
@@ -360,7 +439,6 @@ static HRESULT process_interface(dispex_data_t *data, tid_t tid, ITypeInfo *disp
         }
 
         if(!hook || hook->invoke) {
-            TRACE("adding...\n");
             add_func_info(data, tid, funcdesc, disp_typeinfo ? disp_typeinfo : typeinfo,
                           hook ? hook->invoke : NULL);
         }
@@ -377,17 +455,17 @@ void dispex_info_add_interface(dispex_data_t *info, tid_t tid, const dispex_hook
 
     hres = process_interface(info, tid, NULL, hooks);
     if(FAILED(hres))
-        ERR("process_interface failed: %08x\n", hres);
+        ERR("process_interface failed: %08lx\n", hres);
 }
 
-static int dispid_cmp(const void *p1, const void *p2)
+static int __cdecl dispid_cmp(const void *p1, const void *p2)
 {
     return ((const func_info_t*)p1)->id - ((const func_info_t*)p2)->id;
 }
 
-static int func_name_cmp(const void *p1, const void *p2)
+static int __cdecl func_name_cmp(const void *p1, const void *p2)
 {
-    return strcmpiW((*(func_info_t* const*)p1)->name, (*(func_info_t* const*)p2)->name);
+    return wcsicmp((*(func_info_t* const*)p1)->name, (*(func_info_t* const*)p2)->name);
 }
 
 static dispex_data_t *preprocess_dispex_data(dispex_static_data_t *desc, compat_mode_t compat_mode)
@@ -401,7 +479,7 @@ static dispex_data_t *preprocess_dispex_data(dispex_static_data_t *desc, compat_
     if(desc->disp_tid) {
         hres = get_typeinfo(desc->disp_tid, &dti);
         if(FAILED(hres)) {
-            ERR("Could not get disp type info: %08x\n", hres);
+            ERR("Could not get disp type info: %08lx\n", hres);
             return NULL;
         }
     }
@@ -452,7 +530,7 @@ static dispex_data_t *preprocess_dispex_data(dispex_static_data_t *desc, compat_
     return data;
 }
 
-static int id_cmp(const void *p1, const void *p2)
+static int __cdecl id_cmp(const void *p1, const void *p2)
 {
     return *(const DISPID*)p1 - *(const DISPID*)p2;
 }
@@ -563,7 +641,7 @@ static HRESULT get_dynamic_prop(DispatchEx *This, const WCHAR *name, DWORD flags
         return E_OUTOFMEMORY;
 
     for(prop = data->props; prop < data->props+data->prop_cnt; prop++) {
-        if(flags & fdexNameCaseInsensitive ? !strcmpiW(prop->name, name) : !strcmpW(prop->name, name)) {
+        if(flags & fdexNameCaseInsensitive ? !wcsicmp(prop->name, name) : !wcscmp(prop->name, name)) {
             if(prop->flags & DYNPROP_DELETED) {
                 if(!alloc)
                     return DISP_E_UNKNOWNNAME;
@@ -637,15 +715,17 @@ HRESULT dispex_get_dynid(DispatchEx *This, const WCHAR *name, DISPID *id)
 static HRESULT dispex_value(DispatchEx *This, LCID lcid, WORD flags, DISPPARAMS *params,
         VARIANT *res, EXCEPINFO *ei, IServiceProvider *caller)
 {
+    HRESULT hres;
+
     if(This->info->desc->vtbl && This->info->desc->vtbl->value)
         return This->info->desc->vtbl->value(This, lcid, flags, params, res, ei, caller);
 
     switch(flags) {
     case DISPATCH_PROPERTYGET:
         V_VT(res) = VT_BSTR;
-        V_BSTR(res) = SysAllocString(objectW);
-        if(!V_BSTR(res))
-            return E_OUTOFMEMORY;
+        hres = dispex_to_string(This, &V_BSTR(res));
+        if(FAILED(hres))
+            return hres;
         break;
     default:
         FIXME("Unimplemented flags %x\n", flags);
@@ -658,21 +738,26 @@ static HRESULT dispex_value(DispatchEx *This, LCID lcid, WORD flags, DISPPARAMS
 static HRESULT typeinfo_invoke(DispatchEx *This, func_info_t *func, WORD flags, DISPPARAMS *dp, VARIANT *res,
         EXCEPINFO *ei)
 {
-    DISPPARAMS params = {dp->rgvarg+dp->cNamedArgs, NULL, dp->cArgs-dp->cNamedArgs, 0};
+    DISPPARAMS params = {dp->rgvarg, NULL, dp->cArgs, 0};
     ITypeInfo *ti;
     IUnknown *unk;
     UINT argerr=0;
     HRESULT hres;
 
+    if(params.cArgs > func->argc) {
+        params.rgvarg += params.cArgs - func->argc;
+        params.cArgs = func->argc;
+    }
+
     hres = get_typeinfo(func->tid, &ti);
     if(FAILED(hres)) {
-        ERR("Could not get type info: %08x\n", hres);
+        ERR("Could not get type info: %08lx\n", hres);
         return hres;
     }
 
     hres = IUnknown_QueryInterface(This->outer, tid_ids[func->tid], (void**)&unk);
     if(FAILED(hres)) {
-        ERR("Could not get iface %s: %08x\n", debugstr_mshtml_guid(tid_ids[func->tid]), hres);
+        ERR("Could not get iface %s: %08lx\n", debugstr_mshtml_guid(tid_ids[func->tid]), hres);
         return E_FAIL;
     }
 
@@ -711,7 +796,7 @@ static ULONG WINAPI Function_AddRef(IUnknown *iface)
     func_disp_t *This = impl_from_IUnknown(iface);
     LONG ref = InterlockedIncrement(&This->ref);
 
-    TRACE("(%p) ref=%d\n", This, ref);
+    TRACE("(%p) ref=%ld\n", This, ref);
 
     return ref;
 }
@@ -721,7 +806,7 @@ static ULONG WINAPI Function_Release(IUnknown *iface)
     func_disp_t *This = impl_from_IUnknown(iface);
     LONG ref = InterlockedDecrement(&This->ref);
 
-    TRACE("(%p) ref=%d\n", This, ref);
+    TRACE("(%p) ref=%ld\n", This, ref);
 
     if(!ref) {
         assert(!This->obj);
@@ -757,7 +842,7 @@ static HRESULT function_value(DispatchEx *dispex, LCID lcid, WORD flags, DISPPAR
     case DISPATCH_METHOD:
         if(!This->obj)
             return E_UNEXPECTED;
-        hres = typeinfo_invoke(This->obj, This->info, flags, params, res, ei);
+        hres = dispex_call_builtin(This->obj, This->info->id, params, res, ei, caller);
         break;
     case DISPATCH_PROPERTYGET: {
         unsigned name_len;
@@ -798,16 +883,159 @@ static HRESULT function_value(DispatchEx *dispex, LCID lcid, WORD flags, DISPPAR
     return hres;
 }
 
+static HRESULT function_get_dispid(DispatchEx *dispex, BSTR name, DWORD grfdex, DISPID *pid)
+{
+    if(!wcscmp(name, L"apply")) {
+        *pid = MSHTML_DISPID_CUSTOM_MIN;
+        return S_OK;
+    }
+
+    if(!wcscmp(name, L"call")) {
+        *pid = MSHTML_DISPID_CUSTOM_MIN + 1;
+        return S_OK;
+    }
+
+    return DISP_E_UNKNOWNNAME;
+}
+
+static HRESULT args_to_dp(IDispatch *args_obj, LCID lcid,  DISPPARAMS *dp)
+{
+    DISPPARAMS propget_dp = {NULL};
+    IDispatchEx *dispex;
+    BSTR str;
+    WCHAR buf[12];
+    unsigned i;
+    VARIANT v;
+    DISPID id;
+    HRESULT hres;
+
+    static const WCHAR lengthW[] = {'l','e','n','g','t','h',0};
+    static const WCHAR formatW[] = {'%','u',0};
+
+    hres = IDispatch_QueryInterface(args_obj, &IID_IDispatchEx, (void**)&dispex);
+    if(FAILED(hres)) {
+        FIXME("Could not get IDispatchEx\n");
+        return hres;
+    }
+
+    str = SysAllocString(lengthW);
+    hres = IDispatchEx_GetDispID(dispex, str, 0, &id);
+    SysFreeString(str);
+    if(SUCCEEDED(hres)) {
+        hres = IDispatchEx_InvokeEx(dispex, id, lcid, DISPATCH_PROPERTYGET, &propget_dp, &v, NULL, NULL);
+        if(SUCCEEDED(hres)) {
+            if(V_VT(&v) == VT_I4) {
+                TRACE("length %d\n", V_I4(&v));
+                dp->cArgs = V_I4(&v);
+            }
+            VariantClear(&v);
+        }
+    }
+
+    if(dp->cArgs)
+        dp->rgvarg = heap_alloc(dp->cArgs * sizeof(*dp->rgvarg));
+
+    for(i = 0; i < dp->cArgs; i++) {
+        swprintf(buf, ARRAY_SIZE(buf), formatW, i);
+        str = SysAllocString(buf);
+        hres = IDispatchEx_GetDispID(dispex, str, 0, &id);
+        SysFreeString(str);
+        if(SUCCEEDED(hres)) {
+            hres = IDispatchEx_InvokeEx(dispex, id, lcid, DISPATCH_PROPERTYGET, &propget_dp, dp->rgvarg+dp->cArgs-i-1, NULL, NULL);
+            if(SUCCEEDED(hres)) {
+                TRACE("[%u] = %s\n", i, debugstr_variant(dp->rgvarg+dp->cArgs-i-1));
+            }else {
+                TRACE("InvokeEx failed: %08x\n", hres);
+            }
+        }else {
+            TRACE("no arg %u\n", i);
+            V_VT(dp->rgvarg+dp->cArgs-i-1) = VT_EMPTY;
+        }
+    }
+
+    IDispatchEx_Release(dispex);
+    return S_OK;
+}
+
+static HRESULT function_dispex_invoke(DispatchEx *dispex, DISPID id, LCID lcid,
+        WORD flags, DISPPARAMS *params, VARIANT *res, EXCEPINFO *ei,
+        IServiceProvider *caller)
+{
+    func_disp_t *This = impl_from_DispatchEx(dispex);
+
+    switch(id) {
+    case MSHTML_DISPID_CUSTOM_MIN: {
+        DISPPARAMS dp = {NULL};
+        unsigned i;
+        HRESULT hres;
+
+        if(params->cArgs != 2 || params->cNamedArgs) {
+            FIXME("unsupported apply arguments\n");
+            return E_FAIL;
+        }
+
+        FIXME("%p.apply(%s %s)\n", This, debugstr_variant(params->rgvarg+1), debugstr_variant(params->rgvarg));
+        if((IDispatch*)&This->obj->IDispatchEx_iface != V_DISPATCH(params->rgvarg+1)) {
+            FIXME("obj != this_arg not supported\n");
+            return E_FAIL;
+        }
+        if(V_VT(params->rgvarg) == VT_DISPATCH) {
+            hres = args_to_dp(V_DISPATCH(params->rgvarg), lcid, &dp);
+            if(FAILED(hres))
+                return hres;
+        }
+
+        hres = function_value(&This->dispex, lcid, flags, &dp, res, ei, caller);
+
+        TRACE("function returned %08x\n", hres);
+
+        for(i = 0; i < dp.cArgs; i++)
+            VariantClear(dp.rgvarg + i);
+        heap_free(dp.rgvarg);
+        return hres;
+    }
+    case MSHTML_DISPID_CUSTOM_MIN + 1: {
+        DISPPARAMS dp = *params;
+        IDispatch *disp_this;
+        HRESULT hres;
+
+        if(!params->cArgs || params->cNamedArgs || V_VT(params->rgvarg + params->cArgs - 1) != VT_DISPATCH) {
+            FIXME("unsupported apply arguments\n");
+            return E_FAIL;
+        }
+
+        FIXME("%p.call(%s ...)\n", This, debugstr_variant(params->rgvarg + params->cArgs - 1));
+
+        hres = IDispatch_QueryInterface(V_DISPATCH(params->rgvarg + params->cArgs - 1),
+                                        tid_ids[This->info->tid], (void**)&disp_this);
+        if(FAILED(hres)) {
+            FIXME("call iface not supported\n");
+            return E_NOTIMPL;
+        }
+
+        dp.cArgs--;
+        hres = IDispatch_Invoke(disp_this, This->info->id, &IID_NULL, 0, DISPATCH_METHOD, &dp, res, ei, NULL);
+        IDispatch_Release(disp_this);
+
+        TRACE("function returned %08x\n", hres);
+        return hres;
+    }
+    }
+
+    return DISP_E_MEMBERNOTFOUND;
+}
+
 static const dispex_static_data_vtbl_t function_dispex_vtbl = {
     function_value,
-    NULL,
-    NULL,
+    function_get_dispid,
+    function_dispex_invoke,
     NULL
 };
 
 static const tid_t function_iface_tids[] = {0};
 
 static dispex_static_data_t function_dispex = {
+    L"Function",
     &function_dispex_vtbl,
     NULL_tid,
     function_iface_tids
@@ -822,7 +1050,7 @@ static func_disp_t *create_func_disp(DispatchEx *obj, func_info_t *info)
         return NULL;
 
     ret->IUnknown_iface.lpVtbl = &FunctionUnkVtbl;
-    init_dispex(&ret->dispex, &ret->IUnknown_iface,  &function_dispex);
+    init_dispatch(&ret->dispex, &ret->IUnknown_iface,  &function_dispex, dispex_compat_mode(obj));
     ret->ref = 1;
     ret->obj = obj;
     ret->info = info;
@@ -859,13 +1087,13 @@ static HRESULT invoke_disp_value(DispatchEx *This, IDispatch *func_disp, LCID lc
         hres = IDispatchEx_InvokeEx(dispex, DISPID_VALUE, lcid, flags, &new_dp, res, ei, caller);
         IDispatchEx_Release(dispex);
     }else {
-        ULONG err = 0;
+        UINT err = 0;
         hres = IDispatch_Invoke(func_disp, DISPID_VALUE, &IID_NULL, lcid, flags, &new_dp, res, ei, &err);
     }
     if(SUCCEEDED(hres))
         TRACE("<<< %s\n", debugstr_variant(res));
     else
-        WARN("<<< %08x\n", hres);
+        WARN("<<< %08lx\n", hres);
 
     heap_free(new_dp.rgvarg);
     return hres;
@@ -922,7 +1150,7 @@ static HRESULT get_builtin_func(dispex_data_t *data, DISPID id, func_info_t **re
             max = n-1;
     }
 
-    WARN("invalid id %x\n", id);
+    WARN("invalid id %lx\n", id);
     return DISP_E_UNKNOWNNAME;
 }
 
@@ -936,9 +1164,9 @@ static HRESULT get_builtin_id(DispatchEx *This, BSTR name, DWORD grfdex, DISPID
     while(min <= max) {
         n = (min+max)/2;
 
-        c = strcmpiW(This->info->name_table[n]->name, name);
+        c = wcsicmp(This->info->name_table[n]->name, name);
         if(!c) {
-            if((grfdex & fdexNameCaseSensitive) && strcmpW(This->info->name_table[n]->name, name))
+            if((grfdex & fdexNameCaseSensitive) && wcscmp(This->info->name_table[n]->name, name))
                 break;
 
             *ret = This->info->name_table[n]->id;
@@ -962,7 +1190,7 @@ static HRESULT get_builtin_id(DispatchEx *This, BSTR name, DWORD grfdex, DISPID
     return DISP_E_UNKNOWNNAME;
 }
 
-static HRESULT change_type(VARIANT *dst, VARIANT *src, VARTYPE vt, IServiceProvider *caller)
+HRESULT change_type(VARIANT *dst, VARIANT *src, VARTYPE vt, IServiceProvider *caller)
 {
     V_VT(dst) = VT_EMPTY;
 
@@ -1044,6 +1272,10 @@ static HRESULT builtin_propput(DispatchEx *This, func_info_t *func, DISPPARAMS *
     }
 
     if(!func->put_vtbl_off) {
+        if(dispex_compat_mode(This) >= COMPAT_MODE_IE9) {
+            WARN("No setter\n");
+            return S_OK;
+        }
         FIXME("No setter\n");
         return E_FAIL;
     }
@@ -1078,19 +1310,24 @@ static HRESULT builtin_propput(DispatchEx *This, func_info_t *func, DISPPARAMS *
     return hres;
 }
 
-static HRESULT invoke_builtin_function(DispatchEx *This, func_info_t *func, DISPPARAMS *dp, VARIANT *res, IServiceProvider *caller)
+static HRESULT invoke_builtin_function(DispatchEx *This, func_info_t *func, DISPPARAMS *dp,
+                                       VARIANT *res, EXCEPINFO *ei, IServiceProvider *caller)
 {
     VARIANT arg_buf[MAX_ARGS], *arg_ptrs[MAX_ARGS], *arg, retv, ret_ref, vhres;
     unsigned i, nconv = 0;
     IUnknown *iface;
     HRESULT hres;
 
-    if(dp->cNamedArgs) {
-        FIXME("Named arguments not supported\n");
-        return E_NOTIMPL;
+    if(func->hook) {
+        hres = func->hook(This, DISPATCH_METHOD, dp, res, ei, caller);
+        if(hres != S_FALSE)
+            return hres;
     }
 
-    if(dp->cArgs != func->argc) {
+    if(!func->call_vtbl_off)
+        return typeinfo_invoke(This, func, DISPATCH_METHOD, dp, res, ei);
+
+    if(dp->cArgs + func->default_value_cnt < func->argc) {
         FIXME("Invalid argument count (expected %u, got %u)\n", func->argc, dp->cArgs);
         return E_INVALIDARG;
     }
@@ -1100,6 +1337,12 @@ static HRESULT invoke_builtin_function(DispatchEx *This, func_info_t *func, DISP
         return hres;
 
     for(i=0; i < func->argc; i++) {
+        BOOL own_value = FALSE;
+        if(i >= dp->cArgs) {
+            /* use default value */
+            arg_ptrs[i] = &func->arg_info[i].default_value;
+            continue;
+        }
         arg = dp->rgvarg+dp->cArgs-i-1;
         if(func->arg_types[i] == V_VT(arg)) {
             arg_ptrs[i] = arg;
@@ -1108,6 +1351,24 @@ static HRESULT invoke_builtin_function(DispatchEx *This, func_info_t *func, DISP
             if(FAILED(hres))
                 break;
             arg_ptrs[i] = arg_buf + nconv++;
+            own_value = TRUE;
+        }
+
+        if(func->arg_types[i] == VT_DISPATCH && !IsEqualGUID(&func->arg_info[i].iid, &IID_NULL)
+            && V_DISPATCH(arg_ptrs[i])) {
+            IDispatch *iface;
+            if(!own_value) {
+                arg_buf[nconv] = *arg_ptrs[i];
+                arg_ptrs[i] = arg_buf + nconv++;
+            }
+            hres = IDispatch_QueryInterface(V_DISPATCH(arg_ptrs[i]), &func->arg_info[i].iid, (void**)&iface);
+            if(FAILED(hres)) {
+                WARN("Could not get %s iface: %08lx\n", debugstr_guid(&func->arg_info[i].iid), hres);
+                break;
+            }
+            if(own_value)
+                IDispatch_Release(V_DISPATCH(arg_ptrs[i]));
+            V_DISPATCH(arg_ptrs[i]) = iface;
         }
     }
 
@@ -1124,8 +1385,13 @@ static HRESULT invoke_builtin_function(DispatchEx *This, func_info_t *func, DISP
             case vt:                                    \
                 V_BYREF(&ret_ref) = &access(&retv);     \
                 break
-            BUILTIN_TYPES_SWITCH;
+            BUILTIN_ARG_TYPES_SWITCH;
 #undef CASE_VT
+            case VT_PTR:
+                V_VT(&retv) = VT_DISPATCH;
+                V_VT(&ret_ref) = VT_BYREF | VT_DISPATCH;
+                V_BYREF(&ret_ref) = &V_DISPATCH(&retv);
+                break;
             default:
                 assert(0);
             }
@@ -1182,10 +1448,7 @@ static HRESULT function_invoke(DispatchEx *This, func_info_t *func, WORD flags,
             }
         }
 
-        if(func->call_vtbl_off)
-            hres = invoke_builtin_function(This, func, dp, res, caller);
-        else
-            hres = typeinfo_invoke(This, func, flags, dp, res, ei);
+        hres = invoke_builtin_function(This, func, dp, res, ei, caller);
         break;
     case DISPATCH_PROPERTYGET: {
         func_obj_entry_t *entry;
@@ -1193,9 +1456,9 @@ static HRESULT function_invoke(DispatchEx *This, func_info_t *func, WORD flags,
         if(func->id == DISPID_VALUE) {
             BSTR ret;
 
-            ret = SysAllocString(objectW);
-            if(!ret)
-                return E_OUTOFMEMORY;
+            hres = dispex_to_string(This, &ret);
+            if(FAILED(hres))
+                return hres;
 
             V_VT(res) = VT_BSTR;
             V_BSTR(res) = ret;
@@ -1250,15 +1513,15 @@ static HRESULT invoke_builtin_prop(DispatchEx *This, DISPID id, LCID lcid, WORD
     if(FAILED(hres))
         return hres;
 
+    if(func->func_disp_idx != -1)
+        return function_invoke(This, func, flags, dp, res, ei, caller);
+
     if(func->hook) {
-        hres = func->hook(This, lcid, flags, dp, res, ei, caller);
+        hres = func->hook(This, flags, dp, res, ei, caller);
         if(hres != S_FALSE)
             return hres;
     }
 
-    if(func->func_disp_idx != -1)
-        return function_invoke(This, func, flags, dp, res, ei, caller);
-
     switch(flags) {
     case DISPATCH_PROPERTYPUT:
         if(res)
@@ -1298,6 +1561,19 @@ static HRESULT invoke_builtin_prop(DispatchEx *This, DISPID id, LCID lcid, WORD
     return hres;
 }
 
+HRESULT dispex_call_builtin(DispatchEx *dispex, DISPID id, DISPPARAMS *dp,
+                            VARIANT *res, EXCEPINFO *ei, IServiceProvider *caller)
+{
+    func_info_t *func;
+    HRESULT hres;
+
+    hres = get_builtin_func(dispex->info, id, &func);
+    if(FAILED(hres))
+        return hres;
+
+    return invoke_builtin_function(dispex, func, dp, res, ei, caller);
+}
+
 HRESULT remove_attribute(DispatchEx *This, DISPID id, VARIANT_BOOL *success)
 {
     switch(get_dispid_type(id)) {
@@ -1349,13 +1625,18 @@ HRESULT remove_attribute(DispatchEx *This, DISPID id, VARIANT_BOOL *success)
             *success = VARIANT_TRUE;
             return S_OK;
         }
+        *success = VARIANT_TRUE;
 
         V_VT(&var) = VT_EMPTY;
         hres = builtin_propput(This, func, &dp, NULL);
-        if(FAILED(hres))
-            return hres;
-
-        *success = VARIANT_TRUE;
+        if(FAILED(hres)) {
+            VARIANT *ref;
+            hres = dispex_get_dprop_ref(This, func->name, FALSE, &ref);
+            if(FAILED(hres) || V_VT(ref) != VT_BSTR)
+                *success = VARIANT_FALSE;
+            else
+                VariantClear(ref);
+        }
         return S_OK;
     }
     default:
@@ -1371,6 +1652,34 @@ compat_mode_t dispex_compat_mode(DispatchEx *dispex)
         : dispex->info->desc->vtbl->get_compat_mode(dispex);
 }
 
+HRESULT dispex_to_string(DispatchEx *dispex, BSTR *ret)
+{
+    static const WCHAR prefix[8] = L"[object ";
+    static const WCHAR suffix[] = L"]";
+    WCHAR buf[ARRAY_SIZE(prefix) + 28 + ARRAY_SIZE(suffix)], *p = buf;
+    compat_mode_t compat_mode = dispex_compat_mode(dispex);
+    const WCHAR *name = dispex->info->desc->name;
+    unsigned len;
+
+    if(!ret)
+        return E_INVALIDARG;
+
+    memcpy(p, prefix, sizeof(prefix));
+    p += ARRAY_SIZE(prefix);
+    if(compat_mode < COMPAT_MODE_IE9)
+        p--;
+    else {
+        len = wcslen(name);
+        assert(len <= 28);
+        memcpy(p, name, len * sizeof(WCHAR));
+        p += len;
+    }
+    memcpy(p, suffix, sizeof(suffix));
+
+    *ret = SysAllocString(buf);
+    return *ret ? S_OK : E_OUTOFMEMORY;
+}
+
 static dispex_data_t *ensure_dispex_info(dispex_static_data_t *desc, compat_mode_t compat_mode)
 {
     if(!desc->info_cache[compat_mode]) {
@@ -1433,7 +1742,7 @@ static HRESULT WINAPI DispatchEx_GetTypeInfo(IDispatchEx *iface, UINT iTInfo,
     DispatchEx *This = impl_from_IDispatchEx(iface);
     HRESULT hres;
 
-    TRACE("(%p)->(%u %u %p)\n", This, iTInfo, lcid, ppTInfo);
+    TRACE("(%p)->(%u %lu %p)\n", This, iTInfo, lcid, ppTInfo);
 
     hres = get_typeinfo(This->info->desc->disp_tid, ppTInfo);
     if(FAILED(hres))
@@ -1451,7 +1760,7 @@ static HRESULT WINAPI DispatchEx_GetIDsOfNames(IDispatchEx *iface, REFIID riid,
     UINT i;
     HRESULT hres;
 
-    TRACE("(%p)->(%s %p %u %u %p)\n", This, debugstr_guid(riid), rgszNames, cNames,
+    TRACE("(%p)->(%s %p %u %lu %p)\n", This, debugstr_guid(riid), rgszNames, cNames,
           lcid, rgDispId);
 
     for(i=0; i < cNames; i++) {
@@ -1469,7 +1778,7 @@ static HRESULT WINAPI DispatchEx_Invoke(IDispatchEx *iface, DISPID dispIdMember,
 {
     DispatchEx *This = impl_from_IDispatchEx(iface);
 
-    TRACE("(%p)->(%d %s %d %d %p %p %p %p)\n", This, dispIdMember, debugstr_guid(riid),
+    TRACE("(%p)->(%ld %s %ld %d %p %p %p %p)\n", This, dispIdMember, debugstr_guid(riid),
           lcid, wFlags, pDispParams, pVarResult, pExcepInfo, puArgErr);
 
     return IDispatchEx_InvokeEx(&This->IDispatchEx_iface, dispIdMember, lcid, wFlags, pDispParams,
@@ -1482,10 +1791,10 @@ static HRESULT WINAPI DispatchEx_GetDispID(IDispatchEx *iface, BSTR bstrName, DW
     dynamic_prop_t *dprop;
     HRESULT hres;
 
-    TRACE("(%p)->(%s %x %p)\n", This, debugstr_w(bstrName), grfdex, pid);
+    TRACE("(%p)->(%s %lx %p)\n", This, debugstr_w(bstrName), grfdex, pid);
 
     if(grfdex & ~(fdexNameCaseSensitive|fdexNameCaseInsensitive|fdexNameEnsure|fdexNameImplicit|FDEX_VERSION_MASK))
-        FIXME("Unsupported grfdex %x\n", grfdex);
+        FIXME("Unsupported grfdex %lx\n", grfdex);
 
     if(!ensure_real_info(This))
         return E_OUTOFMEMORY;
@@ -1508,7 +1817,7 @@ static HRESULT WINAPI DispatchEx_InvokeEx(IDispatchEx *iface, DISPID id, LCID lc
     DispatchEx *This = impl_from_IDispatchEx(iface);
     HRESULT hres;
 
-    TRACE("(%p)->(%x %x %x %p %p %p %p)\n", This, id, lcid, wFlags, pdp, pvarRes, pei, pspCaller);
+    TRACE("(%p)->(%lx %lx %x %p %p %p %p)\n", This, id, lcid, wFlags, pdp, pvarRes, pei, pspCaller);
 
     if(!ensure_real_info(This))
         return E_OUTOFMEMORY;
@@ -1588,36 +1897,59 @@ static HRESULT WINAPI DispatchEx_InvokeEx(IDispatchEx *iface, DISPID id, LCID lc
     }
 }
 
-static HRESULT WINAPI DispatchEx_DeleteMemberByName(IDispatchEx *iface, BSTR bstrName, DWORD grfdex)
+static HRESULT WINAPI DispatchEx_DeleteMemberByName(IDispatchEx *iface, BSTR name, DWORD grfdex)
 {
     DispatchEx *This = impl_from_IDispatchEx(iface);
+    DISPID id;
+    HRESULT hres;
 
-    TRACE("(%p)->(%s %x)\n", This, debugstr_w(bstrName), grfdex);
+    TRACE("(%p)->(%s %lx)\n", This, debugstr_w(name), grfdex);
 
-    /* HACK: needed by Office; we shouldn't use IDispatchEx in this case */
-    if(dispex_compat_mode(This) >= COMPAT_MODE_IE9) {
-        FIXME("returning S_OK\n");
-        return S_OK;
+    if(dispex_compat_mode(This) < COMPAT_MODE_IE8) {
+        /* Not implemented by IE */
+        return E_NOTIMPL;
     }
 
-    /* Not implemented by IE */
-    return E_NOTIMPL;
+    hres = IDispatchEx_GetDispID(&This->IDispatchEx_iface, name, grfdex & ~fdexNameEnsure, &id);
+    if(FAILED(hres)) {
+        TRACE("property %s not found\n", debugstr_w(name));
+        return dispex_compat_mode(This) < COMPAT_MODE_IE9 ? hres : S_OK;
+    }
+
+    return IDispatchEx_DeleteMemberByDispID(&This->IDispatchEx_iface, id);
 }
 
 static HRESULT WINAPI DispatchEx_DeleteMemberByDispID(IDispatchEx *iface, DISPID id)
 {
     DispatchEx *This = impl_from_IDispatchEx(iface);
 
-    TRACE("(%p)->(%x)\n", This, id);
+    TRACE("(%p)->(%lx)\n", This, id);
 
-    /* Not implemented by IE */
-    return E_NOTIMPL;
+    if(dispex_compat_mode(This) < COMPAT_MODE_IE8) {
+        /* Not implemented by IE */
+        return E_NOTIMPL;
+    }
+
+    if(is_dynamic_dispid(id)) {
+        DWORD idx = id - DISPID_DYNPROP_0;
+        dynamic_prop_t *prop;
+
+        if(!get_dynamic_data(This) || idx >= This->dynamic_data->prop_cnt)
+            return S_OK;
+
+        prop = This->dynamic_data->props + idx;
+        VariantClear(&prop->var);
+        prop->flags |= DYNPROP_DELETED;
+        return S_OK;
+    }
+
+    return S_OK;
 }
 
 static HRESULT WINAPI DispatchEx_GetMemberProperties(IDispatchEx *iface, DISPID id, DWORD grfdexFetch, DWORD *pgrfdex)
 {
     DispatchEx *This = impl_from_IDispatchEx(iface);
-    FIXME("(%p)->(%x %x %p)\n", This, id, grfdexFetch, pgrfdex);
+    FIXME("(%p)->(%lx %lx %p)\n", This, id, grfdexFetch, pgrfdex);
     return E_NOTIMPL;
 }
 
@@ -1627,7 +1959,7 @@ static HRESULT WINAPI DispatchEx_GetMemberName(IDispatchEx *iface, DISPID id, BS
     func_info_t *func;
     HRESULT hres;
 
-    TRACE("(%p)->(%x %p)\n", This, id, pbstrName);
+    TRACE("(%p)->(%lx %p)\n", This, id, pbstrName);
 
     if(!ensure_real_info(This))
         return E_OUTOFMEMORY;
@@ -1675,7 +2007,7 @@ static HRESULT WINAPI DispatchEx_GetNextDispID(IDispatchEx *iface, DWORD grfdex,
     func_info_t *func;
     HRESULT hres;
 
-    TRACE("(%p)->(%x %x %p)\n", This, grfdex, id, pid);
+    TRACE("(%p)->(%lx %lx %p)\n", This, grfdex, id, pid);
 
     if(!ensure_real_info(This))
         return E_OUTOFMEMORY;
@@ -1829,7 +2161,7 @@ void release_dispex(DispatchEx *This)
     heap_free(This->dynamic_data);
 }
 
-void init_dispex_with_compat_mode(DispatchEx *dispex, IUnknown *outer, dispex_static_data_t *data, compat_mode_t compat_mode)
+void init_dispatch(DispatchEx *dispex, IUnknown *outer, dispex_static_data_t *data, compat_mode_t compat_mode)
 {
     assert(compat_mode < COMPAT_MODE_CNT);
 
