@@ -17,7 +17,6 @@
  */
 
 #include <stdarg.h>
-#include <assert.h>
 #include <limits.h>
 
 #define COBJMACROS
@@ -39,38 +38,11 @@ struct HTMLInputElement {
 
     IHTMLInputElement IHTMLInputElement_iface;
     IHTMLInputTextElement IHTMLInputTextElement_iface;
+    IHTMLInputTextElement2 IHTMLInputTextElement2_iface;
 
     nsIDOMHTMLInputElement *nsinput;
 };
 
-static const WCHAR forW[] = {'f','o','r',0};
-
-static HRESULT return_nsform(HTMLElement *elem, nsIDOMHTMLFormElement *nsform, IHTMLFormElement **p)
-{
-    nsIDOMNode *form_node;
-    HTMLDOMNode *node;
-    nsresult nsres;
-    HRESULT hres;
-
-    if(!nsform) {
-        *p = NULL;
-        return S_OK;
-    }
-
-    nsres = nsIDOMHTMLFormElement_QueryInterface(nsform, &IID_nsIDOMNode, (void**)&form_node);
-    nsIDOMHTMLFormElement_Release(nsform);
-    assert(nsres == NS_OK);
-
-    hres = get_node(elem->node.doc, form_node, TRUE, &node);
-    nsIDOMNode_Release(form_node);
-    if (FAILED(hres))
-        return hres;
-
-    hres = IHTMLDOMNode_QueryInterface(&node->IHTMLDOMNode_iface, &IID_IHTMLElement, (void**)p);
-    node_release(node);
-    return hres;
-}
-
 static inline HTMLInputElement *impl_from_IHTMLInputElement(IHTMLInputElement *iface)
 {
     return CONTAINING_RECORD(iface, HTMLInputElement, IHTMLInputElement_iface);
@@ -155,7 +127,7 @@ static HRESULT WINAPI HTMLInputElement_put_type(IHTMLInputElement *iface, BSTR v
     nsres = nsIDOMHTMLInputElement_SetType(This->nsinput, &type_str);
     nsAString_Finish(&type_str);
     if(NS_FAILED(nsres)) {
-        ERR("SetType failed: %08x\n", nsres);
+        ERR("SetType failed: %08lx\n", nsres);
         return E_FAIL;
     }
 
@@ -187,7 +159,7 @@ static HRESULT WINAPI HTMLInputElement_put_value(IHTMLInputElement *iface, BSTR
     nsres = nsIDOMHTMLInputElement_SetValue(This->nsinput, &val_str);
     nsAString_Finish(&val_str);
     if(NS_FAILED(nsres))
-        ERR("SetValue failed: %08x\n", nsres);
+        ERR("SetValue failed: %08lx\n", nsres);
 
     return S_OK;
 }
@@ -217,7 +189,7 @@ static HRESULT WINAPI HTMLInputElement_put_name(IHTMLInputElement *iface, BSTR v
     nsres = nsIDOMHTMLInputElement_SetName(This->nsinput, &name_str);
     nsAString_Finish(&name_str);
     if(NS_FAILED(nsres)) {
-        ERR("SetName failed: %08x\n", nsres);
+        ERR("SetName failed: %08lx\n", nsres);
         return E_FAIL;
     }
 
@@ -260,7 +232,7 @@ static HRESULT WINAPI HTMLInputElement_put_disabled(IHTMLInputElement *iface, VA
 
     nsres = nsIDOMHTMLInputElement_SetDisabled(This->nsinput, v != VARIANT_FALSE);
     if(NS_FAILED(nsres))
-        ERR("SetDisabled failed: %08x\n", nsres);
+        ERR("SetDisabled failed: %08lx\n", nsres);
 
     return S_OK;
 }
@@ -274,7 +246,7 @@ static HRESULT WINAPI HTMLInputElement_get_disabled(IHTMLInputElement *iface, VA
 
     nsIDOMHTMLInputElement_GetDisabled(This->nsinput, &disabled);
 
-    *p = disabled ? VARIANT_TRUE : VARIANT_FALSE;
+    *p = variant_bool(disabled);
     return S_OK;
 }
 
@@ -287,12 +259,7 @@ static HRESULT WINAPI HTMLInputElement_get_form(IHTMLInputElement *iface, IHTMLF
     TRACE("(%p)->(%p)\n", This, p);
 
     nsres = nsIDOMHTMLInputElement_GetForm(This->nsinput, &nsform);
-    if (NS_FAILED(nsres)) {
-        ERR("GetForm failed: %08x, nsform: %p\n", nsres, nsform);
-        return E_FAIL;
-    }
-
-    return return_nsform(&This->element, nsform, p);
+    return return_nsform(nsres, nsform, p);
 }
 
 static HRESULT WINAPI HTMLInputElement_put_size(IHTMLInputElement *iface, LONG v)
@@ -301,13 +268,13 @@ static HRESULT WINAPI HTMLInputElement_put_size(IHTMLInputElement *iface, LONG v
     UINT32 val = v;
     nsresult nsres;
 
-    TRACE("(%p)->(%d)\n", This, v);
+    TRACE("(%p)->(%ld)\n", This, v);
     if (v <= 0)
         return CTL_E_INVALIDPROPERTYVALUE;
 
     nsres = nsIDOMHTMLInputElement_SetSize(This->nsinput, val);
     if (NS_FAILED(nsres)) {
-        ERR("Set Size(%u) failed: %08x\n", val, nsres);
+        ERR("Set Size(%u) failed: %08lx\n", val, nsres);
         return E_FAIL;
     }
     return S_OK;
@@ -325,7 +292,7 @@ static HRESULT WINAPI HTMLInputElement_get_size(IHTMLInputElement *iface, LONG *
 
     nsres = nsIDOMHTMLInputElement_GetSize(This->nsinput, &val);
     if (NS_FAILED(nsres)) {
-        ERR("Get Size failed: %08x\n", nsres);
+        ERR("Get Size failed: %08lx\n", nsres);
         return E_FAIL;
     }
     *p = val;
@@ -337,7 +304,7 @@ static HRESULT WINAPI HTMLInputElement_put_maxLength(IHTMLInputElement *iface, L
     HTMLInputElement *This = impl_from_IHTMLInputElement(iface);
     nsresult nsres;
 
-    TRACE("(%p)->(%d)\n", This, v);
+    TRACE("(%p)->(%ld)\n", This, v);
 
     nsres = nsIDOMHTMLInputElement_SetMaxLength(This->nsinput, v);
     if(NS_FAILED(nsres)) {
@@ -374,7 +341,7 @@ static HRESULT WINAPI HTMLInputElement_select(IHTMLInputElement *iface)
 
     nsres = nsIDOMHTMLInputElement_Select(This->nsinput);
     if(NS_FAILED(nsres)) {
-        ERR("Select failed: %08x\n", nsres);
+        ERR("Select failed: %08lx\n", nsres);
         return E_FAIL;
     }
 
@@ -425,7 +392,7 @@ static HRESULT WINAPI HTMLInputElement_put_defaultValue(IHTMLInputElement *iface
     nsres = nsIDOMHTMLInputElement_SetDefaultValue(This->nsinput, &nsstr);
     nsAString_Finish(&nsstr);
     if(NS_FAILED(nsres)) {
-        ERR("SetValue failed: %08x\n", nsres);
+        ERR("SetValue failed: %08lx\n", nsres);
         return E_FAIL;
     }
 
@@ -454,7 +421,7 @@ static HRESULT WINAPI HTMLInputElement_put_readOnly(IHTMLInputElement *iface, VA
 
     nsres = nsIDOMHTMLInputElement_SetReadOnly(This->nsinput, v != VARIANT_FALSE);
     if (NS_FAILED(nsres)) {
-        ERR("Set ReadOnly Failed: %08x\n", nsres);
+        ERR("Set ReadOnly Failed: %08lx\n", nsres);
         return E_FAIL;
     }
     return S_OK;
@@ -470,10 +437,11 @@ static HRESULT WINAPI HTMLInputElement_get_readOnly(IHTMLInputElement *iface, VA
 
     nsres = nsIDOMHTMLInputElement_GetReadOnly(This->nsinput, &b);
     if (NS_FAILED(nsres)) {
-        ERR("Get ReadOnly Failed: %08x\n", nsres);
+        ERR("Get ReadOnly Failed: %08lx\n", nsres);
         return E_FAIL;
     }
-    *p = b ? VARIANT_TRUE : VARIANT_FALSE;
+
+    *p = variant_bool(b);
     return S_OK;
 }
 
@@ -507,7 +475,7 @@ static HRESULT WINAPI HTMLInputElement_put_defaultChecked(IHTMLInputElement *ifa
 
     nsres = nsIDOMHTMLInputElement_SetDefaultChecked(This->nsinput, v != VARIANT_FALSE);
     if(NS_FAILED(nsres)) {
-        ERR("SetDefaultChecked failed: %08x\n", nsres);
+        ERR("SetDefaultChecked failed: %08lx\n", nsres);
         return E_FAIL;
     }
 
@@ -524,11 +492,11 @@ static HRESULT WINAPI HTMLInputElement_get_defaultChecked(IHTMLInputElement *ifa
 
     nsres = nsIDOMHTMLInputElement_GetDefaultChecked(This->nsinput, &default_checked);
     if(NS_FAILED(nsres)) {
-        ERR("GetDefaultChecked failed: %08x\n", nsres);
+        ERR("GetDefaultChecked failed: %08lx\n", nsres);
         return E_FAIL;
     }
 
-    *p = default_checked ? VARIANT_TRUE : VARIANT_FALSE;
+    *p = variant_bool(default_checked);
     return S_OK;
 }
 
@@ -541,7 +509,7 @@ static HRESULT WINAPI HTMLInputElement_put_checked(IHTMLInputElement *iface, VAR
 
     nsres = nsIDOMHTMLInputElement_SetChecked(This->nsinput, v != VARIANT_FALSE);
     if(NS_FAILED(nsres)) {
-        ERR("SetChecked failed: %08x\n", nsres);
+        ERR("SetChecked failed: %08lx\n", nsres);
         return E_FAIL;
     }
 
@@ -558,11 +526,11 @@ static HRESULT WINAPI HTMLInputElement_get_checked(IHTMLInputElement *iface, VAR
 
     nsres = nsIDOMHTMLInputElement_GetChecked(This->nsinput, &checked);
     if(NS_FAILED(nsres)) {
-        ERR("GetChecked failed: %08x\n", nsres);
+        ERR("GetChecked failed: %08lx\n", nsres);
         return E_FAIL;
     }
 
-    *p = checked ? VARIANT_TRUE : VARIANT_FALSE;
+    *p = variant_bool(checked);
     TRACE("checked=%x\n", *p);
     return S_OK;
 }
@@ -584,7 +552,7 @@ static HRESULT WINAPI HTMLInputElement_get_border(IHTMLInputElement *iface, VARI
 static HRESULT WINAPI HTMLInputElement_put_vspace(IHTMLInputElement *iface, LONG v)
 {
     HTMLInputElement *This = impl_from_IHTMLInputElement(iface);
-    FIXME("(%p)->(%d)\n", This, v);
+    FIXME("(%p)->(%ld)\n", This, v);
     return E_NOTIMPL;
 }
 
@@ -598,7 +566,7 @@ static HRESULT WINAPI HTMLInputElement_get_vspace(IHTMLInputElement *iface, LONG
 static HRESULT WINAPI HTMLInputElement_put_hspace(IHTMLInputElement *iface, LONG v)
 {
     HTMLInputElement *This = impl_from_IHTMLInputElement(iface);
-    FIXME("(%p)->(%d)\n", This, v);
+    FIXME("(%p)->(%ld)\n", This, v);
     return E_NOTIMPL;
 }
 
@@ -635,7 +603,7 @@ static HRESULT WINAPI HTMLInputElement_put_src(IHTMLInputElement *iface, BSTR v)
     nsres = nsIDOMHTMLInputElement_SetSrc(This->nsinput, &nsstr);
     nsAString_Finish(&nsstr);
     if(NS_FAILED(nsres))
-        ERR("SetSrc failed: %08x\n", nsres);
+        ERR("SetSrc failed: %08lx\n", nsres);
 
     return S_OK;
 }
@@ -653,7 +621,7 @@ static HRESULT WINAPI HTMLInputElement_get_src(IHTMLInputElement *iface, BSTR *p
     nsAString_Init(&src_str, NULL);
     nsres = nsIDOMHTMLInputElement_GetSrc(This->nsinput, &src_str);
     if(NS_FAILED(nsres)) {
-        ERR("GetSrc failed: %08x\n", nsres);
+        ERR("GetSrc failed: %08lx\n", nsres);
         return E_FAIL;
     }
 
@@ -793,7 +761,7 @@ static HRESULT WINAPI HTMLInputElement_get_onabort(IHTMLInputElement *iface, VAR
 static HRESULT WINAPI HTMLInputElement_put_width(IHTMLInputElement *iface, LONG v)
 {
     HTMLInputElement *This = impl_from_IHTMLInputElement(iface);
-    FIXME("(%p)->(%d)\n", This, v);
+    FIXME("(%p)->(%ld)\n", This, v);
     return E_NOTIMPL;
 }
 
@@ -807,7 +775,7 @@ static HRESULT WINAPI HTMLInputElement_get_width(IHTMLInputElement *iface, LONG
 static HRESULT WINAPI HTMLInputElement_put_height(IHTMLInputElement *iface, LONG v)
 {
     HTMLInputElement *This = impl_from_IHTMLInputElement(iface);
-    FIXME("(%p)->(%d)\n", This, v);
+    FIXME("(%p)->(%ld)\n", This, v);
     return E_NOTIMPL;
 }
 
@@ -1068,7 +1036,7 @@ static HRESULT WINAPI HTMLInputTextElement_put_size(IHTMLInputTextElement *iface
 {
     HTMLInputElement *This = impl_from_IHTMLInputTextElement(iface);
 
-    TRACE("(%p)->(%d)\n", This, v);
+    TRACE("(%p)->(%ld)\n", This, v);
 
     return IHTMLInputElement_put_size(&This->IHTMLInputElement_iface, v);
 }
@@ -1086,7 +1054,7 @@ static HRESULT WINAPI HTMLInputTextElement_put_maxLength(IHTMLInputTextElement *
 {
     HTMLInputElement *This = impl_from_IHTMLInputTextElement(iface);
 
-    TRACE("(%p)->(%d)\n", This, v);
+    TRACE("(%p)->(%ld)\n", This, v);
 
     return IHTMLInputElement_put_maxLength(&This->IHTMLInputElement_iface, v);
 }
@@ -1206,6 +1174,158 @@ static const IHTMLInputTextElementVtbl HTMLInputTextElementVtbl = {
     HTMLInputTextElement_createTextRange
 };
 
+static inline HTMLInputElement *impl_from_IHTMLInputTextElement2(IHTMLInputTextElement2 *iface)
+{
+    return CONTAINING_RECORD(iface, HTMLInputElement, IHTMLInputTextElement2_iface);
+}
+
+static HRESULT WINAPI HTMLInputTextElement2_QueryInterface(IHTMLInputTextElement2 *iface, REFIID riid, void **ppv)
+{
+    HTMLInputElement *This = impl_from_IHTMLInputTextElement2(iface);
+    return IHTMLDOMNode_QueryInterface(&This->element.node.IHTMLDOMNode_iface, riid, ppv);
+}
+
+static ULONG WINAPI HTMLInputTextElement2_AddRef(IHTMLInputTextElement2 *iface)
+{
+    HTMLInputElement *This = impl_from_IHTMLInputTextElement2(iface);
+    return IHTMLDOMNode_AddRef(&This->element.node.IHTMLDOMNode_iface);
+}
+
+static ULONG WINAPI HTMLInputTextElement2_Release(IHTMLInputTextElement2 *iface)
+{
+    HTMLInputElement *This = impl_from_IHTMLInputTextElement2(iface);
+    return IHTMLDOMNode_Release(&This->element.node.IHTMLDOMNode_iface);
+}
+
+static HRESULT WINAPI HTMLInputTextElement2_GetTypeInfoCount(IHTMLInputTextElement2 *iface, UINT *pctinfo)
+{
+    HTMLInputElement *This = impl_from_IHTMLInputTextElement2(iface);
+    return IDispatchEx_GetTypeInfoCount(&This->element.node.event_target.dispex.IDispatchEx_iface, pctinfo);
+}
+
+static HRESULT WINAPI HTMLInputTextElement2_GetTypeInfo(IHTMLInputTextElement2 *iface, UINT iTInfo,
+        LCID lcid, ITypeInfo **ppTInfo)
+{
+    HTMLInputElement *This = impl_from_IHTMLInputTextElement2(iface);
+    return IDispatchEx_GetTypeInfo(&This->element.node.event_target.dispex.IDispatchEx_iface, iTInfo, lcid, ppTInfo);
+}
+
+static HRESULT WINAPI HTMLInputTextElement2_GetIDsOfNames(IHTMLInputTextElement2 *iface, REFIID riid,
+        LPOLESTR *rgszNames, UINT cNames, LCID lcid, DISPID *rgDispId)
+{
+    HTMLInputElement *This = impl_from_IHTMLInputTextElement2(iface);
+    return IDispatchEx_GetIDsOfNames(&This->element.node.event_target.dispex.IDispatchEx_iface, riid, rgszNames,
+            cNames, lcid, rgDispId);
+}
+
+static HRESULT WINAPI HTMLInputTextElement2_Invoke(IHTMLInputTextElement2 *iface, DISPID dispIdMember,
+                            REFIID riid, LCID lcid, WORD wFlags, DISPPARAMS *pDispParams,
+                            VARIANT *pVarResult, EXCEPINFO *pExcepInfo, UINT *puArgErr)
+{
+    HTMLInputElement *This = impl_from_IHTMLInputTextElement2(iface);
+    return IDispatchEx_Invoke(&This->element.node.event_target.dispex.IDispatchEx_iface, dispIdMember, riid,
+            lcid, wFlags, pDispParams, pVarResult, pExcepInfo, puArgErr);
+}
+
+static HRESULT WINAPI HTMLInputTextElement2_put_selectionStart(IHTMLInputTextElement2 *iface, LONG v)
+{
+    HTMLInputElement *This = impl_from_IHTMLInputTextElement2(iface);
+    nsresult nsres;
+
+    TRACE("(%p)->(%ld)\n", This, v);
+
+    nsres = nsIDOMHTMLInputElement_SetSelectionStart(This->nsinput, v);
+    if(NS_FAILED(nsres)) {
+        ERR("SetSelectionStart failed: %08lx\n", nsres);
+        return E_FAIL;
+    }
+    return S_OK;
+}
+
+static HRESULT WINAPI HTMLInputTextElement2_get_selectionStart(IHTMLInputTextElement2 *iface, LONG *p)
+{
+    HTMLInputElement *This = impl_from_IHTMLInputTextElement2(iface);
+    LONG selection_start;
+    nsresult nsres;
+
+    TRACE("(%p)->(%p)\n", This, p);
+
+    nsres = nsIDOMHTMLInputElement_GetSelectionStart(This->nsinput, &selection_start);
+    if(NS_FAILED(nsres)) {
+        ERR("GetSelectionStart failed: %08lx\n", nsres);
+        return E_FAIL;
+    }
+
+    *p = selection_start;
+    return S_OK;
+}
+
+static HRESULT WINAPI HTMLInputTextElement2_put_selectionEnd(IHTMLInputTextElement2 *iface, LONG v)
+{
+    HTMLInputElement *This = impl_from_IHTMLInputTextElement2(iface);
+    nsresult nsres;
+
+    TRACE("(%p)->(%ld)\n", This, v);
+
+    nsres = nsIDOMHTMLInputElement_SetSelectionEnd(This->nsinput, v);
+    if(NS_FAILED(nsres)) {
+        ERR("SetSelectionEnd failed: %08lx\n", nsres);
+        return E_FAIL;
+    }
+    return S_OK;
+}
+
+static HRESULT WINAPI HTMLInputTextElement2_get_selectionEnd(IHTMLInputTextElement2 *iface, LONG *p)
+{
+    HTMLInputElement *This = impl_from_IHTMLInputTextElement2(iface);
+    LONG selection_end;
+    nsresult nsres;
+
+    TRACE("(%p)->(%p)\n", This, p);
+
+    nsres = nsIDOMHTMLInputElement_GetSelectionEnd(This->nsinput, &selection_end);
+    if(NS_FAILED(nsres)) {
+        ERR("GetSelectionEnd failed: %08lx\n", nsres);
+        return E_FAIL;
+    }
+
+    *p = selection_end;
+    return S_OK;
+}
+
+static HRESULT WINAPI HTMLInputTextElement2_setSelectionRange(IHTMLInputTextElement2 *iface, LONG start, LONG end)
+{
+    HTMLInputElement *This = impl_from_IHTMLInputTextElement2(iface);
+    nsAString none_str;
+    nsresult nsres;
+
+    TRACE("(%p)->(%ld %ld)\n", This, start, end);
+
+    nsAString_InitDepend(&none_str, L"none");
+    nsres = nsIDOMHTMLInputElement_SetSelectionRange(This->nsinput, start, end, &none_str);
+    nsAString_Finish(&none_str);
+    if(NS_FAILED(nsres)) {
+        ERR("SetSelectionRange failed: %08lx\n", nsres);
+        return E_FAIL;
+    }
+    return S_OK;
+}
+
+static const IHTMLInputTextElement2Vtbl HTMLInputTextElement2Vtbl = {
+    HTMLInputTextElement2_QueryInterface,
+    HTMLInputTextElement2_AddRef,
+    HTMLInputTextElement2_Release,
+    HTMLInputTextElement2_GetTypeInfoCount,
+    HTMLInputTextElement2_GetTypeInfo,
+    HTMLInputTextElement2_GetIDsOfNames,
+    HTMLInputTextElement2_Invoke,
+    HTMLInputTextElement2_put_selectionStart,
+    HTMLInputTextElement2_get_selectionStart,
+    HTMLInputTextElement2_put_selectionEnd,
+    HTMLInputTextElement2_get_selectionEnd,
+    HTMLInputTextElement2_setSelectionRange
+};
+
 static inline HTMLInputElement *impl_from_HTMLDOMNode(HTMLDOMNode *iface)
 {
     return CONTAINING_RECORD(iface, HTMLInputElement, element.node);
@@ -1229,6 +1349,9 @@ static HRESULT HTMLInputElement_QI(HTMLDOMNode *iface, REFIID riid, void **ppv)
     }else if(IsEqualGUID(&IID_IHTMLInputTextElement, riid)) {
         TRACE("(%p)->(IID_IHTMLInputTextElement %p)\n", This, ppv);
         *ppv = &This->IHTMLInputTextElement_iface;
+    }else if(IsEqualGUID(&IID_IHTMLInputTextElement2, riid)) {
+        TRACE("(%p)->(IID_IHTMLInputTextElement2 %p)\n", This, ppv);
+        *ppv = &This->IHTMLInputTextElement2_iface;
     }
 
     if(*ppv) {
@@ -1239,25 +1362,6 @@ static HRESULT HTMLInputElement_QI(HTMLDOMNode *iface, REFIID riid, void **ppv)
     return HTMLElement_QI(&This->element.node, riid, ppv);
 }
 
-static HRESULT HTMLInputElementImpl_fire_event(HTMLDOMNode *iface, eventid_t eid, BOOL *handled)
-{
-    HTMLInputElement *This = impl_from_HTMLDOMNode(iface);
-
-    if(eid == EVENTID_CLICK) {
-        nsresult nsres;
-
-        *handled = TRUE;
-
-        nsres = nsIDOMHTMLElement_Click(This->element.nselem);
-        if(NS_FAILED(nsres)) {
-            ERR("Click failed: %08x\n", nsres);
-            return E_FAIL;
-        }
-    }
-
-    return S_OK;
-}
-
 static HRESULT HTMLInputElementImpl_put_disabled(HTMLDOMNode *iface, VARIANT_BOOL v)
 {
     HTMLInputElement *This = impl_from_HTMLDOMNode(iface);
@@ -1278,19 +1382,12 @@ static BOOL HTMLInputElement_is_text_edit(HTMLDOMNode *iface)
     nsresult nsres;
     BOOL ret = FALSE;
 
-    static const WCHAR buttonW[] = {'b','u','t','t','o','n',0};
-    static const WCHAR hiddenW[] = {'h','i','d','d','e','n',0};
-    static const WCHAR passwordW[] = {'p','a','s','s','w','o','r','d',0};
-    static const WCHAR resetW[] = {'r','e','s','e','t',0};
-    static const WCHAR submitW[] = {'s','u','b','m','i','t',0};
-    static const WCHAR textW[] = {'t','e','x','t',0};
-
     nsAString_Init(&nsstr, NULL);
     nsres = nsIDOMHTMLInputElement_GetType(This->nsinput, &nsstr);
     if(NS_SUCCEEDED(nsres)) {
         nsAString_GetData(&nsstr, &type);
-        ret = !strcmpW(type, buttonW) || !strcmpW(type, hiddenW) || !strcmpW(type, passwordW)
-            || !strcmpW(type, resetW) || !strcmpW(type, submitW) || !strcmpW(type, textW);
+        ret = !wcscmp(type, L"button") || !wcscmp(type, L"hidden") || !wcscmp(type, L"password")
+            || !wcscmp(type, L"reset") || !wcscmp(type, L"submit") || !wcscmp(type, L"text");
     }
     nsAString_Finish(&nsstr);
     return ret;
@@ -1325,7 +1422,6 @@ static const NodeImplVtbl HTMLInputElementImplVtbl = {
     HTMLElement_handle_event,
     HTMLElement_get_attr_col,
     NULL,
-    HTMLInputElementImpl_fire_event,
     HTMLInputElementImpl_put_disabled,
     HTMLInputElementImpl_get_disabled,
     NULL,
@@ -1341,16 +1437,18 @@ static const NodeImplVtbl HTMLInputElementImplVtbl = {
 static const tid_t HTMLInputElement_iface_tids[] = {
     HTMLELEMENT_TIDS,
     IHTMLInputElement_tid,
+    IHTMLInputTextElement2_tid,
     0
 };
 static dispex_static_data_t HTMLInputElement_dispex = {
+    L"HTMLInputElement",
     NULL,
     DispHTMLInputElement_tid,
     HTMLInputElement_iface_tids,
     HTMLElement_init_dispex_info
 };
 
-HRESULT HTMLInputElement_Create(HTMLDocumentNode *doc, nsIDOMHTMLElement *nselem, HTMLElement **elem)
+HRESULT HTMLInputElement_Create(HTMLDocumentNode *doc, nsIDOMElement *nselem, HTMLElement **elem)
 {
     HTMLInputElement *ret;
     nsresult nsres;
@@ -1361,11 +1459,12 @@ HRESULT HTMLInputElement_Create(HTMLDocumentNode *doc, nsIDOMHTMLElement *nselem
 
     ret->IHTMLInputElement_iface.lpVtbl = &HTMLInputElementVtbl;
     ret->IHTMLInputTextElement_iface.lpVtbl = &HTMLInputTextElementVtbl;
+    ret->IHTMLInputTextElement2_iface.lpVtbl = &HTMLInputTextElement2Vtbl;
     ret->element.node.vtbl = &HTMLInputElementImplVtbl;
 
     HTMLElement_Init(&ret->element, doc, nselem, &HTMLInputElement_dispex);
 
-    nsres = nsIDOMHTMLElement_QueryInterface(nselem, &IID_nsIDOMHTMLInputElement, (void**)&ret->nsinput);
+    nsres = nsIDOMElement_QueryInterface(nselem, &IID_nsIDOMHTMLInputElement, (void**)&ret->nsinput);
     assert(nsres == NS_OK);
 
     *elem = &ret->element;
@@ -1447,13 +1546,13 @@ static HRESULT WINAPI HTMLLabelElement_put_htmlFor(IHTMLLabelElement *iface, BST
 
     TRACE("(%p)->(%s)\n", This, debugstr_w(v));
 
-    nsAString_InitDepend(&for_str, forW);
+    nsAString_InitDepend(&for_str, L"for");
     nsAString_InitDepend(&val_str, v);
-    nsres = nsIDOMHTMLElement_SetAttribute(This->element.nselem, &for_str, &val_str);
+    nsres = nsIDOMElement_SetAttribute(This->element.dom_element, &for_str, &val_str);
     nsAString_Finish(&for_str);
     nsAString_Finish(&val_str);
     if(NS_FAILED(nsres)) {
-        ERR("SetAttribute failed: %08x\n", nsres);
+        ERR("SetAttribute failed: %08lx\n", nsres);
         return E_FAIL;
     }
 
@@ -1466,7 +1565,7 @@ static HRESULT WINAPI HTMLLabelElement_get_htmlFor(IHTMLLabelElement *iface, BST
 
     TRACE("(%p)->(%p)\n", This, p);
 
-    return elem_string_attr_getter(&This->element, forW, FALSE, p);
+    return elem_string_attr_getter(&This->element, L"for", FALSE, p);
 }
 
 static HRESULT WINAPI HTMLLabelElement_put_accessKey(IHTMLLabelElement *iface, BSTR v)
@@ -1539,13 +1638,14 @@ static const tid_t HTMLLabelElement_iface_tids[] = {
 };
 
 static dispex_static_data_t HTMLLabelElement_dispex = {
+    L"HTMLLabelElement",
     NULL,
     DispHTMLLabelElement_tid,
     HTMLLabelElement_iface_tids,
     HTMLElement_init_dispex_info
 };
 
-HRESULT HTMLLabelElement_Create(HTMLDocumentNode *doc, nsIDOMHTMLElement *nselem, HTMLElement **elem)
+HRESULT HTMLLabelElement_Create(HTMLDocumentNode *doc, nsIDOMElement *nselem, HTMLElement **elem)
 {
     HTMLLabelElement *ret;
 
@@ -1655,7 +1755,7 @@ static HRESULT WINAPI HTMLButtonElement_put_value(IHTMLButtonElement *iface, BST
     nsres = nsIDOMHTMLButtonElement_SetValue(This->nsbutton, &nsstr);
     nsAString_Finish(&nsstr);
     if(NS_FAILED(nsres)) {
-        ERR("SetValue failed: %08x\n", nsres);
+        ERR("SetValue failed: %08lx\n", nsres);
         return E_FAIL;
     }
 
@@ -1687,7 +1787,7 @@ static HRESULT WINAPI HTMLButtonElement_put_name(IHTMLButtonElement *iface, BSTR
     nsres = nsIDOMHTMLButtonElement_SetName(This->nsbutton, &name_str);
     nsAString_Finish(&name_str);
     if(NS_FAILED(nsres)) {
-        ERR("SetName failed: %08x\n", nsres);
+        ERR("SetName failed: %08lx\n", nsres);
         return E_FAIL;
     }
 
@@ -1730,7 +1830,7 @@ static HRESULT WINAPI HTMLButtonElement_put_disabled(IHTMLButtonElement *iface,
 
     nsres = nsIDOMHTMLButtonElement_SetDisabled(This->nsbutton, !!v);
     if(NS_FAILED(nsres)) {
-        ERR("SetDisabled failed: %08x\n", nsres);
+        ERR("SetDisabled failed: %08lx\n", nsres);
         return E_FAIL;
     }
 
@@ -1747,11 +1847,11 @@ static HRESULT WINAPI HTMLButtonElement_get_disabled(IHTMLButtonElement *iface,
 
     nsres = nsIDOMHTMLButtonElement_GetDisabled(This->nsbutton, &disabled);
     if(NS_FAILED(nsres)) {
-        ERR("GetDisabled failed: %08x\n", nsres);
+        ERR("GetDisabled failed: %08lx\n", nsres);
         return E_FAIL;
     }
 
-    *p = disabled ? VARIANT_TRUE : VARIANT_FALSE;
+    *p = variant_bool(disabled);
     return S_OK;
 }
 
@@ -1764,12 +1864,7 @@ static HRESULT WINAPI HTMLButtonElement_get_form(IHTMLButtonElement *iface, IHTM
     TRACE("(%p)->(%p)\n", This, p);
 
     nsres = nsIDOMHTMLButtonElement_GetForm(This->nsbutton, &nsform);
-    if (NS_FAILED(nsres)) {
-        ERR("GetForm failed: %08x, nsform: %p\n", nsres, nsform);
-        return E_FAIL;
-    }
-
-    return return_nsform(&This->element, nsform, p);
+    return return_nsform(nsres, nsform, p);
 }
 
 static HRESULT WINAPI HTMLButtonElement_createTextRange(IHTMLButtonElement *iface, IHTMLTxtRange **range)
@@ -1871,7 +1966,6 @@ static const NodeImplVtbl HTMLButtonElementImplVtbl = {
     HTMLElement_handle_event,
     HTMLElement_get_attr_col,
     NULL,
-    NULL,
     HTMLButtonElementImpl_put_disabled,
     HTMLButtonElementImpl_get_disabled,
     NULL,
@@ -1891,13 +1985,14 @@ static const tid_t HTMLButtonElement_iface_tids[] = {
 };
 
 static dispex_static_data_t HTMLButtonElement_dispex = {
+    L"HTMLButtonElement",
     NULL,
     DispHTMLButtonElement_tid,
     HTMLButtonElement_iface_tids,
     HTMLElement_init_dispex_info
 };
 
-HRESULT HTMLButtonElement_Create(HTMLDocumentNode *doc, nsIDOMHTMLElement *nselem, HTMLElement **elem)
+HRESULT HTMLButtonElement_Create(HTMLDocumentNode *doc, nsIDOMElement *nselem, HTMLElement **elem)
 {
     HTMLButtonElement *ret;
     nsresult nsres;
@@ -1911,7 +2006,7 @@ HRESULT HTMLButtonElement_Create(HTMLDocumentNode *doc, nsIDOMHTMLElement *nsele
 
     HTMLElement_Init(&ret->element, doc, nselem, &HTMLButtonElement_dispex);
 
-    nsres = nsIDOMHTMLElement_QueryInterface(nselem, &IID_nsIDOMHTMLButtonElement, (void**)&ret->nsbutton);
+    nsres = nsIDOMElement_QueryInterface(nselem, &IID_nsIDOMHTMLButtonElement, (void**)&ret->nsbutton);
     assert(nsres == NS_OK);
 
     *elem = &ret->element;
