@@ -16,11 +16,8 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  */
 
-#include "config.h"
-
 #include <stdarg.h>
 #include <stdio.h>
-#include <assert.h>
 
 #define COBJMACROS
 
@@ -76,7 +73,7 @@ static ULONG WINAPI EnumUnknown_AddRef(IEnumUnknown *iface)
     EnumUnknown *This = impl_from_IEnumUnknown(iface);
     LONG ref = InterlockedIncrement(&This->ref);
 
-    TRACE("(%p) ref=%d\n", This, ref);
+    TRACE("(%p) ref=%ld\n", This, ref);
 
     return ref;
 }
@@ -86,7 +83,7 @@ static ULONG WINAPI EnumUnknown_Release(IEnumUnknown *iface)
     EnumUnknown *This = impl_from_IEnumUnknown(iface);
     LONG ref = InterlockedDecrement(&This->ref);
 
-    TRACE("(%p) ref=%d\n", This, ref);
+    TRACE("(%p) ref=%ld\n", This, ref);
 
     if(!ref)
         heap_free(This);
@@ -98,7 +95,7 @@ static HRESULT WINAPI EnumUnknown_Next(IEnumUnknown *iface, ULONG celt, IUnknown
 {
     EnumUnknown *This = impl_from_IEnumUnknown(iface);
 
-    TRACE("(%p)->(%u %p %p)\n", This, celt, rgelt, pceltFetched);
+    TRACE("(%p)->(%lu %p %p)\n", This, celt, rgelt, pceltFetched);
 
     /* FIXME: It's not clear if we should ever return something here */
     if(pceltFetched)
@@ -109,7 +106,7 @@ static HRESULT WINAPI EnumUnknown_Next(IEnumUnknown *iface, ULONG celt, IUnknown
 static HRESULT WINAPI EnumUnknown_Skip(IEnumUnknown *iface, ULONG celt)
 {
     EnumUnknown *This = impl_from_IEnumUnknown(iface);
-    FIXME("(%p)->(%u)\n", This, celt);
+    FIXME("(%p)->(%lu)\n", This, celt);
     return E_NOTIMPL;
 }
 
@@ -177,16 +174,16 @@ static void update_hostinfo(HTMLDocumentObj *This, DOCHOSTUIINFO *hostinfo)
         nsres = nsIScrollable_SetDefaultScrollbarPreferences(scrollable, ScrollOrientation_Y,
                 (hostinfo->dwFlags & DOCHOSTUIFLAG_SCROLL_NO) ? Scrollbar_Never : Scrollbar_Always);
         if(NS_FAILED(nsres))
-            ERR("Could not set default Y scrollbar prefs: %08x\n", nsres);
+            ERR("Could not set default Y scrollbar prefs: %08lx\n", nsres);
 
         nsres = nsIScrollable_SetDefaultScrollbarPreferences(scrollable, ScrollOrientation_X,
                 hostinfo->dwFlags & DOCHOSTUIFLAG_SCROLL_NO ? Scrollbar_Never : Scrollbar_Auto);
         if(NS_FAILED(nsres))
-            ERR("Could not set default X scrollbar prefs: %08x\n", nsres);
+            ERR("Could not set default X scrollbar prefs: %08lx\n", nsres);
 
         nsIScrollable_Release(scrollable);
     }else {
-        ERR("Could not get nsIScrollable: %08x\n", nsres);
+        ERR("Could not get nsIScrollable: %08lx\n", nsres);
     }
 }
 
@@ -233,19 +230,12 @@ static void load_settings(HTMLDocumentObj *doc)
     DWORD val, size;
     LONG res;
 
-    static const WCHAR ie_keyW[] = {
-        'S','O','F','T','W','A','R','E','\\',
-        'M','i','c','r','o','s','o','f','t','\\',
-        'I','n','t','e','r','n','e','t',' ','E','x','p','l','o','r','e','r',0};
-    static const WCHAR zoomW[] = {'Z','o','o','m',0};
-    static const WCHAR zoom_factorW[] = {'Z','o','o','m','F','a','c','t','o','r',0};
-
-    res = RegOpenKeyW(HKEY_CURRENT_USER, ie_keyW, &settings_key);
+    res = RegOpenKeyW(HKEY_CURRENT_USER, L"SOFTWARE\\Microsoft\\Internet Explorer", &settings_key);
     if(res != ERROR_SUCCESS)
         return;
 
     size = sizeof(val);
-    res = RegGetValueW(settings_key, zoomW, zoom_factorW, RRF_RT_REG_DWORD, NULL, &val, &size);
+    res = RegGetValueW(settings_key, L"Zoom", L"ZoomFactor", RRF_RT_REG_DWORD, NULL, &val, &size);
     RegCloseKey(settings_key);
     if(res == ERROR_SUCCESS)
         set_viewer_zoom(doc->nscontainer, (float)val/100000);
@@ -270,7 +260,7 @@ static HRESULT WINAPI OleObject_SetClientSite(IOleObject *iface, IOleClientSite
     if(This->doc_obj->client) {
         IOleClientSite_Release(This->doc_obj->client);
         This->doc_obj->client = NULL;
-        This->doc_obj->usermode = UNKNOWN_USERMODE;
+        This->doc_obj->nscontainer->usermode = UNKNOWN_USERMODE;
     }
 
     if(This->doc_obj->client_cmdtrg) {
@@ -332,7 +322,7 @@ static HRESULT WINAPI OleObject_SetClientSite(IOleObject *iface, IOleClientSite
         hostinfo.cbSize = sizeof(DOCHOSTUIINFO);
         hres = IDocHostUIHandler_GetHostInfo(This->doc_obj->hostui, &hostinfo);
         if(SUCCEEDED(hres)) {
-            TRACE("hostinfo = {%u %08x %08x %s %s}\n",
+            TRACE("hostinfo = {%lu %08lx %08lx %s %s}\n",
                     hostinfo.cbSize, hostinfo.dwFlags, hostinfo.dwDoubleClick,
                     debugstr_w(hostinfo.pchHostCss), debugstr_w(hostinfo.pchHostNS));
             update_hostinfo(This->doc_obj, &hostinfo);
@@ -369,8 +359,6 @@ static HRESULT WINAPI OleObject_SetClientSite(IOleObject *iface, IOleClientSite
 
     load_settings(This->doc_obj);
 
-    /* CXHACK 15998: Don't call GetWindow() here. */
-    if(0) {
     /* Native calls here GetWindow. What is it for?
      * We don't have anything to do with it here (yet). */
     hres = IOleClientSite_QueryInterface(pClientSite, &IID_IOleWindow, (void**)&ole_window);
@@ -378,7 +366,6 @@ static HRESULT WINAPI OleObject_SetClientSite(IOleObject *iface, IOleClientSite
         IOleWindow_GetWindow(ole_window, &hwnd);
         IOleWindow_Release(ole_window);
     }
-    }
 
     hres = do_query_service((IUnknown*)pClientSite, &IID_IShellBrowser,
             &IID_IBrowserService, (void**)&browser_service);
@@ -436,7 +423,7 @@ static HRESULT WINAPI OleObject_SetClientSite(IOleObject *iface, IOleClientSite
                 OLECMDEXECOPT_DONTPROMPTUSER, &var, NULL);
     }
 
-    if(This->doc_obj->usermode == UNKNOWN_USERMODE)
+    if(This->doc_obj->nscontainer->usermode == UNKNOWN_USERMODE)
         IOleControl_OnAmbientPropertyChange(&This->IOleControl_iface, DISPID_AMBIENT_USERMODE);
 
     IOleControl_OnAmbientPropertyChange(&This->IOleControl_iface,
@@ -483,7 +470,7 @@ static HRESULT WINAPI OleObject_Close(IOleObject *iface, DWORD dwSaveOption)
 {
     HTMLDocument *This = impl_from_IOleObject(iface);
 
-    TRACE("(%p)->(%08x)\n", This, dwSaveOption);
+    TRACE("(%p)->(%08lx)\n", This, dwSaveOption);
 
     if(dwSaveOption == OLECLOSE_PROMPTSAVE)
         FIXME("OLECLOSE_PROMPTSAVE not implemented\n");
@@ -493,8 +480,8 @@ static HRESULT WINAPI OleObject_Close(IOleObject *iface, DWORD dwSaveOption)
 
     HTMLDocument_LockContainer(This->doc_obj, FALSE);
 
-    if(This->advise_holder)
-        IOleAdviseHolder_SendOnClose(This->advise_holder);
+    if(This->doc_obj->advise_holder)
+        IOleAdviseHolder_SendOnClose(This->doc_obj->advise_holder);
     
     return S_OK;
 }
@@ -502,14 +489,14 @@ static HRESULT WINAPI OleObject_Close(IOleObject *iface, DWORD dwSaveOption)
 static HRESULT WINAPI OleObject_SetMoniker(IOleObject *iface, DWORD dwWhichMoniker, IMoniker *pmk)
 {
     HTMLDocument *This = impl_from_IOleObject(iface);
-    FIXME("(%p %d %p)->()\n", This, dwWhichMoniker, pmk);
+    FIXME("(%p %ld %p)->()\n", This, dwWhichMoniker, pmk);
     return E_NOTIMPL;
 }
 
 static HRESULT WINAPI OleObject_GetMoniker(IOleObject *iface, DWORD dwAssign, DWORD dwWhichMoniker, IMoniker **ppmk)
 {
     HTMLDocument *This = impl_from_IOleObject(iface);
-    FIXME("(%p)->(%d %d %p)\n", This, dwAssign, dwWhichMoniker, ppmk);
+    FIXME("(%p)->(%ld %ld %p)\n", This, dwAssign, dwWhichMoniker, ppmk);
     return E_NOTIMPL;
 }
 
@@ -517,14 +504,14 @@ static HRESULT WINAPI OleObject_InitFromData(IOleObject *iface, IDataObject *pDa
                                         DWORD dwReserved)
 {
     HTMLDocument *This = impl_from_IOleObject(iface);
-    FIXME("(%p)->(%p %x %d)\n", This, pDataObject, fCreation, dwReserved);
+    FIXME("(%p)->(%p %x %ld)\n", This, pDataObject, fCreation, dwReserved);
     return E_NOTIMPL;
 }
 
 static HRESULT WINAPI OleObject_GetClipboardData(IOleObject *iface, DWORD dwReserved, IDataObject **ppDataObject)
 {
     HTMLDocument *This = impl_from_IOleObject(iface);
-    FIXME("(%p)->(%d %p)\n", This, dwReserved, ppDataObject);
+    FIXME("(%p)->(%ld %p)\n", This, dwReserved, ppDataObject);
     return E_NOTIMPL;
 }
 
@@ -535,10 +522,10 @@ static HRESULT WINAPI OleObject_DoVerb(IOleObject *iface, LONG iVerb, LPMSG lpms
     IOleDocumentSite *pDocSite;
     HRESULT hres;
 
-    TRACE("(%p)->(%d %p %p %d %p %p)\n", This, iVerb, lpmsg, pActiveSite, lindex, hwndParent, lprcPosRect);
+    TRACE("(%p)->(%ld %p %p %ld %p %p)\n", This, iVerb, lpmsg, pActiveSite, lindex, hwndParent, lprcPosRect);
 
     if(iVerb != OLEIVERB_SHOW && iVerb != OLEIVERB_UIACTIVATE && iVerb != OLEIVERB_INPLACEACTIVATE) { 
-        FIXME("iVerb = %d not supported\n", iVerb);
+        FIXME("iVerb = %ld not supported\n", iVerb);
         return E_NOTIMPL;
     }
 
@@ -550,17 +537,17 @@ static HRESULT WINAPI OleObject_DoVerb(IOleObject *iface, LONG iVerb, LPMSG lpms
         HTMLDocument_LockContainer(This->doc_obj, TRUE);
 
         /* FIXME: Create new IOleDocumentView. See CreateView for more info. */
-        hres = IOleDocumentSite_ActivateMe(pDocSite, &This->IOleDocumentView_iface);
+        hres = IOleDocumentSite_ActivateMe(pDocSite, &This->doc_obj->IOleDocumentView_iface);
         IOleDocumentSite_Release(pDocSite);
     }else {
-        hres = IOleDocumentView_UIActivate(&This->IOleDocumentView_iface, TRUE);
+        hres = IOleDocumentView_UIActivate(&This->doc_obj->IOleDocumentView_iface, TRUE);
         if(SUCCEEDED(hres)) {
             if(lprcPosRect) {
                 RECT rect; /* We need to pass rect as not const pointer */
                 rect = *lprcPosRect;
-                IOleDocumentView_SetRect(&This->IOleDocumentView_iface, &rect);
+                IOleDocumentView_SetRect(&This->doc_obj->IOleDocumentView_iface, &rect);
             }
-            IOleDocumentView_Show(&This->IOleDocumentView_iface, TRUE);
+            IOleDocumentView_Show(&This->doc_obj->IOleDocumentView_iface, TRUE);
         }
     }
 
@@ -604,22 +591,34 @@ static HRESULT WINAPI OleObject_GetUserClassID(IOleObject *iface, CLSID *pClsid)
 static HRESULT WINAPI OleObject_GetUserType(IOleObject *iface, DWORD dwFormOfType, LPOLESTR *pszUserType)
 {
     HTMLDocument *This = impl_from_IOleObject(iface);
-    FIXME("(%p)->(%d %p)\n", This, dwFormOfType, pszUserType);
+    FIXME("(%p)->(%ld %p)\n", This, dwFormOfType, pszUserType);
     return E_NOTIMPL;
 }
 
 static HRESULT WINAPI OleObject_SetExtent(IOleObject *iface, DWORD dwDrawAspect, SIZEL *psizel)
 {
     HTMLDocument *This = impl_from_IOleObject(iface);
-    FIXME("(%p)->(%d %p)\n", This, dwDrawAspect, psizel);
-    return E_NOTIMPL;
+
+    TRACE("(%p)->(%ld %p)\n", This, dwDrawAspect, psizel);
+
+    if (dwDrawAspect != DVASPECT_CONTENT)
+        return E_INVALIDARG;
+
+    This->doc_obj->extent = *psizel;
+    return S_OK;
 }
 
 static HRESULT WINAPI OleObject_GetExtent(IOleObject *iface, DWORD dwDrawAspect, SIZEL *psizel)
 {
     HTMLDocument *This = impl_from_IOleObject(iface);
-    FIXME("(%p)->(%d %p)\n", This, dwDrawAspect, psizel);
-    return E_NOTIMPL;
+
+    TRACE("(%p)->(%ld %p)\n", This, dwDrawAspect, psizel);
+
+    if (dwDrawAspect != DVASPECT_CONTENT)
+        return E_INVALIDARG;
+
+    *psizel = This->doc_obj->extent;
+    return S_OK;
 }
 
 static HRESULT WINAPI OleObject_Advise(IOleObject *iface, IAdviseSink *pAdvSink, DWORD *pdwConnection)
@@ -635,42 +634,42 @@ static HRESULT WINAPI OleObject_Advise(IOleObject *iface, IAdviseSink *pAdvSink,
         return E_INVALIDARG;
     }
 
-    if(!This->advise_holder) {
-        CreateOleAdviseHolder(&This->advise_holder);
-        if(!This->advise_holder)
+    if(!This->doc_obj->advise_holder) {
+        CreateOleAdviseHolder(&This->doc_obj->advise_holder);
+        if(!This->doc_obj->advise_holder)
             return E_OUTOFMEMORY;
     }
 
-    return IOleAdviseHolder_Advise(This->advise_holder, pAdvSink, pdwConnection);
+    return IOleAdviseHolder_Advise(This->doc_obj->advise_holder, pAdvSink, pdwConnection);
 }
 
 static HRESULT WINAPI OleObject_Unadvise(IOleObject *iface, DWORD dwConnection)
 {
     HTMLDocument *This = impl_from_IOleObject(iface);
-    TRACE("(%p)->(%d)\n", This, dwConnection);
+    TRACE("(%p)->(%ld)\n", This, dwConnection);
 
-    if(!This->advise_holder)
+    if(!This->doc_obj->advise_holder)
         return OLE_E_NOCONNECTION;
 
-    return IOleAdviseHolder_Unadvise(This->advise_holder, dwConnection);
+    return IOleAdviseHolder_Unadvise(This->doc_obj->advise_holder, dwConnection);
 }
 
 static HRESULT WINAPI OleObject_EnumAdvise(IOleObject *iface, IEnumSTATDATA **ppenumAdvise)
 {
     HTMLDocument *This = impl_from_IOleObject(iface);
 
-    if(!This->advise_holder) {
+    if(!This->doc_obj->advise_holder) {
         *ppenumAdvise = NULL;
         return S_OK;
     }
 
-    return IOleAdviseHolder_EnumAdvise(This->advise_holder, ppenumAdvise);
+    return IOleAdviseHolder_EnumAdvise(This->doc_obj->advise_holder, ppenumAdvise);
 }
 
 static HRESULT WINAPI OleObject_GetMiscStatus(IOleObject *iface, DWORD dwAspect, DWORD *pdwStatus)
 {
     HTMLDocument *This = impl_from_IOleObject(iface);
-    FIXME("(%p)->(%d %p)\n", This, dwAspect, pdwStatus);
+    FIXME("(%p)->(%ld %p)\n", This, dwAspect, pdwStatus);
     return E_NOTIMPL;
 }
 
@@ -741,7 +740,7 @@ static HRESULT WINAPI OleDocument_CreateView(IOleDocument *iface, IOleInPlaceSit
     HTMLDocument *This = impl_from_IOleDocument(iface);
     HRESULT hres;
 
-    TRACE("(%p)->(%p %p %d %p)\n", This, pIPSite, pstm, dwReserved, ppView);
+    TRACE("(%p)->(%p %p %ld %p)\n", This, pIPSite, pstm, dwReserved, ppView);
 
     if(!ppView)
         return E_INVALIDARG;
@@ -754,7 +753,7 @@ static HRESULT WINAPI OleDocument_CreateView(IOleDocument *iface, IOleInPlaceSit
      */
 
     if(pIPSite) {
-        hres = IOleDocumentView_SetInPlaceSite(&This->IOleDocumentView_iface, pIPSite);
+        hres = IOleDocumentView_SetInPlaceSite(&This->doc_obj->IOleDocumentView_iface, pIPSite);
         if(FAILED(hres))
             return hres;
     }
@@ -762,8 +761,8 @@ static HRESULT WINAPI OleDocument_CreateView(IOleDocument *iface, IOleInPlaceSit
     if(pstm)
         FIXME("pstm is not supported\n");
 
-    IOleDocumentView_AddRef(&This->IOleDocumentView_iface);
-    *ppView = &This->IOleDocumentView_iface;
+    IOleDocumentView_AddRef(&This->doc_obj->IOleDocumentView_iface);
+    *ppView = &This->doc_obj->IOleDocumentView_iface;
     return S_OK;
 }
 
@@ -862,7 +861,7 @@ static HRESULT on_change_dlcontrol(HTMLDocument *This)
     
     hres = get_client_disp_property(This->doc_obj->client, DISPID_AMBIENT_DLCONTROL, &res);
     if(SUCCEEDED(hres))
-        FIXME("unsupported dlcontrol %08x\n", V_I4(&res));
+        FIXME("unsupported dlcontrol %08lx\n", V_I4(&res));
 
     return S_OK;
 }
@@ -889,10 +888,10 @@ static HRESULT WINAPI OleControl_OnAmbientPropertyChange(IOleControl *iface, DIS
 
         if(V_VT(&res) == VT_BOOL) {
             if(V_BOOL(&res)) {
-                This->doc_obj->usermode = BROWSEMODE;
+                This->doc_obj->nscontainer->usermode = BROWSEMODE;
             }else {
                 FIXME("edit mode is not supported\n");
-                This->doc_obj->usermode = EDITMODE;
+                This->doc_obj->nscontainer->usermode = EDITMODE;
             }
         }else {
             FIXME("usermode=%s\n", debugstr_variant(&res));
@@ -927,7 +926,6 @@ static HRESULT WINAPI OleControl_OnAmbientPropertyChange(IOleControl *iface, DIS
         if(V_VT(&res) == VT_BOOL) {
             if(V_BOOL(&res)) {
                 FIXME("silent mode is not supported\n");
-                hres = E_FAIL;
             }
         }else {
             FIXME("silent=%s\n", debugstr_variant(&res));
@@ -940,7 +938,6 @@ static HRESULT WINAPI OleControl_OnAmbientPropertyChange(IOleControl *iface, DIS
             return S_OK;
 
         FIXME("not supported AMBIENT_USERAGENT\n");
-        hres = E_FAIL;
         return S_OK;
     case DISPID_AMBIENT_PALETTE:
         TRACE("(%p)->(DISPID_AMBIENT_PALETTE)\n", This);
@@ -949,11 +946,10 @@ static HRESULT WINAPI OleControl_OnAmbientPropertyChange(IOleControl *iface, DIS
             return S_OK;
 
         FIXME("not supported AMBIENT_PALETTE\n");
-        hres = E_FAIL;
         return S_OK;
     }
 
-    FIXME("(%p) unsupported dispID=%d\n", This, dispID);
+    FIXME("(%p) unsupported dispID=%ld\n", This, dispID);
     return E_FAIL;
 }
 
@@ -974,6 +970,257 @@ static const IOleControlVtbl OleControlVtbl = {
     OleControl_FreezeEvents
 };
 
+/**********************************************************
+ * IOleInPlaceActiveObject implementation
+ */
+
+static inline HTMLDocument *impl_from_IOleInPlaceActiveObject(IOleInPlaceActiveObject *iface)
+{
+    return CONTAINING_RECORD(iface, HTMLDocument, IOleInPlaceActiveObject_iface);
+}
+
+static HRESULT WINAPI OleInPlaceActiveObject_QueryInterface(IOleInPlaceActiveObject *iface, REFIID riid, void **ppv)
+{
+    HTMLDocument *This = impl_from_IOleInPlaceActiveObject(iface);
+    return htmldoc_query_interface(This, riid, ppv);
+}
+
+static ULONG WINAPI OleInPlaceActiveObject_AddRef(IOleInPlaceActiveObject *iface)
+{
+    HTMLDocument *This = impl_from_IOleInPlaceActiveObject(iface);
+    return htmldoc_addref(This);
+}
+
+static ULONG WINAPI OleInPlaceActiveObject_Release(IOleInPlaceActiveObject *iface)
+{
+    HTMLDocument *This = impl_from_IOleInPlaceActiveObject(iface);
+    return htmldoc_release(This);
+}
+
+static HRESULT WINAPI OleInPlaceActiveObject_GetWindow(IOleInPlaceActiveObject *iface, HWND *phwnd)
+{
+    HTMLDocument *This = impl_from_IOleInPlaceActiveObject(iface);
+
+    TRACE("(%p)->(%p)\n", This, phwnd);
+
+    if(!phwnd)
+        return E_INVALIDARG;
+
+    if(!This->doc_obj->in_place_active) {
+        *phwnd = NULL;
+        return E_FAIL;
+    }
+
+    *phwnd = This->doc_obj->hwnd;
+    return S_OK;
+}
+
+static HRESULT WINAPI OleInPlaceActiveObject_ContextSensitiveHelp(IOleInPlaceActiveObject *iface, BOOL fEnterMode)
+{
+    HTMLDocument *This = impl_from_IOleInPlaceActiveObject(iface);
+    FIXME("(%p)->(%x)\n", This, fEnterMode);
+    return E_NOTIMPL;
+}
+
+static HRESULT WINAPI OleInPlaceActiveObject_TranslateAccelerator(IOleInPlaceActiveObject *iface, LPMSG lpmsg)
+{
+    HTMLDocument *This = impl_from_IOleInPlaceActiveObject(iface);
+    FIXME("(%p)->(%p)\n", This, lpmsg);
+    return E_NOTIMPL;
+}
+
+static HRESULT WINAPI OleInPlaceActiveObject_OnFrameWindowActivate(IOleInPlaceActiveObject *iface,
+        BOOL fActivate)
+{
+    HTMLDocument *This = impl_from_IOleInPlaceActiveObject(iface);
+
+    TRACE("(%p)->(%x)\n", This, fActivate);
+
+    if(This->doc_obj->hostui)
+        IDocHostUIHandler_OnFrameWindowActivate(This->doc_obj->hostui, fActivate);
+
+    return S_OK;
+}
+
+static HRESULT WINAPI OleInPlaceActiveObject_OnDocWindowActivate(IOleInPlaceActiveObject *iface, BOOL fActivate)
+{
+    HTMLDocument *This = impl_from_IOleInPlaceActiveObject(iface);
+    FIXME("(%p)->(%x)\n", This, fActivate);
+    return E_NOTIMPL;
+}
+
+static HRESULT WINAPI OleInPlaceActiveObject_ResizeBorder(IOleInPlaceActiveObject *iface, LPCRECT prcBorder,
+                                                IOleInPlaceUIWindow *pUIWindow, BOOL fFrameWindow)
+{
+    HTMLDocument *This = impl_from_IOleInPlaceActiveObject(iface);
+    FIXME("(%p)->(%p %p %x)\n", This, prcBorder, pUIWindow, fFrameWindow);
+    return E_NOTIMPL;
+}
+
+static HRESULT WINAPI OleInPlaceActiveObject_EnableModeless(IOleInPlaceActiveObject *iface, BOOL fEnable)
+{
+    HTMLDocument *This = impl_from_IOleInPlaceActiveObject(iface);
+    FIXME("(%p)->(%x)\n", This, fEnable);
+    return E_NOTIMPL;
+}
+
+static const IOleInPlaceActiveObjectVtbl OleInPlaceActiveObjectVtbl = {
+    OleInPlaceActiveObject_QueryInterface,
+    OleInPlaceActiveObject_AddRef,
+    OleInPlaceActiveObject_Release,
+    OleInPlaceActiveObject_GetWindow,
+    OleInPlaceActiveObject_ContextSensitiveHelp,
+    OleInPlaceActiveObject_TranslateAccelerator,
+    OleInPlaceActiveObject_OnFrameWindowActivate,
+    OleInPlaceActiveObject_OnDocWindowActivate,
+    OleInPlaceActiveObject_ResizeBorder,
+    OleInPlaceActiveObject_EnableModeless
+};
+
+/**********************************************************
+ * IOleInPlaceObjectWindowless implementation
+ */
+
+static inline HTMLDocument *impl_from_IOleInPlaceObjectWindowless(IOleInPlaceObjectWindowless *iface)
+{
+    return CONTAINING_RECORD(iface, HTMLDocument, IOleInPlaceObjectWindowless_iface);
+}
+
+static HRESULT WINAPI OleInPlaceObjectWindowless_QueryInterface(IOleInPlaceObjectWindowless *iface,
+        REFIID riid, void **ppv)
+{
+    HTMLDocument *This = impl_from_IOleInPlaceObjectWindowless(iface);
+    return htmldoc_query_interface(This, riid, ppv);
+}
+
+static ULONG WINAPI OleInPlaceObjectWindowless_AddRef(IOleInPlaceObjectWindowless *iface)
+{
+    HTMLDocument *This = impl_from_IOleInPlaceObjectWindowless(iface);
+    return htmldoc_addref(This);
+}
+
+static ULONG WINAPI OleInPlaceObjectWindowless_Release(IOleInPlaceObjectWindowless *iface)
+{
+    HTMLDocument *This = impl_from_IOleInPlaceObjectWindowless(iface);
+    return htmldoc_release(This);
+}
+
+static HRESULT WINAPI OleInPlaceObjectWindowless_GetWindow(IOleInPlaceObjectWindowless *iface,
+        HWND *phwnd)
+{
+    HTMLDocument *This = impl_from_IOleInPlaceObjectWindowless(iface);
+    return IOleInPlaceActiveObject_GetWindow(&This->IOleInPlaceActiveObject_iface, phwnd);
+}
+
+static HRESULT WINAPI OleInPlaceObjectWindowless_ContextSensitiveHelp(IOleInPlaceObjectWindowless *iface,
+        BOOL fEnterMode)
+{
+    HTMLDocument *This = impl_from_IOleInPlaceObjectWindowless(iface);
+    return IOleInPlaceActiveObject_ContextSensitiveHelp(&This->IOleInPlaceActiveObject_iface, fEnterMode);
+}
+
+static HRESULT WINAPI OleInPlaceObjectWindowless_InPlaceDeactivate(IOleInPlaceObjectWindowless *iface)
+{
+    HTMLDocument *This = impl_from_IOleInPlaceObjectWindowless(iface);
+
+    TRACE("(%p)\n", This);
+
+    if(This->doc_obj->ui_active)
+        IOleDocumentView_UIActivate(&This->doc_obj->IOleDocumentView_iface, FALSE);
+    This->doc_obj->window_active = FALSE;
+
+    if(!This->doc_obj->in_place_active)
+        return S_OK;
+
+    if(This->doc_obj->frame) {
+        IOleInPlaceFrame_Release(This->doc_obj->frame);
+        This->doc_obj->frame = NULL;
+    }
+
+    if(This->doc_obj->hwnd) {
+        ShowWindow(This->doc_obj->hwnd, SW_HIDE);
+        SetWindowPos(This->doc_obj->hwnd, HWND_TOPMOST, 0, 0, 0, 0, SWP_NOSIZE | SWP_NOMOVE | SWP_NOACTIVATE);
+    }
+
+    This->doc_obj->focus = FALSE;
+    notif_focus(This->doc_obj);
+
+    This->doc_obj->in_place_active = FALSE;
+    if(This->doc_obj->ipsite) {
+        IOleInPlaceSiteEx *ipsiteex;
+        HRESULT hres;
+
+        hres = IOleInPlaceSite_QueryInterface(This->doc_obj->ipsite, &IID_IOleInPlaceSiteEx, (void**)&ipsiteex);
+        if(SUCCEEDED(hres)) {
+            IOleInPlaceSiteEx_OnInPlaceDeactivateEx(ipsiteex, TRUE);
+            IOleInPlaceSiteEx_Release(ipsiteex);
+        }else {
+            IOleInPlaceSite_OnInPlaceDeactivate(This->doc_obj->ipsite);
+        }
+    }
+
+    return S_OK;
+}
+
+static HRESULT WINAPI OleInPlaceObjectWindowless_UIDeactivate(IOleInPlaceObjectWindowless *iface)
+{
+    HTMLDocument *This = impl_from_IOleInPlaceObjectWindowless(iface);
+    FIXME("(%p)\n", This);
+    return E_NOTIMPL;
+}
+
+static HRESULT WINAPI OleInPlaceObjectWindowless_SetObjectRects(IOleInPlaceObjectWindowless *iface,
+        const RECT *pos, const RECT *clip)
+{
+    HTMLDocument *This = impl_from_IOleInPlaceObjectWindowless(iface);
+    RECT r;
+
+    TRACE("(%p)->(%s %s)\n", This, wine_dbgstr_rect(pos), wine_dbgstr_rect(clip));
+
+    if(clip && !EqualRect(clip, pos))
+        FIXME("Ignoring clip rect %s\n", wine_dbgstr_rect(clip));
+
+    r = *pos;
+    return IOleDocumentView_SetRect(&This->doc_obj->IOleDocumentView_iface, &r);
+}
+
+static HRESULT WINAPI OleInPlaceObjectWindowless_ReactivateAndUndo(IOleInPlaceObjectWindowless *iface)
+{
+    HTMLDocument *This = impl_from_IOleInPlaceObjectWindowless(iface);
+    FIXME("(%p)\n", This);
+    return E_NOTIMPL;
+}
+
+static HRESULT WINAPI OleInPlaceObjectWindowless_OnWindowMessage(IOleInPlaceObjectWindowless *iface,
+        UINT msg, WPARAM wParam, LPARAM lParam, LRESULT *lpResult)
+{
+    HTMLDocument *This = impl_from_IOleInPlaceObjectWindowless(iface);
+    FIXME("(%p)->(%u %Iu %Iu %p)\n", This, msg, wParam, lParam, lpResult);
+    return E_NOTIMPL;
+}
+
+static HRESULT WINAPI OleInPlaceObjectWindowless_GetDropTarget(IOleInPlaceObjectWindowless *iface,
+        IDropTarget **ppDropTarget)
+{
+    HTMLDocument *This = impl_from_IOleInPlaceObjectWindowless(iface);
+    FIXME("(%p)->(%p)\n", This, ppDropTarget);
+    return E_NOTIMPL;
+}
+
+static const IOleInPlaceObjectWindowlessVtbl OleInPlaceObjectWindowlessVtbl = {
+    OleInPlaceObjectWindowless_QueryInterface,
+    OleInPlaceObjectWindowless_AddRef,
+    OleInPlaceObjectWindowless_Release,
+    OleInPlaceObjectWindowless_GetWindow,
+    OleInPlaceObjectWindowless_ContextSensitiveHelp,
+    OleInPlaceObjectWindowless_InPlaceDeactivate,
+    OleInPlaceObjectWindowless_UIDeactivate,
+    OleInPlaceObjectWindowless_SetObjectRects,
+    OleInPlaceObjectWindowless_ReactivateAndUndo,
+    OleInPlaceObjectWindowless_OnWindowMessage,
+    OleInPlaceObjectWindowless_GetDropTarget
+};
+
 /**********************************************************
  * IObjectWithSite implementation
  */
@@ -1063,7 +1310,7 @@ static HRESULT WINAPI OleContainer_EnumObjects(IOleContainer *iface, DWORD grfFl
     HTMLDocument *This = impl_from_IOleContainer(iface);
     EnumUnknown *ret;
 
-    TRACE("(%p)->(%x %p)\n", This, grfFlags, ppenum);
+    TRACE("(%p)->(%lx %p)\n", This, grfFlags, ppenum);
 
     ret = heap_alloc(sizeof(*ret));
     if(!ret)
@@ -1186,7 +1433,7 @@ static HRESULT WINAPI ObjectSafety_SetInterfaceSafetyOptions(IObjectSafety *ifac
         REFIID riid, DWORD dwOptionSetMask, DWORD dwEnabledOptions)
 {
     HTMLDocument *This = impl_from_IObjectSafety(iface);
-    FIXME("(%p)->(%s %x %x)\n", This, debugstr_guid(riid), dwOptionSetMask, dwEnabledOptions);
+    FIXME("(%p)->(%s %lx %lx)\n", This, debugstr_guid(riid), dwOptionSetMask, dwEnabledOptions);
 
     if(IsEqualGUID(&IID_IPersistMoniker, riid) &&
             dwOptionSetMask==INTERFACESAFE_FOR_UNTRUSTED_DATA &&
@@ -1225,7 +1472,11 @@ void HTMLDocument_OleObj_Init(HTMLDocument *This)
     This->IOleObject_iface.lpVtbl = &OleObjectVtbl;
     This->IOleDocument_iface.lpVtbl = &OleDocumentVtbl;
     This->IOleControl_iface.lpVtbl = &OleControlVtbl;
+    This->IOleInPlaceActiveObject_iface.lpVtbl = &OleInPlaceActiveObjectVtbl;
+    This->IOleInPlaceObjectWindowless_iface.lpVtbl = &OleInPlaceObjectWindowlessVtbl;
     This->IObjectWithSite_iface.lpVtbl = &ObjectWithSiteVtbl;
     This->IOleContainer_iface.lpVtbl = &OleContainerVtbl;
     This->IObjectSafety_iface.lpVtbl = &ObjectSafetyVtbl;
+    This->doc_obj->extent.cx = 1;
+    This->doc_obj->extent.cy = 1;
 }
