@@ -16,11 +16,8 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  */
 
-#include "config.h"
-
 #include <stdarg.h>
 #include <stdio.h>
-#include <assert.h>
 
 #define COBJMACROS
 
@@ -44,6 +41,7 @@ typedef struct {
     DWORD id;
     DWORD time;
     DWORD interval;
+    enum timer_type type;
     IDispatch *disp;
 
     struct list entry;
@@ -165,7 +163,7 @@ static BOOL queue_timer(thread_data_t *thread_data, task_timer_t *timer)
     return FALSE;
 }
 
-HRESULT set_task_timer(HTMLInnerWindow *window, LONG msec, BOOL interval, IDispatch *disp, LONG *id)
+HRESULT set_task_timer(HTMLInnerWindow *window, LONG msec, enum timer_type timer_type, IDispatch *disp, LONG *id)
 {
     thread_data_t *thread_data;
     task_timer_t *timer;
@@ -187,7 +185,8 @@ HRESULT set_task_timer(HTMLInnerWindow *window, LONG msec, BOOL interval, IDispa
     timer->id = id_cnt++;
     timer->window = window;
     timer->time = tc + msec;
-    timer->interval = interval ? msec : 0;
+    timer->interval = timer_type == TIMER_INTERVAL ? msec : 0;
+    timer->type = timer_type;
     list_init(&timer->entry);
 
     IDispatch_AddRef(disp);
@@ -219,9 +218,20 @@ HRESULT clear_task_timer(HTMLInnerWindow *window, DWORD id)
     return S_OK;
 }
 
-static void call_timer_disp(IDispatch *disp)
+static const char *debugstr_timer_type(enum timer_type type)
+{
+    switch(type) {
+    case TIMER_TIMEOUT:  return "timeout";
+    case TIMER_INTERVAL: return "interval";
+    case TIMER_ANIMATION_FRAME: return "animation-frame";
+    DEFAULT_UNREACHABLE;
+    }
+}
+
+static void call_timer_disp(IDispatch *disp, enum timer_type timer_type)
 {
     DISPPARAMS dp = {NULL, NULL, 0, 0};
+    VARIANT timestamp;
     EXCEPINFO ei;
     VARIANT res;
     HRESULT hres;
@@ -229,12 +239,19 @@ static void call_timer_disp(IDispatch *disp)
     V_VT(&res) = VT_EMPTY;
     memset(&ei, 0, sizeof(ei));
 
-    TRACE(">>>\n");
+    if(timer_type == TIMER_ANIMATION_FRAME) {
+        dp.cArgs = 1;
+        dp.rgvarg = &timestamp;
+        V_VT(&timestamp) = VT_R8;
+        V_R8(&timestamp) = get_time_stamp();
+    }
+
+    TRACE("%p %s >>>\n", disp, debugstr_timer_type(timer_type));
     hres = IDispatch_Invoke(disp, DISPID_VALUE, &IID_NULL, 0, DISPATCH_METHOD, &dp, &res, &ei, NULL);
     if(hres == S_OK)
-        TRACE("<<<\n");
+        TRACE("%p %s <<<\n", disp, debugstr_timer_type(timer_type));
     else
-        WARN("<<< %08x\n", hres);
+        WARN("%p %s <<< %08lx\n", disp, debugstr_timer_type(timer_type), hres);
 
     VariantClear(&res);
 }
@@ -242,6 +259,7 @@ static void call_timer_disp(IDispatch *disp)
 static LRESULT process_timer(void)
 {
     thread_data_t *thread_data;
+    enum timer_type timer_type;
     IDispatch *disp;
     DWORD tc;
     task_timer_t *timer=NULL, *last_timer;
@@ -274,6 +292,7 @@ static LRESULT process_timer(void)
 
         disp = timer->disp;
         IDispatch_AddRef(disp);
+        timer_type = timer->type;
 
         if(timer->interval) {
             timer->time += timer->interval;
@@ -282,7 +301,7 @@ static LRESULT process_timer(void)
             release_task_timer(thread_data->thread_hwnd, timer);
         }
 
-        call_timer_disp(disp);
+        call_timer_disp(disp, timer_type);
 
         IDispatch_Release(disp);
     }while(!list_empty(&thread_data->timer_list));
@@ -331,7 +350,7 @@ static LRESULT WINAPI hidden_proc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lPa
     }
 
     if(msg > WM_USER)
-        FIXME("(%p %d %lx %lx)\n", hwnd, msg, wParam, lParam);
+        FIXME("(%p %d %Ix %Ix)\n", hwnd, msg, wParam, lParam);
 
     return DefWindowProcW(hwnd, msg, wParam, lParam);
 }
@@ -339,22 +358,20 @@ static LRESULT WINAPI hidden_proc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lPa
 static HWND create_thread_hwnd(void)
 {
     static ATOM hidden_wnd_class = 0;
-    static const WCHAR wszInternetExplorer_Hidden[] = {'I','n','t','e','r','n','e','t',
-            ' ','E','x','p','l','o','r','e','r','_','H','i','d','d','e','n',0};
 
     if(!hidden_wnd_class) {
         WNDCLASSEXW wndclass = {
             sizeof(WNDCLASSEXW), 0,
             hidden_proc,
             0, 0, hInst, NULL, NULL, NULL, NULL,
-            wszInternetExplorer_Hidden,
+            L"Internet Explorer_Hidden",
             NULL
         };
 
         hidden_wnd_class = RegisterClassExW(&wndclass);
     }
 
-    return CreateWindowExW(0, wszInternetExplorer_Hidden, NULL, WS_POPUP,
+    return CreateWindowExW(0, L"Internet Explorer_Hidden", NULL, WS_POPUP,
                            0, 0, 0, 0, NULL, NULL, hInst, NULL);
 }
 
@@ -386,7 +403,7 @@ HRESULT create_marshaled_doc(HWND main_thread_hwnd, REFIID riid, void **ppv)
     res = SendMessageW(main_thread_hwnd, WM_CREATEDOC, 0, (LPARAM)&params);
     TRACE("SendMessage ret %x\n", res);
     if(FAILED(params.hres)) {
-        WARN("EM_CREATEDOC failed: %08x\n", params.hres);
+        WARN("EM_CREATEDOC failed: %08lx\n", params.hres);
         IStream_Release(params.stream);
         return hres;
     }
@@ -397,7 +414,7 @@ HRESULT create_marshaled_doc(HWND main_thread_hwnd, REFIID riid, void **ppv)
         hres = CoUnmarshalInterface(params.stream, riid, ppv);
     IStream_Release(params.stream);
     if(FAILED(hres))
-        WARN("CoUnmarshalInterface failed: %08x\n", hres);
+        WARN("CoUnmarshalInterface failed: %08lx\n", hres);
     return hres;
 }
 
@@ -433,3 +450,14 @@ thread_data_t *get_thread_data(BOOL create)
 
     return thread_data;
 }
+
+ULONGLONG get_time_stamp(void)
+{
+    FILETIME time;
+
+    /* 1601 to 1970 is 369 years plus 89 leap days */
+    const ULONGLONG time_epoch = (ULONGLONG)(369 * 365 + 89) * 86400 * 1000;
+
+    GetSystemTimeAsFileTime(&time);
+    return (((ULONGLONG)time.dwHighDateTime << 32) + time.dwLowDateTime) / 10000 - time_epoch;
+}
