@@ -19,11 +19,12 @@
  */
 
 #include "config.h"
-#include "wine/port.h"
 
 #include <stdarg.h>
 #include <stdlib.h>
 #include <string.h>
+#include <sys/types.h>
+#include <math.h>
 
 #include "windef.h"
 #include "winbase.h"
@@ -36,6 +37,7 @@
 #include "opengl_ext.h"
 #include "wine/gdi_driver.h"
 #include "wine/glu.h"
+#include "wine/static_strings.h"
 #include "wine/debug.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(wgl);
@@ -105,6 +107,8 @@ static CRITICAL_SECTION wgl_section = { &critsect_debug, -1, 0, 0, 0, 0 };
 
 static const MAT2 identity = { {0,1},{0,0},{0,0},{0,1} };
 
+DECLARE_STATIC_STRINGS(gl_strings);
+
 static inline HANDLE next_handle( struct wgl_handle *ptr, enum wgl_handle_type type )
 {
     WORD generation = HIWORD( ptr->handle ) + 1;
@@ -410,9 +414,9 @@ HDC WINAPI wglGetCurrentDC(void)
 }
 
 /***********************************************************************
- *		wglCreateContext (OPENGL32.@)
+ *		wgl_create_context wrapper for hooking
  */
-HGLRC WINAPI wglCreateContext(HDC hdc)
+static HGLRC wgl_create_context(HDC hdc)
 {
     HGLRC ret = 0;
     struct wgl_context *drv_ctx;
@@ -431,6 +435,14 @@ HGLRC WINAPI wglCreateContext(HDC hdc)
     return ret;
 }
 
+/***********************************************************************
+ *		wglCreateContext (OPENGL32.@)
+ */
+HGLRC WINAPI wglCreateContext(HDC hdc)
+{
+    return wgl_create_context(hdc);
+}
+
 /***********************************************************************
  *		wglGetCurrentContext (OPENGL32.@)
  */
@@ -479,7 +491,7 @@ INT WINAPI wglChoosePixelFormat(HDC hdc, const PIXELFORMATDESCRIPTOR* ppfd)
     {
         if (!wglDescribePixelFormat( hdc, i, sizeof(format), &format )) continue;
 
-        if (ppfd->iPixelType != format.iPixelType)
+        if ((ppfd->iPixelType == PFD_TYPE_COLORINDEX) != (format.iPixelType == PFD_TYPE_COLORINDEX))
         {
             TRACE( "pixel type mismatch for iPixelFormat=%d\n", i );
             continue;
@@ -567,7 +579,7 @@ INT WINAPI wglChoosePixelFormat(HDC hdc, const PIXELFORMATDESCRIPTOR* ppfd)
                 continue;
             }
         }
-        if (ppfd->cDepthBits)
+        if (ppfd->cDepthBits && !(ppfd->dwFlags & PFD_DEPTH_DONTCARE))
         {
             if (((ppfd->cDepthBits > best.cDepthBits) && (format.cDepthBits > best.cDepthBits)) ||
                 ((format.cDepthBits >= ppfd->cDepthBits) && (format.cDepthBits < best.cDepthBits)))
@@ -603,6 +615,9 @@ INT WINAPI wglChoosePixelFormat(HDC hdc, const PIXELFORMATDESCRIPTOR* ppfd)
                 continue;
             }
         }
+        if (ppfd->dwFlags & PFD_DEPTH_DONTCARE && format.cDepthBits < best.cDepthBits)
+            goto found;
+
         continue;
 
     found:
@@ -679,7 +694,7 @@ HGLRC WINAPI wglCreateLayerContext(HDC hdc,
   TRACE("(%p,%d)\n", hdc, iLayerPlane);
 
   if (iLayerPlane == 0) {
-      return wglCreateContext(hdc);
+      return wgl_create_context(hdc);
   }
   FIXME("no handler for layer %d\n", iLayerPlane);
 
@@ -712,7 +727,7 @@ int WINAPI wglGetLayerPaletteEntries(HDC hdc,
   return 0;
 }
 
-static BOOL filter_extensions(const char *extensions, GLubyte **exts_list, GLuint **disabled_exts);
+static BOOL filter_extensions(const char *HOSTPTR extensions, GLubyte **exts_list, GLuint **disabled_exts);
 
 void WINAPI glGetIntegerv(GLenum pname, GLint *data)
 {
@@ -746,7 +761,7 @@ const GLubyte * WINAPI glGetStringi(GLenum name, GLuint index)
     TRACE("(%d, %d)\n", name, index);
     if (!funcs->ext.p_glGetStringi)
     {
-        void **func_ptr = (void **)&funcs->ext.p_glGetStringi;
+        void * HOSTPTR *func_ptr = (void *HOSTPTR *)&funcs->ext.p_glGetStringi;
 
         *func_ptr = funcs->wgl.p_wglGetProcAddress("glGetStringi");
     }
@@ -763,10 +778,10 @@ const GLubyte * WINAPI glGetStringi(GLenum name, GLuint index)
 
             while (index + disabled_count >= *disabled_exts++)
                 disabled_count++;
-            return funcs->ext.p_glGetStringi(name, index + disabled_count);
+            return wine_static_ustring_add(&gl_strings, funcs->ext.p_glGetStringi(name, index + disabled_count));
         }
     }
-    return funcs->ext.p_glGetStringi(name, index);
+    return wine_static_ustring_add(&gl_strings, funcs->ext.p_glGetStringi(name, index));
 }
 
 /* check if the extension is present in the list */
@@ -797,9 +812,9 @@ static BOOL has_extension( const char *list, const char *ext, size_t len )
     return FALSE;
 }
 
-static int compar(const void *elt_a, const void *elt_b) {
-  return strcmp(((const OpenGL_extension *) elt_a)->name,
-		((const OpenGL_extension *) elt_b)->name);
+static int compar(const void * HOSTPTR elt_a, const void * HOSTPTR elt_b) {
+  return strcmp(((const OpenGL_extension * HOSTPTR) elt_a)->name,
+		((const OpenGL_extension * HOSTPTR) elt_b)->name);
 }
 
 /* Check if a GL extension is supported */
@@ -840,7 +855,7 @@ static BOOL is_extension_supported(const char* extension)
          * Check if we are searching for a core GL function */
         if(strncmp(extension, "GL_VERSION_", 11) == 0)
         {
-            const GLubyte *gl_version = funcs->gl.p_glGetString(GL_VERSION);
+            const GLubyte *HOSTPTR gl_version = funcs->gl.p_glGetString(GL_VERSION);
             const char *version = extension + 11; /* Move past 'GL_VERSION_' */
 
             if(!gl_version) {
@@ -869,7 +884,7 @@ static BOOL is_extension_supported(const char* extension)
 PROC WINAPI wglGetProcAddress( LPCSTR name )
 {
     struct opengl_funcs *funcs = NtCurrentTeb()->glTable;
-    void **func_ptr;
+    void *HOSTPTR *func_ptr;
     OpenGL_extension  ext;
     const OpenGL_extension *ext_ret;
 
@@ -885,17 +900,17 @@ PROC WINAPI wglGetProcAddress( LPCSTR name )
     }
 
     ext.name = name;
-    ext_ret = bsearch(&ext, extension_registry, extension_registry_size, sizeof(ext), compar);
+    ext_ret = ADDRSPACECAST(void *, bsearch(&ext, extension_registry, extension_registry_size, sizeof(ext), compar));
     if (!ext_ret)
     {
         WARN("Function %s unknown\n", name);
         return NULL;
     }
 
-    func_ptr = (void **)&funcs->ext + (ext_ret - extension_registry);
+    func_ptr = (void *HOSTPTR *)&funcs->ext + (ext_ret - extension_registry);
     if (!*func_ptr)
     {
-        void *driver_func = funcs->wgl.p_wglGetProcAddress( name );
+        void *HOSTPTR driver_func = funcs->wgl.p_wglGetProcAddress( name );
 
         if (!is_extension_supported(ext_ret->extension))
         {
@@ -953,6 +968,15 @@ int WINAPI wglSetLayerPaletteEntries(HDC hdc,
   return 0;
 }
 
+/***********************************************************************
+ *		wglGetDefaultProcAddress (OPENGL32.@)
+ */
+PROC WINAPI wglGetDefaultProcAddress( LPCSTR name )
+{
+    FIXME( "%s: stub\n", debugstr_a(name));
+    return NULL;
+}
+
 /***********************************************************************
  *		wglSwapLayerBuffers (OPENGL32.@)
  */
@@ -1224,8 +1248,8 @@ BOOL WINAPI wglUseFontBitmapsW(HDC hdc, DWORD first, DWORD count, DWORD listBase
 
 static void fixed_to_double(POINTFX fixed, UINT em_size, GLdouble vertex[3])
 {
-    vertex[0] = (fixed.x.value + (GLdouble)fixed.x.fract / (1 << 16)) / em_size;  
-    vertex[1] = (fixed.y.value + (GLdouble)fixed.y.fract / (1 << 16)) / em_size;  
+    vertex[0] = (fixed.x.value + (GLdouble)fixed.x.fract / (1 << 16)) / em_size;
+    vertex[1] = (fixed.y.value + (GLdouble)fixed.y.fract / (1 << 16)) / em_size;
     vertex[2] = 0.0;
 }
 
@@ -1405,7 +1429,7 @@ static BOOL wglUseFontOutlines_common(HDC hdc,
             lpgmf->gmfCellIncY = (float)gm.gmCellIncY / em_size;
 
             TRACE("%fx%f at %f,%f inc %f,%f\n", lpgmf->gmfBlackBoxX, lpgmf->gmfBlackBoxY,
-                  lpgmf->gmfptGlyphOrigin.x, lpgmf->gmfptGlyphOrigin.y, lpgmf->gmfCellIncX, lpgmf->gmfCellIncY); 
+                  lpgmf->gmfptGlyphOrigin.x, lpgmf->gmfptGlyphOrigin.y, lpgmf->gmfCellIncX, lpgmf->gmfCellIncY);
             lpgmf++;
         }
 
@@ -1599,10 +1623,10 @@ GLint WINAPI glDebugEntry( GLint unknown1, GLint unknown2 )
     return 0;
 }
 
-static GLubyte *filter_extensions_list(const char *extensions, const char *disabled)
+static GLubyte *filter_extensions_list(const char *HOSTPTR extensions, const char *disabled)
 {
     char *p, *str;
-    const char *end;
+    const char *HOSTPTR end;
 
     p = str = HeapAlloc(GetProcessHeap(), 0, strlen(extensions) + 2);
     if (!str)
@@ -1639,14 +1663,14 @@ static GLubyte *filter_extensions_list(const char *extensions, const char *disab
 static GLuint *filter_extensions_index(const char *disabled)
 {
     const struct opengl_funcs *funcs = NtCurrentTeb()->glTable;
-    const char *ext, *end, *gl_ext;
+    const char *ext, *end, *HOSTPTR gl_ext;
     GLuint *disabled_exts, *new_disabled_exts;
     unsigned int i = 0, j, disabled_size;
     GLint extensions_count;
 
     if (!funcs->ext.p_glGetStringi)
     {
-        void **func_ptr = (void **)&funcs->ext.p_glGetStringi;
+        void *HOSTPTR *func_ptr = (void *HOSTPTR *)&funcs->ext.p_glGetStringi;
 
         *func_ptr = funcs->wgl.p_wglGetProcAddress("glGetStringi");
         if (!funcs->ext.p_glGetStringi)
@@ -1663,7 +1687,7 @@ static GLuint *filter_extensions_index(const char *disabled)
 
     for (j = 0; j < extensions_count; ++j)
     {
-        gl_ext = (const char *)funcs->ext.p_glGetStringi(GL_EXTENSIONS, j);
+        gl_ext = (const char *HOSTPTR)funcs->ext.p_glGetStringi(GL_EXTENSIONS, j);
         ext = disabled;
         for (;;)
         {
@@ -1703,7 +1727,7 @@ static GLuint *filter_extensions_index(const char *disabled)
 }
 
 /* build the extension string by filtering out the disabled extensions */
-static BOOL filter_extensions(const char *extensions, GLubyte **exts_list, GLuint **disabled_exts)
+static BOOL filter_extensions(const char *HOSTPTR extensions, GLubyte **exts_list, GLuint **disabled_exts)
 {
     static const char *disabled;
 
@@ -1771,16 +1795,16 @@ static BOOL filter_extensions(const char *extensions, GLubyte **exts_list, GLuin
 const GLubyte * WINAPI glGetString( GLenum name )
 {
     const struct opengl_funcs *funcs = NtCurrentTeb()->glTable;
-    const GLubyte *ret = funcs->gl.p_glGetString( name );
+    const GLubyte *HOSTPTR ret = funcs->gl.p_glGetString( name );
 
     if (name == GL_EXTENSIONS && ret)
     {
         struct wgl_handle *ptr = get_current_context_ptr();
         if (ptr->u.context->extensions ||
-            filter_extensions((const char *)ret, &ptr->u.context->extensions, &ptr->u.context->disabled_exts))
-            ret = ptr->u.context->extensions;
+            filter_extensions((const char *HOSTPTR)ret, &ptr->u.context->extensions, &ptr->u.context->disabled_exts))
+            return ptr->u.context->extensions;
     }
-    return ret;
+    return wine_static_ustring_add(&gl_strings, ret);
 }
 
 /* wrapper for glDebugMessageCallback* functions */
@@ -1788,8 +1812,11 @@ static void gl_debug_message_callback( GLenum source, GLenum type, GLuint id, GL
                                        GLsizei length, const GLchar *message,const void *userParam )
 {
     struct wgl_handle *ptr = (struct wgl_handle *)userParam;
+    GLchar *temp;
     if (!ptr->u.context->debug_callback) return;
-    ptr->u.context->debug_callback( source, type, id, severity, length, message, ptr->u.context->debug_user );
+    if ((temp = mirror_callback_string(message)))
+        ptr->u.context->debug_callback( source, type, id, severity, length, temp, ptr->u.context->debug_user );
+    gl_temp_free(temp);
 }
 
 /***********************************************************************
@@ -1850,6 +1877,9 @@ BOOL WINAPI DllMain( HINSTANCE hinst, DWORD reason, LPVOID reserved )
     case DLL_THREAD_ATTACH:
         NtCurrentTeb()->glTable = &null_opengl_funcs;
         break;
+    case DLL_PROCESS_DETACH:
+        wine_static_string_free(&gl_strings);
+        break;
     }
     return TRUE;
 }
