@@ -19,9 +19,6 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  */
 
-#include "config.h"
-#include "wine/port.h"
-
 #include <stdlib.h>
 #include <string.h>
 #include <stdarg.h>
@@ -39,7 +36,6 @@
 #include "wingdi.h"
 #include "pidl.h"
 #include "shlguid.h"
-#include "undocshell.h"
 #include "shell32_main.h"
 #include "shresdef.h"
 #include "shlwapi.h"
@@ -163,7 +159,7 @@ static ULONG WINAPI ISF_MyComputer_fnAddRef (IShellFolder2 * iface)
     IMyComputerFolderImpl *This = impl_from_IShellFolder2(iface);
     ULONG refCount = InterlockedIncrement(&This->ref);
 
-    TRACE ("(%p)->(count=%u)\n", This, refCount - 1);
+    TRACE ("(%p)->(count=%lu)\n", This, refCount - 1);
 
     return refCount;
 }
@@ -173,7 +169,7 @@ static ULONG WINAPI ISF_MyComputer_fnRelease (IShellFolder2 * iface)
     IMyComputerFolderImpl *This = impl_from_IShellFolder2(iface);
     ULONG refCount = InterlockedDecrement(&This->ref);
 
-    TRACE ("(%p)->(count=%u)\n", This, refCount + 1);
+    TRACE ("(%p)->(count=%lu)\n", This, refCount + 1);
 
     if (!refCount)
     {
@@ -238,7 +234,7 @@ static HRESULT WINAPI ISF_MyComputer_fnParseDisplayName (IShellFolder2 *iface,
 
     *ppidl = pidlTemp;
 
-    TRACE ("(%p)->(-- ret=0x%08x)\n", This, hr);
+    TRACE ("(%p)->(-- ret=0x%08lx)\n", This, hr);
 
     return hr;
 }
@@ -246,13 +242,6 @@ static HRESULT WINAPI ISF_MyComputer_fnParseDisplayName (IShellFolder2 *iface,
 /* retrieve a map of drives that should be displayed */
 static DWORD get_drive_map(void)
 {
-    static const WCHAR policiesW[] = {'S','o','f','t','w','a','r','e','\\',
-                                      'M','i','c','r','o','s','o','f','t','\\',
-                                      'W','i','n','d','o','w','s','\\',
-                                      'C','u','r','r','e','n','t','V','e','r','s','i','o','n','\\',
-                                      'P','o','l','i','c','i','e','s','\\',
-                                      'E','x','p','l','o','r','e','r',0};
-    static const WCHAR nodrivesW[] = {'N','o','D','r','i','v','e','s',0};
     static DWORD drive_mask;
     static BOOL init_done = FALSE;
 
@@ -261,17 +250,17 @@ static DWORD get_drive_map(void)
         DWORD type, size, data, mask = 0;
         HKEY hkey;
 
-        if (!RegOpenKeyW( HKEY_LOCAL_MACHINE, policiesW, &hkey ))
+        if (!RegOpenKeyW( HKEY_LOCAL_MACHINE, L"Software\\Microsoft\\Windows\\CurrentVersion\\Policies\\Explorer", &hkey ))
         {
             size = sizeof(data);
-            if (!RegQueryValueExW( hkey, nodrivesW, NULL, &type, (LPBYTE)&data, &size ) && type == REG_DWORD)
+            if (!RegQueryValueExW( hkey, L"NoDrives", NULL, &type, (LPBYTE)&data, &size ) && type == REG_DWORD)
                 mask |= data;
             RegCloseKey( hkey );
         }
-        if (!RegOpenKeyW( HKEY_CURRENT_USER, policiesW, &hkey ))
+        if (!RegOpenKeyW( HKEY_CURRENT_USER, L"Software\\Microsoft\\Windows\\CurrentVersion\\Policies\\Explorer", &hkey ))
         {
             size = sizeof(data);
-            if (!RegQueryValueExW( hkey, nodrivesW, NULL, &type, (LPBYTE)&data, &size ) && type == REG_DWORD)
+            if (!RegQueryValueExW( hkey, L"NoDrives", NULL, &type, (LPBYTE)&data, &size ) && type == REG_DWORD)
                 mask |= data;
             RegCloseKey( hkey );
         }
@@ -285,22 +274,16 @@ static DWORD get_drive_map(void)
 /**************************************************************************
  *  CreateMyCompEnumList()
  */
-static const WCHAR MyComputer_NameSpaceW[] = { 'S','O','F','T','W','A','R','E',
- '\\','M','i','c','r','o','s','o','f','t','\\','W','i','n','d','o','w','s','\\',
- 'C','u','r','r','e','n','t','V','e','r','s','i','o','n','\\','E','x','p','l',
- 'o','r','e','r','\\','M','y','C','o','m','p','u','t','e','r','\\','N','a','m',
- 'e','s','p','a','c','e','\0' };
-
 static BOOL CreateMyCompEnumList(IEnumIDListImpl *list, DWORD dwFlags)
 {
     BOOL ret = TRUE;
 
-    TRACE("(%p)->(flags=0x%08x)\n", list, dwFlags);
+    TRACE("(%p)->(flags=0x%08lx)\n", list, dwFlags);
 
     /* enumerate the folders */
     if (dwFlags & SHCONTF_FOLDERS)
     {
-        WCHAR wszDriveName[] = {'A', ':', '\\', '\0'};
+        WCHAR wszDriveName[] = L"A:\\";
         DWORD dwDrivemap = get_drive_map();
         HKEY hkey;
         UINT i;
@@ -316,7 +299,8 @@ static BOOL CreateMyCompEnumList(IEnumIDListImpl *list, DWORD dwFlags)
         TRACE("-- (%p)-> enumerate (mycomputer shell extensions)\n",list);
         for (i=0; i<2; i++) {
             if (ret && !RegOpenKeyExW(i == 0 ? HKEY_LOCAL_MACHINE : HKEY_CURRENT_USER,
-                                      MyComputer_NameSpaceW, 0, KEY_READ, &hkey))
+                                      L"SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Explorer\\MyComputer\\Namespace",
+                                      0, KEY_READ, &hkey))
             {
                 WCHAR iid[50];
                 int i=0;
@@ -356,7 +340,7 @@ static HRESULT WINAPI ISF_MyComputer_fnEnumObjects (IShellFolder2 *iface,
     IMyComputerFolderImpl *This = impl_from_IShellFolder2(iface);
     IEnumIDListImpl *list;
 
-    TRACE("(%p)->(HWND=%p flags=0x%08x pplist=%p)\n", This,
+    TRACE("(%p)->(HWND=%p flags=0x%08lx pplist=%p)\n", This,
           hwndOwner, dwFlags, ppEnumIDList);
 
     if (!(list = IEnumIDList_Constructor()))
@@ -380,7 +364,7 @@ static HRESULT WINAPI ISF_MyComputer_fnBindToObject (IShellFolder2 *iface,
     TRACE("(%p)->(pidl=%p,%p,%s,%p)\n", This,
           pidl, pbcReserved, shdebugstr_guid (riid), ppvOut);
 
-    return SHELL32_BindToChild (This->pidlRoot, NULL, pidl, riid, ppvOut);
+    return SHELL32_BindToChild (This->pidlRoot, &CLSID_ShellFSFolder, NULL, pidl, riid, ppvOut);
 }
 
 /**************************************************************************
@@ -408,9 +392,9 @@ static HRESULT WINAPI ISF_MyComputer_fnCompareIDs (IShellFolder2 *iface,
     IMyComputerFolderImpl *This = impl_from_IShellFolder2(iface);
     HRESULT hr;
 
-    TRACE ("(%p)->(0x%08lx,pidl1=%p,pidl2=%p)\n", This, lParam, pidl1, pidl2);
+    TRACE ("(%p)->(0x%08Ix,pidl1=%p,pidl2=%p)\n", This, lParam, pidl1, pidl2);
     hr = SHELL32_CompareIDs(&This->IShellFolder2_iface, lParam, pidl1, pidl2);
-    TRACE ("-- 0x%08x\n", hr);
+    TRACE ("-- 0x%08lx\n", hr);
     return hr;
 }
 
@@ -439,8 +423,7 @@ static HRESULT WINAPI ISF_MyComputer_fnCreateViewObject (IShellFolder2 *iface,
     }
     else if (IsEqualIID (riid, &IID_IContextMenu))
     {
-        WARN ("IContextMenu not implemented\n");
-        hr = E_NOTIMPL;
+        hr = BackgroundMenu_Constructor((IShellFolder*)iface, FALSE, riid, ppvOut);
     }
     else if (IsEqualIID (riid, &IID_IShellView))
     {
@@ -464,7 +447,7 @@ static HRESULT WINAPI ISF_MyComputer_fnGetAttributesOf (IShellFolder2 * iface,
     IMyComputerFolderImpl *This = impl_from_IShellFolder2(iface);
     HRESULT hr = S_OK;
 
-    TRACE ("(%p)->(cidl=%d apidl=%p mask=%p (0x%08x))\n",
+    TRACE ("(%p)->(cidl=%d apidl=%p mask=%p (0x%08lx))\n",
            This, cidl, apidl, rgfInOut, rgfInOut ? *rgfInOut : 0);
 
     if (!rgfInOut)
@@ -495,7 +478,7 @@ static HRESULT WINAPI ISF_MyComputer_fnGetAttributesOf (IShellFolder2 * iface,
     /* make sure SFGAO_VALIDATE is cleared, some apps depend on that */
     *rgfInOut &= ~SFGAO_VALIDATE;
 
-    TRACE ("-- result=0x%08x\n", *rgfInOut);
+    TRACE ("-- result=0x%08lx\n", *rgfInOut);
     return hr;
 }
 
@@ -572,7 +555,7 @@ static HRESULT WINAPI ISF_MyComputer_fnGetUIObjectOf (IShellFolder2 * iface,
         hr = E_OUTOFMEMORY;
 
     *ppvOut = pObj;
-    TRACE ("(%p)->hr=0x%08x\n", This, hr);
+    TRACE ("(%p)->hr=0x%08lx\n", This, hr);
     return hr;
 }
 
@@ -587,7 +570,7 @@ static HRESULT WINAPI ISF_MyComputer_fnGetDisplayNameOf (IShellFolder2 *iface,
     LPWSTR pszPath;
     HRESULT hr = S_OK;
 
-    TRACE ("(%p)->(pidl=%p,0x%08x,%p)\n", This, pidl, dwFlags, strRet);
+    TRACE ("(%p)->(pidl=%p,0x%08lx,%p)\n", This, pidl, dwFlags, strRet);
     pdump (pidl);
 
     if (!strRet)
@@ -616,15 +599,8 @@ static HRESULT WINAPI ISF_MyComputer_fnGetDisplayNameOf (IShellFolder2 *iface,
             clsid = _ILGetGUIDPointer (pidl);
             if (clsid)
             {
-                if (GET_SHGDN_FOR (dwFlags) & SHGDN_FORPARSING)
+                if ((GET_SHGDN_FOR (dwFlags) & (SHGDN_FORPARSING | SHGDN_FORADDRESSBAR)) == SHGDN_FORPARSING)
                 {
-                    static const WCHAR clsidW[] =
-                     { 'C','L','S','I','D','\\',0 };
-                    static const WCHAR shellfolderW[] =
-                     { '\\','s','h','e','l','l','f','o','l','d','e','r',0 };
-                    static const WCHAR wantsForParsingW[] =
-                     { 'W','a','n','t','s','F','o','r','P','a','r','s','i','n',
-                     'g',0 };
                     BOOL bWantsForParsing = FALSE;
                     WCHAR szRegPath[100];
                     LONG r;
@@ -640,11 +616,10 @@ static HRESULT WINAPI ISF_MyComputer_fnGetDisplayNameOf (IShellFolder2 *iface,
                      * Get the "WantsFORPARSING" flag from the registry
                      */
 
-                    lstrcpyW (szRegPath, clsidW);
+                    lstrcpyW (szRegPath, L"CLSID\\");
                     SHELL32_GUIDToStringW (clsid, &szRegPath[6]);
-                    lstrcatW (szRegPath, shellfolderW);
-                    r = SHGetValueW (HKEY_CLASSES_ROOT, szRegPath, 
-                                     wantsForParsingW, NULL, NULL, NULL);
+                    lstrcatW (szRegPath, L"\\shellfolder");
+                    r = SHGetValueW (HKEY_CLASSES_ROOT, szRegPath, L"WantsForParsing", NULL, NULL, NULL);
                     if (r == ERROR_SUCCESS)
                         bWantsForParsing = TRUE;
 
@@ -695,34 +670,14 @@ static HRESULT WINAPI ISF_MyComputer_fnGetDisplayNameOf (IShellFolder2 *iface,
             /* long view "lw_name (C:)" */
             if (!(dwFlags & SHGDN_FORPARSING))
             {
-                static const WCHAR wszOpenBracket[] = {' ','(',0};
-                static const WCHAR wszCloseBracket[] = {')',0};
                 WCHAR wszDrive[32 /* label */ + 6 /* ' (C:)'\0 */] = {0};
 
                 GetVolumeInformationW (pszPath, wszDrive, ARRAY_SIZE(wszDrive) - 5, NULL, NULL,
                         NULL, NULL, 0);
-
-                /* Display unix path if volume has no label */
-                if (!wszDrive[0])
-                {
-                    char *unix_path = wine_get_unix_file_name(pszPath);
-                    if (unix_path)
-                    {
-                        char real_unix_path[PATH_MAX];
-                        int len;
-
-                        realpath(unix_path, real_unix_path);
-                        HeapFree( GetProcessHeap(), 0, unix_path );
-                        len = MultiByteToWideChar(CP_UNIXCP, 0, real_unix_path, -1,
-                                            wszDrive, sizeof(wszDrive)/sizeof(wszDrive[0]) - 6);
-                        wszDrive[len] = '\0';
-                    }
-                }
-
-                strcatW (wszDrive, wszOpenBracket);
-                lstrcpynW (wszDrive + strlenW(wszDrive), pszPath, 3);
-                strcatW (wszDrive, wszCloseBracket);
-                strcpyW (pszPath, wszDrive);
+                lstrcatW (wszDrive, L" (");
+                lstrcpynW (wszDrive + lstrlenW(wszDrive), pszPath, 3);
+                lstrcatW (wszDrive, L")");
+                lstrcpyW (pszPath, wszDrive);
             }
         }
         else 
@@ -780,7 +735,7 @@ static HRESULT WINAPI ISF_MyComputer_fnSetNameOf (
                LPCOLESTR lpName, DWORD dwFlags, LPITEMIDLIST * pPidlOut)
 {
     IMyComputerFolderImpl *This = impl_from_IShellFolder2(iface);
-    FIXME ("(%p)->(%p,pidl=%p,%s,%u,%p)\n", This,
+    FIXME ("(%p)->(%p,pidl=%p,%s,%lu,%p)\n", This,
            hwndOwner, pidl, debugstr_w (lpName), dwFlags, pPidlOut);
     return E_FAIL;
 }
@@ -804,7 +759,7 @@ static HRESULT WINAPI ISF_MyComputer_fnGetDefaultColumn(IShellFolder2 *iface, DW
 {
     IMyComputerFolderImpl *This = impl_from_IShellFolder2(iface);
 
-    TRACE("(%p)->(%#x, %p, %p)\n", This, reserved, sort, display);
+    TRACE("(%p)->(%#lx, %p, %p)\n", This, reserved, sort, display);
 
     return E_NOTIMPL;
 }
@@ -854,13 +809,6 @@ static HRESULT WINAPI ISF_MyComputer_fnGetDetailsOf (IShellFolder2 *iface,
 
     switch (iColumn)
     {
-        case 0:        /* name */
-            hr = IShellFolder2_GetDisplayNameOf (iface, pidl,
-                       SHGDN_NORMAL | SHGDN_INFOLDER, &psd->str);
-            break;
-        case 1:        /* type */
-            _ILGetFileType (pidl, psd->str.u.cStr, MAX_PATH);
-            break;
         case 2:        /* total size */
             if (_ILIsDrive (pidl))
             {
@@ -877,6 +825,9 @@ static HRESULT WINAPI ISF_MyComputer_fnGetDetailsOf (IShellFolder2 *iface,
                 StrFormatByteSizeA (ulBytes.u.LowPart, psd->str.u.cStr, MAX_PATH);
             }
             break;
+
+        default:
+            return shellfolder_get_file_details( iface, pidl, mycomputer_header, iColumn, psd );
     }
 
     return hr;
@@ -936,7 +887,7 @@ static HRESULT WINAPI IMCFldr_PersistFolder2_QueryInterface (
 static ULONG WINAPI IMCFldr_PersistFolder2_AddRef (IPersistFolder2 * iface)
 {
     IMyComputerFolderImpl *This = impl_from_IPersistFolder2(iface);
-    TRACE ("(%p)->(count=%u)\n", This, This->ref);
+    TRACE ("(%p)->(count=%lu)\n", This, This->ref);
     return IShellFolder2_AddRef (&This->IShellFolder2_iface);
 }
 
@@ -946,7 +897,7 @@ static ULONG WINAPI IMCFldr_PersistFolder2_AddRef (IPersistFolder2 * iface)
 static ULONG WINAPI IMCFldr_PersistFolder2_Release (IPersistFolder2 * iface)
 {
     IMyComputerFolderImpl *This = impl_from_IPersistFolder2(iface);
-    TRACE ("(%p)->(count=%u)\n", This, This->ref);
+    TRACE ("(%p)->(count=%lu)\n", This, This->ref);
     return IShellFolder2_Release (&This->IShellFolder2_iface);
 }
 
