@@ -18,9 +18,6 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  */
 
-#include "config.h"
-#include "wine/port.h"
-
 #include <ctype.h>
 #include <errno.h>
 #include <limits.h>
@@ -34,7 +31,6 @@
 #include "wingdi.h"
 #include "winuser.h"
 #include "winnls.h"
-#include "wine/unicode.h"
 #include "controls.h"
 #include "win.h"
 #include "user_private.h"
@@ -131,8 +127,8 @@ static const WORD *DIALOG_GetControl32( const WORD *p, DLG_CONTROL_INFO *info,
 
     if (dialogEx)
     {
-        /* id is a DWORD for DIALOGEX */
-        info->id = GET_DWORD(p);
+        /* id is 4 bytes for DIALOGEX */
+        info->id = GET_LONG(p);
         p += 2;
     }
     else
@@ -143,14 +139,14 @@ static const WORD *DIALOG_GetControl32( const WORD *p, DLG_CONTROL_INFO *info,
 
     if (GET_WORD(p) == 0xffff)
     {
-        static const WCHAR class_names[6][10] =
+        static const WCHAR *class_names[6] =
         {
-            { 'B','u','t','t','o','n', },             /* 0x80 */
-            { 'E','d','i','t', },                     /* 0x81 */
-            { 'S','t','a','t','i','c', },             /* 0x82 */
-            { 'L','i','s','t','B','o','x', },         /* 0x83 */
-            { 'S','c','r','o','l','l','B','a','r', }, /* 0x84 */
-            { 'C','o','m','b','o','B','o','x', }      /* 0x85 */
+            L"Button",    /* 0x80 */
+            L"Edit",      /* 0x81 */
+            L"Static",    /* 0x82 */
+            L"ListBox",   /* 0x83 */
+            L"ScrollBar", /* 0x84 */
+            L"ComboBox"   /* 0x85 */
         };
         WORD id = GET_WORD(p+1);
         /* Windows treats dialog control class ids 0-5 same way as 0x80-0x85 */
@@ -167,7 +163,7 @@ static const WORD *DIALOG_GetControl32( const WORD *p, DLG_CONTROL_INFO *info,
     else
     {
         info->className = p;
-        p += strlenW( info->className ) + 1;
+        p += lstrlenW( info->className ) + 1;
     }
 
     if (GET_WORD(p) == 0xffff)  /* Is it an integer id? */
@@ -178,7 +174,7 @@ static const WORD *DIALOG_GetControl32( const WORD *p, DLG_CONTROL_INFO *info,
     else
     {
         info->windowName = p;
-        p += strlenW( info->windowName ) + 1;
+        p += lstrlenW( info->windowName ) + 1;
     }
 
     TRACE("    %s %s %ld, %d, %d, %d, %d, %08x, %08x, %08x\n",
@@ -197,7 +193,7 @@ static const WORD *DIALOG_GetControl32( const WORD *p, DLG_CONTROL_INFO *info,
             TRACE("\n");
             TRACE("  END\n" );
         }
-        info->data = p + 1;
+        info->data = p;
         p += GET_WORD(p) / sizeof(WORD);
     }
     else info->data = NULL;
@@ -283,6 +279,7 @@ static BOOL DIALOG_CreateControls32( HWND hwnd, LPCSTR template, const DLG_TEMPL
             WARN("control %s %s creation failed\n", debugstr_w(info.className),
                  debugstr_w(info.windowName));
             if (dlgTemplate->style & DS_NOFAILCREATE) continue;
+            SetLastError(0);
             return FALSE;
         }
 
@@ -360,7 +357,7 @@ static LPCSTR DIALOG_ParseTemplate32( LPCSTR template, DLG_TEMPLATE * result )
     default:
         result->menuName = p;
         TRACE(" MENU %s\n", debugstr_w(result->menuName) );
-        p += strlenW( result->menuName ) + 1;
+        p += lstrlenW( result->menuName ) + 1;
         break;
     }
 
@@ -380,14 +377,14 @@ static LPCSTR DIALOG_ParseTemplate32( LPCSTR template, DLG_TEMPLATE * result )
     default:
         result->className = p;
         TRACE(" CLASS %s\n", debugstr_w( result->className ));
-        p += strlenW( result->className ) + 1;
+        p += lstrlenW( result->className ) + 1;
         break;
     }
 
     /* Get the window caption */
 
     result->caption = p;
-    p += strlenW( result->caption ) + 1;
+    p += lstrlenW( result->caption ) + 1;
     TRACE(" CAPTION %s\n", debugstr_w( result->caption ) );
 
     /* Get the font name */
@@ -422,7 +419,7 @@ static LPCSTR DIALOG_ParseTemplate32( LPCSTR template, DLG_TEMPLATE * result )
                 result->italic = LOBYTE(GET_WORD(p)); p++;
             }
             result->faceName = p;
-            p += strlenW( result->faceName ) + 1;
+            p += lstrlenW( result->faceName ) + 1;
 
             TRACE(" FONT %d, %s, %d, %s\n",
                   result->pointSize, debugstr_w( result->faceName ),
@@ -511,7 +508,7 @@ static HWND DIALOG_CreateIndirect( HINSTANCE hInst, LPCVOID dlgTemplate,
             }
             SelectObject( dc, hOldFont );
         }
-        ReleaseDC(0, dc);
+        NtUserReleaseDC( 0, dc );
         TRACE("units = %d,%d\n", xBaseUnit, yBaseUnit );
     }
 
@@ -642,7 +639,7 @@ static HWND DIALOG_CreateIndirect( HINSTANCE hInst, LPCVOID dlgTemplate,
     if (!hwnd)
     {
         if (hUserFont) DeleteObject( hUserFont );
-        if (hMenu) DestroyMenu( hMenu );
+        if (hMenu) NtUserDestroyMenu( hMenu );
         if (disabled_owner) EnableWindow( disabled_owner, TRUE );
         return 0;
     }
@@ -670,6 +667,8 @@ static HWND DIALOG_CreateIndirect( HINSTANCE hInst, LPCVOID dlgTemplate,
 
     if (DIALOG_CreateControls32( hwnd, dlgTemplate, &template, hInst, unicode ))
     {
+        HWND capture;
+
         /* Send initialisation messages and set focus */
 
         if (dlgProc)
@@ -688,19 +687,27 @@ static HWND DIALOG_CreateIndirect( HINSTANCE hInst, LPCVOID dlgTemplate,
                 {
                     if (SendMessageW( focus, WM_GETDLGCODE, 0, 0 ) & DLGC_HASSETSEL)
                         SendMessageW( focus, EM_SETSEL, 0, MAXLONG );
-                    SetFocus( focus );
+                    NtUserSetFocus( focus );
+                }
+                else
+                {
+                    if (!(template.style & WS_CHILD))
+                        NtUserSetFocus( hwnd );
                 }
             }
         }
 
+        if (modal_owner && (capture = GetCapture()))
+            SendMessageW( capture, WM_CANCELMODE, 0, 0 );
+
         if (template.style & WS_VISIBLE && !(GetWindowLongW( hwnd, GWL_STYLE ) & WS_VISIBLE))
         {
-           ShowWindow( hwnd, SW_SHOWNORMAL );   /* SW_SHOW doesn't always work */
+           NtUserShowWindow( hwnd, SW_SHOWNORMAL );   /* SW_SHOW doesn't always work */
         }
         return hwnd;
     }
     if (disabled_owner) EnableWindow( disabled_owner, TRUE );
-    if( IsWindow(hwnd) ) DestroyWindow( hwnd );
+    if (IsWindow(hwnd)) NtUserDestroyWindow( hwnd );
     return 0;
 }
 
@@ -786,7 +793,7 @@ INT DIALOG_DoDialogBox( HWND hwnd, HWND owner )
                 if (bFirstEmpty)
                 {
                     /* ShowWindow the first time the queue goes empty */
-                    ShowWindow( hwnd, SW_SHOWNORMAL );
+                    NtUserShowWindow( hwnd, SW_SHOWNORMAL );
                     bFirstEmpty = FALSE;
                 }
                 if (!(GetWindowLongW( hwnd, GWL_STYLE ) & DS_NOIDLEMSG))
@@ -814,13 +821,13 @@ INT DIALOG_DoDialogBox( HWND hwnd, HWND owner )
 
             if (bFirstEmpty && msg.message == WM_TIMER)
             {
-                ShowWindow( hwnd, SW_SHOWNORMAL );
+                NtUserShowWindow( hwnd, SW_SHOWNORMAL );
                 bFirstEmpty = FALSE;
             }
         }
     }
     retval = dlgInfo->idResult;
-    DestroyWindow( hwnd );
+    NtUserDestroyWindow( hwnd );
     return retval;
 }
 
@@ -839,9 +846,8 @@ INT_PTR WINAPI DialogBoxParamA( HINSTANCE hInst, LPCSTR name,
 
     if (!(hrsrc = FindResourceA( hInst, name, (LPSTR)RT_DIALOG ))) return -1;
     if (!(ptr = LoadResource(hInst, hrsrc))) return -1;
-    hwnd = DIALOG_CreateIndirect( hInst, ptr, owner, dlgProc, param, FALSE, &owner );
-    if (hwnd) return DIALOG_DoDialogBox( hwnd, owner );
-    return -1;
+    if (!(hwnd = DIALOG_CreateIndirect( hInst, ptr, owner, dlgProc, param, FALSE, &owner ))) return -1;
+    return DIALOG_DoDialogBox( hwnd, owner );
 }
 
 
@@ -859,9 +865,8 @@ INT_PTR WINAPI DialogBoxParamW( HINSTANCE hInst, LPCWSTR name,
 
     if (!(hrsrc = FindResourceW( hInst, name, (LPWSTR)RT_DIALOG ))) return -1;
     if (!(ptr = LoadResource(hInst, hrsrc))) return -1;
-    hwnd = DIALOG_CreateIndirect( hInst, ptr, owner, dlgProc, param, TRUE, &owner );
-    if (hwnd) return DIALOG_DoDialogBox( hwnd, owner );
-    return -1;
+    if (!(hwnd = DIALOG_CreateIndirect( hInst, ptr, owner, dlgProc, param, TRUE, &owner ))) return -1;
+    return DIALOG_DoDialogBox( hwnd, owner );
 }
 
 
@@ -921,13 +926,13 @@ BOOL WINAPI EndDialog( HWND hwnd, INT_PTR retval )
     /* Windows sets the focus to the dialog itself in EndDialog */
 
     if (IsChild(hwnd, GetFocus()))
-       SetFocus( hwnd );
+       NtUserSetFocus( hwnd );
 
     /* Don't have to send a ShowWindow(SW_HIDE), just do
        SetWindowPos with SWP_HIDEWINDOW as done in Windows */
 
-    SetWindowPos(hwnd, NULL, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE
-                 | SWP_NOZORDER | SWP_NOACTIVATE | SWP_HIDEWINDOW);
+    NtUserSetWindowPos( hwnd, NULL, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE
+                        | SWP_NOZORDER | SWP_NOACTIVATE | SWP_HIDEWINDOW );
 
     if (hwnd == GetActiveWindow())
     {
@@ -961,19 +966,19 @@ static BOOL DIALOG_IsAccelerator( HWND hwnd, HWND hwndDlg, WPARAM wParam )
         {
             dlgCode = SendMessageW( hwndControl, WM_GETDLGCODE, 0, 0 );
             if ( (dlgCode & (DLGC_BUTTON | DLGC_STATIC)) &&
-                 GetWindowTextW( hwndControl, buffer, sizeof(buffer)/sizeof(WCHAR) ))
+                 GetWindowTextW( hwndControl, buffer, ARRAY_SIZE( buffer )))
             {
                 /* find the accelerator key */
                 LPWSTR p = buffer - 2;
 
                 do
                 {
-                    p = strchrW( p + 2, '&' );
+                    p = wcschr( p + 2, '&' );
                 }
                 while (p != NULL && p[1] == '&');
 
                 /* and check if it's the one we're looking for */
-                if (p != NULL && toupperW( p[1] ) == toupperW( wParam ) )
+                if (p != NULL && towupper( p[1] ) == towupper( wParam ) )
                 {
                     if ((dlgCode & DLGC_STATIC) || (style & 0x0f) == BS_GROUPBOX )
                     {
@@ -1193,7 +1198,7 @@ BOOL WINAPI IsDialogMessageW( HWND hwndDlg, LPMSG msg )
                  * do so but I presume someone has)
                  */
                 if (fIsDialog)
-                    SendMessageW( hwndDlg, WM_NEXTDLGCTL, (GetKeyState(VK_SHIFT) & 0x8000), 0 );
+                    SendMessageW( hwndDlg, WM_NEXTDLGCTL, (NtUserGetKeyState(VK_SHIFT) & 0x8000), 0 );
                 else
                 {
                     /* It would appear that GetNextDlgTabItem can handle being
@@ -1203,7 +1208,7 @@ BOOL WINAPI IsDialogMessageW( HWND hwndDlg, LPMSG msg )
                     HWND hwndFocus = GetFocus();
                     HWND hwndNext = GetNextDlgTabItem (hwndDlg,
                             hwndFocus == hwndDlg ? NULL : hwndFocus,
-                            GetKeyState (VK_SHIFT) & 0x8000);
+                            NtUserGetKeyState (VK_SHIFT) & 0x8000);
                     if (hwndNext)
                     {
                         dlgCode = SendMessageW (hwndNext, WM_GETDLGCODE,
@@ -1216,12 +1221,12 @@ BOOL WINAPI IsDialogMessageW( HWND hwndDlg, LPMSG msg )
                             {
                                 INT length;
                                 SendMessageW (hwndNext, WM_GETTEXT, maxlen, (LPARAM) buffer);
-                                length = strlenW (buffer);
+                                length = lstrlenW (buffer);
                                 HeapFree (GetProcessHeap(), 0, buffer);
                                 SendMessageW (hwndNext, EM_SETSEL, 0, length);
                             }
                         }
-                        SetFocus (hwndNext);
+                        NtUserSetFocus( hwndNext );
                         DIALOG_FixChildrenOnChangeFocus (hwndDlg, hwndNext);
                     }
                     else
@@ -1238,8 +1243,16 @@ BOOL WINAPI IsDialogMessageW( HWND hwndDlg, LPMSG msg )
             if (!(dlgCode & DLGC_WANTARROWS))
             {
                 BOOL fPrevious = (msg->wParam == VK_LEFT || msg->wParam == VK_UP);
-                HWND hwndNext = GetNextDlgGroupItem (hwndDlg, GetFocus(), fPrevious );
-                SendMessageW( hwndDlg, WM_NEXTDLGCTL, (WPARAM)hwndNext, 1 );
+                HWND hwndNext = GetNextDlgGroupItem( hwndDlg, msg->hwnd, fPrevious );
+                if (hwndNext && SendMessageW( hwndNext, WM_GETDLGCODE, msg->wParam, (LPARAM)msg ) == (DLGC_BUTTON | DLGC_RADIOBUTTON))
+                {
+                    NtUserSetFocus( hwndNext );
+                    if ((GetWindowLongW( hwndNext, GWL_STYLE ) & BS_TYPEMASK) == BS_AUTORADIOBUTTON &&
+                        SendMessageW( hwndNext, BM_GETCHECK, 0, 0 ) != BST_CHECKED)
+                        SendMessageW( hwndNext, BM_CLICK, 1, 0 );
+                }
+                else
+                    SendMessageW( hwndDlg, WM_NEXTDLGCTL, (WPARAM)hwndNext, 1 );
                 return TRUE;
             }
             break;
@@ -1253,10 +1266,11 @@ BOOL WINAPI IsDialogMessageW( HWND hwndDlg, LPMSG msg )
         case VK_RETURN:
             {
                 DWORD dw;
-                if ((GetFocus() == msg->hwnd) &&
-                    (SendMessageW (msg->hwnd, WM_GETDLGCODE, 0, 0) & DLGC_DEFPUSHBUTTON))
+                HWND hwndFocus = GetFocus();
+                if (IsChild( hwndDlg, hwndFocus ) &&
+                    (SendMessageW( hwndFocus, WM_GETDLGCODE, 0, 0 ) & DLGC_DEFPUSHBUTTON))
                 {
-                    SendMessageW (hwndDlg, WM_COMMAND, MAKEWPARAM (GetDlgCtrlID(msg->hwnd),BN_CLICKED), (LPARAM)msg->hwnd); 
+                    SendMessageW( hwndDlg, WM_COMMAND, MAKEWPARAM( GetDlgCtrlID( hwndFocus ), BN_CLICKED ), (LPARAM)hwndFocus );
                 }
                 else if (DC_HASDEFID == HIWORD(dw = SendMessageW (hwndDlg, DM_GETDEFID, 0, 0)))
                 {
@@ -1267,7 +1281,6 @@ BOOL WINAPI IsDialogMessageW( HWND hwndDlg, LPMSG msg )
                 else
                 {
                     SendMessageW( hwndDlg, WM_COMMAND, IDOK, (LPARAM)GetDlgItem( hwndDlg, IDOK ) );
-
                 }
             }
             return TRUE;
@@ -1506,22 +1519,22 @@ BOOL WINAPI CheckRadioButton( HWND hwndDlg, int firstID,
  */
 DWORD WINAPI GetDialogBaseUnits(void)
 {
-    static DWORD units;
+    static LONG cx, cy;
 
-    if (!units)
+    if (!cx)
     {
         HDC hdc;
-        SIZE size;
 
         if ((hdc = GetDC(0)))
         {
-            size.cx = GdiGetCharDimensions( hdc, NULL, &size.cy );
-            if (size.cx) units = MAKELONG( size.cx, size.cy );
-            ReleaseDC( 0, hdc );
+            cx = GdiGetCharDimensions( hdc, NULL, &cy );
+            NtUserReleaseDC( 0, hdc );
         }
-        TRACE("base units = %d,%d\n", LOWORD(units), HIWORD(units) );
+        TRACE( "base units = %d,%d\n", cx, cy );
     }
-    return units;
+
+    return MAKELONG( MulDiv( cx, GetDpiForSystem(), USER_DEFAULT_SCREEN_DPI ),
+                     MulDiv( cy, GetDpiForSystem(), USER_DEFAULT_SCREEN_DPI ));
 }
 
 
@@ -1583,9 +1596,9 @@ HWND WINAPI GetNextDlgGroupItem( HWND hwndDlg, HWND hwndCtrl, BOOL fPrevious )
      */
     retvalue = hwndCtrl;
     hwnd = hwndCtrl;
-    while (hwndNext = GetWindow (hwnd, GW_HWNDNEXT),
-           1)
+    while (1)
     {
+        hwndNext = GetWindow (hwnd, GW_HWNDNEXT);
         while (!hwndNext)
         {
             /* Climb out until there is a next sibling of the ancestor or we
@@ -1771,17 +1784,17 @@ static BOOL DIALOG_DlgDirSelect( HWND hwnd, LPWSTR str, INT len,
         }
         else
         {
-            buffer[strlenW(buffer)-1] = '\\';
+            buffer[lstrlenW(buffer)-1] = '\\';
             ptr = buffer + 1;
         }
     }
     else
     {
         /* Filenames without a dot extension must have one tacked at the end */
-        if (strchrW(buffer, '.') == NULL)
+        if (wcschr(buffer, '.') == NULL)
         {
-            buffer[strlenW(buffer)+1] = '\0';
-            buffer[strlenW(buffer)] = '.';
+            buffer[lstrlenW(buffer)+1] = '\0';
+            buffer[lstrlenW(buffer)] = '.';
         }
         ptr = buffer;
     }
@@ -1808,7 +1821,8 @@ static INT DIALOG_DlgDirListW( HWND hDlg, LPWSTR spec, INT idLBox,
 {
     HWND hwnd;
     LPWSTR orig_spec = spec;
-    WCHAR any[] = {'*','.','*',0};
+    WCHAR any[] = L"*.*";
+    WCHAR star[] = L"*";
 
 #define SENDMSG(msg,wparam,lparam) \
     ((attrib & DDL_POSTMSGS) ? PostMessageW( hwnd, msg, wparam, lparam ) \
@@ -1817,14 +1831,20 @@ static INT DIALOG_DlgDirListW( HWND hDlg, LPWSTR spec, INT idLBox,
     TRACE("%p %s %d %d %04x\n", hDlg, debugstr_w(spec), idLBox, idStatic, attrib );
 
     /* If the path exists and is a directory, chdir to it */
-    if (!spec || !spec[0] || SetCurrentDirectoryW( spec )) spec = any;
+    if (!spec || !spec[0] || SetCurrentDirectoryW( spec )) spec = star;
     else
     {
         WCHAR *p, *p2;
+
+        if (!wcschr(spec, '*') && !wcschr(spec, '?'))
+        {
+            SetLastError(ERROR_NO_WILDCARD_CHARACTERS);
+            return FALSE;
+        }
         p = spec;
-        if ((p2 = strchrW( p, ':' ))) p = p2 + 1;
-        if ((p2 = strrchrW( p, '\\' ))) p = p2;
-        if ((p2 = strrchrW( p, '/' ))) p = p2;
+        if ((p2 = wcschr( p, ':' ))) p = p2 + 1;
+        if ((p2 = wcsrchr( p, '\\' ))) p = p2;
+        if ((p2 = wcsrchr( p, '/' ))) p = p2;
         if (p != spec)
         {
             WCHAR sep = *p;
@@ -1869,7 +1889,7 @@ static INT DIALOG_DlgDirListW( HWND hDlg, LPWSTR spec, INT idLBox,
     if (idStatic && ((hwnd = GetDlgItem( hDlg, idStatic )) != 0))
     {
         WCHAR temp[MAX_PATH];
-        GetCurrentDirectoryW( sizeof(temp)/sizeof(WCHAR), temp );
+        GetCurrentDirectoryW( ARRAY_SIZE( temp ), temp );
         CharLowerW( temp );
         /* Can't use PostMessage() here, because the string is on the stack */
         SetDlgItemTextW( hDlg, idStatic, temp );
