@@ -4,6 +4,7 @@
  * Copyright 1995 Thomas Sandford
  * Copyright 1997 Marcus Meissner
  * Copyright 1998 Turchanov Sergey
+ * Copyright 2019 Micah N Gorrell for CodeWeavers
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -20,25 +21,22 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  */
 
-#include "config.h"
-
 #include <stdarg.h>
 
 #include "windef.h"
 #include "wine/windef16.h"
 #include "winbase.h"
 #include "wingdi.h"
-#include "winuser.h"
-#include "winnls.h"
-#include "winternl.h"
 #include "controls.h"
+#include "imm.h"
 #include "user_private.h"
 
-#include "wine/unicode.h"
 #include "wine/debug.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(win);
 
+BOOL WINAPI ImmSetActiveContext(HWND, HIMC, BOOL);
+
 #define IMM_INIT_MAGIC 0x19650412
 static HWND (WINAPI *imm_get_ui_window)(HKL);
 BOOL (WINAPI *imm_register_window)(HWND) = NULL;
@@ -243,157 +241,6 @@ DWORD WINAPI SetLogonNotifyWindow(HWINSTA hwinsta,HWND hwnd)
     return 1;
 }
 
-/* CrossOver Hack 13441 */
-static const WCHAR adapter_device_string[] = {'W','i','n','e',' ','D','i','s','p','l','a','y',' ',
-                                              'A','d','a','p','t','e','r',0};
-static const WCHAR adapter_device_deviceid[] = {'P','C','I','\\','V','E','N','_','0','0','0','0','&',
-                                                'D','E','V','_','0','0','0','0',0};
-static const WCHAR display_device_name[] = {'%','s','\\','M','o','n','i','t','o','r','0',0};
-static const WCHAR display_device_string[] = {'W','i','n','e',' ','D','i','s','p','l','a','y',0};
-static const WCHAR display_device_deviceid[] = {'M','O','N','I','T','O','R','\\','W','I','N','E','%','0','4','d',0};
-
-struct display_devices_enum_info
-{
-    LPCWSTR adapter;
-    DWORD target;
-    DWORD non_primary_seen;
-    LPDISPLAY_DEVICEW device;
-};
-
-/***********************************************************************
- *		display_devices_enum
- *
- * Helper callback for EnumDisplayDevicesW()
- */
-static BOOL CALLBACK display_devices_enum( HMONITOR monitor, HDC hdc, LPRECT rect, LPARAM lp )
-{
-    struct display_devices_enum_info *info = (struct display_devices_enum_info *)lp;
-    MONITORINFOEXW mon_info;
-    BOOL match;
-
-    mon_info.cbSize = sizeof(mon_info);
-    GetMonitorInfoW( monitor, (MONITORINFO*)&mon_info );
-
-    if (!(mon_info.dwFlags & MONITORINFOF_PRIMARY))
-        info->non_primary_seen++;
-
-    if (info->adapter)
-    {
-        match = !strcmpiW( info->adapter, mon_info.szDevice );
-        if (match)
-        {
-            snprintfW( info->device->DeviceName, sizeof(info->device->DeviceName) / sizeof(WCHAR),
-                       display_device_name, mon_info.szDevice );
-            lstrcpynW( info->device->DeviceString, display_device_string, sizeof(info->device->DeviceString) / sizeof(WCHAR) );
-
-            if (info->device->cb >= offsetof(DISPLAY_DEVICEW, DeviceID) + sizeof(info->device->DeviceID))
-            {
-                snprintfW( info->device->DeviceID, sizeof(info->device->DeviceID) / sizeof(WCHAR),
-                           display_device_deviceid, (mon_info.dwFlags & MONITORINFOF_PRIMARY) ? 0 : info->non_primary_seen );
-            }
-        }
-    }
-    else
-    {
-        if (mon_info.dwFlags & MONITORINFOF_PRIMARY)
-            match = (info->target == 0);
-        else
-            match = (info->target == info->non_primary_seen);
-
-        if (match)
-        {
-            lstrcpynW( info->device->DeviceName, mon_info.szDevice, sizeof(info->device->DeviceName) / sizeof(WCHAR) );
-            lstrcpynW( info->device->DeviceString, adapter_device_string, sizeof(info->device->DeviceString) / sizeof(WCHAR) );
-
-            if (info->device->cb >= offsetof(DISPLAY_DEVICEW, DeviceID) + sizeof(info->device->DeviceID))
-                lstrcpynW( info->device->DeviceID, adapter_device_deviceid, sizeof(info->device->DeviceID) / sizeof(WCHAR) );
-        }
-    }
-
-    return !match;
-}
-
-/***********************************************************************
- *		EnumDisplayDevicesA (USER32.@)
- */
-BOOL WINAPI EnumDisplayDevicesA( LPCSTR lpDevice, DWORD i, LPDISPLAY_DEVICEA lpDispDev,
-                                 DWORD dwFlags )
-{
-    UNICODE_STRING deviceW;
-    DISPLAY_DEVICEW ddW;
-    BOOL ret;
-
-    if(lpDevice)
-        RtlCreateUnicodeStringFromAsciiz(&deviceW, lpDevice); 
-    else
-        deviceW.Buffer = NULL;
-
-    ddW.cb = sizeof(ddW);
-    ret = EnumDisplayDevicesW(deviceW.Buffer, i, &ddW, dwFlags);
-    RtlFreeUnicodeString(&deviceW);
-
-    if(!ret) return ret;
-
-    WideCharToMultiByte(CP_ACP, 0, ddW.DeviceName, -1, lpDispDev->DeviceName, sizeof(lpDispDev->DeviceName), NULL, NULL);
-    WideCharToMultiByte(CP_ACP, 0, ddW.DeviceString, -1, lpDispDev->DeviceString, sizeof(lpDispDev->DeviceString), NULL, NULL);
-    lpDispDev->StateFlags = ddW.StateFlags;
-
-    if(lpDispDev->cb >= offsetof(DISPLAY_DEVICEA, DeviceID) + sizeof(lpDispDev->DeviceID))
-        WideCharToMultiByte(CP_ACP, 0, ddW.DeviceID, -1, lpDispDev->DeviceID, sizeof(lpDispDev->DeviceID), NULL, NULL);
-    if(lpDispDev->cb >= offsetof(DISPLAY_DEVICEA, DeviceKey) + sizeof(lpDispDev->DeviceKey))
-        WideCharToMultiByte(CP_ACP, 0, ddW.DeviceKey, -1, lpDispDev->DeviceKey, sizeof(lpDispDev->DeviceKey), NULL, NULL);
-
-    return TRUE;
-}
-
-/***********************************************************************
- *		EnumDisplayDevicesW (USER32.@)
- */
-BOOL WINAPI EnumDisplayDevicesW( LPCWSTR lpDevice, DWORD i, LPDISPLAY_DEVICEW lpDisplayDevice,
-                                 DWORD dwFlags )
-{
-    /* CrossOver Hack 13441 */
-    struct display_devices_enum_info info;
-
-    TRACE("(%s,%d,%p,0x%08x)\n",debugstr_w(lpDevice),i,lpDisplayDevice,dwFlags);
-
-    if (lpDevice && i)
-        return FALSE;
-
-    lpDisplayDevice->StateFlags =
-        DISPLAY_DEVICE_ATTACHED_TO_DESKTOP |
-        DISPLAY_DEVICE_VGA_COMPATIBLE;
-
-    if (!lpDevice && i == 0)
-        lpDisplayDevice->StateFlags |= DISPLAY_DEVICE_PRIMARY_DEVICE;
-
-    info.adapter = lpDevice;
-    info.target = i;
-    info.non_primary_seen = 0;
-    info.device = lpDisplayDevice;
-    if (EnumDisplayMonitors( 0, NULL, display_devices_enum, (LPARAM)&info ))
-        return FALSE;
-
-    if(lpDisplayDevice->cb >= offsetof(DISPLAY_DEVICEW, DeviceKey) + sizeof(lpDisplayDevice->DeviceKey))
-        lpDisplayDevice->DeviceKey[0] = 0;
-
-    TRACE("DeviceName %s DeviceString %s DeviceID %s DeviceKey %s\n", debugstr_w(lpDisplayDevice->DeviceName),
-          debugstr_w(lpDisplayDevice->DeviceString), debugstr_w(lpDisplayDevice->DeviceID), debugstr_w(lpDisplayDevice->DeviceKey));
-
-    return TRUE;
-}
-
-/***********************************************************************
- *              QueryDisplayConfig (USER32.@)
- */
-LONG WINAPI QueryDisplayConfig(UINT32 flags, UINT32 *numpathelements, DISPLAYCONFIG_PATH_INFO *pathinfo,
-                               UINT32 *numinfoelements, DISPLAYCONFIG_MODE_INFO *modeinfo,
-                               DISPLAYCONFIG_TOPOLOGY_ID *topologyid)
-{
-   FIXME("(%08x %p %p %p %p %p)\n", flags, numpathelements, pathinfo, numinfoelements, modeinfo, topologyid);
-   return ERROR_CALL_NOT_IMPLEMENTED;
-}
-
 /***********************************************************************
  *		RegisterSystemThread (USER32.@)
  */
@@ -431,59 +278,6 @@ DWORD WINAPI RegisterTasklist (DWORD x)
     return TRUE;
 }
 
-
-/***********************************************************************
- *		RegisterDeviceNotificationA (USER32.@)
- *
- * See RegisterDeviceNotificationW.
- */
-HDEVNOTIFY WINAPI RegisterDeviceNotificationA(HANDLE hnd, LPVOID notifyfilter, DWORD flags)
-{
-    FIXME("(hwnd=%p, filter=%p,flags=0x%08x) returns a fake device notification handle!\n",
-          hnd,notifyfilter,flags );
-    return (HDEVNOTIFY) 0xcafecafe;
-}
-
-/***********************************************************************
- *		RegisterDeviceNotificationW (USER32.@)
- *
- * Registers a window with the system so that it will receive
- * notifications about a device.
- *
- * PARAMS
- *     hRecipient           [I] Window or service status handle that
- *                              will receive notifications.
- *     pNotificationFilter  [I] DEV_BROADCAST_HDR followed by some
- *                              type-specific data.
- *     dwFlags              [I] See notes
- *
- * RETURNS
- *
- * A handle to the device notification.
- *
- * NOTES
- *
- * The dwFlags parameter can be one of two values:
- *| DEVICE_NOTIFY_WINDOW_HANDLE  - hRecipient is a window handle
- *| DEVICE_NOTIFY_SERVICE_HANDLE - hRecipient is a service status handle
- */
-HDEVNOTIFY WINAPI RegisterDeviceNotificationW(HANDLE hRecipient, LPVOID pNotificationFilter, DWORD dwFlags)
-{
-    FIXME("(hwnd=%p, filter=%p,flags=0x%08x) returns a fake device notification handle!\n",
-          hRecipient,pNotificationFilter,dwFlags );
-    return (HDEVNOTIFY) 0xcafeaffe;
-}
-
-/***********************************************************************
- *		UnregisterDeviceNotification (USER32.@)
- *
- */
-BOOL  WINAPI UnregisterDeviceNotification(HDEVNOTIFY hnd)
-{
-    FIXME("(handle=%p), STUB!\n", hnd);
-    return TRUE;
-}
-
 /***********************************************************************
  *           GetAppCompatFlags   (USER32.@)
  */
@@ -531,8 +325,7 @@ VOID WINAPI LoadLocalFonts(VOID)
  */
 BOOL WINAPI User32InitializeImmEntryTable(DWORD magic)
 {
-    static const WCHAR imm32_dllW[] = {'i','m','m','3','2','.','d','l','l',0};
-    HMODULE imm32 = GetModuleHandleW(imm32_dllW);
+    HMODULE imm32 = GetModuleHandleW(L"imm32.dll");
 
     TRACE("(%x)\n", magic);
 
@@ -650,6 +443,24 @@ BOOL WINAPI UnregisterPowerSettingNotification(HPOWERNOTIFY handle)
     return TRUE;
 }
 
+/**********************************************************************
+ * RegisterSuspendResumeNotification (USER32.@)
+ */
+HPOWERNOTIFY WINAPI RegisterSuspendResumeNotification(HANDLE recipient, DWORD flags)
+{
+    FIXME("%p, %#x: stub.\n", recipient, flags);
+    return (HPOWERNOTIFY)0xdeadbeef;
+}
+
+/**********************************************************************
+ * UnregisterSuspendResumeNotification (USER32.@)
+ */
+BOOL WINAPI UnregisterSuspendResumeNotification(HPOWERNOTIFY handle)
+{
+    FIXME("%p: stub.\n", handle);
+    return TRUE;
+}
+
 /*****************************************************************************
  * GetGestureConfig (USER32.@)
  */
@@ -688,21 +499,6 @@ BOOL WINAPI IsWindowRedirectedForPrint( HWND hwnd )
     return FALSE;
 }
 
-/**********************************************************************
- * GetDisplayConfigBufferSizes [USER32.@]
- */
-LONG WINAPI GetDisplayConfigBufferSizes(UINT32 flags, UINT32 *num_path_info, UINT32 *num_mode_info)
-{
-    FIXME("(0x%x %p %p): stub\n", flags, num_path_info, num_mode_info);
-
-    if (!num_path_info || !num_mode_info)
-        return ERROR_INVALID_PARAMETER;
-
-    *num_path_info = 0;
-    *num_mode_info = 0;
-    return ERROR_NOT_SUPPORTED;
-}
-
 /**********************************************************************
  * RegisterPointerDeviceNotifications [USER32.@]
  */
@@ -738,11 +534,26 @@ BOOL WINAPI RegisterTouchHitTestingWindow(HWND hwnd, ULONG value)
     return TRUE;
 }
 
+/**********************************************************************
+ * GetPointerType [USER32.@]
+ */
+BOOL WINAPI GetPointerType(UINT32 id, POINTER_INPUT_TYPE *type)
+{
+    FIXME("(%d %p): stub\n", id, type);
+
+    if(!id || !type)
+    {
+        SetLastError(ERROR_INVALID_PARAMETER);
+        return FALSE;
+    }
+
+    *type = PT_MOUSE;
+    return TRUE;
+}
 
-static const WCHAR imeW[] = {'I','M','E',0};
 const struct builtin_class_descr IME_builtin_class =
 {
-    imeW,               /* name */
+    L"IME",             /* name */
     0,                  /* style  */
     WINPROC_IME,        /* proc */
     2*sizeof(LONG_PTR), /* extra */
@@ -780,6 +591,27 @@ static BOOL is_ime_ui_msg( UINT msg )
     }
 }
 
+static LRESULT ime_internal_msg( WPARAM wParam, LPARAM lParam)
+{
+    HWND hwnd = (HWND)lParam;
+    HIMC himc;
+
+    switch(wParam)
+    {
+    case IME_INTERNAL_ACTIVATE:
+    case IME_INTERNAL_DEACTIVATE:
+        himc = ImmGetContext(hwnd);
+        ImmSetActiveContext(hwnd, himc, wParam == IME_INTERNAL_ACTIVATE);
+        ImmReleaseContext(hwnd, himc);
+        break;
+    default:
+        FIXME("wParam = %lx\n", wParam);
+        break;
+    }
+
+    return 0;
+}
+
 LRESULT WINAPI ImeWndProcA( HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam )
 {
     HWND uiwnd;
@@ -787,9 +619,12 @@ LRESULT WINAPI ImeWndProcA( HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam )
     if (msg==WM_CREATE)
         return TRUE;
 
+    if (msg==WM_IME_INTERNAL)
+        return ime_internal_msg(wParam, lParam);
+
     if (imm_get_ui_window && is_ime_ui_msg(msg))
     {
-        if ((uiwnd = imm_get_ui_window(GetKeyboardLayout(0))))
+        if ((uiwnd = imm_get_ui_window( NtUserGetKeyboardLayout(0) )))
             return SendMessageA(uiwnd, msg, wParam, lParam);
         return FALSE;
     }
@@ -804,9 +639,12 @@ LRESULT WINAPI ImeWndProcW( HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam )
     if (msg==WM_CREATE)
         return TRUE;
 
+    if (msg==WM_IME_INTERNAL)
+        return ime_internal_msg(wParam, lParam);
+
     if (imm_get_ui_window && is_ime_ui_msg(msg))
     {
-        if ((uiwnd = imm_get_ui_window(GetKeyboardLayout(0))))
+        if ((uiwnd = imm_get_ui_window( NtUserGetKeyboardLayout(0) )))
             return SendMessageW(uiwnd, msg, wParam, lParam);
         return FALSE;
     }
