@@ -18,8 +18,6 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  */
 
-#include "config.h"
-
 #include <stdarg.h>
 #include <stdlib.h>
 
@@ -32,9 +30,6 @@
 #include "controls.h"
 #include "wine/debug.h"
 
-/* For CrossOver Hack #15440 */
-#include "wine/unicode.h"
-
 WINE_DEFAULT_DEBUG_CHANNEL(nonclient);
 
 #define SC_ABOUTWINE            (SC_SCREENSAVE+1)
@@ -249,8 +244,8 @@ BOOL WINAPI DrawCaptionTempW (HWND hwnd, HDC hdc, const RECT *rect, HFONT hFont,
         pt.y = (rc.bottom + rc.top - GetSystemMetrics(SM_CYSMICON)) / 2;
 
         if (!hIcon) hIcon = NC_IconForWindow(hwnd);
-        DrawIconEx (hdc, pt.x, pt.y, hIcon, GetSystemMetrics(SM_CXSMICON),
-                    GetSystemMetrics(SM_CYSMICON), 0, 0, DI_NORMAL);
+        NtUserDrawIconEx( hdc, pt.x, pt.y, hIcon, GetSystemMetrics(SM_CXSMICON),
+                          GetSystemMetrics(SM_CYSMICON), 0, 0, DI_NORMAL );
         rc.left = pt.x + GetSystemMetrics( SM_CXSMICON );
     }
 
@@ -319,8 +314,6 @@ BOOL WINAPI DECLSPEC_HOTPATCH AdjustWindowRectEx( LPRECT rect, DWORD style, BOOL
 {
     NONCLIENTMETRICSW ncm;
 
-    if (style & WS_MINIMIZE) return TRUE;
-
     TRACE("(%s) %08x %d %08x\n", wine_dbgstr_rect(rect), style, menu, exStyle );
 
     ncm.cbSize = sizeof(ncm);
@@ -339,8 +332,6 @@ BOOL WINAPI DECLSPEC_HOTPATCH AdjustWindowRectExForDpi( LPRECT rect, DWORD style
 {
     NONCLIENTMETRICSW ncm;
 
-    if (style & WS_MINIMIZE) return TRUE;
-
     TRACE("(%s) %08x %d %08x %u\n", wine_dbgstr_rect(rect), style, menu, exStyle, dpi );
 
     ncm.cbSize = sizeof(ncm);
@@ -356,19 +347,14 @@ BOOL WINAPI DECLSPEC_HOTPATCH AdjustWindowRectExForDpi( LPRECT rect, DWORD style
  *
  * Handle a WM_NCCALCSIZE message. Called from DefWindowProc().
  */
-LRESULT NC_HandleNCCalcSize( HWND hwnd, WPARAM wparam, RECT *winRect )
+void NC_HandleNCCalcSize( HWND hwnd, WPARAM wparam, RECT *winRect )
 {
     RECT tmpRect = { 0, 0, 0, 0 };
-    LRESULT result = 0;
-    LONG cls_style = GetClassLongW(hwnd, GCL_STYLE);
     LONG style = GetWindowLongW( hwnd, GWL_STYLE );
     LONG exStyle = GetWindowLongW( hwnd, GWL_EXSTYLE );
 
     if (winRect == NULL)
-        return 0;
-
-    if (cls_style & CS_VREDRAW) result |= WVR_VREDRAW;
-    if (cls_style & CS_HREDRAW) result |= WVR_HREDRAW;
+        return;
 
     if (!(style & WS_MINIMIZE))
     {
@@ -418,37 +404,11 @@ LRESULT NC_HandleNCCalcSize( HWND hwnd, WPARAM wparam, RECT *winRect )
         if (winRect->left > winRect->right)
             winRect->right = winRect->left;
     }
-    else /* CrossOver Hack #15440.  Powerpnt 2010 requires that
-            minimized windows have an empty client rect, which is the
-            case with win95-style minimization.  Wine however uses
-            win31-style so this will break things, however it seems ok
-            for Powerpnt. */
+    else
     {
-        static int is_ppt = -1;
-        WCHAR class_name[80];
-        static const WCHAR PPTFrameClass[] = {'P','P','T','F','r','a','m','e','C','l','a','s','s',0};
-
-        if (is_ppt == -1)
-        {
-            WCHAR name[MAX_PATH], *p;
-            static const WCHAR POWERPNT[] = {'P','O','W','E','R','P','N','T','.','E','X','E',0};
-
-            GetModuleFileNameW( NULL, name, sizeof(name) / sizeof(name[0]) );
-            p = strrchrW( name, '\\' );
-            if (!p) p = name;
-            else p++;
-            is_ppt = !strcmpiW( p, POWERPNT );
-        }
-
-        if (is_ppt && GetClassNameW( hwnd, class_name, sizeof(class_name) / sizeof(class_name[0]) ) &&
-            !strcmpiW( class_name, PPTFrameClass ))
-        {
-            winRect->right = winRect->left;
-            winRect->bottom = winRect->top;
-        }
+        winRect->right = winRect->left;
+        winRect->bottom = winRect->top;
     }
-
-    return result;
 }
 
 
@@ -463,8 +423,6 @@ static void NC_GetInsideRect( HWND hwnd, enum coords_relative relative, RECT *re
 {
     WIN_GetRectangles( hwnd, relative, rect, NULL );
 
-    if (style & WS_MINIMIZE) return;
-
     /* Remove frame from rectangle */
     if (HAS_THICKFRAME( style, ex_style ))
     {
@@ -508,7 +466,6 @@ LRESULT NC_HandleNCHitTest( HWND hwnd, POINT pt )
 
     style = GetWindowLongW( hwnd, GWL_STYLE );
     ex_style = GetWindowLongW( hwnd, GWL_EXSTYLE );
-    if (style & WS_MINIMIZE) return HTCAPTION;
 
     if (PtInRect( &rcClient, pt )) return HTCLIENT;
 
@@ -673,6 +630,41 @@ LRESULT NC_HandleNCHitTest( HWND hwnd, POINT pt )
     return HTNOWHERE;
 }
 
+LRESULT NC_HandleNCMouseMove(HWND hwnd, WPARAM wParam, LPARAM lParam)
+{
+    RECT rect;
+    POINT pt;
+
+    TRACE("hwnd=%p wparam=%#lx lparam=%#lx\n", hwnd, wParam, lParam);
+
+    if (wParam != HTHSCROLL && wParam != HTVSCROLL)
+        return 0;
+
+    WIN_GetRectangles(hwnd, COORDS_CLIENT, &rect, NULL);
+
+    pt.x = (short)LOWORD(lParam);
+    pt.y = (short)HIWORD(lParam);
+    ScreenToClient(hwnd, &pt);
+    pt.x -= rect.left;
+    pt.y -= rect.top;
+    SCROLL_HandleScrollEvent(hwnd, wParam == HTHSCROLL ? SB_HORZ : SB_VERT, WM_NCMOUSEMOVE, pt);
+    return 0;
+}
+
+LRESULT NC_HandleNCMouseLeave(HWND hwnd)
+{
+    LONG style = GetWindowLongW(hwnd, GWL_STYLE);
+    POINT pt = {0, 0};
+
+    TRACE("hwnd=%p\n", hwnd);
+
+    if (style & WS_HSCROLL)
+        SCROLL_HandleScrollEvent(hwnd, SB_HORZ, WM_NCMOUSELEAVE, pt);
+    if (style & WS_VSCROLL)
+        SCROLL_HandleScrollEvent(hwnd, SB_VERT, WM_NCMOUSELEAVE, pt);
+
+    return 0;
+}
 
 /******************************************************************************
  *
@@ -695,9 +687,9 @@ BOOL NC_DrawSysButton (HWND hwnd, HDC hdc, BOOL down)
         NC_GetInsideRect( hwnd, COORDS_WINDOW, &rect, style, ex_style );
         pt.x = rect.left + 2;
         pt.y = rect.top + (GetSystemMetrics(SM_CYCAPTION) - GetSystemMetrics(SM_CYSMICON)) / 2;
-        DrawIconEx (hdc, pt.x, pt.y, hIcon,
-                    GetSystemMetrics(SM_CXSMICON),
-                    GetSystemMetrics(SM_CYSMICON), 0, 0, DI_NORMAL);
+        NtUserDrawIconEx( hdc, pt.x, pt.y, hIcon,
+                          GetSystemMetrics(SM_CXSMICON),
+                          GetSystemMetrics(SM_CYSMICON), 0, 0, DI_NORMAL );
     }
     return (hIcon != 0);
 }
@@ -785,13 +777,15 @@ static void NC_DrawMaxButton(HWND hwnd,HDC hdc,BOOL down, BOOL bGrayed)
 static void  NC_DrawMinButton(HWND hwnd,HDC hdc,BOOL down, BOOL bGrayed)
 {
     RECT rect;
-    UINT flags = DFCS_CAPTIONMIN;
+    UINT flags;
     DWORD style = GetWindowLongW( hwnd, GWL_STYLE );
     DWORD ex_style = GetWindowLongW( hwnd, GWL_EXSTYLE );
 
     /* never draw minimize box when window has WS_EX_TOOLWINDOW style */
     if (ex_style & WS_EX_TOOLWINDOW) return;
 
+    flags = (style & WS_MINIMIZE) ? DFCS_CAPTIONRESTORE : DFCS_CAPTIONMIN;
+
     NC_GetInsideRect( hwnd, COORDS_WINDOW, &rect, style, ex_style );
     if (style & WS_SYSMENU)
         rect.right -= GetSystemMetrics(SM_CXSIZE);
@@ -988,9 +982,6 @@ static void  NC_DoNCPaint( HWND  hwnd, HRGN  clip )
     flags = wndPtr->flags;
     WIN_ReleasePtr( wndPtr );
 
-    if ( dwStyle & WS_MINIMIZE ||
-         !WIN_IsWindowDrawable( hwnd, 0 )) return; /* Nothing to do */
-
     active  = flags & WIN_NCACTIVATED;
 
     TRACE("%p %d\n", hwnd, active );
@@ -1008,14 +999,18 @@ static void  NC_DoNCPaint( HWND  hwnd, HRGN  clip )
     if (clip > (HRGN)1)
     {
         CombineRgn( hrgn, clip, hrgn, RGN_DIFF );
-        hdc = GetDCEx( hwnd, hrgn, DCX_USESTYLE | DCX_WINDOW | DCX_INTERSECTRGN );
+        hdc = NtUserGetDCEx( hwnd, hrgn, DCX_USESTYLE | DCX_WINDOW | DCX_INTERSECTRGN );
     }
     else
     {
-        hdc = GetDCEx( hwnd, hrgn, DCX_USESTYLE | DCX_WINDOW | DCX_EXCLUDERGN );
+        hdc = NtUserGetDCEx( hwnd, hrgn, DCX_USESTYLE | DCX_WINDOW | DCX_EXCLUDERGN );
     }
 
-    if (!hdc) return;
+    if (!hdc)
+    {
+        DeleteObject( hrgn );
+        return;
+    }
 
     WIN_GetRectangles( hwnd, COORDS_WINDOW, &rect, NULL );
     GetClipBox( hdc, &rectClip );
@@ -1062,11 +1057,7 @@ static void  NC_DoNCPaint( HWND  hwnd, HRGN  clip )
 	DrawEdge (hdc, &rect, EDGE_SUNKEN, BF_RECT | BF_ADJUST);
 
     /* Draw the scroll-bars */
-
-    if (dwStyle & WS_VSCROLL)
-        SCROLL_DrawScrollBar( hwnd, hdc, SB_VERT, TRUE, TRUE );
-    if (dwStyle & WS_HSCROLL)
-        SCROLL_DrawScrollBar( hwnd, hdc, SB_HORZ, TRUE, TRUE );
+    SCROLL_DrawNCScrollBar( hwnd, hdc, dwStyle & WS_HSCROLL, dwStyle & WS_VSCROLL );
 
     /* Draw the "size-box" */
     if ((dwStyle & WS_VSCROLL) && (dwStyle & WS_HSCROLL))
@@ -1077,10 +1068,10 @@ static void  NC_DoNCPaint( HWND  hwnd, HRGN  clip )
         else
             r.left = r.right - GetSystemMetrics(SM_CXVSCROLL) + 1;
         r.top  = r.bottom - GetSystemMetrics(SM_CYHSCROLL) + 1;
-        FillRect( hdc, &r,  GetSysColorBrush(COLOR_SCROLLBAR) );
+        FillRect( hdc, &r, GetSysColorBrush( COLOR_BTNFACE ) );
     }
 
-    ReleaseDC( hwnd, hdc );
+    NtUserReleaseDC( hwnd, hdc );
 }
 
 
@@ -1093,15 +1084,12 @@ static void  NC_DoNCPaint( HWND  hwnd, HRGN  clip )
  */
 LRESULT NC_HandleNCPaint( HWND hwnd , HRGN clip)
 {
-    HWND parent = GetAncestor( hwnd, GA_PARENT );
+    HWND parent = NtUserGetAncestor( hwnd, GA_PARENT );
     DWORD dwStyle = GetWindowLongW( hwnd, GWL_STYLE );
 
     if( dwStyle & WS_VISIBLE )
     {
-	if( dwStyle & WS_MINIMIZE )
-	    WINPOS_RedrawIconTitle( hwnd );
-	else
-	    NC_DoNCPaint( hwnd, clip );
+        NC_DoNCPaint( hwnd, clip );
 
         if (parent == GetDesktopWindow())
             PostMessageW( parent, WM_PARENTNOTIFY, WM_NCPAINT, (LPARAM)hwnd );
@@ -1130,12 +1118,9 @@ LRESULT NC_HandleNCActivate( HWND hwnd, WPARAM wParam, LPARAM lParam )
      */
     if (lParam != -1)
     {
-        if (IsIconic(hwnd))
-            WINPOS_RedrawIconTitle( hwnd );
-        else
-            NC_DoNCPaint( hwnd, (HRGN)1 );
+        NC_DoNCPaint( hwnd, (HRGN)1 );
 
-        if (GetAncestor( hwnd, GA_PARENT ) == GetDesktopWindow())
+        if (NtUserGetAncestor( hwnd, GA_PARENT ) == GetDesktopWindow())
             PostMessageW( GetDesktopWindow(), WM_PARENTNOTIFY, WM_NCACTIVATE, (LPARAM)hwnd );
     }
 
@@ -1167,7 +1152,7 @@ LRESULT NC_HandleSetCursor( HWND hwnd, WPARAM wParam, LPARAM lParam )
         {
             HCURSOR hCursor = (HCURSOR)GetClassLongPtrW(hwnd, GCLP_HCURSOR);
             if(hCursor) {
-                SetCursor(hCursor);
+                NtUserSetCursor(hCursor);
                 return TRUE;
             }
             return FALSE;
@@ -1175,23 +1160,23 @@ LRESULT NC_HandleSetCursor( HWND hwnd, WPARAM wParam, LPARAM lParam )
 
     case HTLEFT:
     case HTRIGHT:
-        return (LRESULT)SetCursor( LoadCursorA( 0, (LPSTR)IDC_SIZEWE ) );
+        return (LRESULT)NtUserSetCursor( LoadCursorA( 0, (LPSTR)IDC_SIZEWE ) );
 
     case HTTOP:
     case HTBOTTOM:
-        return (LRESULT)SetCursor( LoadCursorA( 0, (LPSTR)IDC_SIZENS ) );
+        return (LRESULT)NtUserSetCursor( LoadCursorA( 0, (LPSTR)IDC_SIZENS ) );
 
     case HTTOPLEFT:
     case HTBOTTOMRIGHT:
-        return (LRESULT)SetCursor( LoadCursorA( 0, (LPSTR)IDC_SIZENWSE ) );
+        return (LRESULT)NtUserSetCursor( LoadCursorA( 0, (LPSTR)IDC_SIZENWSE ) );
 
     case HTTOPRIGHT:
     case HTBOTTOMLEFT:
-        return (LRESULT)SetCursor( LoadCursorA( 0, (LPSTR)IDC_SIZENESW ) );
+        return (LRESULT)NtUserSetCursor( LoadCursorA( 0, (LPSTR)IDC_SIZENESW ) );
     }
 
     /* Default cursor: arrow */
-    return (LRESULT)SetCursor( LoadCursorA( 0, (LPSTR)IDC_ARROW ) );
+    return (LRESULT)NtUserSetCursor( LoadCursorA( 0, (LPSTR)IDC_ARROW ) );
 }
 
 /***********************************************************************
@@ -1256,7 +1241,7 @@ static void NC_TrackMinMaxBox( HWND hwnd, WORD wParam )
         paintButton = NC_DrawMaxButton;
     }
 
-    SetCapture( hwnd );
+    NtUserSetCapture( hwnd );
 
     (*paintButton)( hwnd, hdc, TRUE, FALSE);
 
@@ -1282,7 +1267,7 @@ static void NC_TrackMinMaxBox( HWND hwnd, WORD wParam )
         (*paintButton)(hwnd, hdc, FALSE, FALSE);
 
     ReleaseCapture();
-    ReleaseDC( hwnd, hdc );
+    NtUserReleaseDC( hwnd, hdc );
 
     /* If the minimize or maximize items of the sysmenu are not there */
     /* or if the style is not present, do nothing */
@@ -1290,7 +1275,8 @@ static void NC_TrackMinMaxBox( HWND hwnd, WORD wParam )
         return;
 
     if (wParam == HTMINBUTTON)
-        SendMessageW( hwnd, WM_SYSCOMMAND, SC_MINIMIZE, MAKELONG(msg.pt.x,msg.pt.y) );
+        SendMessageW( hwnd, WM_SYSCOMMAND,
+                      IsIconic(hwnd) ? SC_RESTORE : SC_MINIMIZE, MAKELONG(msg.pt.x,msg.pt.y) );
     else
         SendMessageW( hwnd, WM_SYSCOMMAND,
                       IsZoomed(hwnd) ? SC_RESTORE:SC_MAXIMIZE, MAKELONG(msg.pt.x,msg.pt.y) );
@@ -1320,7 +1306,7 @@ static void NC_TrackCloseButton (HWND hwnd, WPARAM wParam, LPARAM lParam)
 
     hdc = GetWindowDC( hwnd );
 
-    SetCapture( hwnd );
+    NtUserSetCapture( hwnd );
 
     NC_DrawCloseButton (hwnd, hdc, TRUE, FALSE);
 
@@ -1346,7 +1332,7 @@ static void NC_TrackCloseButton (HWND hwnd, WPARAM wParam, LPARAM lParam)
         NC_DrawCloseButton (hwnd, hdc, FALSE, FALSE);
 
     ReleaseCapture();
-    ReleaseDC( hwnd, hdc );
+    NtUserReleaseDC( hwnd, hdc );
     if (!pressed) return;
 
     SendMessageW( hwnd, WM_SYSCOMMAND, SC_CLOSE, lParam );
@@ -1394,7 +1380,7 @@ LRESULT NC_HandleNCLButtonDown( HWND hwnd, WPARAM wParam, LPARAM lParam )
             {
                 if ((GetWindowLongW( top, GWL_STYLE ) & (WS_POPUP|WS_CHILD)) != WS_CHILD)
                     break;
-                parent = GetAncestor( top, GA_PARENT );
+                parent = NtUserGetAncestor( top, GA_PARENT );
                 if (!parent || parent == GetDesktopWindow()) break;
                 top = parent;
             }
@@ -1405,17 +1391,14 @@ LRESULT NC_HandleNCLButtonDown( HWND hwnd, WPARAM wParam, LPARAM lParam )
         }
 
     case HTSYSMENU:
-         if( style & WS_SYSMENU )
-         {
-             if( !(style & WS_MINIMIZE) )
-             {
-                HDC hDC = GetWindowDC(hwnd);
-                NC_DrawSysButton( hwnd, hDC, TRUE );
-                ReleaseDC( hwnd, hDC );
-             }
-             SendMessageW( hwnd, WM_SYSCOMMAND, SC_MOUSEMENU + HTSYSMENU, lParam );
-         }
-         break;
+        if (style & WS_SYSMENU)
+        {
+            HDC hDC = GetWindowDC( hwnd );
+            NC_DrawSysButton( hwnd, hDC, TRUE );
+            NtUserReleaseDC( hwnd, hDC );
+            SendMessageW( hwnd, WM_SYSCOMMAND, SC_MOUSEMENU + HTSYSMENU, lParam );
+        }
+        break;
 
     case HTMENU:
         SendMessageW( hwnd, WM_SYSCOMMAND, SC_MOUSEMENU, lParam );
@@ -1478,7 +1461,7 @@ LRESULT NC_HandleNCRButtonDown( HWND hwnd, WPARAM wParam, LPARAM lParam )
     {
     case HTCAPTION:
     case HTSYSMENU:
-        SetCapture( hwnd );
+        NtUserSetCapture( hwnd );
         for (;;)
         {
             if (!GetMessageW( &msg, 0, WM_MOUSEFIRST, WM_MOUSELAST )) break;
@@ -1558,12 +1541,7 @@ LRESULT NC_HandleSysCommand( HWND hwnd, WPARAM wParam, LPARAM lParam )
 {
     TRACE("hwnd %p WM_SYSCOMMAND %lx %lx\n", hwnd, wParam, lParam );
 
-    if (!IsWindowEnabled( hwnd )) return 0;
-
-    if (HOOK_CallHooks( WH_CBT, HCBT_SYSCOMMAND, wParam, lParam, TRUE ))
-        return 0;
-
-    if (!USER_Driver->pSysCommand( hwnd, wParam, lParam ))
+    if (!NtUserMessageCall( hwnd, WM_SYSCOMMAND, wParam, lParam, 0, NtUserDefWindowProc, FALSE ))
         return 0;
 
     switch (wParam & 0xfff0)
@@ -1573,24 +1551,6 @@ LRESULT NC_HandleSysCommand( HWND hwnd, WPARAM wParam, LPARAM lParam )
         WINPOS_SysCommandSizeMove( hwnd, wParam );
         break;
 
-    case SC_MINIMIZE:
-        if (hwnd == GetActiveWindow())
-            ShowOwnedPopups(hwnd,FALSE);
-        ShowWindow( hwnd, SW_MINIMIZE );
-        break;
-
-    case SC_MAXIMIZE:
-        if (IsIconic(hwnd) && hwnd == GetActiveWindow())
-            ShowOwnedPopups(hwnd,TRUE);
-        ShowWindow( hwnd, SW_MAXIMIZE );
-        break;
-
-    case SC_RESTORE:
-        if (IsIconic(hwnd) && hwnd == GetActiveWindow())
-            ShowOwnedPopups(hwnd,TRUE);
-        ShowWindow( hwnd, SW_RESTORE );
-        break;
-
     case SC_CLOSE:
         return SendMessageW( hwnd, WM_CLOSE, 0, 0 );
 
@@ -1629,9 +1589,12 @@ LRESULT NC_HandleSysCommand( HWND hwnd, WPARAM wParam, LPARAM lParam )
             if (hmodule)
             {
                 BOOL (WINAPI *aboutproc)(HWND, LPCSTR, LPCSTR, HICON);
+                extern const char * CDECL wine_get_version(void);
+                char app[256];
 
+                sprintf( app, "Wine %s", wine_get_version() );
                 aboutproc = (void *)GetProcAddress( hmodule, "ShellAboutA" );
-                if (aboutproc) aboutproc( hwnd, PACKAGE_STRING, NULL, 0 );
+                if (aboutproc) aboutproc( hwnd, app, NULL, 0 );
                 FreeLibrary( hmodule );
             }
         }
