@@ -25,12 +25,12 @@
  */
 
 #include "config.h"
-#include "wine/port.h"
 
 #include <assert.h>
 #include <stdarg.h>
 #include <stdlib.h>
 #include <string.h>
+#include <dlfcn.h>
 #ifdef HAVE_EGL_EGL_H
 #include <EGL/egl.h>
 #endif
@@ -42,7 +42,6 @@
 #include "wine/wgl.h"
 #undef GLAPIENTRY
 #include "wine/wgl_driver.h"
-#include "wine/library.h"
 #include "wine/debug.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(android);
@@ -246,7 +245,7 @@ static BOOL set_pixel_format( HDC hdc, int format, BOOL allow_change )
     release_gl_drawable( gl );
 
     if (prev && prev != format && !allow_change) return FALSE;
-    if (__wine_set_pixel_format( hwnd, format )) return TRUE;
+    if (NtUserSetWindowPixelFormat( hwnd, format )) return TRUE;
     destroy_gl_drawable( hwnd );
     return FALSE;
 }
@@ -419,7 +418,7 @@ static BOOL android_wglSetPixelFormatWINE( HDC hdc, int format )
 /***********************************************************************
  *		android_wglCopyContext
  */
-static BOOL android_wglCopyContext( struct wgl_context *src, struct wgl_context *dst, UINT mask )
+static BOOL WINAPI android_wglCopyContext( struct wgl_context *src, struct wgl_context *dst, UINT mask )
 {
     FIXME( "%p -> %p mask %#x unsupported\n", src, dst, mask );
     return FALSE;
@@ -428,7 +427,7 @@ static BOOL android_wglCopyContext( struct wgl_context *src, struct wgl_context
 /***********************************************************************
  *		android_wglCreateContext
  */
-static struct wgl_context *android_wglCreateContext( HDC hdc )
+static struct wgl_context * WINAPI android_wglCreateContext( HDC hdc )
 {
     int egl_attribs[3] = { EGL_CONTEXT_CLIENT_VERSION, egl_client_version, EGL_NONE };
 
@@ -438,7 +437,7 @@ static struct wgl_context *android_wglCreateContext( HDC hdc )
 /***********************************************************************
  *		android_wglDeleteContext
  */
-static BOOL android_wglDeleteContext( struct wgl_context *ctx )
+static BOOL WINAPI android_wglDeleteContext( struct wgl_context *ctx )
 {
     EnterCriticalSection( &drawable_section );
     list_remove( &ctx->entry );
@@ -450,7 +449,7 @@ static BOOL android_wglDeleteContext( struct wgl_context *ctx )
 /***********************************************************************
  *		android_wglDescribePixelFormat
  */
-static int android_wglDescribePixelFormat( HDC hdc, int fmt, UINT size, PIXELFORMATDESCRIPTOR *pfd )
+static int WINAPI android_wglDescribePixelFormat( HDC hdc, int fmt, UINT size, PIXELFORMATDESCRIPTOR *pfd )
 {
     EGLint val;
     EGLConfig config;
@@ -496,7 +495,7 @@ static int android_wglDescribePixelFormat( HDC hdc, int fmt, UINT size, PIXELFOR
 /***********************************************************************
  *		android_wglGetPixelFormat
  */
-static int android_wglGetPixelFormat( HDC hdc )
+static int WINAPI android_wglGetPixelFormat( HDC hdc )
 {
     struct gl_drawable *gl;
     int ret = 0;
@@ -514,7 +513,7 @@ static int android_wglGetPixelFormat( HDC hdc )
 /***********************************************************************
  *		android_wglGetProcAddress
  */
-static PROC android_wglGetProcAddress( LPCSTR name )
+static PROC WINAPI android_wglGetProcAddress( LPCSTR name )
 {
     PROC ret;
     if (!strncmp( name, "wgl", 3 )) return NULL;
@@ -526,7 +525,7 @@ static PROC android_wglGetProcAddress( LPCSTR name )
 /***********************************************************************
  *		android_wglMakeCurrent
  */
-static BOOL android_wglMakeCurrent( HDC hdc, struct wgl_context *ctx )
+static BOOL WINAPI android_wglMakeCurrent( HDC hdc, struct wgl_context *ctx )
 {
     BOOL ret = FALSE;
     struct gl_drawable *gl;
@@ -566,7 +565,7 @@ done:
 /***********************************************************************
  *		android_wglSetPixelFormat
  */
-static BOOL android_wglSetPixelFormat( HDC hdc, int format, const PIXELFORMATDESCRIPTOR *pfd )
+static BOOL WINAPI android_wglSetPixelFormat( HDC hdc, int format, const PIXELFORMATDESCRIPTOR *pfd )
 {
     return set_pixel_format( hdc, format, FALSE );
 }
@@ -574,7 +573,7 @@ static BOOL android_wglSetPixelFormat( HDC hdc, int format, const PIXELFORMATDES
 /***********************************************************************
  *		android_wglShareLists
  */
-static BOOL android_wglShareLists( struct wgl_context *org, struct wgl_context *dest )
+static BOOL WINAPI android_wglShareLists( struct wgl_context *org, struct wgl_context *dest )
 {
     FIXME( "%p %p\n", org, dest );
     return FALSE;
@@ -583,7 +582,7 @@ static BOOL android_wglShareLists( struct wgl_context *org, struct wgl_context *
 /***********************************************************************
  *		android_wglSwapBuffers
  */
-static BOOL android_wglSwapBuffers( HDC hdc )
+static BOOL WINAPI android_wglSwapBuffers( HDC hdc )
 {
     struct wgl_context *ctx = NtCurrentTeb()->glContext;
 
@@ -638,25 +637,6 @@ static void init_extensions(void)
     egl_funcs.ext.p_wglGetCurrentReadDCARB   = (void *)1;  /* never called */
     egl_funcs.ext.p_wglMakeContextCurrentARB = android_wglMakeContextCurrentARB;
 
-#if 0  /* FIXME */
-    register_extension("WGL_ARB_pbuffer");
-    egl_funcs.ext.p_wglCreatePbufferARB    = android_wglCreatePbufferARB;
-    egl_funcs.ext.p_wglDestroyPbufferARB   = android_wglDestroyPbufferARB;
-    egl_funcs.ext.p_wglGetPbufferDCARB     = android_wglGetPbufferDCARB;
-    egl_funcs.ext.p_wglQueryPbufferARB     = android_wglQueryPbufferARB;
-    egl_funcs.ext.p_wglReleasePbufferDCARB = android_wglReleasePbufferDCARB;
-    egl_funcs.ext.p_wglSetPbufferAttribARB = android_wglSetPbufferAttribARB;
-#endif
-
-#if 0  /* FIXME */
-    register_extension("WGL_ARB_pixel_format");
-    egl_funcs.ext.p_wglChoosePixelFormatARB      = android_wglChoosePixelFormatARB;
-    egl_funcs.ext.p_wglGetPixelFormatAttribfvARB = android_wglGetPixelFormatAttribfvARB;
-    egl_funcs.ext.p_wglGetPixelFormatAttribivARB = android_wglGetPixelFormatAttribivARB;
-#endif
-
-    /* EXT Extensions */
-
     register_extension("WGL_EXT_extensions_string");
     egl_funcs.ext.p_wglGetExtensionsStringEXT = android_wglGetExtensionsStringEXT;
 
@@ -674,11 +654,11 @@ static void init_extensions(void)
 
     /* load standard functions and extensions exported from the OpenGL library */
 
-#define USE_GL_FUNC(func) if ((ptr = wine_dlsym( opengl_handle, #func, NULL, 0 ))) egl_funcs.gl.p_##func = ptr;
+#define USE_GL_FUNC(func) if ((ptr = dlsym( opengl_handle, #func ))) egl_funcs.gl.p_##func = ptr;
     ALL_WGL_FUNCS
 #undef USE_GL_FUNC
 
-#define LOAD_FUNCPTR(func) egl_funcs.ext.p_##func = wine_dlsym( opengl_handle, #func, NULL, 0 )
+#define LOAD_FUNCPTR(func) egl_funcs.ext.p_##func = dlsym( opengl_handle, #func )
     LOAD_FUNCPTR( glActiveShaderProgram );
     LOAD_FUNCPTR( glActiveTexture );
     LOAD_FUNCPTR( glAttachShader );
@@ -968,24 +948,23 @@ static BOOL egl_init(void)
     static int retval = -1;
     EGLConfig *configs;
     EGLint major, minor, count, i, pass;
-    char buffer[200];
 
     if (retval != -1) return retval;
     retval = 0;
 
-    if (!(egl_handle = wine_dlopen( SONAME_LIBEGL, RTLD_NOW|RTLD_GLOBAL, buffer, sizeof(buffer) )))
+    if (!(egl_handle = dlopen( SONAME_LIBEGL, RTLD_NOW|RTLD_GLOBAL )))
     {
-        ERR( "failed to load %s: %s\n", SONAME_LIBEGL, buffer );
+        ERR( "failed to load %s: %s\n", SONAME_LIBEGL, dlerror() );
         return FALSE;
     }
-    if (!(opengl_handle = wine_dlopen( SONAME_LIBGLESV2, RTLD_NOW|RTLD_GLOBAL, buffer, sizeof(buffer) )))
+    if (!(opengl_handle = dlopen( SONAME_LIBGLESV2, RTLD_NOW|RTLD_GLOBAL )))
     {
-        ERR( "failed to load %s: %s\n", SONAME_LIBGLESV2, buffer );
+        ERR( "failed to load %s: %s\n", SONAME_LIBGLESV2, dlerror() );
         return FALSE;
     }
 
 #define LOAD_FUNCPTR(func) do { \
-        if (!(p_##func = wine_dlsym( egl_handle, #func, NULL, 0 ))) \
+        if (!(p_##func = dlsym( egl_handle, #func ))) \
         { ERR( "can't find symbol %s\n", #func); return FALSE; }    \
     } while(0)
     LOAD_FUNCPTR( eglCreateContext );
@@ -1003,12 +982,6 @@ static BOOL egl_init(void)
     LOAD_FUNCPTR( eglSwapInterval );
 #undef LOAD_FUNCPTR
 
-    /* initializing OpenGL seems to break loading of libandroid_runtime.so
-     * (dlopen failed: cannot locate symbol "glWeightPointerOES" referenced by "/system/lib/libandroid_runtime.so"...)
-     * so we preload it here */
-    if (!wine_dlopen( "libandroid_runtime.so", RTLD_NOW|RTLD_GLOBAL, buffer, sizeof(buffer) ))
-        WARN( "failed to load libandroid_runtime.so: '%s'\n", buffer );
-
     display = p_eglGetDisplay( EGL_DEFAULT_DISPLAY );
     if (!p_eglInitialize( display, &major, &minor )) return 0;
     TRACE( "display %p version %u.%u\n", display, major, minor );
@@ -1070,7 +1043,6 @@ static void glstub_##name(void) \
 }
 
 ALL_WGL_FUNCS
-ALL_GLU_FUNCS
 #undef USE_GL_FUNC
 
 static struct opengl_funcs egl_funcs =
@@ -1088,9 +1060,7 @@ static struct opengl_funcs egl_funcs =
         android_wglSwapBuffers,
     },
 #define USE_GL_FUNC(name) (void *)glstub_##name,
-    { ALL_WGL_FUNCS },
-    { },
-    { ALL_GLU_FUNCS }
+    { ALL_WGL_FUNCS }
 #undef USE_GL_FUNC
 };
 
