@@ -24,26 +24,21 @@
 #define NONAMELESSSTRUCT
 
 #include "config.h"
-#include "wine/port.h"
 
 #include <assert.h>
 #include <fcntl.h>
-#include <math.h>
 #include <poll.h>
 #include <errno.h>
 #include <stdarg.h>
 #include <stdlib.h>
 #include <stdio.h>
-#ifdef HAVE_UNISTD_H
-# include <unistd.h>
-#endif
+#include <unistd.h>
 
 #define OEMRESOURCE
 #include "windef.h"
 #include "winbase.h"
 #include "wingdi.h"
 #include "winuser.h"
-#include "wine/unicode.h"
 
 #include "android.h"
 #include "wine/server.h"
@@ -236,6 +231,7 @@ void desktop_changed( JNIEnv *env, jobject obj, jint width, jint height )
     data.type = DESKTOP_CHANGED;
     data.desktop.width = width;
     data.desktop.height = height;
+    p__android_log_print( ANDROID_LOG_INFO, "wine", "desktop_changed: %ux%u", width, height );
     send_event( &data );
 }
 
@@ -252,6 +248,7 @@ void config_changed( JNIEnv *env, jobject obj, jint dpi )
     memset( &data, 0, sizeof(data) );
     data.type = CONFIG_CHANGED;
     data.cfg.dpi = dpi;
+    p__android_log_print( ANDROID_LOG_INFO, "wine", "config_changed: %u dpi", dpi );
     send_event( &data );
 }
 
@@ -278,6 +275,8 @@ void surface_changed( JNIEnv *env, jobject obj, jint win, jobject surface, jbool
         data.surface.window = win;
         data.surface.width = width;
         data.surface.height = height;
+        p__android_log_print( ANDROID_LOG_INFO, "wine", "surface_changed: %p %s %ux%u",
+                              data.surface.hwnd, client ? "client" : "whole", width, height );
     }
     data.type = SURFACE_CHANGED;
     send_event( &data );
@@ -331,7 +330,7 @@ jboolean motion_event( JNIEnv *env, jobject obj, jint win, jint action, jint x,
             data.motion.input.u.mi.dwFlags |= MOUSEEVENTF_RIGHTDOWN;
         if ((state & ~prev_state) & AMOTION_EVENT_BUTTON_TERTIARY)
             data.motion.input.u.mi.dwFlags |= MOUSEEVENTF_MIDDLEDOWN;
-        if (!state) /* touch event */
+        if (!(state & ~prev_state)) /* touch event */
             data.motion.input.u.mi.dwFlags |= MOUSEEVENTF_LEFTDOWN;
         break;
     case AMOTION_EVENT_ACTION_UP:
@@ -347,11 +346,8 @@ jboolean motion_event( JNIEnv *env, jobject obj, jint win, jint action, jint x,
             data.motion.input.u.mi.dwFlags |= MOUSEEVENTF_LEFTUP;
         break;
     case AMOTION_EVENT_ACTION_SCROLL:
-        if ((action & 0x10000) == 0x10000)
-            data.motion.input.u.mi.dwFlags |= MOUSEEVENTF_HWHEEL;
-        else
-            data.motion.input.u.mi.dwFlags |= MOUSEEVENTF_WHEEL;
-        data.motion.input.u.mi.mouseData = vscroll;
+        data.motion.input.u.mi.dwFlags |= MOUSEEVENTF_WHEEL;
+        data.motion.input.u.mi.mouseData = vscroll < 0 ? -WHEEL_DELTA : WHEEL_DELTA;
         break;
     case AMOTION_EVENT_ACTION_MOVE:
     case AMOTION_EVENT_ACTION_HOVER_MOVE:
@@ -363,64 +359,6 @@ jboolean motion_event( JNIEnv *env, jobject obj, jint win, jint action, jint x,
     return JNI_TRUE;
 }
 
-void set_focus( JNIEnv *env, jobject obj, jint win )
-{
-    union event_data data;
-
-    data.type = SET_FOCUS;
-    data.focus.hwnd = LongToHandle( win );
-    send_event( &data );
-}
-
-void send_syscommand( JNIEnv *env, jobject obj, jint win, jint param )
-{
-    union event_data data;
-
-    data.type = SEND_SYSCOMMAND;
-    data.syscommand.hwnd = LongToHandle( win );
-    data.syscommand.wp = param;
-    send_event( &data );
-}
-
-void clipdata_update( JNIEnv *env, jobject obj, jint flags, jobjectArray mimetypes )
-{
-    union event_data data;
-    int i;
-
-    data.type = CLIPDATA_UPDATE;
-    data.clipdata.flags = flags;
-
-    if (mimetypes)
-    {
-        int count = (*env)->GetArrayLength( env, mimetypes );
-
-        data.clipdata.mimetypes = malloc( sizeof(LPWSTR*) * (count + 1) );
-        for (i = 0; i < count; i++)
-        {
-            jobject s = (*env)->GetObjectArrayElement( env, mimetypes, i );
-            const jchar *mimetype_str = (*env)->GetStringChars( env, s, NULL );
-            jsize len = (*env)->GetStringLength( env, s );
-            data.clipdata.mimetypes[i] = malloc( sizeof(WCHAR) * (len + 1) );
-            lstrcpynW( data.clipdata.mimetypes[i], (WCHAR*)mimetype_str, len + 1 );
-            (*env)->ReleaseStringChars( env, s, mimetype_str );
-        }
-        data.clipdata.mimetypes[i] = 0;
-    }
-    else
-        data.clipdata.mimetypes = NULL;
-
-    send_event( &data );
-}
-
-void send_window_close( JNIEnv *env, jobject obj, jint win )
-{
-    union event_data data;
-
-    data.type = SEND_WINDOW_CLOSE;
-    data.syscommand.hwnd = LongToHandle( win );
-    send_event( &data );
-}
-
 
 /***********************************************************************
  *           init_event_queue
@@ -579,7 +517,7 @@ static int process_events( DWORD mask )
                     }
                     SERVER_END_REQ;
                 }
-                __wine_send_input( capture ? capture : event->data.motion.hwnd, &event->data.motion.input );
+                __wine_send_input( capture ? capture : event->data.motion.hwnd, &event->data.motion.input, NULL );
             }
             break;
 
@@ -593,77 +531,9 @@ static int process_events( DWORD mask )
                       event->data.kbd.input.u.ki.wVk, event->data.kbd.input.u.ki.wVk,
                       event->data.kbd.input.u.ki.wScan );
             update_keyboard_lock_state( event->data.kbd.input.u.ki.wVk, event->data.kbd.lock_state );
-            __wine_send_input( 0, &event->data.kbd.input );
+            __wine_send_input( 0, &event->data.kbd.input, NULL );
             break;
 
-        case SET_FOCUS:
-            TRACE("SET_FOCUS %p\n", event->data.focus.hwnd);
-            SendNotifyMessageW( event->data.focus.hwnd, WM_ANDROID_SET_FOCUS, 0, 0 );
-            break;
-
-        case SEND_SYSCOMMAND:
-            TRACE( "SEND_SYSCOMMAND %p %08lx\n", event->data.syscommand.hwnd, event->data.syscommand.wp );
-            SendNotifyMessageW( event->data.syscommand.hwnd, WM_SYSCOMMAND, event->data.syscommand.wp, 0 );
-            break;
-
-        case CLIPDATA_UPDATE:
-            TRACE("CLIPDATA_UPDATE flags %x mimetypes %p\n", event->data.clipdata.flags, event->data.clipdata.mimetypes);
-            handle_clipdata_update( event->data.clipdata.flags, event->data.clipdata.mimetypes );
-            break;
-
-        case IME_TEXT:
-            ERR( "IME_TEXT target %u, length %u\n", event->data.ime_text.target , event->data.ime_text.length );
-            handle_IME_TEXT( event->data.ime_text.target, event->data.ime_text.length);
-            break;
-
-        case IME_FINISH:
-            TRACE( "IME_FINISH target %u, length %u\n", event->data.ime_finish.target, event->data.ime_text.length );
-            handle_IME_FINISH( event->data.ime_finish.target, event->data.ime_finish.length );
-            break;
-
-        case IME_CANCEL:
-            TRACE( "IME_CANCEL\n" );
-            handle_IME_CANCEL();
-            break;
-
-        case IME_START:
-            TRACE( "IME_START\n" );
-            handle_IME_START();
-            break;
-
-        case RUN_CMDLINE:
-            handle_run_cmdline( event->data.runcmd.cmdline, event->data.runcmd.env );
-            free( event->data.runcmd.cmdline );
-            if (event->data.runcmd.env)
-            {
-                LPWSTR *strs = event->data.runcmd.env;
-                while (*strs) free( *strs++ );
-                free( event->data.runcmd.env );
-            }
-            break;
-        case RUN_CMDARRAY:
-            handle_run_cmdarray( event->data.runcmdarr.cmdarray, event->data.runcmdarr.env );
-            if (event->data.runcmdarr.cmdarray)
-            {
-                LPWSTR *strs = event->data.runcmdarr.cmdarray;
-                while (*strs) free( *strs++ );
-                free( event->data.runcmdarr.cmdarray );
-            }
-            if (event->data.runcmdarr.env)
-            {
-                LPWSTR *strs = event->data.runcmdarr.env;
-                while (*strs) free( *strs++ );
-                free( event->data.runcmdarr.env );
-            }
-            break;
-        case CLEAR_META:
-            TRACE( "CLEAR_META" );
-            handle_clear_meta_key_states( event->data.clearmeta.states );
-            break;
-        case SEND_WINDOW_CLOSE:
-            TRACE( "SEND_WINDOW_CLOSE %p\n", event->data.syscommand.hwnd );
-            SendMessageW( event->data.syscommand.hwnd, WM_CLOSE, 0, 0 );
-            break;
         default:
             FIXME( "got event %u\n", event->data.type );
         }
@@ -987,18 +857,27 @@ static void set_color_key( struct android_window_surface *surface, COLORREF key
 static void set_surface_region( struct window_surface *window_surface, HRGN win_region )
 {
     struct android_window_surface *surface = get_android_surface( window_surface );
+    struct android_win_data *win_data;
     HRGN region = win_region;
     RGNDATA *data = NULL;
     DWORD size;
+    int offset_x, offset_y;
 
     if (window_surface->funcs != &android_surface_funcs) return;  /* we may get the null surface */
 
+    if (!(win_data = get_win_data( surface->hwnd ))) return;
+    offset_x = win_data->window_rect.left - win_data->whole_rect.left;
+    offset_y = win_data->window_rect.top - win_data->whole_rect.top;
+    release_win_data( win_data );
+
     if (win_region == (HRGN)1)  /* hack: win_region == 1 means retrieve region from server */
     {
-        region = CreateRectRgnIndirect( &surface->header.rect );
+        region = CreateRectRgn( 0, 0, win_data->window_rect.right - win_data->window_rect.left,
+                                win_data->window_rect.bottom - win_data->window_rect.top );
         if (GetWindowRgn( surface->hwnd, region ) == ERROR && !surface->region) goto done;
     }
 
+    OffsetRgn( region, offset_x, offset_y );
     if (surface->region) CombineRgn( region, region, surface->region, RGN_AND );
 
     if (!(size = GetRegionData( region, 0, NULL ))) goto done;
@@ -1193,31 +1072,6 @@ failed:
 }
 
 
-/***********************************************************************
- *              fetch_window_icon
- */
-static void fetch_window_icon( HWND hwnd, HICON icon )
-{
-    ICONINFO ii;
-    unsigned int width = 0, height = 0, *bits = NULL;
-
-    if (!icon) icon = (HICON)SendMessageW( hwnd, WM_GETICON, ICON_BIG, 0 );
-    if (!icon) icon = (HICON)GetClassLongPtrW( hwnd, GCLP_HICON );
-
-    if (GetIconInfo( icon, &ii ))
-    {
-        HDC hdc = CreateCompatibleDC( 0 );
-        bits = get_bitmap_argb( hdc, ii.hbmColor, ii.hbmMask, &width, &height );
-        DeleteDC( hdc );
-        DeleteObject( ii.hbmColor );
-        DeleteObject( ii.hbmMask );
-    }
-
-    ioctl_set_window_icon( hwnd, width, height, bits );
-    HeapFree( GetProcessHeap(), 0, bits );
-}
-
-
 enum android_system_cursors
 {
     TYPE_ARROW = 1000,
@@ -1318,9 +1172,9 @@ static int get_cursor_system_id( const ICONINFOEXW *info )
     if (info->szResName[0]) return 0;  /* only integer resources are supported here */
     if (!(module = GetModuleHandleW( info->szModName ))) return 0;
 
-    for (i = 0; i < sizeof(module_cursors)/sizeof(module_cursors[0]); i++)
+    for (i = 0; i < ARRAY_SIZE( module_cursors ); i++)
         if (GetModuleHandleW( module_cursors[i].name ) == module) break;
-    if (i == sizeof(module_cursors)/sizeof(module_cursors[0])) return 0;
+    if (i == ARRAY_SIZE( module_cursors )) return 0;
 
     cursors = module_cursors[i].cursors;
     for (i = 0; cursors[i].id; i++)
@@ -1339,9 +1193,6 @@ static LRESULT CALLBACK desktop_wndproc_wrapper( HWND hwnd, UINT msg, WPARAM wp,
     case WM_PARENTNOTIFY:
         if (LOWORD(wp) == WM_DESTROY) destroy_ioctl_window( (HWND)lp, FALSE );
         break;
-    case WM_CLOSE:
-        ioctl_close_desktop();
-        break;
     }
     return desktop_orig_wndproc( hwnd, msg, wp, lp );
 }
@@ -1350,8 +1201,8 @@ static LRESULT CALLBACK desktop_wndproc_wrapper( HWND hwnd, UINT msg, WPARAM wp,
 /***********************************************************************
  *           ANDROID_MsgWaitForMultipleObjectsEx
  */
-DWORD CDECL ANDROID_MsgWaitForMultipleObjectsEx( DWORD count, const HANDLE *handles,
-                                                 DWORD timeout, DWORD mask, DWORD flags )
+DWORD ANDROID_MsgWaitForMultipleObjectsEx( DWORD count, const HANDLE *handles,
+                                           DWORD timeout, DWORD mask, DWORD flags )
 {
     if (GetCurrentThreadId() == desktop_tid)
     {
@@ -1363,23 +1214,10 @@ DWORD CDECL ANDROID_MsgWaitForMultipleObjectsEx( DWORD count, const HANDLE *hand
                                      timeout, flags & MWMO_ALERTABLE );
 }
 
-/*****************************************************************
- *		SetFocus
- */
-void CDECL ANDROID_SetFocus( HWND hwnd )
-{
-    IME_UpdateAssociation(hwnd);
-
-    hwnd = GetAncestor( hwnd, GA_ROOT );
-    ioctl_set_window_focus( hwnd );
-    SetWindowPos( hwnd, HWND_TOP, 0, 0, 0, 0, SWP_NOACTIVATE|SWP_NOMOVE|SWP_NOSIZE );
-}
-
-
 /**********************************************************************
  *           ANDROID_CreateWindow
  */
-BOOL CDECL ANDROID_CreateWindow( HWND hwnd )
+BOOL ANDROID_CreateWindow( HWND hwnd )
 {
     TRACE( "%p\n", hwnd );
 
@@ -1391,7 +1229,6 @@ BOOL CDECL ANDROID_CreateWindow( HWND hwnd )
         start_android_device();
         if (!(data = alloc_win_data( hwnd ))) return FALSE;
         release_win_data( data );
-        init_clipboard();
     }
     return TRUE;
 }
@@ -1400,7 +1237,7 @@ BOOL CDECL ANDROID_CreateWindow( HWND hwnd )
 /***********************************************************************
  *           ANDROID_DestroyWindow
  */
-void CDECL ANDROID_DestroyWindow( HWND hwnd )
+void ANDROID_DestroyWindow( HWND hwnd )
 {
     struct android_win_data *data;
 
@@ -1429,13 +1266,6 @@ static struct android_win_data *create_win_data( HWND hwnd, const RECT *window_r
     if (!(data = alloc_win_data( hwnd ))) return NULL;
 
     data->parent = (parent == GetDesktopWindow()) ? 0 : parent;
-    if (!data->parent)
-    {
-        WCHAR text[1024];
-        if (InternalGetWindowText( hwnd, text, sizeof(text)/sizeof(WCHAR) ))
-            ioctl_set_window_text( hwnd, text );
-    }
-
     data->whole_rect = data->window_rect = *window_rect;
     data->client_rect = *client_rect;
     return data;
@@ -1457,9 +1287,9 @@ static inline BOOL get_surface_rect( const RECT *visible_rect, RECT *surface_rec
 /***********************************************************************
  *           ANDROID_WindowPosChanging
  */
-void CDECL ANDROID_WindowPosChanging( HWND hwnd, HWND insert_after, UINT swp_flags,
-                                     const RECT *window_rect, const RECT *client_rect, RECT *visible_rect,
-                                     struct window_surface **surface )
+BOOL ANDROID_WindowPosChanging( HWND hwnd, HWND insert_after, UINT swp_flags,
+                                const RECT *window_rect, const RECT *client_rect, RECT *visible_rect,
+                                struct window_surface **surface )
 {
     struct android_win_data *data = get_win_data( hwnd );
     RECT surface_rect;
@@ -1472,7 +1302,7 @@ void CDECL ANDROID_WindowPosChanging( HWND hwnd, HWND insert_after, UINT swp_fla
            hwnd, wine_dbgstr_rect(window_rect), wine_dbgstr_rect(client_rect),
            GetWindowLongW( hwnd, GWL_STYLE ), swp_flags );
 
-    if (!data && !(data = create_win_data( hwnd, window_rect, client_rect ))) return;
+    if (!data && !(data = create_win_data( hwnd, window_rect, client_rect ))) return TRUE;
 
     *visible_rect = *window_rect;
 
@@ -1505,20 +1335,21 @@ void CDECL ANDROID_WindowPosChanging( HWND hwnd, HWND insert_after, UINT swp_fla
 
 done:
     release_win_data( data );
+    return TRUE;
 }
 
 
 /***********************************************************************
  *           ANDROID_WindowPosChanged
  */
-void CDECL ANDROID_WindowPosChanged( HWND hwnd, HWND insert_after, UINT swp_flags,
-                                    const RECT *window_rect, const RECT *client_rect,
-                                    const RECT *visible_rect, const RECT *valid_rects,
-                                    struct window_surface *surface )
+void ANDROID_WindowPosChanged( HWND hwnd, HWND insert_after, UINT swp_flags,
+                               const RECT *window_rect, const RECT *client_rect,
+                               const RECT *visible_rect, const RECT *valid_rects,
+                               struct window_surface *surface )
 {
     struct android_win_data *data;
     DWORD new_style = GetWindowLongW( hwnd, GWL_STYLE );
-    BOOL is_child;
+    HWND owner = 0;
 
     if (!(data = get_win_data( hwnd ))) return;
 
@@ -1532,31 +1363,24 @@ void CDECL ANDROID_WindowPosChanged( HWND hwnd, HWND insert_after, UINT swp_flag
         if (data->surface) window_surface_release( data->surface );
         data->surface = surface;
     }
-    is_child = (data->parent != 0);
-
+    if (!data->parent) owner = GetWindow( hwnd, GW_OWNER );
     release_win_data( data );
 
-    TRACE( "win %p window %s client %s style %08x after %p flags %08x\n", hwnd,
+    if (!(swp_flags & SWP_NOZORDER)) insert_after = GetWindow( hwnd, GW_HWNDPREV );
+
+    TRACE( "win %p window %s client %s style %08x owner %p after %p flags %08x\n", hwnd,
            wine_dbgstr_rect(window_rect), wine_dbgstr_rect(client_rect),
-           new_style, insert_after, swp_flags );
+           new_style, owner, insert_after, swp_flags );
 
-    if (is_child)
-    {
-        if (!(swp_flags & SWP_NOZORDER)) insert_after = GetWindow( hwnd, GW_HWNDPREV );
-        ioctl_window_pos_changed( hwnd, window_rect, client_rect, visible_rect, new_style, swp_flags, insert_after, 0 );
-        return;
-    }
-
-    ioctl_window_pos_changed( hwnd, window_rect, client_rect, visible_rect, new_style, swp_flags,
-                              insert_after, GetWindow( hwnd, GW_OWNER ));
-    if (swp_flags & SWP_SHOWWINDOW) fetch_window_icon( hwnd, 0 );
+    ioctl_window_pos_changed( hwnd, window_rect, client_rect, visible_rect,
+                              new_style, swp_flags, insert_after, owner );
 }
 
 
 /***********************************************************************
  *           ANDROID_ShowWindow
  */
-UINT CDECL ANDROID_ShowWindow( HWND hwnd, INT cmd, RECT *rect, UINT swp )
+UINT ANDROID_ShowWindow( HWND hwnd, INT cmd, RECT *rect, UINT swp )
 {
     if (IsRectEmpty( rect )) return swp;
     if (!IsIconic( hwnd )) return swp;
@@ -1573,7 +1397,7 @@ UINT CDECL ANDROID_ShowWindow( HWND hwnd, INT cmd, RECT *rect, UINT swp )
 /*****************************************************************
  *	     ANDROID_SetParent
  */
-void CDECL ANDROID_SetParent( HWND hwnd, HWND parent, HWND old_parent )
+void ANDROID_SetParent( HWND hwnd, HWND parent, HWND old_parent )
 {
     struct android_win_data *data;
 
@@ -1591,7 +1415,7 @@ void CDECL ANDROID_SetParent( HWND hwnd, HWND parent, HWND old_parent )
 /***********************************************************************
  *           ANDROID_SetCapture
  */
-void CDECL ANDROID_SetCapture( HWND hwnd, UINT flags )
+void ANDROID_SetCapture( HWND hwnd, UINT flags )
 {
     if (!(flags & (GUI_INMOVESIZE | GUI_INMENUMODE))) return;
     ioctl_set_capture( hwnd );
@@ -1601,7 +1425,7 @@ void CDECL ANDROID_SetCapture( HWND hwnd, UINT flags )
 /***********************************************************************
  *           ANDROID_SetCursor
  */
-void CDECL ANDROID_SetCursor( HCURSOR handle )
+void ANDROID_SetCursor( HCURSOR handle )
 {
     static HCURSOR last_cursor;
     static DWORD last_cursor_change;
@@ -1646,7 +1470,7 @@ void CDECL ANDROID_SetCursor( HCURSOR handle )
 /***********************************************************************
  *           ANDROID_SetWindowStyle
  */
-void CDECL ANDROID_SetWindowStyle( HWND hwnd, INT offset, STYLESTRUCT *style )
+void ANDROID_SetWindowStyle( HWND hwnd, INT offset, STYLESTRUCT *style )
 {
     struct android_win_data *data;
     DWORD changed = style->styleNew ^ style->styleOld;
@@ -1667,29 +1491,10 @@ void CDECL ANDROID_SetWindowStyle( HWND hwnd, INT offset, STYLESTRUCT *style )
 }
 
 
-/*****************************************************************
- *		ANDROID_SetWindowIcon
- */
-void CDECL ANDROID_SetWindowIcon( HWND hwnd, UINT type, HICON icon )
-{
-    if (type != ICON_BIG) return;  /* small icons not supported */
-    fetch_window_icon( hwnd, icon );
-}
-
-
-/*****************************************************************
- *		ANDROID_SetWindowText
- */
-void CDECL ANDROID_SetWindowText( HWND hwnd, WCHAR *text /* technically const */ )
-{
-    ioctl_set_window_text( hwnd, text );
-}
-
-
 /***********************************************************************
  *           ANDROID_SetWindowRgn
  */
-void CDECL ANDROID_SetWindowRgn( HWND hwnd, HRGN hrgn, BOOL redraw )
+void ANDROID_SetWindowRgn( HWND hwnd, HRGN hrgn, BOOL redraw )
 {
     struct android_win_data *data;
 
@@ -1705,7 +1510,7 @@ void CDECL ANDROID_SetWindowRgn( HWND hwnd, HRGN hrgn, BOOL redraw )
 /***********************************************************************
  *	     ANDROID_SetLayeredWindowAttributes
  */
-void CDECL ANDROID_SetLayeredWindowAttributes( HWND hwnd, COLORREF key, BYTE alpha, DWORD flags )
+void ANDROID_SetLayeredWindowAttributes( HWND hwnd, COLORREF key, BYTE alpha, DWORD flags )
 {
     struct android_win_data *data;
 
@@ -1723,8 +1528,8 @@ void CDECL ANDROID_SetLayeredWindowAttributes( HWND hwnd, COLORREF key, BYTE alp
 /*****************************************************************************
  *           ANDROID_UpdateLayeredWindow
  */
-BOOL CDECL ANDROID_UpdateLayeredWindow( HWND hwnd, const UPDATELAYEREDWINDOWINFO *info,
-                                        const RECT *window_rect )
+BOOL ANDROID_UpdateLayeredWindow( HWND hwnd, const UPDATELAYEREDWINDOWINFO *info,
+                                  const RECT *window_rect )
 {
     struct window_surface *surface;
     struct android_win_data *data;
@@ -1807,81 +1612,11 @@ done:
     return ret;
 }
 
-static inline BOOL can_activate_window( HWND hwnd )
-{
-    LONG style = GetWindowLongW( hwnd, GWL_STYLE );
-    RECT rect;
-
-    if (!(style & WS_VISIBLE)) return FALSE;
-    if ((style & (WS_POPUP|WS_CHILD)) == WS_CHILD) return FALSE;
-    if (style & WS_MINIMIZE) return FALSE;
-    if (GetWindowLongW( hwnd, GWL_EXSTYLE ) & WS_EX_NOACTIVATE) return FALSE;
-    if (hwnd == GetDesktopWindow()) return FALSE;
-    if (GetWindowRect( hwnd, &rect ) && IsRectEmpty( &rect )) return FALSE;
-    return !(style & WS_DISABLED);
-}
-
-static BOOL try_activate_window( HWND hwnd )
-{
-    if (can_activate_window( hwnd ))
-    {
-        LRESULT ma = SendMessageW( hwnd, WM_MOUSEACTIVATE,
-                                   (WPARAM)GetAncestor( hwnd, GA_ROOT ),
-                                   MAKELONG(HTCAPTION, WM_LBUTTONDOWN) );
-
-        if (ma != MA_NOACTIVATEANDEAT && ma != MA_NOACTIVATE)
-        {
-            if (SetForegroundWindow( hwnd ))
-            {
-                SetWindowPos( hwnd, HWND_TOP, 0, 0, 0, 0, SWP_NOSIZE|SWP_NOMOVE );
-                return TRUE;
-            }
-        }
-    }
-
-    return FALSE;
-}
-
-static BOOL CALLBACK enum_activate_owned( HWND hwnd, LPARAM lparam )
-{
-    HWND owner = (HWND)lparam;
-
-    if (GetWindow( hwnd, GW_OWNER ) == owner)
-    {
-        if (try_activate_window( hwnd ))
-            return FALSE;
-
-        if (!EnumWindows( enum_activate_owned, (LPARAM)hwnd ))
-            return FALSE;
-    }
-
-    return TRUE;
-}
-
-void handle_set_focus( HWND hwnd )
-{
-    DWORD style;
-
-    if (hwnd == GetDesktopWindow())
-    {
-        SendMessageW( GetForegroundWindow(), WM_CANCELMODE, 0, 0 );
-        SetForegroundWindow( hwnd );
-        return;
-    }
-
-    if (try_activate_window( hwnd )) return;
-
-    /* Try to activate a window owned by a disabled window. */
-    style = GetWindowLongW( hwnd, GWL_STYLE );
-    if (style & WS_DISABLED)
-        EnumWindows( enum_activate_owned, (LPARAM)hwnd );
-}
-
 
 /**********************************************************************
  *           ANDROID_WindowMessage
  */
-LRESULT CDECL ANDROID_WindowMessage( HWND hwnd, UINT msg, WPARAM wp, LPARAM lp )
+LRESULT ANDROID_WindowMessage( HWND hwnd, UINT msg, WPARAM wp, LPARAM lp )
 {
     struct android_win_data *data;
 
@@ -1905,11 +1640,6 @@ LRESULT CDECL ANDROID_WindowMessage( HWND hwnd, UINT msg, WPARAM wp, LPARAM lp )
             release_win_data( data );
         }
         return 0;
-    case WM_ANDROID_IME_CONTROL:
-        return Ime_Control(hwnd, msg, wp, lp);
-    case WM_ANDROID_SET_FOCUS:
-        handle_set_focus( hwnd );
-        return 0;
     default:
         FIXME( "got window msg %x hwnd %p wp %lx lp %lx\n", msg, hwnd, wp, lp );
         return 0;
@@ -1922,13 +1652,6 @@ LRESULT CDECL ANDROID_WindowMessage( HWND hwnd, UINT msg, WPARAM wp, LPARAM lp )
  */
 BOOL CDECL ANDROID_create_desktop( UINT width, UINT height )
 {
-    static const WCHAR shellW[] = {'s','h','e','l','l',0};
-    WCHAR name[MAX_PATH];
-
-    if (!GetUserObjectInformationW( GetThreadDesktop( GetCurrentThreadId() ),
-                                    UOI_NAME, name, sizeof(name), NULL ))
-        name[0] = 0;
-
     desktop_orig_wndproc = (WNDPROC)SetWindowLongPtrW( GetDesktopWindow(), GWLP_WNDPROC,
                                                        (LONG_PTR)desktop_wndproc_wrapper );
 
@@ -1942,6 +1665,5 @@ BOOL CDECL ANDROID_create_desktop( UINT width, UINT height )
         }
         process_events( QS_ALLINPUT );
     }
-    /* use desktop mode only for the "shell" desktop */
-    return !lstrcmpiW( name, shellW );
+    return TRUE;
 }
