@@ -60,6 +60,7 @@ int right_command_is_ctrl = 0;
 BOOL allow_software_rendering = FALSE;
 BOOL disable_window_decorations = FALSE;
 int allow_immovable_windows = TRUE;
+int use_confinement_cursor_clipping = TRUE;
 int cursor_clipping_locks_windows = TRUE;
 int use_precise_scrolling = TRUE;
 int gl_surface_mode = GL_SURFACE_IN_FRONT_OPAQUE;
@@ -79,7 +80,8 @@ CFDictionaryRef localized_strings;
 const char* debugstr_cf(CFTypeRef t)
 {
     CFStringRef s;
-    const char* ret;
+    const char* HOSTPTR cstring = NULL;
+    const char* ret = NULL;
 
     if (!t) return "(null)";
 
@@ -87,13 +89,13 @@ const char* debugstr_cf(CFTypeRef t)
         s = t;
     else
         s = CFCopyDescription(t);
-    ret = CFStringGetCStringPtr(s, kCFStringEncodingUTF8);
-    if (ret) ret = debugstr_a(ret);
+    cstring = CFStringGetCStringPtr(s, kCFStringEncodingUTF8);
+    if (cstring) ret = debugstr_a(cstring);
     if (!ret)
     {
-        const UniChar* u = CFStringGetCharactersPtr(s);
+        const UniChar* HOSTPTR u = CFStringGetCharactersPtr(s);
         if (u)
-            ret = debugstr_wn((const WCHAR*)u, CFStringGetLength(s));
+            ret = debugstr_wn((const WCHAR* HOSTPTR)u, CFStringGetLength(s));
     }
     if (!ret)
     {
@@ -210,6 +212,9 @@ static void setup_options(void)
     if (!get_config_key(hkey, appkey, "AllowImmovableWindows", buffer, sizeof(buffer)))
         allow_immovable_windows = IS_OPTION_TRUE(buffer[0]);
 
+    if (!get_config_key(hkey, appkey, "UseConfinementCursorClipping", buffer, sizeof(buffer)))
+        use_confinement_cursor_clipping = IS_OPTION_TRUE(buffer[0]);
+
     if (!get_config_key(hkey, appkey, "CursorClippingLocksWindows", buffer, sizeof(buffer)))
         cursor_clipping_locks_windows = IS_OPTION_TRUE(buffer[0]);
 
@@ -330,10 +335,8 @@ static BOOL process_attach(void)
         return FALSE;
     }
 
-    /* CrossOver Hack 10188: Actually, this disables that hack.  Don't pass
-                             system tray icons to our launcher since the Mac
-                             driver handles them itself. */
-    unsetenv("CX_SYSTRAY_SOCKET");
+    init_user_driver();
+    macdrv_init_display_devices(FALSE);
 
     return TRUE;
 }
@@ -342,7 +345,7 @@ static BOOL process_attach(void)
 /***********************************************************************
  *              ThreadDetach   (MACDRV.@)
  */
-void CDECL macdrv_ThreadDetach(void)
+void macdrv_ThreadDetach(void)
 {
     struct macdrv_thread_data *data = macdrv_thread_data();
 
@@ -418,6 +421,7 @@ struct macdrv_thread_data *macdrv_init_thread_data(void)
     set_queue_display_fd(macdrv_get_event_queue_fd(data->queue));
     TlsSetValue(thread_data_tls_index, data);
 
+    ActivateKeyboardLayout(data->active_keyboard_layout, 0);
     return data;
 }
 
@@ -443,7 +447,7 @@ BOOL WINAPI DllMain(HINSTANCE hinst, DWORD reason, LPVOID reserved)
 /***********************************************************************
  *              SystemParametersInfo (MACDRV.@)
  */
-BOOL CDECL macdrv_SystemParametersInfo( UINT action, UINT int_param, void *ptr_param, UINT flags )
+BOOL macdrv_SystemParametersInfo( UINT action, UINT int_param, void *ptr_param, UINT flags )
 {
     switch (action)
     {
