@@ -22,8 +22,8 @@
 
 #include "config.h"
 
-#include "macdrv.h"
 #define OEMRESOURCE
+#include "macdrv.h"
 #include "winuser.h"
 #include "winreg.h"
 #include "wine/server.h"
@@ -165,7 +165,7 @@ static void send_mouse_input(HWND hwnd, macdrv_window cocoa_window, UINT flags,
     input.mi.time           = time;
     input.mi.dwExtraInfo    = 0;
 
-    __wine_send_input(top_level_hwnd, &input);
+    __wine_send_input(top_level_hwnd, &input, NULL);
 }
 
 
@@ -262,7 +262,7 @@ CFArrayRef create_monochrome_cursor(HDC hdc, const ICONINFOEXW *icon, int width,
     BITMAPINFO *info = (BITMAPINFO *)buffer;
     unsigned int width_bytes = (width + 31) / 32 * 4;
     unsigned long *and_bits = NULL, *xor_bits;
-    unsigned long *data_bits;
+    unsigned long * HOSTPTR data_bits;
     int count, i;
     CGColorSpaceRef colorspace;
     CFMutableDataRef data;
@@ -347,7 +347,7 @@ CFArrayRef create_monochrome_cursor(HDC hdc, const ICONINFOEXW *icon, int width,
     /* image data = AND mask */
     CFDataAppendBytes(data, (UInt8*)and_bits, info->bmiHeader.biSizeImage / 2);
     /* image data ^= XOR mask */
-    data_bits = (unsigned long*)CFDataGetMutableBytePtr(data);
+    data_bits = (unsigned long* HOSTPTR)CFDataGetMutableBytePtr(data);
     count = (info->bmiHeader.biSizeImage / 2) / sizeof(*data_bits);
     for (i = 0; i < count; i++)
         data_bits[i] ^= xor_bits[i];
@@ -396,7 +396,7 @@ CFArrayRef create_monochrome_cursor(HDC hdc, const ICONINFOEXW *icon, int width,
     /* mask data = AND mask */
     CFDataAppendBytes(data, (UInt8*)and_bits, info->bmiHeader.biSizeImage / 2);
     /* mask data &= ~XOR mask */
-    data_bits = (unsigned long*)CFDataGetMutableBytePtr(data);
+    data_bits = (unsigned long* HOSTPTR)CFDataGetMutableBytePtr(data);
     for (i = 0; i < count; i++)
         data_bits[i] &= ~xor_bits[i];
     HeapFree(GetProcessHeap(), 0, and_bits);
@@ -439,7 +439,7 @@ CFArrayRef create_monochrome_cursor(HDC hdc, const ICONINFOEXW *icon, int width,
 
     values[0] = cgmasked;
     values[1] = hot_spot_dict;
-    frame = CFDictionaryCreate(NULL, (const void**)keys, values, ARRAY_SIZE(keys),
+    frame = CFDictionaryCreate(NULL, (const void* HOSTPTR *)keys, values, ARRAY_SIZE(keys),
                                &kCFCopyStringDictionaryKeyCallBacks,
                                &kCFTypeDictionaryValueCallBacks);
     CFRelease(hot_spot_dict);
@@ -450,7 +450,7 @@ CFArrayRef create_monochrome_cursor(HDC hdc, const ICONINFOEXW *icon, int width,
         return NULL;
     }
 
-    frames = CFArrayCreate(NULL, (const void**)&frame, 1, &kCFTypeArrayCallBacks);
+    frames = CFArrayCreate(NULL, (const void* HOSTPTR *)&frame, 1, &kCFTypeArrayCallBacks);
     CFRelease(frame);
     if (!frames)
     {
@@ -643,7 +643,7 @@ cleanup:
 /***********************************************************************
  *              DestroyCursorIcon (MACDRV.@)
  */
-void CDECL macdrv_DestroyCursorIcon(HCURSOR cursor)
+void macdrv_DestroyCursorIcon(HCURSOR cursor)
 {
     TRACE("cursor %p\n", cursor);
 
@@ -659,7 +659,7 @@ void CDECL macdrv_DestroyCursorIcon(HCURSOR cursor)
  *
  * Set the cursor clipping rectangle.
  */
-BOOL CDECL macdrv_ClipCursor(LPCRECT clip)
+BOOL macdrv_ClipCursor(LPCRECT clip)
 {
     CGRect rect;
 
@@ -683,7 +683,7 @@ BOOL CDECL macdrv_ClipCursor(LPCRECT clip)
 /***********************************************************************
  *              GetCursorPos (MACDRV.@)
  */
-BOOL CDECL macdrv_GetCursorPos(LPPOINT pos)
+BOOL macdrv_GetCursorPos(LPPOINT pos)
 {
     CGPoint pt;
     BOOL ret;
@@ -702,7 +702,7 @@ BOOL CDECL macdrv_GetCursorPos(LPPOINT pos)
 /***********************************************************************
  *              SetCapture (MACDRV.@)
  */
- void CDECL macdrv_SetCapture(HWND hwnd, UINT flags)
+ void macdrv_SetCapture(HWND hwnd, UINT flags)
 {
     struct macdrv_thread_data *thread_data = macdrv_thread_data();
     HWND top = GetAncestor(hwnd, GA_ROOT);
@@ -720,7 +720,7 @@ BOOL CDECL macdrv_GetCursorPos(LPPOINT pos)
 /***********************************************************************
  *              SetCursor (MACDRV.@)
  */
-void CDECL macdrv_SetCursor(HCURSOR cursor)
+void macdrv_SetCursor(HCURSOR cursor)
 {
     CFStringRef cursor_name = NULL;
     CFArrayRef cursor_frames = NULL;
@@ -813,7 +813,7 @@ done:
 /***********************************************************************
  *              SetCursorPos (MACDRV.@)
  */
-BOOL CDECL macdrv_SetCursorPos(INT x, INT y)
+BOOL macdrv_SetCursorPos(INT x, INT y)
 {
     BOOL ret = macdrv_set_cursor_position(CGPointMake(x, y));
     if (ret)
