@@ -64,16 +64,14 @@
  */
 
 #include "config.h"
-#include "wine/port.h"
 
 #include <string.h>
 #include <stdarg.h>
 #include <stdio.h>
 #include <stdlib.h>
-#ifdef HAVE_UNISTD_H
-# include <unistd.h>
-#endif
+#include <unistd.h>
 #include <fcntl.h>
+#include <dlfcn.h>
 #include <limits.h>
 #include <time.h>
 #include <assert.h>
@@ -87,7 +85,6 @@
 #include "shlobj.h"
 #include "shellapi.h"
 #include "shlwapi.h"
-#include "wine/library.h"
 #include "wine/list.h"
 #include "wine/debug.h"
 #include "wine/unicode.h"
@@ -96,7 +93,7 @@ WINE_DEFAULT_DEBUG_CHANNEL(clipboard);
 
 /* Maximum wait time for selection notify */
 #define SELECTION_RETRIES 500  /* wait for .5 seconds */
-#define SELECTION_WAIT    1000 /* us */
+#define SELECTION_WAIT    1    /* ms */
 
 #define SELECTION_UPDATE_DELAY 2000   /* delay between checks of the X11 selection */
 
@@ -459,7 +456,7 @@ static BOOL convert_selection( Display *display, Window win, Atom selection,
         Bool res = XCheckTypedWindowEvent( display, win, SelectionNotify, &event );
         if (res && event.xselection.selection == selection && event.xselection.target == format->atom)
             return read_property( display, win, event.xselection.property, type, data, size );
-        usleep( SELECTION_WAIT );
+        Sleep( SELECTION_WAIT );
     }
     ERR( "Timed out waiting for SelectionNotify event\n" );
     return FALSE;
@@ -674,7 +671,7 @@ static HANDLE unicode_text_from_string( UINT codepage, const void *data, size_t
     MultiByteToWideChar( codepage, 0, data, size, strW + count, count );
     for (i = j = 0; i < count; i++)
     {
-        if (strW[i + count] == '\n') strW[j++] = '\r';
+        if (strW[i + count] == '\n' && (!i || strW[i + count - 1] != '\r')) strW[j++] = '\r';
         strW[j++] = strW[i + count];
     }
     strW[j++] = 0;
@@ -1054,7 +1051,7 @@ static HANDLE import_selection( Display *display, Window win, Atom selection,
  */
 void X11DRV_CLIPBOARD_ImportSelection( Display *display, Window win, Atom selection,
                                        Atom *targets, UINT count,
-                                       void (*callback)( Atom, UINT, HANDLE ))
+                                       void (*callback)( UINT, HANDLE ))
 {
     UINT i;
     HANDLE handle;
@@ -1067,7 +1064,7 @@ void X11DRV_CLIPBOARD_ImportSelection( Display *display, Window win, Atom select
         if (!(format = find_x11_format( targets[i] ))) continue;
         if (!format->id) continue;
         if (!(handle = import_selection( display, win, selection, format ))) continue;
-        callback( targets[i], format->id, handle );
+        callback( format->id, handle );
     }
 }
 
@@ -1320,8 +1317,8 @@ static BOOL export_text_html( Display *display, Window win, Atom prop, Atom targ
          * any classes/styles defined outside the fragment are in the text/html
          * export. Ideally, we should parse the HTML to find them and ensure that
          * they are included but that "real" elements outside the fragment are not. */
-        if (!strncmp( p, "StartHTML:", 14 )) start = atoi( p + 14 );
-        else if (!strncmp( p, "EndHTML:", 12 )) end = atoi( p + 12 );
+        if (!strncmp( p, "StartHTML:", 10 )) start = atoi( p + 10 );
+        else if (!strncmp( p, "EndHTML:", 8 )) end = atoi( p + 8 );
         if (!(p = strpbrk( p, "\r\n" ))) break;
         while (*p == '\r' || *p == '\n') p++;
     }
@@ -1745,7 +1742,7 @@ static BOOL read_property( Display *display, Window w, Atom prop,
                 if (res && xe.xproperty.atom == prop &&
                     xe.xproperty.state == PropertyNewValue)
                     break;
-                usleep(SELECTION_WAIT);
+                Sleep(SELECTION_WAIT);
             }
 
             if (i >= SELECTION_RETRIES ||
@@ -1902,13 +1899,12 @@ static BOOL request_selection_contents( Display *display, BOOL changed )
                last_size != size ||
                memcmp( last_data, data, size ));
 
-    if (!changed)
+    if (!changed || !OpenClipboard( clipboard_hwnd ))
     {
         HeapFree( GetProcessHeap(), 0, data );
         return FALSE;
     }
 
-    if (!OpenClipboard( clipboard_hwnd )) return FALSE;
     TRACE( "selection changed, importing\n" );
     EmptyClipboard();
     is_clipboard_owner = TRUE;
@@ -2040,14 +2036,14 @@ static void xfixes_init(void)
     int major = 3, minor = 0;
     void *handle;
 
-    handle = wine_dlopen(SONAME_LIBXFIXES, RTLD_NOW, NULL, 0);
+    handle = dlopen(SONAME_LIBXFIXES, RTLD_NOW);
     if (!handle) return;
 
-    pXFixesQueryExtension = wine_dlsym(handle, "XFixesQueryExtension", NULL, 0);
+    pXFixesQueryExtension = dlsym(handle, "XFixesQueryExtension");
     if (!pXFixesQueryExtension) return;
-    pXFixesQueryVersion = wine_dlsym(handle, "XFixesQueryVersion", NULL, 0);
+    pXFixesQueryVersion = dlsym(handle, "XFixesQueryVersion");
     if (!pXFixesQueryVersion) return;
-    pXFixesSelectSelectionInput = wine_dlsym(handle, "XFixesSelectSelectionInput", NULL, 0);
+    pXFixesSelectSelectionInput = dlsym(handle, "XFixesSelectSelectionInput");
     if (!pXFixesSelectSelectionInput) return;
 
     if (!pXFixesQueryExtension(clipboard_display, &event_base, &error_base))
@@ -2119,9 +2115,8 @@ static DWORD WINAPI clipboard_thread( void *arg )
     clipboard_thread_id = GetCurrentThreadId();
     AddClipboardFormatListener( clipboard_hwnd );
     register_builtin_formats();
-    request_selection_contents( clipboard_display, TRUE );
-
     xfixes_init();
+    request_selection_contents( clipboard_display, TRUE );
 
     TRACE( "clipboard thread %04x running\n", GetCurrentThreadId() );
     while (GetMessageW( &msg, 0, 0, 0 )) DispatchMessageW( &msg );
@@ -2132,7 +2127,7 @@ static DWORD WINAPI clipboard_thread( void *arg )
 /**************************************************************************
  *		X11DRV_UpdateClipboard
  */
-void CDECL X11DRV_UpdateClipboard(void)
+void X11DRV_UpdateClipboard(void)
 {
     static ULONG last_update;
     ULONG now;
