@@ -20,19 +20,15 @@
  */
 
 #include "config.h"
-#include "wine/port.h"
 
 #include <fcntl.h>
 #include <stdarg.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#ifdef HAVE_SYS_TIME_H
-# include <sys/time.h>
-#endif
-#ifdef HAVE_UNISTD_H
-# include <unistd.h>
-#endif
+#include <sys/time.h>
+#include <unistd.h>
+#include <dlfcn.h>
 #include <X11/cursorfont.h>
 #include <X11/Xlib.h>
 #ifdef HAVE_XKB
@@ -42,6 +38,8 @@
 #include <X11/extensions/Xrender.h>
 #endif
 
+#include "ntstatus.h"
+#define WIN32_NO_STATUS
 #include "windef.h"
 #include "winbase.h"
 #include "winreg.h"
@@ -51,7 +49,8 @@
 #include "wine/server.h"
 #include "wine/unicode.h"
 #include "wine/debug.h"
-#include "wine/library.h"
+#include "wine/list.h"
+#include "wine/heap.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(x11drv);
 WINE_DECLARE_DEBUG_CHANNEL(synchronous);
@@ -82,7 +81,6 @@ BOOL client_side_with_render = TRUE;
 BOOL shape_layered_windows = TRUE;
 int copy_default_colors = 128;
 int alloc_system_colors = 256;
-DWORD thread_data_tls_index = TLS_OUT_OF_INDEXES;
 int xrender_error_base = 0;
 HMODULE x11drv_module = 0;
 char *process_name = NULL;
@@ -94,7 +92,20 @@ static int err_callback_result;              /* error callback result */
 static unsigned long err_serial;             /* serial number of first request */
 static int (*old_error_handler)( Display *, XErrorEvent * );
 static BOOL use_xim = TRUE;
-static char input_style[20];
+static WCHAR input_style[20];
+
+static pthread_mutex_t d3dkmt_mutex = PTHREAD_MUTEX_INITIALIZER;
+static pthread_mutex_t error_mutex = PTHREAD_MUTEX_INITIALIZER;
+
+struct d3dkmt_vidpn_source
+{
+    D3DKMT_VIDPNSOURCEOWNER_TYPE type;      /* VidPN source owner type */
+    D3DDDI_VIDEO_PRESENT_SOURCE_ID id;      /* VidPN present source id */
+    D3DKMT_HANDLE device;                   /* Kernel mode device context */
+    struct list entry;                      /* List entry */
+};
+
+static struct list d3dkmt_vidpn_sources = LIST_INIT( d3dkmt_vidpn_sources );   /* VidPN source information list */
 
 #define IS_OPTION_TRUE(ch) \
     ((ch) == 'y' || (ch) == 'Y' || (ch) == 't' || (ch) == 'T' || (ch) == '1')
@@ -107,6 +118,7 @@ static const char * const atom_names[NB_XATOMS - FIRST_XATOM] =
 {
     "CLIPBOARD",
     "COMPOUND_TEXT",
+    "EDID",
     "INCR",
     "MANAGER",
     "MULTIPLE",
@@ -155,6 +167,7 @@ static const char * const atom_names[NB_XATOMS - FIRST_XATOM] =
     "_NET_WM_WINDOW_TYPE_NORMAL",
     "_NET_WM_WINDOW_TYPE_UTILITY",
     "_NET_WORKAREA",
+    "_GTK_WORKAREAS_D0",
     "_XEMBED",
     "_XEMBED_INFO",
     "XdndAware",
@@ -188,9 +201,7 @@ static const char * const atom_names[NB_XATOMS - FIRST_XATOM] =
     "text/plain",
     "text/rtf",
     "text/richtext",
-    "text/uri-list",
-    "_CX_WORKAREA", /* CodeWeavers Hack bug 5752 */
-    "_CX_APPLEWM_TAG", /* CodeWeavers Hack bug 9517 */
+    "text/uri-list"
 };
 
 /***********************************************************************
@@ -235,6 +246,7 @@ static inline BOOL ignore_error( Display *display, XErrorEvent *event )
  */
 void X11DRV_expect_error( Display *display, x11drv_error_callback callback, void *arg )
 {
+    pthread_mutex_lock( &error_mutex );
     err_callback         = callback;
     err_callback_display = display;
     err_callback_arg     = arg;
@@ -251,8 +263,10 @@ void X11DRV_expect_error( Display *display, x11drv_error_callback callback, void
  */
 int X11DRV_check_error(void)
 {
+    int res = err_callback_result;
     err_callback = NULL;
-    return err_callback_result;
+    pthread_mutex_unlock( &error_mutex );
+    return res;
 }
 
 
@@ -309,16 +323,98 @@ static void init_pixmap_formats( Display *display )
 }
 
 
+HKEY reg_open_key( HKEY root, const WCHAR *name, ULONG name_len )
+{
+    UNICODE_STRING nameW = { name_len, name_len, (WCHAR *)name };
+    OBJECT_ATTRIBUTES attr;
+    HANDLE ret;
+
+    attr.Length = sizeof(attr);
+    attr.RootDirectory = root;
+    attr.ObjectName = &nameW;
+    attr.Attributes = 0;
+    attr.SecurityDescriptor = NULL;
+    attr.SecurityQualityOfService = NULL;
+
+    return NtOpenKeyEx( &ret, MAXIMUM_ALLOWED, &attr, 0 ) ? 0 : ret;
+}
+
+
+HKEY open_hkcu_key( const char *name )
+{
+    WCHAR bufferW[256];
+    static HKEY hkcu;
+
+    if (!hkcu)
+    {
+        char buffer[256];
+        DWORD_PTR sid_data[(sizeof(TOKEN_USER) + SECURITY_MAX_SID_SIZE) / sizeof(DWORD_PTR)];
+        DWORD i, len = sizeof(sid_data);
+        SID *sid;
+
+        if (NtQueryInformationToken( GetCurrentThreadEffectiveToken(), TokenUser, sid_data, len, &len ))
+            return 0;
+
+        sid = ((TOKEN_USER *)sid_data)->User.Sid;
+        len = sprintf( buffer, "\\Registry\\User\\S-%u-%u", sid->Revision,
+                       MAKELONG( MAKEWORD( sid->IdentifierAuthority.Value[5],
+                                           sid->IdentifierAuthority.Value[4] ),
+                                 MAKEWORD( sid->IdentifierAuthority.Value[3],
+                                           sid->IdentifierAuthority.Value[2] )));
+        for (i = 0; i < sid->SubAuthorityCount; i++)
+            len += sprintf( buffer + len, "-%u", sid->SubAuthority[i] );
+
+        ascii_to_unicode( bufferW, buffer, len );
+        hkcu = reg_open_key( NULL, bufferW, len * sizeof(WCHAR) );
+    }
+
+    return reg_open_key( hkcu, bufferW, asciiz_to_unicode( bufferW, name ) - sizeof(WCHAR) );
+}
+
+
+ULONG query_reg_value( HKEY hkey, const WCHAR *name, KEY_VALUE_PARTIAL_INFORMATION *info, ULONG size )
+{
+    unsigned int name_size = name ? lstrlenW( name ) * sizeof(WCHAR) : 0;
+    UNICODE_STRING nameW = { name_size, name_size, (WCHAR *)name };
+
+    if (NtQueryValueKey( hkey, &nameW, KeyValuePartialInformation,
+                         info, size, &size ))
+        return 0;
+
+    return size - FIELD_OFFSET(KEY_VALUE_PARTIAL_INFORMATION, Data);
+}
+
+
 /***********************************************************************
  *		get_config_key
  *
  * Get a config key from either the app-specific or the default config
  */
 static inline DWORD get_config_key( HKEY defkey, HKEY appkey, const char *name,
-                                    char *buffer, DWORD size )
+                                    WCHAR *buffer, DWORD size )
 {
-    if (appkey && !RegQueryValueExA( appkey, name, 0, NULL, (LPBYTE)buffer, &size )) return 0;
-    if (defkey && !RegQueryValueExA( defkey, name, 0, NULL, (LPBYTE)buffer, &size )) return 0;
+    WCHAR nameW[128];
+    char buf[2048];
+    KEY_VALUE_PARTIAL_INFORMATION *info = (void *)buf;
+
+    asciiz_to_unicode( nameW, name );
+
+    if (appkey && query_reg_value( appkey, nameW, info, sizeof(buf) ))
+    {
+        size = min( info->DataLength, size - sizeof(WCHAR) );
+        memcpy( buffer, info->Data, size );
+        buffer[size / sizeof(WCHAR)] = 0;
+        return 0;
+    }
+
+    if (defkey && query_reg_value( defkey, nameW, info, sizeof(buf) ))
+    {
+        size = min( info->DataLength, size - sizeof(WCHAR) );
+        memcpy( buffer, info->Data, size );
+        buffer[size / sizeof(WCHAR)] = 0;
+        return 0;
+    }
+
     return ERROR_FILE_NOT_FOUND;
 }
 
@@ -330,22 +426,23 @@ static inline DWORD get_config_key( HKEY defkey, HKEY appkey, const char *name,
  */
 static void setup_options(void)
 {
+    static const WCHAR steamwebhelperW[] = {'s','t','e','a','m','w','e','b','h','e','l','p','e','r','.','e','x','e',0};
+    static const WCHAR steamW[] = {'s','t','e','a','m','.','e','x','e',0};
     static const WCHAR x11driverW[] = {'\\','X','1','1',' ','D','r','i','v','e','r',0};
-    char buffer[64];
-    WCHAR bufferW[MAX_PATH+16];
+    WCHAR buffer[MAX_PATH+16];
     HKEY hkey, appkey = 0;
     DWORD len;
 
     /* @@ Wine registry key: HKCU\Software\Wine\X11 Driver */
-    if (RegOpenKeyA( HKEY_CURRENT_USER, "Software\\Wine\\X11 Driver", &hkey )) hkey = 0;
+    hkey = open_hkcu_key( "Software\\Wine\\X11 Driver" );
 
     /* open the app-specific key */
 
-    len = (GetModuleFileNameW( 0, bufferW, MAX_PATH ));
+    len = GetModuleFileNameW( 0, buffer, MAX_PATH );
     if (len && len < MAX_PATH)
     {
         HKEY tmpkey;
-        WCHAR *p, *appname = bufferW;
+        WCHAR *p, *appname = buffer;
         if ((p = strrchrW( appname, '/' ))) appname = p + 1;
         if ((p = strrchrW( appname, '\\' ))) appname = p + 1;
         CharLowerW(appname);
@@ -354,11 +451,14 @@ static void setup_options(void)
             WideCharToMultiByte( CP_UNIXCP, 0, appname, -1, process_name, len, NULL, NULL );
         strcatW( appname, x11driverW );
         /* @@ Wine registry key: HKCU\Software\Wine\AppDefaults\app.exe\X11 Driver */
-        if (!RegOpenKeyA( HKEY_CURRENT_USER, "Software\\Wine\\AppDefaults", &tmpkey ))
+        if ((tmpkey = open_hkcu_key( "Software\\Wine\\AppDefaults" )))
         {
-            if (RegOpenKeyW( tmpkey, appname, &appkey )) appkey = 0;
-            RegCloseKey( tmpkey );
+            appkey = reg_open_key( tmpkey, appname, lstrlenW( appname ) * sizeof(WCHAR) );
+            NtClose( tmpkey );
         }
+
+        if (!lstrcmpW(appname, steamwebhelperW) || !strcmpW(appname, steamW))
+            enable_shm_surface = TRUE;
     }
 
     if (!get_config_key( hkey, appkey, "Managed", buffer, sizeof(buffer) ))
@@ -392,7 +492,7 @@ static void setup_options(void)
         grab_fullscreen = IS_OPTION_TRUE( buffer[0] );
 
     if (!get_config_key( hkey, appkey, "ScreenDepth", buffer, sizeof(buffer) ))
-        default_visual.depth = atoi(buffer);
+        default_visual.depth = strtolW( buffer, NULL, 0 );
 
     if (!get_config_key( hkey, appkey, "ClientSideGraphics", buffer, sizeof(buffer) ))
         client_side_graphics = IS_OPTION_TRUE( buffer[0] );
@@ -410,13 +510,13 @@ static void setup_options(void)
         private_color_map = IS_OPTION_TRUE( buffer[0] );
 
     if (!get_config_key( hkey, appkey, "PrimaryMonitor", buffer, sizeof(buffer) ))
-        primary_monitor = atoi( buffer );
+        primary_monitor = strtolW( buffer, NULL, 0 );
 
     if (!get_config_key( hkey, appkey, "CopyDefaultColors", buffer, sizeof(buffer) ))
-        copy_default_colors = atoi(buffer);
+        copy_default_colors = strtolW( buffer, NULL, 0 );
 
     if (!get_config_key( hkey, appkey, "AllocSystemColors", buffer, sizeof(buffer) ))
-        alloc_system_colors = atoi(buffer);
+        alloc_system_colors = strtolW( buffer, NULL, 0 );
 
     get_config_key( hkey, appkey, "InputStyle", input_style, sizeof(input_style) );
 
@@ -429,8 +529,8 @@ static void setup_options(void)
         }
     }
 
-    if (appkey) RegCloseKey( appkey );
-    if (hkey) RegCloseKey( hkey );
+    NtClose( appkey );
+    NtClose( hkey );
 }
 
 #ifdef SONAME_LIBXCOMPOSITE
@@ -452,7 +552,7 @@ static int xcomp_error_base;
 
 static void X11DRV_XComposite_Init(void)
 {
-    void *xcomposite_handle = wine_dlopen(SONAME_LIBXCOMPOSITE, RTLD_NOW, NULL, 0);
+    void *xcomposite_handle = dlopen(SONAME_LIBXCOMPOSITE, RTLD_NOW);
     if (!xcomposite_handle)
     {
         TRACE("Unable to open %s, XComposite disabled\n", SONAME_LIBXCOMPOSITE);
@@ -461,23 +561,22 @@ static void X11DRV_XComposite_Init(void)
     }
 
 #define LOAD_FUNCPTR(f) \
-    if((p##f = wine_dlsym(xcomposite_handle, #f, NULL, 0)) == NULL) \
-        goto sym_not_found;
-    LOAD_FUNCPTR(XCompositeQueryExtension)
-    LOAD_FUNCPTR(XCompositeQueryVersion)
-    LOAD_FUNCPTR(XCompositeVersion)
-    LOAD_FUNCPTR(XCompositeRedirectWindow)
-    LOAD_FUNCPTR(XCompositeRedirectSubwindows)
-    LOAD_FUNCPTR(XCompositeUnredirectWindow)
-    LOAD_FUNCPTR(XCompositeUnredirectSubwindows)
-    LOAD_FUNCPTR(XCompositeCreateRegionFromBorderClip)
-    LOAD_FUNCPTR(XCompositeNameWindowPixmap)
+    if((p##f = dlsym(xcomposite_handle, #f)) == NULL) goto sym_not_found
+    LOAD_FUNCPTR(XCompositeQueryExtension);
+    LOAD_FUNCPTR(XCompositeQueryVersion);
+    LOAD_FUNCPTR(XCompositeVersion);
+    LOAD_FUNCPTR(XCompositeRedirectWindow);
+    LOAD_FUNCPTR(XCompositeRedirectSubwindows);
+    LOAD_FUNCPTR(XCompositeUnredirectWindow);
+    LOAD_FUNCPTR(XCompositeUnredirectSubwindows);
+    LOAD_FUNCPTR(XCompositeCreateRegionFromBorderClip);
+    LOAD_FUNCPTR(XCompositeNameWindowPixmap);
 #undef LOAD_FUNCPTR
 
     if(!pXCompositeQueryExtension(gdi_display, &xcomp_event_base,
                                   &xcomp_error_base)) {
         TRACE("XComposite extension could not be queried; disabled\n");
-        wine_dlclose(xcomposite_handle, NULL, 0);
+        dlclose(xcomposite_handle);
         xcomposite_handle = NULL;
         usexcomposite = FALSE;
         return;
@@ -487,7 +586,7 @@ static void X11DRV_XComposite_Init(void)
 
 sym_not_found:
     TRACE("Unable to load function pointers from %s, XComposite disabled\n", SONAME_LIBXCOMPOSITE);
-    wine_dlclose(xcomposite_handle, NULL, 0);
+    dlclose(xcomposite_handle);
     xcomposite_handle = NULL;
     usexcomposite = FALSE;
 }
@@ -551,25 +650,22 @@ static void init_visuals( Display *display, int screen )
  */
 static BOOL process_attach(void)
 {
-    char error[1024];
     Display *display;
-    void *libx11 = wine_dlopen( SONAME_LIBX11, RTLD_NOW|RTLD_GLOBAL, error, sizeof(error) );
+    void *libx11 = dlopen( SONAME_LIBX11, RTLD_NOW|RTLD_GLOBAL );
 
     if (!libx11)
     {
-        ERR( "failed to load %s: %s\n", SONAME_LIBX11, error );
+        ERR( "failed to load %s: %s\n", SONAME_LIBX11, dlerror() );
         return FALSE;
     }
-    pXGetEventData = wine_dlsym( libx11, "XGetEventData", NULL, 0 );
-    pXFreeEventData = wine_dlsym( libx11, "XFreeEventData", NULL, 0 );
+    pXGetEventData = dlsym( libx11, "XGetEventData" );
+    pXFreeEventData = dlsym( libx11, "XFreeEventData" );
 #ifdef SONAME_LIBXEXT
-    wine_dlopen( SONAME_LIBXEXT, RTLD_NOW|RTLD_GLOBAL, NULL, 0 );
+    dlopen( SONAME_LIBXEXT, RTLD_NOW|RTLD_GLOBAL );
 #endif
 
     setup_options();
 
-    if ((thread_data_tls_index = TlsAlloc()) == TLS_OUT_OF_INDEXES) return FALSE;
-
     /* Open display */
 
     if (!XInitThreads()) ERR( "XInitThreads failed, trouble ahead\n" );
@@ -586,9 +682,7 @@ static BOOL process_attach(void)
 
     XInternAtoms( display, (char **)atom_names, NB_XATOMS - FIRST_XATOM, False, X11DRV_Atoms );
 
-    winContext = XUniqueContext();
-    win_data_context = XUniqueContext();
-    cursor_context = XUniqueContext();
+    init_win_context();
 
     if (TRACE_ON(synchronous)) XSynchronize( display, True );
 
@@ -611,6 +705,8 @@ static BOOL process_attach(void)
     X11DRV_InitKeyboard( gdi_display );
     if (use_xim) use_xim = X11DRV_InitXIM( input_style );
 
+    init_user_driver();
+    X11DRV_DisplayDevices_Init(FALSE);
     return TRUE;
 }
 
@@ -618,18 +714,19 @@ static BOOL process_attach(void)
 /***********************************************************************
  *           ThreadDetach (X11DRV.@)
  */
-void CDECL X11DRV_ThreadDetach(void)
+void X11DRV_ThreadDetach(void)
 {
-    struct x11drv_thread_data *data = TlsGetValue( thread_data_tls_index );
+    struct x11drv_thread_data *data = x11drv_thread_data();
 
     if (data)
     {
+        vulkan_thread_detach();
         if (data->xim) XCloseIM( data->xim );
         if (data->font_set) XFreeFontSet( data->display, data->font_set );
         XCloseDisplay( data->display );
         HeapFree( GetProcessHeap(), 0, data );
         /* clear data in case we get re-entered from user32 before the thread is truly dead */
-        TlsSetValue( thread_data_tls_index, NULL );
+        NtUserGetThreadInfo()->driver_data = NULL;
     }
 }
 
@@ -690,7 +787,7 @@ struct x11drv_thread_data *x11drv_init_thread_data(void)
     if (TRACE_ON(synchronous)) XSynchronize( data->display, True );
 
     set_queue_display_fd( data->display );
-    TlsSetValue( thread_data_tls_index, data );
+    NtUserGetThreadInfo()->driver_data = data;
 
     if (use_xim) X11DRV_SetupXIM();
 
@@ -720,7 +817,7 @@ BOOL WINAPI DllMain( HINSTANCE hinst, DWORD reason, LPVOID reserved )
 /***********************************************************************
  *              SystemParametersInfo (X11DRV.@)
  */
-BOOL CDECL X11DRV_SystemParametersInfo( UINT action, UINT int_param, void *ptr_param, UINT flags )
+BOOL X11DRV_SystemParametersInfo( UINT action, UINT int_param, void *ptr_param, UINT flags )
 {
     switch (action)
     {
@@ -752,3 +849,143 @@ BOOL CDECL X11DRV_SystemParametersInfo( UINT action, UINT int_param, void *ptr_p
     }
     return FALSE;  /* let user32 handle it */
 }
+
+/**********************************************************************
+ *           X11DRV_D3DKMTSetVidPnSourceOwner
+ */
+NTSTATUS CDECL X11DRV_D3DKMTSetVidPnSourceOwner( const D3DKMT_SETVIDPNSOURCEOWNER *desc )
+{
+    struct d3dkmt_vidpn_source *source, *source2;
+    NTSTATUS status = STATUS_SUCCESS;
+    BOOL found;
+    UINT i;
+
+    TRACE("(%p)\n", desc);
+
+    pthread_mutex_lock( &d3dkmt_mutex );
+
+    /* Check parameters */
+    for (i = 0; i < desc->VidPnSourceCount; ++i)
+    {
+        LIST_FOR_EACH_ENTRY( source, &d3dkmt_vidpn_sources, struct d3dkmt_vidpn_source, entry )
+        {
+            if (source->id == desc->pVidPnSourceId[i])
+            {
+                /* Same device */
+                if (source->device == desc->hDevice)
+                {
+                    if ((source->type == D3DKMT_VIDPNSOURCEOWNER_EXCLUSIVE
+                         && (desc->pType[i] == D3DKMT_VIDPNSOURCEOWNER_SHARED
+                             || desc->pType[i] == D3DKMT_VIDPNSOURCEOWNER_EMULATED))
+                        || (source->type == D3DKMT_VIDPNSOURCEOWNER_EMULATED
+                            && desc->pType[i] == D3DKMT_VIDPNSOURCEOWNER_EXCLUSIVE))
+                    {
+                        status = STATUS_INVALID_PARAMETER;
+                        goto done;
+                    }
+                }
+                /* Different devices */
+                else
+                {
+                    if ((source->type == D3DKMT_VIDPNSOURCEOWNER_EXCLUSIVE
+                         || source->type == D3DKMT_VIDPNSOURCEOWNER_EMULATED)
+                        && (desc->pType[i] == D3DKMT_VIDPNSOURCEOWNER_EXCLUSIVE
+                            || desc->pType[i] == D3DKMT_VIDPNSOURCEOWNER_EMULATED))
+                    {
+                        status = STATUS_GRAPHICS_VIDPN_SOURCE_IN_USE;
+                        goto done;
+                    }
+                }
+            }
+        }
+
+        /* On Windows, it seems that all video present sources are owned by DMM clients, so any attempt to set
+         * D3DKMT_VIDPNSOURCEOWNER_SHARED come back STATUS_GRAPHICS_VIDPN_SOURCE_IN_USE */
+        if (desc->pType[i] == D3DKMT_VIDPNSOURCEOWNER_SHARED)
+        {
+            status = STATUS_GRAPHICS_VIDPN_SOURCE_IN_USE;
+            goto done;
+        }
+
+        /* FIXME: D3DKMT_VIDPNSOURCEOWNER_EXCLUSIVEGDI unsupported */
+        if (desc->pType[i] == D3DKMT_VIDPNSOURCEOWNER_EXCLUSIVEGDI || desc->pType[i] > D3DKMT_VIDPNSOURCEOWNER_EMULATED)
+        {
+            status = STATUS_INVALID_PARAMETER;
+            goto done;
+        }
+    }
+
+    /* Remove owner */
+    if (!desc->VidPnSourceCount && !desc->pType && !desc->pVidPnSourceId)
+    {
+        LIST_FOR_EACH_ENTRY_SAFE( source, source2, &d3dkmt_vidpn_sources, struct d3dkmt_vidpn_source, entry )
+        {
+            if (source->device == desc->hDevice)
+            {
+                list_remove( &source->entry );
+                heap_free( source );
+            }
+        }
+        goto done;
+    }
+
+    /* Add owner */
+    for (i = 0; i < desc->VidPnSourceCount; ++i)
+    {
+        found = FALSE;
+        LIST_FOR_EACH_ENTRY( source, &d3dkmt_vidpn_sources, struct d3dkmt_vidpn_source, entry )
+        {
+            if (source->device == desc->hDevice && source->id == desc->pVidPnSourceId[i])
+            {
+                found = TRUE;
+                break;
+            }
+        }
+
+        if (found)
+            source->type = desc->pType[i];
+        else
+        {
+            source = heap_alloc( sizeof( *source ) );
+            if (!source)
+            {
+                status = STATUS_NO_MEMORY;
+                goto done;
+            }
+
+            source->id = desc->pVidPnSourceId[i];
+            source->type = desc->pType[i];
+            source->device = desc->hDevice;
+            list_add_tail( &d3dkmt_vidpn_sources, &source->entry );
+        }
+    }
+
+done:
+    pthread_mutex_unlock( &d3dkmt_mutex );
+    return status;
+}
+
+/**********************************************************************
+ *           X11DRV_D3DKMTCheckVidPnExclusiveOwnership
+ */
+NTSTATUS CDECL X11DRV_D3DKMTCheckVidPnExclusiveOwnership( const D3DKMT_CHECKVIDPNEXCLUSIVEOWNERSHIP *desc )
+{
+    struct d3dkmt_vidpn_source *source;
+
+    TRACE("(%p)\n", desc);
+
+    if (!desc || !desc->hAdapter)
+        return STATUS_INVALID_PARAMETER;
+
+    pthread_mutex_lock( &d3dkmt_mutex );
+    LIST_FOR_EACH_ENTRY( source, &d3dkmt_vidpn_sources, struct d3dkmt_vidpn_source, entry )
+    {
+        if (source->id == desc->VidPnSourceId && source->type == D3DKMT_VIDPNSOURCEOWNER_EXCLUSIVE)
+        {
+            pthread_mutex_unlock( &d3dkmt_mutex );
+            return STATUS_GRAPHICS_PRESENT_OCCLUDED;
+        }
+    }
+    pthread_mutex_unlock( &d3dkmt_mutex );
+    return STATUS_SUCCESS;
+}
