@@ -32,7 +32,6 @@
 #include <stdio.h>
 #include <assert.h>
 
-#include "wine/library.h"
 #include "windef.h"
 #include "winbase.h"
 #include "wininet.h"
@@ -82,7 +81,7 @@ static DWORD netconn_verify_cert(netconn_t *conn, PCCERT_CONTEXT cert, HCERTSTOR
         }
 
         if (chain->TrustStatus.dwErrorStatus & ~supportedErrors) {
-            WARN("error status %x\n", chain->TrustStatus.dwErrorStatus & ~supportedErrors);
+            WARN("error status %lx\n", chain->TrustStatus.dwErrorStatus & ~supportedErrors);
             err = conn->mask_errors && err ? ERROR_INTERNET_SEC_CERT_ERRORS : ERROR_INTERNET_SEC_INVALID_CERT;
             errors &= supportedErrors;
             if(!conn->mask_errors)
@@ -182,9 +181,9 @@ static DWORD netconn_verify_cert(netconn_t *conn, PCCERT_CONTEXT cert, HCERTSTOR
                     conn->security_flags |= _SECURITY_FLAG_CERT_INVALID_CN;
                 err = conn->mask_errors && err ? ERROR_INTERNET_SEC_CERT_ERRORS : ERROR_INTERNET_SEC_CERT_CN_INVALID;
             }else if(policyStatus.dwError) {
-                WARN("policyStatus.dwError %x\n", policyStatus.dwError);
+                WARN("policyStatus.dwError %lx\n", policyStatus.dwError);
                 if(conn->mask_errors)
-                    WARN("unknown error flags for policy status %x\n", policyStatus.dwError);
+                    WARN("unknown error flags for policy status %lx\n", policyStatus.dwError);
                 err = conn->mask_errors && err ? ERROR_INTERNET_SEC_CERT_ERRORS : ERROR_INTERNET_SEC_INVALID_CERT;
             }
         }else {
@@ -192,26 +191,8 @@ static DWORD netconn_verify_cert(netconn_t *conn, PCCERT_CONTEXT cert, HCERTSTOR
         }
     }
 
-    if (err) /* CrossOver hack tracked by bug 6776 */
-    {
-        HKEY hkey;
-        DWORD type, value, size = sizeof(value);
-
-        /* @@ Wine registry key: HKCU\Software\Wine\wininet */
-        if (!RegOpenKeyA( HKEY_CURRENT_USER, "Software\\Wine\\wininet", &hkey ))
-        {
-            if (!RegQueryValueExA( hkey, "accept_invalid_certs", 0, &type, (BYTE *)&value, &size ) &&
-                type == REG_DWORD && value)
-            {
-                WARN("certificate is invalid, accepting it anyway\n");
-                err = ERROR_SUCCESS;
-            }
-            RegCloseKey( hkey );
-        }
-    }
-
     if(err) {
-        WARN("failed %u\n", err);
+        WARN("failed %lu\n", err);
         CertFreeCertificateChain(chain);
         if(conn->server->cert_chain) {
             CertFreeCertificateChain(conn->server->cert_chain);
@@ -270,7 +251,7 @@ static BOOL ensure_cred_handle(void)
     LeaveCriticalSection(&init_sechandle_cs);
 
     if(res != SEC_E_OK) {
-        WARN("Failed: %08x\n", res);
+        WARN("Failed: %08lx\n", res);
         return FALSE;
     }
 
@@ -288,7 +269,7 @@ static BOOL WINAPI winsock_startup(INIT_ONCE *once, void *param, void **context)
     if(res == ERROR_SUCCESS)
         winsock_loaded = TRUE;
     else
-        ERR("WSAStartup failed: %u\n", res);
+        ERR("WSAStartup failed: %lu\n", res);
     return TRUE;
 }
 
@@ -493,7 +474,7 @@ static DWORD netcon_secure_connect_setup(netconn_t *connection, BOOL compat_mode
         if(out_buf.cbBuffer) {
             assert(status == SEC_I_CONTINUE_NEEDED);
 
-            TRACE("sending %u bytes\n", out_buf.cbBuffer);
+            TRACE("sending %lu bytes\n", out_buf.cbBuffer);
 
             size = sock_send(connection->socket, out_buf.pvBuffer, out_buf.cbBuffer, 0);
             if(size != out_buf.cbBuffer) {
@@ -541,13 +522,13 @@ static DWORD netcon_secure_connect_setup(netconn_t *connection, BOOL compat_mode
             break;
         }
 
-        TRACE("recv %lu bytes\n", size);
+        TRACE("recv %Iu bytes\n", size);
 
         in_bufs[0].cbBuffer += size;
         in_bufs[0].pvBuffer = read_buf;
         status = InitializeSecurityContextW(cred, &ctx, connection->server->name,  isc_req_flags, 0, 0, &in_desc,
                 0, NULL, &out_desc, &attrs, NULL);
-        TRACE("InitializeSecurityContext ret %08x\n", status);
+        TRACE("InitializeSecurityContext ret %08lx\n", status);
 
         if(status == SEC_E_OK) {
             if(SecIsValidHandle(&connection->ssl_ctx))
@@ -568,7 +549,7 @@ static DWORD netcon_secure_connect_setup(netconn_t *connection, BOOL compat_mode
                 res = netconn_verify_cert(connection, cert, cert->hCertStore);
                 CertFreeCertificateContext(cert);
                 if(res != ERROR_SUCCESS) {
-                    WARN("cert verify failed: %u\n", res);
+                    WARN("cert verify failed: %lu\n", res);
                     break;
                 }
             }else {
@@ -588,7 +569,7 @@ static DWORD netcon_secure_connect_setup(netconn_t *connection, BOOL compat_mode
     heap_free(read_buf);
 
     if(status != SEC_E_OK || res != ERROR_SUCCESS) {
-        WARN("Failed to establish SSL connection: %08x (%u)\n", status, res);
+        WARN("Failed to establish SSL connection: %08lx (%lu)\n", status, res);
         heap_free(connection->ssl_buf);
         connection->ssl_buf = NULL;
         return res ? res : ERROR_INTERNET_SECURITY_CHANNEL_ERROR;
@@ -794,7 +775,7 @@ static BOOL read_ssl_chunk(netconn_t *conn, void *buf, SIZE_T buf_size, BOOL blo
             buf_len += size;
             continue;
         default:
-            WARN("failed: %08x\n", res);
+            WARN("failed: %08lx\n", res);
             return ERROR_INTERNET_CONNECTION_ABORTED;
         }
     } while(res != SEC_E_OK);
@@ -881,7 +862,7 @@ DWORD NETCON_recv(netconn_t *connection, void *buf, size_t len, BOOL blocking, i
             }
         }while(!size && !eof);
 
-        TRACE("received %ld bytes\n", size);
+        TRACE("received %Id bytes\n", size);
         *recvd = size;
         return res;
     }
@@ -917,7 +898,7 @@ int NETCON_GetCipherStrength(netconn_t *connection)
 
     res = QueryContextAttributesW(&connection->ssl_ctx, SECPKG_ATTR_CONNECTION_INFO, (void*)&conn_info);
     if(res != SEC_E_OK)
-        WARN("QueryContextAttributesW failed: %08x\n", res);
+        WARN("QueryContextAttributesW failed: %08lx\n", res);
     return res == SEC_E_OK ? conn_info.dwCipherStrength : 0;
 }
 
