@@ -19,6 +19,8 @@
 #ifndef __WINE_D3D8_H
 #define __WINE_D3D8_H
 
+#include "wine/winheader_enter.h"
+
 #ifndef DIRECT3D_VERSION
 #define DIRECT3D_VERSION  0x0800
 #endif
@@ -1014,7 +1016,7 @@ DECLARE_INTERFACE_(IDirect3DDevice8,IUnknown)
 #define IDirect3DDevice8_DrawIndexedPrimitive(p,a,b,c,d,e)         (p)->lpVtbl->DrawIndexedPrimitive(p,a,b,c,d,e)
 #define IDirect3DDevice8_DrawPrimitiveUP(p,a,b,c,d)                (p)->lpVtbl->DrawPrimitiveUP(p,a,b,c,d)
 #define IDirect3DDevice8_DrawIndexedPrimitiveUP(p,a,b,c,d,e,f,g,h) (p)->lpVtbl->DrawIndexedPrimitiveUP(p,a,b,c,d,e,f,g,h)
-#define IDirect3DDevice8_ProcessVertices(p,a,b,c,d,e)              (p)->lpVtbl->processVertices(p,a,b,c,d,e)
+#define IDirect3DDevice8_ProcessVertices(p,a,b,c,d,e)              (p)->lpVtbl->ProcessVertices(p,a,b,c,d,e)
 #define IDirect3DDevice8_CreateVertexShader(p,a,b,c,d)             (p)->lpVtbl->CreateVertexShader(p,a,b,c,d)
 #define IDirect3DDevice8_SetVertexShader(p,a)                      (p)->lpVtbl->SetVertexShader(p,a)
 #define IDirect3DDevice8_GetVertexShader(p,a)                      (p)->lpVtbl->GetVertexShader(p,a)
@@ -1150,4 +1152,6 @@ IDirect3D8* WINAPI Direct3DCreate8(UINT SDKVersion);
 } /* extern "C" */
 #endif /* defined(__cplusplus) */
 
+#include "wine/winheader_exit.h"
+
 #endif /* __WINE_D3D8_H */
