@@ -19,6 +19,8 @@
 #ifndef _NTSECPKG_H
 #define _NTSECPKG_H
 
+#include "wine/winheader_enter.h"
+
 #ifdef __cplusplus
 extern "C" {
 #endif
@@ -38,6 +40,7 @@ extern "C" {
 #define SECPKG_INTERFACE_VERSION_5                  0x100000
 #define SECPKG_INTERFACE_VERSION_6                  0x200000
 #define SECPKG_INTERFACE_VERSION_7                  0x400000
+#define SECPKG_INTERFACE_VERSION_8                  0x800000
 
 /* enum definitions for Secure Service Provider/Authentication Packages */
 typedef enum _LSA_TOKEN_INFORMATION_TYPE {
@@ -305,6 +308,28 @@ typedef struct _LSA_SECPKG_FUNCTION_TABLE {
 } LSA_SECPKG_FUNCTION_TABLE,
  *PLSA_SECPKG_FUNCTION_TABLE;
 
+/* Functions for redirected credentials */
+typedef NTSTATUS (NTAPI *PLSA_REDIRECTED_LOGON_INIT)(HANDLE,
+ const UNICODE_STRING *, ULONG, const LUID *);
+typedef NTSTATUS (NTAPI *PLSA_REDIRECTED_LOGON_CALLBACK)(HANDLE, PVOID, ULONG,
+ PVOID *, ULONG *);
+typedef VOID (NTAPI *PLSA_REDIRECTED_LOGON_CLEANUP_CALLBACK)(HANDLE);
+typedef NTSTATUS (NTAPI *PLSA_REDIRECTED_LOGON_GET_LOGON_CREDS)(HANDLE, PBYTE *,
+ PULONG);
+typedef NTSTATUS (NTAPI *PLSA_REDIRECTED_LOGON_GET_SUPP_CREDS)(HANDLE,
+ PSECPKG_SUPPLEMENTAL_CRED_ARRAY *);
+
+typedef struct _SECPKG_REDIRECTED_LOGON_BUFFER {
+    GUID RedirectedLogonGuid;
+    HANDLE RedirectedLogonHandle;
+    PLSA_REDIRECTED_LOGON_INIT Init;
+    PLSA_REDIRECTED_LOGON_CALLBACK Callback;
+    PLSA_REDIRECTED_LOGON_CLEANUP_CALLBACK CleanupCallback;
+    PLSA_REDIRECTED_LOGON_GET_LOGON_CREDS GetLogonCreds;
+    PLSA_REDIRECTED_LOGON_GET_SUPP_CREDS GetSupplementalCreds;
+} SECPKG_REDIRECTED_LOGON_BUFFER,
+ *PSECPKG_REDIRECTED_LOGON_BUFFER;
+
 /* LSA-mode functions implemented by SSP/AP obtainable by a dispatch table */
 typedef NTSTATUS (NTAPI *PLSA_AP_INITIALIZE_PACKAGE)(ULONG, PLSA_DISPATCH_TABLE,
  PLSA_STRING, PLSA_STRING, PLSA_STRING *);
@@ -374,6 +399,12 @@ typedef NTSTATUS (NTAPI SpUpdateCredentialsFn)(LSA_SEC_HANDLE, GUID *, ULONG,
 typedef NTSTATUS (NTAPI SpValidateTargetInfoFn)(PLSA_CLIENT_REQUEST, PVOID,
  PVOID, ULONG, PSECPKG_TARGETINFO);
 typedef NTSTATUS (NTAPI LSA_AP_POST_LOGON_USER)(PSECPKG_POST_LOGON_USER_INFO);
+typedef NTSTATUS (NTAPI SpGetRemoteCredGuardLogonBufferFn)(LSA_SEC_HANDLE,
+ const UNICODE_STRING *, PHANDLE, PLSA_REDIRECTED_LOGON_CALLBACK *,
+ PLSA_REDIRECTED_LOGON_CLEANUP_CALLBACK *, PULONG, PVOID *);
+typedef NTSTATUS (NTAPI SpGetRemoteCredGuardSupplementalCredsFn)(LSA_SEC_HANDLE,
+ const UNICODE_STRING *, PHANDLE, PLSA_REDIRECTED_LOGON_CALLBACK *,
+ PLSA_REDIRECTED_LOGON_CLEANUP_CALLBACK *, PULONG, PVOID *);
 
 /* User-mode functions implemented by SSP/AP obtainable by a dispatch table */
 typedef NTSTATUS (NTAPI SpInstanceInitFn)(ULONG, PSECPKG_DLL_FUNCTIONS,
@@ -446,6 +477,9 @@ typedef struct SECPKG_FUNCTION_TABLE {
     /* Packages with version SECPKG_INTERFACE_VERSION_6 end here */
     LSA_AP_POST_LOGON_USER* PostLogonUser;
     /* Packages with version SECPKG_INTERFACE_VERSION_7 end here */
+    SpGetRemoteCredGuardLogonBufferFn* GetRemoteCredGuardLogonBuffer;
+    SpGetRemoteCredGuardSupplementalCredsFn* GetRemoteCredGuardSupplementalCreds;
+    /* Packages with version SECPKG_INTERFACE_VERSION_8 end here */
 } SECPKG_FUNCTION_TABLE,
  *PSECPKG_FUNCTION_TABLE;
 
@@ -479,4 +513,6 @@ typedef NTSTATUS (WINAPI *SpUserModeInitializeFn)(ULONG, PULONG,
 #ifdef __cplusplus
 }
 #endif
+#include "wine/winheader_exit.h"
+
 #endif /* _NTSECPKG_H */
