@@ -19,6 +19,8 @@
 #ifndef __WINE_SHLOBJ_H
 #define __WINE_SHLOBJ_H
 
+#include "wine/winheader_enter.h"
+
 #include <ole2.h>
 #include <commctrl.h>
 #include <prsht.h>
@@ -152,6 +154,30 @@ BOOL WINAPI SHObjectProperties(HWND,DWORD,LPCWSTR,LPCWSTR);
 
 int WINAPI PathCleanupSpec(LPCWSTR,LPWSTR);
 
+/* SHOpenWithDialog API */
+
+typedef enum
+{
+    OAIF_ALLOW_REGISTRATION = 0x00000001,
+    OAIF_REGISTER_EXT       = 0x00000002,
+    OAIF_EXEC               = 0x00000004,
+    OAIF_FORCE_REGISTRATION = 0x00000008,
+    OAIF_HIDE_REGISTRATION  = 0x00000020,
+    OAIF_URL_PROTOCOL       = 0x00000040,
+    OAIF_FILE_IS_URI        = 0x00000080
+} OPEN_AS_INFO_FLAGS;
+
+#include <pshpack8.h>
+typedef struct
+{
+    LPCWSTR pcszFile;
+    LPCWSTR pcszClass;
+    OPEN_AS_INFO_FLAGS oaifInFlags;
+} OPENASINFO;
+#include <poppack.h>
+
+HRESULT WINAPI SHOpenWithDialog(HWND,const OPENASINFO*);
+
 /* Shell_MergeMenus flags */
 #define MM_ADDSEPARATOR     0x00000001
 #define MM_SUBMENUSHAVEIDS  0x00000002
@@ -1351,30 +1377,39 @@ typedef struct _SHChangeNotifyEntry
 
 void WINAPI SHChangeNotify(LONG wEventId, UINT uFlags, LPCVOID dwItem1, LPCVOID dwItem2);
 
-typedef enum {
-    SLDF_HAS_ID_LIST = 0x00000001,
-    SLDF_HAS_LINK_INFO = 0x00000002,
-    SLDF_HAS_NAME = 0x00000004,
-    SLDF_HAS_RELPATH = 0x00000008,
-    SLDF_HAS_WORKINGDIR = 0x00000010,
-    SLDF_HAS_ARGS = 0x00000020,
-    SLDF_HAS_ICONLOCATION = 0x00000040,
-    SLDF_UNICODE = 0x00000080,
-    SLDF_FORCE_NO_LINKINFO = 0x00000100,
-    SLDF_HAS_EXP_SZ = 0x00000200,
-    SLDF_RUN_IN_SEPARATE = 0x00000400,
-    SLDF_HAS_LOGO3ID = 0x00000800,
-    SLDF_HAS_DARWINID = 0x00001000,
-    SLDF_RUNAS_USER = 0x00002000,
-    SLDF_HAS_EXP_ICON_SZ = 0x00004000,
-    SLDF_NO_PIDL_ALIAS = 0x00008000,
-    SLDF_FORCE_UNCNAME = 0x00010000,
-    SLDF_RUN_WITH_SHIMLAYER = 0x00020000,
-    SLDF_FORCE_NO_LINKTRACK = 0x00040000,
-    SLDF_ENABLE_TARGET_METADATA = 0x00080000,
+typedef enum
+{
+    SLDF_DEFAULT                               = 0x00000000,
+    SLDF_HAS_ID_LIST                           = 0x00000001,
+    SLDF_HAS_LINK_INFO                         = 0x00000002,
+    SLDF_HAS_NAME                              = 0x00000004,
+    SLDF_HAS_RELPATH                           = 0x00000008,
+    SLDF_HAS_WORKINGDIR                        = 0x00000010,
+    SLDF_HAS_ARGS                              = 0x00000020,
+    SLDF_HAS_ICONLOCATION                      = 0x00000040,
+    SLDF_UNICODE                               = 0x00000080,
+    SLDF_FORCE_NO_LINKINFO                     = 0x00000100,
+    SLDF_HAS_EXP_SZ                            = 0x00000200,
+    SLDF_RUN_IN_SEPARATE                       = 0x00000400,
+    SLDF_HAS_LOGO3ID                           = 0x00000800,
+    SLDF_HAS_DARWINID                          = 0x00001000,
+    SLDF_RUNAS_USER                            = 0x00002000,
+    SLDF_HAS_EXP_ICON_SZ                       = 0x00004000,
+    SLDF_NO_PIDL_ALIAS                         = 0x00008000,
+    SLDF_FORCE_UNCNAME                         = 0x00010000,
+    SLDF_RUN_WITH_SHIMLAYER                    = 0x00020000,
+    SLDF_FORCE_NO_LINKTRACK                    = 0x00040000,
+    SLDF_ENABLE_TARGET_METADATA                = 0x00080000,
+    SLDF_DISABLE_LINK_PATH_TRACKING            = 0x00100000,
     SLDF_DISABLE_KNOWNFOLDER_RELATIVE_TRACKING = 0x00200000,
-    SLDF_VALID = 0x003ff7ff,
-    SLDF_RESERVED = 0x80000000,
+    SDLF_NO_KF_ALIAS                           = 0x00400000,
+    SDLF_ALLOW_LINK_TO_LINK                    = 0x00800000,
+    SDLF_UNALIAS_ON_SAVE                       = 0x01000000,
+    SDLF_PREFER_ENVIRONMENT_PATH               = 0x02000000,
+    SDLF_KEEP_LOCAL_IDLIST_FOR_UNC_TARGET      = 0x04000000,
+    SDLF_PERSIST_VOLUME_ID_ACTIVE              = 0x08000000,
+    SLDF_VALID                                 = 0x0ffff7ff,
+    SLDF_RESERVED                              = 0x80000000,
 } SHELL_LINK_DATA_FLAGS;
 
 typedef struct tagDATABLOCKHEADER
@@ -1383,6 +1418,35 @@ typedef struct tagDATABLOCKHEADER
     DWORD dwSignature;
 } DATABLOCK_HEADER, *LPDATABLOCK_HEADER, *LPDBLIST;
 
+typedef struct {
+    DATABLOCK_HEADER dbh;
+    WORD wFillAttribute;
+    WORD wPopupFillAttribute;
+    COORD dwScreenBufferSize;
+    COORD dwWindowSize;
+    COORD dwWindowOrigin;
+    DWORD nFont;
+    DWORD nInputBufferSize;
+    COORD dwFontSize;
+    UINT uFontFamily;
+    UINT uFontWeight;
+    WCHAR FaceName[LF_FACESIZE];
+    UINT uCursorSize;
+    BOOL bFullScreen;
+    BOOL bQuickEdit;
+    BOOL bInsertMode;
+    BOOL bAutoPosition;
+    UINT uHistoryBufferSize;
+    UINT uNumberOfHistoryBuffers;
+    BOOL bHistoryNoDup;
+    COLORREF ColorTable[16];
+} NT_CONSOLE_PROPS, *LPNT_CONSOLE_PROPS;
+
+typedef struct {
+    DATABLOCK_HEADER dbh;
+    UINT uCodePage;
+} NT_FE_CONSOLE_PROPS, *LPNT_FE_CONSOLE_PROPS;
+
 typedef struct {
     DATABLOCK_HEADER dbh;
     CHAR szDarwinID[MAX_PATH];
@@ -1451,6 +1515,22 @@ int WINAPI SHCreateDirectoryExW(HWND, LPCWSTR, LPSECURITY_ATTRIBUTES);
 /****************************************************************************
 * SHGetSetFolderCustomSettings API
 */
+
+#define FCS_READ       0x00000001
+#define FCS_FORCEWRITE 0x00000002
+#define FCS_WRITE      (FCS_READ | FCS_FORCEWRITE)
+
+#define FCS_FLAG_DRAGDROP    0x00000002
+
+#define FCSM_VIEWID          0x00000001
+#define FCSM_WEBVIEWTEMPLATE 0x00000002
+#define FCSM_INFOTIP         0x00000004
+#define FCSM_CLSID           0x00000008
+#define FCSM_ICONFILE        0x00000010
+#define FCSM_LOGO            0x00000020
+#define FCSM_FLAGS           0x00000040
+
+#include <pshpack8.h>
 typedef struct {
     DWORD dwSize;
     DWORD dwMask;
@@ -1468,8 +1548,9 @@ typedef struct {
     LPWSTR pszLogo;
     DWORD cchLogo;
 } SHFOLDERCUSTOMSETTINGS, *LPSHFOLDERCUSTOMSETTINGS;
+#include <poppack.h>
 
-HRESULT WINAPI SHGetSetFolderCustomSettings(LPSHFOLDERCUSTOMSETTINGS pfcs, LPCSTR pszPath, DWORD dwReadWrite);
+HRESULT WINAPI SHGetSetFolderCustomSettings(LPSHFOLDERCUSTOMSETTINGS pfcs, PCWSTR pszPath, DWORD dwReadWrite);
 
 /****************************************************************************
 * SHGetSpecialFolderLocation API
@@ -1676,10 +1757,9 @@ BOOL WINAPI WriteCabinetState(CABINETSTATE *);
 
 /* PathResolve flags */
 #define PRF_VERIFYEXISTS         0x01
-#define PRF_EXECUTABLE           0x02
 #define PRF_TRYPROGRAMEXTENSIONS 0x03
 #define PRF_FIRSTDIRDEF          0x04
-#define PRF_DONTFINDLINK         0x08
+#define PRF_DONTFINDLNK          0x08
 #define PRF_REQUIREABSOLUTE      0x10
 
 VOID WINAPI PathGetShortPath(LPWSTR pszPath);
@@ -1776,10 +1856,15 @@ HRESULT WINAPI CDefFolderMenu_Create2(LPCITEMIDLIST pidlFolder, HWND hwnd, UINT
                                       LPFNDFMCALLBACK lpfn, UINT nKeys, const HKEY *ahkeys,
                                       IContextMenu **ppcm);
 
+int WINAPI PickIconDlg(HWND owner, WCHAR *path, UINT path_len, int *index);
+HRESULT WINAPI SHLimitInputEdit(HWND hwnd, IShellFolder *folder);
+
 #include <poppack.h>
 
 #ifdef __cplusplus
 } /* extern "C" */
 #endif /* defined(__cplusplus) */
 
+#include "wine/winheader_exit.h"
+
 #endif /* __WINE_SHLOBJ_H */
