@@ -19,6 +19,8 @@
 #ifndef _WINE_WININET_H_
 #define _WINE_WININET_H_
 
+#include "wine/winheader_enter.h"
+
 #ifdef __cplusplus
 extern "C" {
 #endif
@@ -1463,6 +1465,8 @@ INTERNETAPI DWORD WINAPI PrivacyGetZonePreferenceW(DWORD,DWORD,LPDWORD,LPWSTR,LP
 #define ERROR_INTERNET_SEC_CERT_ERRORS          (INTERNET_ERROR_BASE + 55)
 #define ERROR_INTERNET_SEC_CERT_NO_REV          (INTERNET_ERROR_BASE + 56)
 #define ERROR_INTERNET_SEC_CERT_REV_FAILED      (INTERNET_ERROR_BASE + 57)
+#define ERROR_HTTP_HSTS_REDIRECT_REQUIRED       (INTERNET_ERROR_BASE + 60)
+#define ERROR_INTERNET_SEC_CERT_WEAK_SIGNATURE  (INTERNET_ERROR_BASE + 62)
 #define ERROR_FTP_TRANSFER_IN_PROGRESS          (INTERNET_ERROR_BASE + 110)
 #define ERROR_FTP_DROPPED                       (INTERNET_ERROR_BASE + 111)
 #define ERROR_FTP_NO_PASSIVE_MODE               (INTERNET_ERROR_BASE + 112)
@@ -1501,7 +1505,9 @@ INTERNETAPI DWORD WINAPI PrivacyGetZonePreferenceW(DWORD,DWORD,LPDWORD,LPWSTR,LP
 #define ERROR_INTERNET_NEED_MSN_SSPI_PKG          (INTERNET_ERROR_BASE + 173)
 #define ERROR_INTERNET_LOGIN_FAILURE_DISPLAY_ENTITY_BODY  (INTERNET_ERROR_BASE + 174)
 #define ERROR_INTERNET_DECODING_FAILED            (INTERNET_ERROR_BASE + 175)
-#define INTERNET_ERROR_LAST                       ERROR_INTERNET_DECODING_FAILED
+#define ERROR_INTERNET_CLIENT_AUTH_CERT_NEEDED_PROXY (INTERNET_ERROR_BASE + 187)
+#define ERROR_INTERNET_SECURE_FAILURE_PROXY     (INTERNET_ERROR_BASE + 188)
+#define INTERNET_ERROR_LAST                     ERROR_INTERNET_SECURE_FAILURE_PROXY
 
 
 #define NORMAL_CACHE_ENTRY              0x00000001
@@ -1772,4 +1778,6 @@ BOOL WINAPI DetectAutoProxyUrl(LPSTR, DWORD, DWORD);
 }
 #endif
 
+#include "wine/winheader_exit.h"
+
 #endif
