@@ -19,6 +19,8 @@
 #ifndef __WINE_WINTRUST_H
 #define __WINE_WINTRUST_H
 
+#include "wine/winheader_enter.h"
+
 #include <wincrypt.h>
 
 
@@ -451,10 +453,7 @@ typedef struct _CRYPT_PROVUI_FUNCS {
 extern "C" {
 #endif
 
-#if defined(__GNUC__)
-#define WT_PROVIDER_CERTTRUST_FUNCTION (const WCHAR []) \
-    {'W','i','n','t','r','u','s','t','C','e','r','t','i','f','i','c','a','t','e','T','r','u','s','t', 0}
-#elif defined(_MSC_VER)
+#if defined(_MSC_VER) || defined(__MINGW32__)
 #define WT_PROVIDER_CERTTRUST_FUNCTION L"WintrustCertificateTrust"
 #else
 static const WCHAR WT_PROVIDER_CERTTRUST_FUNCTION[] =
@@ -665,4 +664,6 @@ typedef struct _WIN_TRUST_SUBJECT_FILE_AND_DISPLAY
 }
 #endif
 
+#include "wine/winheader_exit.h"
+
 #endif
