@@ -25,34 +25,52 @@
 #ifdef __APPLE__
 
 #include "config.h"
-#include "wine/port.h"
 
 #include <stdarg.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <sys/types.h>
-#ifdef HAVE_SYS_STAT_H
-# include <sys/stat.h>
-#endif
+#include <sys/stat.h>
 #include <fcntl.h>
-#ifdef HAVE_SYS_MMAN_H
-# include <sys/mman.h>
-#endif
+#include <sys/mman.h>
 #ifdef HAVE_SYS_SYSCALL_H
 # include <sys/syscall.h>
 #endif
-#ifdef HAVE_UNISTD_H
-# include <unistd.h>
-#endif
+#include <unistd.h>
+#include <dlfcn.h>
 #ifdef HAVE_MACH_O_LOADER_H
 #include <mach/thread_status.h>
+#include <mach/vm_statistics.h>
 #include <mach-o/loader.h>
 #include <mach-o/ldsyms.h>
 #endif
 
+#include "wine/asm.h"
 #include "main.h"
 
+#if defined(__x86_64__)
+/* Rosetta on Apple Silicon allocates memory starting at 0x100000000 (the 4GB line)
+ * before the preloader runs, which prevents any nonrelocatable EXEs with that
+ * base address from running.
+ *
+ * This empty linker section forces Rosetta's allocations (currently ~132 MB)
+ * to start at 0x114000000, and they should end below 0x120000000.
+ */
+__asm__(".zerofill WINE_4GB_RESERVE,WINE_4GB_RESERVE,___wine_4gb_reserve,0x14000000");
+
+static const struct wine_preload_info zerofill_sections[] =
+{
+    { (void *)0x000100000000, 0x14000000 },  /* WINE_4GB_RESERVE section */
+    { 0, 0 }                                 /* end of list */
+};
+#else
+static const struct wine_preload_info zerofill_sections[] =
+{
+    { 0, 0 }                                 /* end of list */
+};
+#endif
+
 #ifndef LC_MAIN
 #define LC_MAIN 0x80000028
 struct entry_point_command
@@ -78,7 +96,9 @@ static struct wine_preload_info preload_info[] =
     { (void *)0x000000010000, 0x00100000 },  /* DOS area */
     { (void *)0x000000110000, 0x67ef0000 },  /* low memory area */
     { (void *)0x00007ff00000, 0x000f0000 },  /* shared user data */
-    { (void *)0x7ffef0000000, 0x01ff0000 },  /* top-down allocations + virtual heap */
+    { (void *)0x000080000000, 0x80000000 },  /* 2-4GB large-address-aware area */
+    { (void *)0x000100000000, 0x14000000 },  /* WINE_4GB_RESERVE section */
+    { (void *)0x7ff000000000, 0x01ff0000 },  /* top-down allocations + virtual heap */
 #endif /* __i386__ */
     { 0, 0 },                            /* PE exe range set with WINEPRELOADRESERVE */
     { 0, 0 }                             /* end of list */
@@ -96,7 +116,6 @@ void __stack_chk_fail(void) { return; }
 
 #ifdef __i386__
 
-static const size_t page_size = 0x1000;
 static const size_t page_mask = 0xfff;
 #define target_mach_header      mach_header
 #define target_segment_command  segment_command
@@ -184,7 +203,6 @@ __ASM_GLOBAL_FUNC( start,
 
 #elif defined(__x86_64__)
 
-static const size_t page_size = 0x1000;
 static const size_t page_mask = 0xfff;
 #define target_mach_header      mach_header_64
 #define target_segment_command  segment_command_64
@@ -280,9 +298,6 @@ SYSCALL_FUNC( wld_mmap, 197 /* SYS_mmap */ );
 void *wld_munmap( void *start, size_t len );
 SYSCALL_FUNC( wld_munmap, 73 /* SYS_munmap */ );
 
-int wld_mincore( void *addr, size_t length, unsigned char *vec );
-SYSCALL_FUNC( wld_mincore, 78 /* SYS_mincore */ );
-
 static intptr_t (*p_dyld_get_image_slide)( const struct target_mach_header* mh );
 
 #define MAKE_FUNCPTR(f) static typeof(f) * p##f
@@ -429,6 +444,10 @@ static int preloader_overlaps_range( const void *start, const void *end )
             struct target_segment_command *seg = (struct target_segment_command*)cmd;
             const void *seg_start = (const void*)(seg->vmaddr + slide);
             const void *seg_end = (const char*)seg_start + seg->vmsize;
+            static const char reserved_segname[] = "WINE_4GB_RESERVE";
+
+            if (!wld_strncmp( seg->segname, reserved_segname, sizeof(reserved_segname)-1 ))
+                continue;
 
             if (end > seg_start && start <= seg_end)
             {
@@ -548,28 +567,17 @@ static void *get_entry_point( struct target_mach_header *mh, intptr_t slide, int
     return NULL;
 };
 
-static int is_region_empty( struct wine_preload_info *info )
+static int is_zerofill( struct wine_preload_info *info )
 {
-    unsigned char vec[1024];
-    size_t pos, size, block = 1024 * page_size;
     int i;
 
-    for (pos = 0; pos < info->size; pos += size)
+    for (i = 0; zerofill_sections[i].size; i++)
     {
-        size = (pos + block <= info->size) ? block : (info->size - pos);
-        if (wld_mincore( (char *)info->addr + pos, size, vec ) == -1)
-        {
-            if (size <= page_size) continue;
-            block = page_size; size = 0;  /* retry with smaller block size */
-        }
-        else
-        {
-            for (i = 0; i < size / page_size; i++)
-                if (vec[i] & 1) return 0;
-        }
+        if ((zerofill_sections[i].addr == info->addr) &&
+            (zerofill_sections[i].size == info->size))
+            return 1;
     }
-
-    return 1;
+    return 0;
 }
 
 static int map_region( struct wine_preload_info *info )
@@ -577,20 +585,11 @@ static int map_region( struct wine_preload_info *info )
     int flags = MAP_PRIVATE | MAP_ANON;
     void *ret;
 
-    if (!info->addr) flags |= MAP_FIXED;
+    if (!info->addr || is_zerofill( info )) flags |= MAP_FIXED;
 
-    for (;;)
-    {
-        ret = wld_mmap( info->addr, info->size, PROT_NONE, flags, -1, 0 );
-        if (ret == info->addr) return 1;
-        if (ret != (void *)-1) wld_munmap( ret, info->size );
-        if (flags & MAP_FIXED) break;
-
-        /* Some versions of macOS ignore the address hint passed to mmap -
-         * use mincore() to check if its empty and then use MAP_FIXED */
-        if (!is_region_empty( info )) break;
-        flags |= MAP_FIXED;
-    }
+    ret = wld_mmap( info->addr, info->size, PROT_NONE, flags, VM_MAKE_TAG(240), 0 );
+    if (ret == info->addr) return 1;
+    if (ret != (void *)-1) wld_munmap( ret, info->size );
 
     /* don't warn for zero page */
     if (info->addr >= (void *)0x1000)
