@@ -28,7 +28,6 @@
 #include <commctrl.h>
 #include <winnt.h>
 
-#include "wine/unicode.h"
 #include "resource.h"
 #include "taskmgr.h"
 #include "perfdata.h"
@@ -77,35 +76,6 @@ static void Draw3dRect(HDC hDC, int x, int y, int cx, int cy, COLORREF clrTopLef
     FillSolidRect2(hDC, x, y + cy, cx, -1, clrBottomRight);
 }
 
-void Font_DrawText(HDC hDC, LPWSTR lpwszText, int x, int y)
-{
-    HDC        hFontDC;
-    HBITMAP    hFontBitmap;
-    HBITMAP    hOldBitmap;
-    int        i;
-
-    hFontDC = CreateCompatibleDC(hDC);
-    hFontBitmap = LoadBitmapW(hInst, MAKEINTRESOURCEW(IDB_FONT));
-    hOldBitmap = SelectObject(hFontDC, hFontBitmap);
-
-    for (i = 0; lpwszText[i]; i++) {
-        if ((lpwszText[i] >= '0') && (lpwszText[i] <= '9')) {
-            BitBlt(hDC, x + (i * 8), y, 8, 11, hFontDC, (lpwszText[i] - '0') * 8, 0, SRCCOPY);
-        }
-        else if (lpwszText[i] == 'K')
-        {
-            BitBlt(hDC, x + (i * 8), y, 8, 11, hFontDC, 80, 0, SRCCOPY);
-        }
-        else if (lpwszText[i] == '%')
-        {
-            BitBlt(hDC, x + (i * 8), y, 8, 11, hFontDC, 88, 0, SRCCOPY);
-        }
-    }
-    SelectObject(hFontDC, hOldBitmap);
-    DeleteObject(hFontBitmap);
-    DeleteDC(hFontDC);
-}
-
 static BOOL OnCreate(HWND hWnd)
 {
     HMENU   hMenu;
@@ -727,8 +697,8 @@ LPWSTR GetLastErrorText(LPWSTR lpwszBuf, DWORD dwSize)
     if (!dwRet || ( dwSize < dwRet+14)) {
         lpwszBuf[0] = '\0';
     } else {
-        lpwszTemp[strlenW(lpwszTemp)-2] = '\0';  /* remove cr and newline character */
-        sprintfW(lpwszBuf, wszFormat, lpwszTemp, GetLastError());
+        lpwszTemp[lstrlenW(lpwszTemp)-2] = '\0';  /* remove cr and newline character */
+        swprintf(lpwszBuf, dwSize, wszFormat, lpwszTemp, GetLastError());
     }
     if (lpwszTemp) {
         LocalFree(lpwszTemp);
@@ -1029,6 +999,8 @@ int APIENTRY WinMain(HINSTANCE hInstance,
     HANDLE hToken; 
     TOKEN_PRIVILEGES tkp; 
 
+    InitCommonControls();
+
     /* Initialize global variables */
     hInst = hInstance;
 
