@@ -20,7 +20,6 @@
  */
 
 #include "config.h"
-#include "wine/port.h"
 
 #include <assert.h>
 #include <fcntl.h>
@@ -32,8 +31,10 @@
 #include <limits.h>
 #include <dirent.h>
 #include <errno.h>
-#ifdef HAVE_POLL_H
-# include <poll.h>
+#include <unistd.h>
+#include <poll.h>
+#ifdef HAVE_SYS_INOTIFY_H
+#include <sys/inotify.h>
 #endif
 
 #include "ntstatus.h"
@@ -65,56 +66,6 @@
 
 /* inotify support */
 
-#ifdef HAVE_SYS_INOTIFY_H
-#include <sys/inotify.h>
-#define USE_INOTIFY
-#elif defined(__linux__) && defined(__i386__)
-
-#define SYS_inotify_init	291
-#define SYS_inotify_add_watch	292
-#define SYS_inotify_rm_watch	293
-
-struct inotify_event {
-    int           wd;
-    unsigned int  mask;
-    unsigned int  cookie;
-    unsigned int  len;
-    char          name[1];
-};
-
-#define IN_ACCESS        0x00000001
-#define IN_MODIFY        0x00000002
-#define IN_ATTRIB        0x00000004
-#define IN_CLOSE_WRITE   0x00000008
-#define IN_CLOSE_NOWRITE 0x00000010
-#define IN_OPEN          0x00000020
-#define IN_MOVED_FROM    0x00000040
-#define IN_MOVED_TO      0x00000080
-#define IN_CREATE        0x00000100
-#define IN_DELETE        0x00000200
-#define IN_DELETE_SELF   0x00000400
-
-#define IN_ISDIR         0x40000000
-
-static inline int inotify_init( void )
-{
-    return syscall( SYS_inotify_init );
-}
-
-static inline int inotify_add_watch( int fd, const char *name, unsigned int mask )
-{
-    return syscall( SYS_inotify_add_watch, fd, name, mask );
-}
-
-static inline int inotify_rm_watch( int fd, int wd )
-{
-    return syscall( SYS_inotify_rm_watch, fd, wd );
-}
-
-#define USE_INOTIFY
-
-#endif
-
 struct inode;
 
 static void free_inode( struct inode *inode );
@@ -135,7 +86,7 @@ struct dir
     uid_t          uid;      /* file stat.st_uid */
     struct list    entry;    /* entry in global change notifications list */
     unsigned int   filter;   /* notification filter */
-    int            notified; /* SIGIO counter */
+    volatile int   notified; /* SIGIO counter */
     int            want_data; /* return change data */
     int            subtree;  /* do we want to watch subdirectories? */
     struct list    change_records;   /* data for the change */
@@ -150,28 +101,30 @@ static struct security_descriptor *dir_get_sd( struct object *obj );
 static int dir_set_sd( struct object *obj, const struct security_descriptor *sd,
                        unsigned int set_info );
 static void dir_dump( struct object *obj, int verbose );
-static struct object_type *dir_get_type( struct object *obj );
 static int dir_close_handle( struct object *obj, struct process *process, obj_handle_t handle );
 static void dir_destroy( struct object *obj );
 
 static const struct object_ops dir_ops =
 {
     sizeof(struct dir),       /* size */
+    &file_type,               /* type */
     dir_dump,                 /* dump */
-    dir_get_type,             /* get_type */
     add_queue,                /* add_queue */
     remove_queue,             /* remove_queue */
     default_fd_signaled,      /* signaled */
+    default_fd_get_esync_fd,  /* get_esync_fd */
     no_satisfied,             /* satisfied */
     no_signal,                /* signal */
     dir_get_fd,               /* get_fd */
-    default_fd_map_access,    /* map_access */
+    default_map_access,       /* map_access */
     dir_get_sd,               /* get_sd */
     dir_set_sd,               /* set_sd */
+    no_get_full_name,         /* get_full_name */
     no_lookup_name,           /* lookup_name */
     no_link_name,             /* link_name */
     NULL,                     /* unlink_name */
     no_open_file,             /* open_file */
+    no_kernel_obj_list,       /* get_kernel_obj_list */
     dir_close_handle,         /* close_handle */
     dir_destroy               /* destroy */
 };
@@ -187,7 +140,10 @@ static const struct fd_ops dir_fd_ops =
     no_fd_read,                  /* read */
     no_fd_write,                 /* write */
     no_fd_flush,                 /* flush */
+    default_fd_get_file_info,    /* get_file_info */
+    no_fd_get_volume_info,       /* get_volume_info */
     default_fd_ioctl,            /* ioctl */
+    default_fd_cancel_async,     /* cancel_async */
     default_fd_queue_async,      /* queue_async */
     default_fd_reselect_async    /* reselect_async */
 };
@@ -335,13 +291,6 @@ static void dir_dump( struct object *obj, int verbose )
     fprintf( stderr, "Dirfile fd=%p filter=%08x\n", dir->fd, dir->filter );
 }
 
-static struct object_type *dir_get_type( struct object *obj )
-{
-    static const WCHAR name[] = {'F','i','l','e'};
-    static const struct unicode_str str = { name, sizeof(name) };
-    return get_object_type( &str );
-}
-
 /* enter here directly from SIGIO signal handler */
 void do_change_notify( int unix_fd )
 {
@@ -351,7 +300,7 @@ void do_change_notify( int unix_fd )
     LIST_FOR_EACH_ENTRY( dir, &change_list, struct dir, entry )
     {
         if (get_unix_fd( dir->fd ) != unix_fd) continue;
-        interlocked_xchg_add( &dir->notified, 1 );
+        dir->notified = 1;
         break;
     }
 }
@@ -363,8 +312,9 @@ void sigio_callback(void)
 
     LIST_FOR_EACH_ENTRY( dir, &change_list, struct dir, entry )
     {
-        if (interlocked_xchg( &dir->notified, 0 ))
-            fd_async_wake_up( dir->fd, ASYNC_TYPE_WAIT, STATUS_ALERTED );
+        if (!dir->notified) continue;
+        dir->notified = 0;
+        fd_async_wake_up( dir->fd, ASYNC_TYPE_WAIT, STATUS_ALERTED );
     }
 }
 
@@ -415,7 +365,7 @@ static int dir_set_sd( struct object *obj, const struct security_descriptor *sd,
                        unsigned int set_info )
 {
     struct dir *dir = (struct dir *)obj;
-    const SID *owner;
+    const struct sid *owner;
     struct stat st;
     mode_t mode;
     int unix_fd;
@@ -434,7 +384,7 @@ static int dir_set_sd( struct object *obj, const struct security_descriptor *sd,
             set_error( STATUS_INVALID_SECURITY_DESCR );
             return 0;
         }
-        if (!obj->sd || !security_equal_sid( owner, sd_get_owner( obj->sd ) ))
+        if (!obj->sd || !equal_sid( owner, sd_get_owner( obj->sd ) ))
         {
             /* FIXME: get Unix uid and call fchown */
         }
@@ -471,7 +421,6 @@ static int dir_close_handle( struct object *obj, struct process *process, obj_ha
 {
     struct dir *dir = (struct dir *)obj;
 
-    if (!fd_close_handle( obj, process, handle )) return 0;
     if (obj->handle_count == 1) release_dir_cache_entry( dir ); /* closing last handle, release cache */
     return 1;  /* ok to close */
 }
@@ -518,7 +467,7 @@ static enum server_fd_type dir_get_fd_type( struct fd *fd )
     return FD_TYPE_DIR;
 }
 
-#ifdef USE_INOTIFY
+#ifdef HAVE_SYS_INOTIFY_H
 
 #define HASH_SIZE 31
 
@@ -740,11 +689,9 @@ static unsigned int filter_from_event( struct inotify_event *ie )
     if (ie->mask & (IN_MOVED_FROM | IN_MOVED_TO | IN_DELETE | IN_CREATE))
         filter |= FILE_NOTIFY_CHANGE_FILE_NAME | FILE_NOTIFY_CHANGE_DIR_NAME;
     if (ie->mask & IN_MODIFY)
-        filter |= FILE_NOTIFY_CHANGE_SIZE | FILE_NOTIFY_CHANGE_LAST_WRITE;
+        filter |= FILE_NOTIFY_CHANGE_SIZE | FILE_NOTIFY_CHANGE_LAST_WRITE | FILE_NOTIFY_CHANGE_LAST_ACCESS;
     if (ie->mask & IN_ATTRIB)
         filter |= FILE_NOTIFY_CHANGE_ATTRIBUTES | FILE_NOTIFY_CHANGE_SECURITY;
-    if (ie->mask & IN_ACCESS)
-        filter |= FILE_NOTIFY_CHANGE_LAST_ACCESS;
     if (ie->mask & IN_CREATE)
         filter |= FILE_NOTIFY_CHANGE_CREATION;
 
@@ -947,11 +894,9 @@ static void inotify_poll_event( struct fd *fd, int event )
     for( ofs = 0; ofs < r - offsetof(struct inotify_event, name); )
     {
         ie = (struct inotify_event*) &buffer[ofs];
-        if (!ie->len)
-            break;
         ofs += offsetof( struct inotify_event, name[ie->len] );
         if (ofs > r) break;
-        inotify_notify_all( ie );
+        if (ie->len) inotify_notify_all( ie );
     }
 }
 
@@ -979,7 +924,7 @@ static int map_flags( unsigned int filter )
     if (filter & FILE_NOTIFY_CHANGE_LAST_WRITE)
         mask |= IN_MODIFY;
     if (filter & FILE_NOTIFY_CHANGE_LAST_ACCESS)
-        mask |= IN_ACCESS;
+        mask |= IN_MODIFY;
     if (filter & FILE_NOTIFY_CHANGE_SECURITY)
         mask |= IN_ATTRIB;
 
@@ -1176,7 +1121,7 @@ static int dir_add_to_existing_notify( struct dir *dir )
     return 0;
 }
 
-#endif  /* USE_INOTIFY */
+#endif  /* HAVE_SYS_INOTIFY_H */
 
 struct object *create_dir_obj( struct fd *fd, unsigned int access, mode_t mode )
 {
@@ -1246,8 +1191,8 @@ DECL_HANDLER(read_directory_changes)
         return;
 
     /* requests don't timeout */
-    if (!(async = create_async( current, &req->async, NULL ))) goto end;
-    if (!fd_queue_async( dir->fd, async, ASYNC_TYPE_WAIT )) goto end;
+    if (!(async = create_async( dir->fd, current, &req->async, NULL ))) goto end;
+    fd_queue_async( dir->fd, async, ASYNC_TYPE_WAIT );
 
     /* assign it once */
     if (!dir->filter)
@@ -1269,8 +1214,8 @@ DECL_HANDLER(read_directory_changes)
 
     set_error(STATUS_PENDING);
 
+    release_object( async );
 end:
-    if (async) release_object( async );
     release_object( dir );
 }
 
