@@ -20,7 +20,6 @@
  */
 
 #include "config.h"
-#include "wine/port.h"
 
 #include <assert.h>
 #include <errno.h>
@@ -41,7 +40,6 @@
 #include "process.h"
 #include "thread.h"
 #include "request.h"
-#include "wine/library.h"
 
 #ifdef USE_MACH
 
@@ -50,6 +48,7 @@
 #include <mach/thread_act.h>
 #include <mach/mach_vm.h>
 #include <servers/bootstrap.h>
+#include <sys/sysctl.h>
 
 static mach_port_t server_mach_port;
 
@@ -90,7 +89,7 @@ void init_tracing_mechanism(void)
                                  server_mach_port,
                                  MACH_MSG_TYPE_MAKE_SEND ) != KERN_SUCCESS)
             fatal_error("Error inserting rights\n");
-    if (bootstrap_register(bp, (char*)wine_get_server_dir(), server_mach_port) != KERN_SUCCESS)
+    if (bootstrap_register(bp, server_dir, server_mach_port) != KERN_SUCCESS)
         fatal_error("Can't check in server_mach_port\n");
     mach_port_deallocate(mach_task_self(), bp);
 }
@@ -153,6 +152,26 @@ void init_thread_context( struct thread *thread )
 {
 }
 
+/* CX HACK 21217 */
+static int is_apple_silicon( void )
+{
+    static int apple_silicon_status, did_check = 0;
+    if (!did_check)
+    {
+        /* returns 0 for native process or on error, 1 for translated */
+        int ret = 0;
+        size_t size = sizeof(ret);
+        if (sysctlbyname( "sysctl.proc_translated", &ret, &size, NULL, 0 ) == -1)
+            apple_silicon_status = 0;
+        else
+            apple_silicon_status = ret;
+
+        did_check = 1;
+    }
+
+    return apple_silicon_status;
+}
+
 /* retrieve the thread x86 registers */
 void get_thread_context( struct thread *thread, context_t *context, unsigned int flags )
 {
@@ -195,25 +214,22 @@ void get_thread_context( struct thread *thread, context_t *context, unsigned int
         else
 #endif
         {
-/* work around silly renaming of struct members in OS X 10.5 */
-#if __DARWIN_UNIX03 && defined(_STRUCT_X86_DEBUG_STATE32)
             context->debug.i386_regs.dr0 = state.uds.ds32.__dr0;
             context->debug.i386_regs.dr1 = state.uds.ds32.__dr1;
             context->debug.i386_regs.dr2 = state.uds.ds32.__dr2;
             context->debug.i386_regs.dr3 = state.uds.ds32.__dr3;
             context->debug.i386_regs.dr6 = state.uds.ds32.__dr6;
             context->debug.i386_regs.dr7 = state.uds.ds32.__dr7;
-#else
-            context->debug.i386_regs.dr0 = state.uds.ds32.dr0;
-            context->debug.i386_regs.dr1 = state.uds.ds32.dr1;
-            context->debug.i386_regs.dr2 = state.uds.ds32.dr2;
-            context->debug.i386_regs.dr3 = state.uds.ds32.dr3;
-            context->debug.i386_regs.dr6 = state.uds.ds32.dr6;
-            context->debug.i386_regs.dr7 = state.uds.ds32.dr7;
-#endif
         }
         context->flags |= SERVER_CTX_DEBUG_REGISTERS;
     }
+    else if (is_apple_silicon())
+    {
+        /* CX HACK 21217: Fake debug registers on Apple Silicon */
+        fprintf( stderr, "%04x: thread_get_state failed on Apple Silicon - faking zero debug registers\n", thread->id );
+        memset( &context->debug, 0, sizeof(context->debug) );
+        context->flags |= SERVER_CTX_DEBUG_REGISTERS;
+    }
     mach_port_deallocate( mach_task_self(), port );
 #endif
 }
@@ -241,7 +257,7 @@ void set_thread_context( struct thread *thread, const context_t *context, unsign
 
 
 #ifdef __x86_64__
-    if (thread->process->cpu == CPU_x86_64)
+    if (thread->process->machine == IMAGE_FILE_MACHINE_AMD64)
     {
         /* Mac OS doesn't allow setting the global breakpoint flags */
         dr7 = (context->debug.x86_64_regs.dr7 & ~0xaa) | ((context->debug.x86_64_regs.dr7 & 0xaa) >> 1);
@@ -264,7 +280,6 @@ void set_thread_context( struct thread *thread, const context_t *context, unsign
 
         state.dsh.flavor = x86_DEBUG_STATE32;
         state.dsh.count = sizeof(state.uds.ds32) / sizeof(int);
-#if __DARWIN_UNIX03 && defined(_STRUCT_X86_DEBUG_STATE32)
         state.uds.ds32.__dr0 = context->debug.i386_regs.dr0;
         state.uds.ds32.__dr1 = context->debug.i386_regs.dr1;
         state.uds.ds32.__dr2 = context->debug.i386_regs.dr2;
@@ -273,29 +288,8 @@ void set_thread_context( struct thread *thread, const context_t *context, unsign
         state.uds.ds32.__dr5 = 0;
         state.uds.ds32.__dr6 = context->debug.i386_regs.dr6;
         state.uds.ds32.__dr7 = dr7;
-#else
-        state.uds.ds32.dr0 = context->debug.i386_regs.dr0;
-        state.uds.ds32.dr1 = context->debug.i386_regs.dr1;
-        state.uds.ds32.dr2 = context->debug.i386_regs.dr2;
-        state.uds.ds32.dr3 = context->debug.i386_regs.dr3;
-        state.uds.ds32.dr4 = 0;
-        state.uds.ds32.dr5 = 0;
-        state.uds.ds32.dr6 = context->debug.i386_regs.dr6;
-        state.uds.ds32.dr7 = dr7;
-#endif
-    }
-    if (!thread_set_state( port, x86_DEBUG_STATE, (thread_state_t)&state, count ))
-    {
-        if (thread->context)  /* update the cached values */
-        {
-#ifdef __x86_64__
-            if (thread->process->cpu == CPU_x86_64)
-                thread->context->debug.x86_64_regs = context->debug.x86_64_regs;
-            else
-#endif
-                thread->context->debug.i386_regs = context->debug.i386_regs;
-        }
     }
+    thread_set_state( port, x86_DEBUG_STATE, (thread_state_t)&state, count );
     mach_port_deallocate( mach_task_self(), port );
 #endif
 }
