@@ -23,7 +23,6 @@
  */
 
 #include "config.h"
-#include "wine/port.h"
 
 #include <assert.h>
 #include <ctype.h>
@@ -35,8 +34,13 @@
 #include <string.h>
 #include <stdlib.h>
 #include <sys/stat.h>
+#include <sys/types.h>
 #include <unistd.h>
 
+#if defined(__APPLE__) && defined(__x86_64__)
+#include <sys/utsname.h>
+#endif
+
 #include "ntstatus.h"
 #define WIN32_NO_STATUS
 #include "object.h"
@@ -48,18 +52,32 @@
 #include "security.h"
 
 #include "winternl.h"
-#include "wine/library.h"
 
 struct notify
 {
     struct list       entry;    /* entry in list of notifications */
-    struct event     *event;    /* event to set when changing this key */
+    struct event    **events;   /* events to set when changing this key */
+    unsigned int      event_count; /* number of events */
     int               subtree;  /* true if subtree notification */
     unsigned int      filter;   /* which events to notify on */
     obj_handle_t      hkey;     /* hkey associated with this notification */
     struct process   *process;  /* process in which the hkey is valid */
 };
 
+static const WCHAR key_name[] = {'K','e','y'};
+
+struct type_descr key_type =
+{
+    { key_name, sizeof(key_name) },   /* name */
+    KEY_ALL_ACCESS | SYNCHRONIZE,     /* valid_access */
+    {                                 /* mapping */
+        STANDARD_RIGHTS_READ | KEY_NOTIFY | KEY_ENUMERATE_SUB_KEYS | KEY_QUERY_VALUE,
+        STANDARD_RIGHTS_WRITE | KEY_CREATE_SUB_KEY | KEY_SET_VALUE,
+        STANDARD_RIGHTS_EXECUTE | KEY_CREATE_LINK | KEY_NOTIFY | KEY_ENUMERATE_SUB_KEYS | KEY_QUERY_VALUE,
+        KEY_ALL_ACCESS
+    },
+};
+
 /* a registry key */
 struct key
 {
@@ -87,6 +105,7 @@ struct key
 #define KEY_SYMLINK  0x0008  /* key is a symbolic link */
 #define KEY_WOW64    0x0010  /* key contains a Wow6432Node subkey */
 #define KEY_WOWSHARE 0x0020  /* key is a Wow64 shared key (used for Software\Classes) */
+#define KEY_PREDEF   0x0040  /* key is marked as predefined */
 
 /* a key value */
 struct key_value
@@ -131,6 +150,10 @@ struct save_branch_info
 static int save_branch_count;
 static struct save_branch_info save_branch_info[MAX_SAVE_BRANCH_INFO];
 
+unsigned int supported_machines_count = 0;
+unsigned short supported_machines[8];
+unsigned short native_machine = 0;
+int wow64_using_32bit_prefix = 0;
 
 /* information about a file being loaded */
 struct file_load_info
@@ -146,30 +169,33 @@ struct file_load_info
 
 
 static void key_dump( struct object *obj, int verbose );
-static struct object_type *key_get_type( struct object *obj );
 static unsigned int key_map_access( struct object *obj, unsigned int access );
 static struct security_descriptor *key_get_sd( struct object *obj );
+static WCHAR *key_get_full_name( struct object *obj, data_size_t *len );
 static int key_close_handle( struct object *obj, struct process *process, obj_handle_t handle );
 static void key_destroy( struct object *obj );
 
 static const struct object_ops key_ops =
 {
     sizeof(struct key),      /* size */
+    &key_type,               /* type */
     key_dump,                /* dump */
-    key_get_type,            /* get_type */
     no_add_queue,            /* add_queue */
     NULL,                    /* remove_queue */
     NULL,                    /* signaled */
+    NULL,                    /* get_esync_fd */
     NULL,                    /* satisfied */
     no_signal,               /* signal */
     no_get_fd,               /* get_fd */
     key_map_access,          /* map_access */
     key_get_sd,              /* get_sd */
     default_set_sd,          /* set_sd */
+    key_get_full_name,       /* get_full_name */
     no_lookup_name,          /* lookup_name */
     no_link_name,            /* link_name */
     NULL,                    /* unlink_name */
     no_open_file,            /* open_file */
+    no_kernel_obj_list,      /* get_kernel_obj_list */
     key_close_handle,        /* close_handle */
     key_destroy              /* destroy */
 };
@@ -177,8 +203,7 @@ static const struct object_ops key_ops =
 
 static inline int is_wow6432node( const WCHAR *name, unsigned int len )
 {
-    return (len == sizeof(wow6432node) &&
-            !memicmpW( name, wow6432node, sizeof(wow6432node)/sizeof(WCHAR) ));
+    return (len == sizeof(wow6432node) && !memicmp_strW( name, wow6432node, sizeof( wow6432node )));
 }
 
 /*
@@ -198,7 +223,7 @@ static void dump_path( const struct key *key, const struct key *base, FILE *f )
         dump_path( key->parent, base, f );
         fprintf( f, "\\\\" );
     }
-    dump_strW( key->name, key->namelen / sizeof(WCHAR), f, "[]" );
+    dump_strW( key->name, key->namelen, f, "[]" );
 }
 
 /* dump a value to a text file */
@@ -210,7 +235,7 @@ static void dump_value( const struct key_value *value, FILE *f )
     if (value->namelen)
     {
         fputc( '\"', f );
-        count = 1 + dump_strW( value->name, value->namelen / sizeof(WCHAR), f, "\"\"" );
+        count = 1 + dump_strW( value->name, value->namelen, f, "\"\"" );
         count += fprintf( f, "\"=" );
     }
     else count = fprintf( f, "@=" );
@@ -226,7 +251,7 @@ static void dump_value( const struct key_value *value, FILE *f )
         if (((WCHAR *)value->data)[value->len / sizeof(WCHAR) - 1]) break;
         if (value->type != REG_SZ) fprintf( f, "str(%x):", value->type );
         fputc( '\"', f );
-        dump_strW( (WCHAR *)value->data, value->len / sizeof(WCHAR), f, "\"\"" );
+        dump_strW( (WCHAR *)value->data, value->len, f, "\"\"" );
         fprintf( f, "\"\n" );
         return;
 
@@ -272,7 +297,7 @@ static void save_subkeys( const struct key *key, const struct key *base, FILE *f
         if (key->class)
         {
             fprintf( f, "#class=\"" );
-            dump_strW( key->class, key->classlen / sizeof(WCHAR), f, "\"\"" );
+            dump_strW( key->class, key->classlen, f, "\"\"" );
             fprintf( f, "\"\n" );
         }
         if (key->flags & KEY_SYMLINK) fputs( "#link\n", f );
@@ -303,22 +328,20 @@ static void key_dump( struct object *obj, int verbose )
     fprintf( stderr, "\n" );
 }
 
-static struct object_type *key_get_type( struct object *obj )
-{
-    static const WCHAR name[] = {'K','e','y'};
-    static const struct unicode_str str = { name, sizeof(name) };
-    return get_object_type( &str );
-}
-
 /* notify waiter and maybe delete the notification */
 static void do_notification( struct key *key, struct notify *notify, int del )
 {
-    if (notify->event)
+    unsigned int i;
+
+    for (i = 0; i < notify->event_count; ++i)
     {
-        set_event( notify->event );
-        release_object( notify->event );
-        notify->event = NULL;
+        set_event( notify->events[i] );
+        release_object( notify->events[i] );
     }
+    free( notify->events );
+    notify->events = NULL;
+    notify->event_count = 0;
+
     if (del)
     {
         list_remove( &notify->entry );
@@ -339,13 +362,9 @@ static inline struct notify *find_notify( struct key *key, struct process *proce
 
 static unsigned int key_map_access( struct object *obj, unsigned int access )
 {
-    if (access & GENERIC_READ)    access |= KEY_READ;
-    if (access & GENERIC_WRITE)   access |= KEY_WRITE;
-    if (access & GENERIC_EXECUTE) access |= KEY_EXECUTE;
-    if (access & GENERIC_ALL)     access |= KEY_ALL_ACCESS;
+    access = default_map_access( obj, access );
     /* filter the WOW64 masks, as they aren't real access bits */
-    return access & ~(GENERIC_READ | GENERIC_WRITE | GENERIC_EXECUTE | GENERIC_ALL |
-                      KEY_WOW64_64KEY | KEY_WOW64_32KEY);
+    return access & ~(KEY_WOW64_64KEY | KEY_WOW64_32KEY);
 }
 
 static struct security_descriptor *key_get_sd( struct object *obj )
@@ -356,12 +375,12 @@ static struct security_descriptor *key_get_sd( struct object *obj )
 
     if (!key_default_sd)
     {
-        size_t users_sid_len = security_sid_len( security_builtin_users_sid );
-        size_t admins_sid_len = security_sid_len( security_builtin_admins_sid );
-        size_t dacl_len = sizeof(ACL) + 2 * offsetof( ACCESS_ALLOWED_ACE, SidStart )
-                          + users_sid_len + admins_sid_len;
-        ACCESS_ALLOWED_ACE *aaa;
-        ACL *dacl;
+        struct acl *dacl;
+        struct ace *ace;
+        struct sid *sid;
+        size_t users_sid_len = sid_len( &builtin_users_sid );
+        size_t admins_sid_len = sid_len( &builtin_admins_sid );
+        size_t dacl_len = sizeof(*dacl) + 2 * sizeof(*ace) + users_sid_len + admins_sid_len;
 
         key_default_sd = mem_alloc( sizeof(*key_default_sd) + 2 * admins_sid_len + dacl_len );
         key_default_sd->control   = SE_DACL_PRESENT;
@@ -369,31 +388,51 @@ static struct security_descriptor *key_get_sd( struct object *obj )
         key_default_sd->group_len = admins_sid_len;
         key_default_sd->sacl_len  = 0;
         key_default_sd->dacl_len  = dacl_len;
-        memcpy( key_default_sd + 1, security_builtin_admins_sid, admins_sid_len );
-        memcpy( (char *)(key_default_sd + 1) + admins_sid_len, security_builtin_admins_sid, admins_sid_len );
+        sid = (struct sid *)(key_default_sd + 1);
+        sid = copy_sid( sid, &builtin_admins_sid );
+        sid = copy_sid( sid, &builtin_admins_sid );
 
-        dacl = (ACL *)((char *)(key_default_sd + 1) + 2 * admins_sid_len);
-        dacl->AclRevision = ACL_REVISION;
-        dacl->Sbz1 = 0;
-        dacl->AclSize = dacl_len;
-        dacl->AceCount = 2;
-        dacl->Sbz2 = 0;
-        aaa = (ACCESS_ALLOWED_ACE *)(dacl + 1);
-        aaa->Header.AceType = ACCESS_ALLOWED_ACE_TYPE;
-        aaa->Header.AceFlags = INHERIT_ONLY_ACE | CONTAINER_INHERIT_ACE;
-        aaa->Header.AceSize = offsetof( ACCESS_ALLOWED_ACE, SidStart ) + users_sid_len;
-        aaa->Mask = GENERIC_READ;
-        memcpy( &aaa->SidStart, security_builtin_users_sid, users_sid_len );
-        aaa = (ACCESS_ALLOWED_ACE *)((char *)aaa + aaa->Header.AceSize);
-        aaa->Header.AceType = ACCESS_ALLOWED_ACE_TYPE;
-        aaa->Header.AceFlags = 0;
-        aaa->Header.AceSize = offsetof( ACCESS_ALLOWED_ACE, SidStart ) + admins_sid_len;
-        aaa->Mask = KEY_ALL_ACCESS;
-        memcpy( &aaa->SidStart, security_builtin_admins_sid, admins_sid_len );
+        dacl = (struct acl *)((char *)(key_default_sd + 1) + 2 * admins_sid_len);
+        dacl->revision = ACL_REVISION;
+        dacl->pad1     = 0;
+        dacl->size     = dacl_len;
+        dacl->count    = 2;
+        dacl->pad2     = 0;
+        ace = set_ace( ace_first( dacl ), &builtin_users_sid, ACCESS_ALLOWED_ACE_TYPE,
+                       INHERIT_ONLY_ACE | CONTAINER_INHERIT_ACE, GENERIC_READ );
+        set_ace( ace_next( ace ), &builtin_admins_sid, ACCESS_ALLOWED_ACE_TYPE, 0, KEY_ALL_ACCESS );
     }
     return key_default_sd;
 }
 
+static WCHAR *key_get_full_name( struct object *obj, data_size_t *ret_len )
+{
+    static const WCHAR backslash = '\\';
+    struct key *key = (struct key *) obj;
+    data_size_t len = sizeof(root_name) - sizeof(WCHAR);
+    char *ret;
+
+    if (key->flags & KEY_DELETED)
+    {
+        set_error( STATUS_KEY_DELETED );
+        return NULL;
+    }
+
+    for (key = (struct key *)obj; key != root_key; key = key->parent) len += key->namelen + sizeof(WCHAR);
+    if (!(ret = malloc( len ))) return NULL;
+
+    *ret_len = len;
+    key = (struct key *)obj;
+    for (key = (struct key *)obj; key != root_key; key = key->parent)
+    {
+        memcpy( ret + len - key->namelen, key->name, key->namelen );
+        len -= key->namelen + sizeof(WCHAR);
+        memcpy( ret + len, &backslash, sizeof(WCHAR) );
+    }
+    memcpy( ret, root_name, sizeof(root_name) - sizeof(WCHAR) );
+    return (WCHAR *)ret;
+}
+
 /* close the notification associated with a handle */
 static int key_close_handle( struct object *obj, struct process *process, obj_handle_t handle )
 {
@@ -438,10 +477,9 @@ static inline void get_req_path( struct unicode_str *str, int skip_root )
     str->str = get_req_data();
     str->len = (get_req_data_size() / sizeof(WCHAR)) * sizeof(WCHAR);
 
-    if (skip_root && str->len >= sizeof(root_name) &&
-        !memicmpW( str->str, root_name, sizeof(root_name)/sizeof(WCHAR) ))
+    if (skip_root && str->len >= sizeof(root_name) && !memicmp_strW( str->str, root_name, sizeof(root_name) ))
     {
-        str->str += sizeof(root_name)/sizeof(WCHAR);
+        str->str += ARRAY_SIZE( root_name );
         str->len -= sizeof(root_name);
     }
 }
@@ -647,7 +685,7 @@ static struct key *find_subkey( const struct key *key, const struct unicode_str
     {
         i = (min + max) / 2;
         len = min( key->subkeys[i]->namelen, name->len );
-        res = memicmpW( key->subkeys[i]->name, name->str, len / sizeof(WCHAR) );
+        res = memicmp_strW( key->subkeys[i]->name, name->str, len );
         if (!res) res = key->subkeys[i]->namelen - name->len;
         if (!res)
         {
@@ -691,8 +729,8 @@ static struct key *follow_symlink( struct key *key, int iteration )
     path.str = value->data;
     path.len = (value->len / sizeof(WCHAR)) * sizeof(WCHAR);
     if (path.len <= sizeof(root_name)) return NULL;
-    if (memicmpW( path.str, root_name, sizeof(root_name)/sizeof(WCHAR) )) return NULL;
-    path.str += sizeof(root_name) / sizeof(WCHAR);
+    if (memicmp_strW( path.str, root_name, sizeof(root_name) )) return NULL;
+    path.str += ARRAY_SIZE( root_name );
     path.len -= sizeof(root_name);
 
     key = root_key;
@@ -762,6 +800,7 @@ static struct key *open_key( struct key *key, const struct unicode_str *name, un
         return NULL;
     }
     if (debug_level > 1) dump_operation( key, NULL, "Open" );
+    if (key->flags & KEY_PREDEF) set_error( STATUS_PREDEFINED_HANDLE );
     grab_object( key );
     return key;
 }
@@ -792,6 +831,7 @@ static struct key *create_key( struct key *key, const struct unicode_str *name,
             return NULL;
         }
         if (debug_level > 1) dump_operation( key, NULL, "Open" );
+        if (key->flags & KEY_PREDEF) set_error( STATUS_PREDEFINED_HANDLE );
         grab_object( key );
         return key;
     }
@@ -877,17 +917,21 @@ static struct key *create_key_recursive( struct key *key, const struct unicode_s
 }
 
 /* query information about a key or a subkey */
-static void enum_key( const struct key *key, int index, int info_class,
-                      struct enum_key_reply *reply )
+static void enum_key( struct key *key, int index, int info_class, struct enum_key_reply *reply )
 {
-    static const WCHAR backslash[] = { '\\' };
     int i;
     data_size_t len, namelen, classlen;
     data_size_t max_subkey = 0, max_class = 0;
     data_size_t max_value = 0, max_data = 0;
-    const struct key *k;
+    WCHAR *fullname = NULL;
     char *data;
 
+    if (key->flags & KEY_PREDEF)
+    {
+        set_error( STATUS_INVALID_HANDLE );
+        return;
+    }
+
     if (index != -1)  /* -1 means use the specified key directly */
     {
         if ((index < 0) || (index > key->last_subkey))
@@ -904,11 +948,7 @@ static void enum_key( const struct key *key, int index, int info_class,
     switch(info_class)
     {
     case KeyNameInformation:
-        namelen = 0;
-        for (k = key; k != root_key; k = k->parent)
-            namelen += k->namelen + sizeof(backslash);
-        if (!namelen) return;
-        namelen += sizeof(root_name) - sizeof(backslash);
+        if (!(fullname = key->obj.ops->get_full_name( &key->obj, &namelen ))) return;
         /* fall through */
     case KeyBasicInformation:
         classlen = 0; /* only return the name */
@@ -960,18 +1000,8 @@ static void enum_key( const struct key *key, int index, int info_class,
         }
         else if (info_class == KeyNameInformation)
         {
-            data_size_t pos = namelen;
             reply->namelen = namelen;
-            for (k = key; k != root_key; k = k->parent)
-            {
-                pos -= k->namelen;
-                if (pos < len) memcpy( data + pos, k->name,
-                                       min( k->namelen, len - pos ) );
-                pos -= sizeof(backslash);
-                if (pos < len) memcpy( data + pos, backslash,
-                                       min( sizeof(backslash), len - pos ) );
-            }
-            memcpy( data, root_name, min( sizeof(root_name) - sizeof(backslash), len ) );
+            memcpy( data, fullname, len );
         }
         else
         {
@@ -979,6 +1009,7 @@ static void enum_key( const struct key *key, int index, int info_class,
             memcpy( data, key->name, len );
         }
     }
+    free( fullname );
     if (debug_level > 1) dump_operation( key, NULL, "Enum" );
 }
 
@@ -996,6 +1027,12 @@ static int delete_key( struct key *key, int recurse )
     }
     assert( parent );
 
+    if (key->flags & KEY_PREDEF)
+    {
+        set_error( STATUS_INVALID_HANDLE );
+        return -1;
+    }
+
     while (recurse && (key->last_subkey>=0))
         if (0 > delete_key(key->subkeys[key->last_subkey], 1))
             return -1;
@@ -1054,7 +1091,7 @@ static struct key_value *find_value( const struct key *key, const struct unicode
     {
         i = (min + max) / 2;
         len = min( key->values[i].namelen, name->len );
-        res = memicmpW( key->values[i].name, name->str, len / sizeof(WCHAR) );
+        res = memicmp_strW( key->values[i].name, name->str, len );
         if (!res) res = key->values[i].namelen - name->len;
         if (!res)
         {
@@ -1102,6 +1139,12 @@ static void set_value( struct key *key, const struct unicode_str *name,
     void *ptr = NULL;
     int index;
 
+    if (key->flags & KEY_PREDEF)
+    {
+        set_error( STATUS_INVALID_HANDLE );
+        return;
+    }
+
     if ((value = find_value( key, name, &index )))
     {
         /* check if the new value is identical to the existing one */
@@ -1116,7 +1159,7 @@ static void set_value( struct key *key, const struct unicode_str *name,
     if (key->flags & KEY_SYMLINK)
     {
         if (type != REG_LINK || name->len != symlink_str.len ||
-            memicmpW( name->str, symlink_str.str, name->len / sizeof(WCHAR) ))
+            memicmp_strW( name->str, symlink_str.str, name->len ))
         {
             set_error( STATUS_ACCESS_DENIED );
             return;
@@ -1148,6 +1191,12 @@ static void get_value( struct key *key, const struct unicode_str *name, int *typ
     struct key_value *value;
     int index;
 
+    if (key->flags & KEY_PREDEF)
+    {
+        set_error( STATUS_INVALID_HANDLE );
+        return;
+    }
+
     if ((value = find_value( key, name, &index )))
     {
         *type = value->type;
@@ -1167,6 +1216,12 @@ static void enum_value( struct key *key, int i, int info_class, struct enum_key_
 {
     struct key_value *value;
 
+    if (key->flags & KEY_PREDEF)
+    {
+        set_error( STATUS_INVALID_HANDLE );
+        return;
+    }
+
     if (i < 0 || i > key->last_value) set_error( STATUS_NO_MORE_ENTRIES );
     else
     {
@@ -1219,6 +1274,12 @@ static void delete_value( struct key *key, const struct unicode_str *name )
     struct key_value *value;
     int i, index, nb_values;
 
+    if (key->flags & KEY_PREDEF)
+    {
+        set_error( STATUS_INVALID_HANDLE );
+        return;
+    }
+
     if (!(value = find_value( key, name, &index )))
     {
         set_error( STATUS_OBJECT_NAME_NOT_FOUND );
@@ -1614,7 +1675,7 @@ static int get_prefix_len( struct key *key, const char *name, struct file_load_i
     len = (p - info->tmp) * sizeof(WCHAR);
     for (res = 1; key != root_key; res++)
     {
-        if (len == key->namelen && !memicmpW( info->tmp, key->name, len / sizeof(WCHAR) )) break;
+        if (len == key->namelen && !memicmp_strW( info->tmp, key->name, len )) break;
         key = key->parent;
     }
     if (key == root_key) res = 0;  /* no matching name */
@@ -1734,59 +1795,64 @@ static int load_init_registry_from_file( const char *filename, struct key *key )
 
     save_branch_info[save_branch_count].path = filename;
     save_branch_info[save_branch_count++].key = (struct key *)grab_object( key );
-    make_object_static( &key->obj );
+    make_object_permanent( &key->obj );
     return (f != NULL);
 }
 
-static WCHAR *format_user_registry_path( const SID *sid, struct unicode_str *path )
+static WCHAR *format_user_registry_path( const struct sid *sid, struct unicode_str *path )
 {
-    static const WCHAR prefixW[] = {'U','s','e','r','\\','S',0};
-    static const WCHAR formatW[] = {'-','%','u',0};
-    WCHAR buffer[7 + 10 + 10 + 10 * SID_MAX_SUB_AUTHORITIES];
-    WCHAR *p = buffer;
+    char buffer[7 + 11 + 11 + 11 * ARRAY_SIZE(sid->sub_auth)], *p = buffer;
     unsigned int i;
 
-    strcpyW( p, prefixW );
-    p += strlenW( prefixW );
-    p += sprintfW( p, formatW, sid->Revision );
-    p += sprintfW( p, formatW, MAKELONG( MAKEWORD( sid->IdentifierAuthority.Value[5],
-                                                   sid->IdentifierAuthority.Value[4] ),
-                                         MAKEWORD( sid->IdentifierAuthority.Value[3],
-                                                   sid->IdentifierAuthority.Value[2] )));
-    for (i = 0; i < sid->SubAuthorityCount; i++)
-        p += sprintfW( p, formatW, sid->SubAuthority[i] );
-
-    path->len = (p - buffer) * sizeof(WCHAR);
-    path->str = p = memdup( buffer, path->len );
-    return p;
+    p += sprintf( p, "User\\S-%u-%u", sid->revision,
+                  ((unsigned int)sid->id_auth[2] << 24) |
+                  ((unsigned int)sid->id_auth[3] << 16) |
+                  ((unsigned int)sid->id_auth[4] << 8) |
+                  ((unsigned int)sid->id_auth[5]) );
+    for (i = 0; i < sid->sub_count; i++) p += sprintf( p, "-%u", sid->sub_auth[i] );
+    return ascii_to_unicode_str( buffer, path );
 }
 
-/* get the cpu architectures that can be supported in the current prefix */
-unsigned int get_prefix_cpu_mask(void)
+/* whether to use wow64 for i386 EXEs or launch a 32-bit wine */
+static int needs_wow64(void)
 {
-    /* Allowed server/client/prefix combinations:
-     *
-     *              prefix
-     *            32     64
-     *  server +------+------+ client
-     *         |  ok  | fail | 32
-     *      32 +------+------+---
-     *         | fail | fail | 64
-     *      ---+------+------+---
-     *         |  ok  |  ok  | 32
-     *      64 +------+------+---
-     *         | fail |  ok  | 64
-     *      ---+------+------+---
-     */
-    switch (prefix_type)
+#if defined(__APPLE__) && defined(__x86_64__)
+    struct utsname name;
+    unsigned major, minor;
+
+    return (uname(&name) == 0 &&
+            sscanf(name.release, "%u.%u", &major, &minor) == 2 &&
+            major >= 19 /* macOS 10.15 Catalina */);
+#else
+    return 0;
+#endif
+}
+
+static void init_supported_machines(void)
+{
+    unsigned int count = 0;
+#ifdef __i386__
+    if (prefix_type == PREFIX_32BIT) supported_machines[count++] = IMAGE_FILE_MACHINE_I386;
+#elif defined(__x86_64__)
+    if (prefix_type == PREFIX_64BIT || needs_wow64()) supported_machines[count++] = IMAGE_FILE_MACHINE_AMD64;
+    supported_machines[count++] = IMAGE_FILE_MACHINE_I386;
+
+    /* CX HACK 20810: detect when using a 32-bit prefix in Wow64 mode and disable various checks */
+    wow64_using_32bit_prefix = (prefix_type == PREFIX_32BIT) && needs_wow64();
+#elif defined(__arm__)
+    if (prefix_type == PREFIX_32BIT) supported_machines[count++] = IMAGE_FILE_MACHINE_ARMNT;
+#elif defined(__aarch64__)
+    if (prefix_type == PREFIX_64BIT)
     {
-    case PREFIX_64BIT:
-        /* 64-bit prefix requires 64-bit server */
-        return sizeof(void *) > sizeof(int) ? ~0 : 0;
-    case PREFIX_32BIT:
-    default:
-        return ~CPU_64BIT_MASK;  /* only 32-bit cpus supported on 32-bit prefix */
+        supported_machines[count++] = IMAGE_FILE_MACHINE_ARM64;
+        supported_machines[count++] = IMAGE_FILE_MACHINE_I386;
     }
+    supported_machines[count++] = IMAGE_FILE_MACHINE_ARMNT;
+#else
+#error Unsupported machine
+#endif
+    supported_machines_count = count;
+    native_machine = supported_machines[0];
 }
 
 /* registry initialisation */
@@ -1794,17 +1860,33 @@ void init_registry(void)
 {
     static const WCHAR HKLM[] = { 'M','a','c','h','i','n','e' };
     static const WCHAR HKU_default[] = { 'U','s','e','r','\\','.','D','e','f','a','u','l','t' };
-    static const WCHAR classes[] = {'S','o','f','t','w','a','r','e','\\',
-                                    'C','l','a','s','s','e','s','\\',
-                                    'W','o','w','6','4','3','2','N','o','d','e'};
+    static const WCHAR classes_i386[] = {'S','o','f','t','w','a','r','e','\\',
+                                         'C','l','a','s','s','e','s','\\',
+                                         'W','o','w','6','4','3','2','N','o','d','e'};
+    static const WCHAR classes_amd64[] = {'S','o','f','t','w','a','r','e','\\',
+                                          'C','l','a','s','s','e','s','\\',
+                                          'W','o','w','6','4','6','4','N','o','d','e'};
+    static const WCHAR classes_arm[] = {'S','o','f','t','w','a','r','e','\\',
+                                        'C','l','a','s','s','e','s','\\',
+                                        'W','o','w','A','A','3','2','N','o','d','e'};
+    static const WCHAR classes_arm64[] = {'S','o','f','t','w','a','r','e','\\',
+                                          'C','l','a','s','s','e','s','\\',
+                                          'W','o','w','A','A','6','4','N','o','d','e'};
+    static const WCHAR perflib[] = {'S','o','f','t','w','a','r','e','\\',
+                                    'M','i','c','r','o','s','o','f','t','\\',
+                                    'W','i','n','d','o','w','s',' ','N','T','\\',
+                                    'C','u','r','r','e','n','t','V','e','r','s','i','o','n','\\',
+                                    'P','e','r','f','l','i','b','\\',
+                                    '0','0','9'};
     static const struct unicode_str root_name = { NULL, 0 };
     static const struct unicode_str HKLM_name = { HKLM, sizeof(HKLM) };
     static const struct unicode_str HKU_name = { HKU_default, sizeof(HKU_default) };
-    static const struct unicode_str classes_name = { classes, sizeof(classes) };
+    static const struct unicode_str perflib_name = { perflib, sizeof(perflib) };
 
     WCHAR *current_user_path;
     struct unicode_str current_user_str;
     struct key *key, *hklm, *hkcu;
+    unsigned int i;
     char *p;
 
     /* switch to the config dir */
@@ -1814,7 +1896,7 @@ void init_registry(void)
     /* create the root key */
     root_key = alloc_key( &root_name, current_time );
     assert( root_key );
-    make_object_static( &root_key->obj );
+    make_object_permanent( &root_key->obj );
 
     /* load system.reg into Registry\Machine */
 
@@ -1831,6 +1913,8 @@ void init_registry(void)
     else if (prefix_type == PREFIX_UNKNOWN)
         prefix_type = PREFIX_32BIT;
 
+    init_supported_machines();
+
     /* load userdef.reg into Registry\User\.Default */
 
     if (!(key = create_key_recursive( root_key, &HKU_name, current_time )))
@@ -1842,17 +1926,26 @@ void init_registry(void)
     /* load user.reg into HKEY_CURRENT_USER */
 
     /* FIXME: match default user in token.c. should get from process token instead */
-    current_user_path = format_user_registry_path( security_local_user_sid, &current_user_str );
+    current_user_path = format_user_registry_path( &local_user_sid, &current_user_str );
     if (!current_user_path ||
         !(hkcu = create_key_recursive( root_key, &current_user_str, current_time )))
         fatal_error( "could not create HKEY_CURRENT_USER registry key\n" );
     free( current_user_path );
     load_init_registry_from_file( "user.reg", hkcu );
 
-    /* set the shared flag on Software\Classes\Wow6432Node */
-    if (prefix_type == PREFIX_64BIT)
+    /* set the shared flag on Software\Classes\Wow6432Node for all platforms */
+    for (i = 1; i < supported_machines_count; i++)
     {
-        if ((key = create_key_recursive( hklm, &classes_name, current_time )))
+        struct unicode_str name;
+
+        switch (supported_machines[i])
+        {
+        case IMAGE_FILE_MACHINE_I386:  name.str = classes_i386;  name.len = sizeof(classes_i386);  break;
+        case IMAGE_FILE_MACHINE_ARMNT: name.str = classes_arm;   name.len = sizeof(classes_arm);   break;
+        case IMAGE_FILE_MACHINE_AMD64: name.str = classes_amd64; name.len = sizeof(classes_amd64); break;
+        case IMAGE_FILE_MACHINE_ARM64: name.str = classes_arm64; name.len = sizeof(classes_arm64); break;
+        }
+        if ((key = create_key_recursive( hklm, &name, current_time )))
         {
             key->flags |= KEY_WOWSHARE;
             release_object( key );
@@ -1860,12 +1953,35 @@ void init_registry(void)
         /* FIXME: handle HKCU too */
     }
 
+    if ((key = create_key_recursive( hklm, &perflib_name, current_time )))
+    {
+        key->flags |= KEY_PREDEF;
+        release_object( key );
+    }
+
     release_object( hklm );
     release_object( hkcu );
 
     /* start the periodic save timer */
     set_periodic_save_timer();
 
+    /* create windows directories */
+
+    if (!mkdir( "drive_c/windows", 0777 ))
+    {
+        mkdir( "drive_c/windows/system32", 0777 );
+        for (i = 1; i < supported_machines_count; i++)
+        {
+            switch (supported_machines[i])
+            {
+            case IMAGE_FILE_MACHINE_I386:  mkdir( "drive_c/windows/syswow64", 0777 ); break;
+            case IMAGE_FILE_MACHINE_ARMNT: mkdir( "drive_c/windows/sysarm32", 0777 ); break;
+            case IMAGE_FILE_MACHINE_AMD64: mkdir( "drive_c/windows/sysx8664", 0777 ); break;
+            case IMAGE_FILE_MACHINE_ARM64: mkdir( "drive_c/windows/sysarm64", 0777 ); break;
+            }
+        }
+    }
+
     /* go back to the server dir */
     if (fchdir( server_dir_fd ) == -1) fatal_error( "chdir to server dir: %s\n", strerror( errno ));
 }
@@ -2031,7 +2147,10 @@ void flush_registry(void)
 /* determine if the thread is wow64 (32-bit client running on 64-bit prefix) */
 static int is_wow64_thread( struct thread *thread )
 {
-    return (prefix_type == PREFIX_64BIT && !(CPU_FLAG(thread->process->cpu) & CPU_64BIT_MASK));
+    /* CX HACK 21221: Apps in 32-bit prefixes running in Wow64 mode should not
+     * be subject to Wow6432Node redirections. */
+    return !wow64_using_32bit_prefix &&
+           (is_machine_64bit( native_machine ) && !is_machine_64bit( thread->process->machine ));
 }
 
 
@@ -2052,9 +2171,9 @@ DECL_HANDLER(create_key)
     class.len = (class.len / sizeof(WCHAR)) * sizeof(WCHAR);
 
     if (!objattr->rootdir && name.len >= sizeof(root_name) &&
-        !memicmpW( name.str, root_name, sizeof(root_name)/sizeof(WCHAR) ))
+        !memicmp_strW( name.str, root_name, sizeof(root_name) ))
     {
-        name.str += sizeof(root_name)/sizeof(WCHAR);
+        name.str += ARRAY_SIZE( root_name );
         name.len -= sizeof(root_name);
     }
 
@@ -2203,16 +2322,16 @@ DECL_HANDLER(load_registry)
 
     if (!objattr) return;
 
-    if (!thread_single_check_privilege( current, &SeRestorePrivilege ))
+    if (!thread_single_check_privilege( current, SeRestorePrivilege ))
     {
         set_error( STATUS_PRIVILEGE_NOT_HELD );
         return;
     }
 
     if (!objattr->rootdir && name.len >= sizeof(root_name) &&
-        !memicmpW( name.str, root_name, sizeof(root_name)/sizeof(WCHAR) ))
+        !memicmp_strW( name.str, root_name, sizeof(root_name) ))
     {
-        name.str += sizeof(root_name)/sizeof(WCHAR);
+        name.str += ARRAY_SIZE( root_name );
         name.len -= sizeof(root_name);
     }
 
@@ -2230,18 +2349,30 @@ DECL_HANDLER(load_registry)
 
 DECL_HANDLER(unload_registry)
 {
-    struct key *key;
+    struct key *key, *parent;
+    struct unicode_str name;
+    unsigned int access = 0;
 
-    if (!thread_single_check_privilege( current, &SeRestorePrivilege ))
+    if (!thread_single_check_privilege( current, SeRestorePrivilege ))
     {
         set_error( STATUS_PRIVILEGE_NOT_HELD );
         return;
     }
 
-    if ((key = get_hkey_obj( req->hkey, 0 )))
+    if (!is_wow64_thread( current )) access = (access & ~KEY_WOW64_32KEY) | KEY_WOW64_64KEY;
+
+    if ((parent = get_parent_hkey_obj( req->parent )))
     {
-        delete_key( key, 1 );     /* FIXME */
-        release_object( key );
+        get_req_path( &name, !req->parent );
+        if ((key = open_key( parent, &name, access, req->attributes )))
+        {
+            if (key->obj.handle_count)
+                set_error( STATUS_CANNOT_DELETE );
+            else
+                delete_key( key, 1 );     /* FIXME */
+            release_object( key );
+        }
+        release_object( parent );
     }
 }
 
@@ -2250,7 +2381,7 @@ DECL_HANDLER(save_registry)
 {
     struct key *key;
 
-    if (!thread_single_check_privilege( current, &SeBackupPrivilege ))
+    if (!thread_single_check_privilege( current, SeBackupPrivilege ))
     {
         set_error( STATUS_PRIVILEGE_NOT_HELD );
         return;
@@ -2277,20 +2408,13 @@ DECL_HANDLER(set_registry_notification)
         if (event)
         {
             notify = find_notify( key, current->process, req->hkey );
-            if (notify)
-            {
-                if (notify->event)
-                    release_object( notify->event );
-                grab_object( event );
-                notify->event = event;
-            }
-            else
+            if (!notify)
             {
                 notify = mem_alloc( sizeof(*notify) );
                 if (notify)
                 {
-                    grab_object( event );
-                    notify->event   = event;
+                    notify->events  = NULL;
+                    notify->event_count = 0;
                     notify->subtree = req->subtree;
                     notify->filter  = req->filter;
                     notify->hkey    = req->hkey;
@@ -2300,8 +2424,16 @@ DECL_HANDLER(set_registry_notification)
             }
             if (notify)
             {
-                reset_event( event );
-                set_error( STATUS_PENDING );
+                struct event **new_array;
+
+                if ((new_array = realloc( notify->events, (notify->event_count + 1) * sizeof(*notify->events) )))
+                {
+                    notify->events = new_array;
+                    notify->events[notify->event_count++] = (struct event *)grab_object( event );
+                    reset_event( event );
+                    set_error( STATUS_PENDING );
+                }
+                else set_error( STATUS_NO_MEMORY );
             }
             release_object( event );
         }
