@@ -29,14 +29,7 @@
 
 #include <stdio.h>
 #include <stdlib.h>
-#include <string.h>
-
-#ifndef max
-#define max(a,b)   (((a) > (b)) ? (a) : (b))
-#endif
-#ifndef min
-#define min(a,b)   (((a) < (b)) ? (a) : (b))
-#endif
+#include "../tools.h"
 
 typedef enum
 {
@@ -47,7 +40,6 @@ typedef enum
     TYPE_STDCALL,      /* stdcall function (Win32) */
     TYPE_CDECL,        /* cdecl function (Win32) */
     TYPE_VARARGS,      /* varargs function (Win32) */
-    TYPE_THISCALL,     /* thiscall function (Win32 on i386) */
     TYPE_EXTERN,       /* external symbol (Win32) */
     TYPE_NBTYPES
 } ORD_TYPE;
@@ -99,11 +91,11 @@ typedef struct
 {
     ORD_TYPE    type;
     int         ordinal;
+    int         hint;
     int         lineno;
     int         flags;
     char       *name;         /* public name of this function */
     char       *link_name;    /* name of the C symbol to link to */
-    char       *impl_name;    /* name of the C symbol of the real implementation (thunks only) */
     char       *export_name;  /* name exported under for noname exports */
     union
     {
@@ -113,6 +105,34 @@ typedef struct
     } u;
 } ORDDEF;
 
+struct apiset_entry
+{
+    unsigned int name_off;
+    unsigned int name_len;
+    unsigned int hash;
+    unsigned int hash_len;
+    unsigned int val_count;
+    struct apiset_value
+    {
+        unsigned int name_off;
+        unsigned int name_len;
+        unsigned int val_off;
+        unsigned int val_len;
+    } values[4];
+};
+
+struct apiset
+{
+    unsigned int count;
+    unsigned int size;
+    struct apiset_entry *entries;
+    unsigned int str_pos;
+    unsigned int str_size;
+    char *strings;
+};
+
+static const unsigned int apiset_hash_factor = 31;
+
 typedef struct
 {
     char            *src_name;           /* file name of the source spec file */
@@ -130,62 +150,62 @@ typedef struct
     int              alloc_entry_points; /* number of allocated entry points */
     int              nb_names;           /* number of entry points with names */
     unsigned int     nb_resources;       /* number of resources */
-    int              nb_syscalls;        /* number of syscalls */
     int              characteristics;    /* characteristics for the PE header */
     int              dll_characteristics;/* DLL characteristics for the PE header */
     int              subsystem;          /* subsystem id */
     int              subsystem_major;    /* subsystem version major number */
     int              subsystem_minor;    /* subsystem version minor number */
+    int              syscall_table;      /* syscall table id */
+    int              unicode_app;        /* default to unicode entry point */
     ORDDEF          *entry_points;       /* dll entry points */
     ORDDEF         **names;              /* array of entry point names (points into entry_points) */
     ORDDEF         **ordinals;           /* array of dll ordinals (points into entry_points) */
     struct resource *resources;          /* array of dll resources (format differs between Win16/Win32) */
-    ORDDEF         **syscalls;           /* array of syscalls (points into entry_points) */
+    struct apiset    apiset;             /* list of defined api sets */
 } DLLSPEC;
 
-enum target_cpu
-{
-    CPU_x86, CPU_x86_64, CPU_POWERPC, CPU_ARM, CPU_ARM64, CPU_LAST = CPU_ARM64
-};
-
-enum target_platform
-{
-    PLATFORM_UNSPECIFIED,
-    PLATFORM_APPLE,
-    PLATFORM_FREEBSD,
-    PLATFORM_SOLARIS,
-    PLATFORM_WINDOWS
-};
-
 extern char *target_alias;
-extern enum target_cpu target_cpu;
-extern enum target_platform target_platform;
+extern struct target target;
 
-struct strarray
+static inline unsigned int get_ptr_size(void)
 {
-    const char **str;
-    unsigned int count;
-    unsigned int max;
-};
+    return get_target_ptr_size( target );
+}
+
+/* return the size of a pointer on the target CPU */
+static inline unsigned int get_host_ptr_size(void)
+{
+    if (target.cpu == CPU_x86_32on64)
+        return 8;
+    return get_ptr_size();
+}
+
+static inline int is_pe(void)
+{
+    return target.platform == PLATFORM_MINGW || target.platform == PLATFORM_WINDOWS;
+}
 
 /* entry point flags */
-#define FLAG_NORELAY   0x01  /* don't use relay debugging for this function */
-#define FLAG_NONAME    0x02  /* don't export function by name */
-#define FLAG_RET16     0x04  /* function returns a 16-bit value */
-#define FLAG_RET64     0x08  /* function returns a 64-bit value */
-#define FLAG_REGISTER  0x10  /* use register calling convention */
-#define FLAG_PRIVATE   0x20  /* function is private (cannot be imported) */
-#define FLAG_ORDINAL   0x40  /* function should be imported by ordinal */
+#define FLAG_NORELAY   0x0001  /* don't use relay debugging for this function */
+#define FLAG_NONAME    0x0002  /* don't export function by name */
+#define FLAG_RET16     0x0004  /* function returns a 16-bit value */
+#define FLAG_RET64     0x0008  /* function returns a 64-bit value */
+#define FLAG_REGISTER  0x0010  /* use register calling convention */
+#define FLAG_PRIVATE   0x0020  /* function is private (cannot be imported) */
+#define FLAG_ORDINAL   0x0040  /* function should be imported by ordinal */
+#define FLAG_THISCALL  0x0080  /* function uses thiscall calling convention */
+#define FLAG_FASTCALL  0x0100  /* function uses fastcall calling convention */
+#define FLAG_SYSCALL   0x0200  /* function is a system call */
+#define FLAG_IMPORT    0x0400  /* export is imported from another module */
 
-#define FLAG_FORWARD   0x100  /* function is a forwarded name */
-#define FLAG_EXT_LINK  0x200  /* function links to an external symbol */
-#define FLAG_EXPORT32  0x400  /* 32-bit export in 16-bit spec file */
-#define FLAG_SYSCALL   0x800  /* function should be called through a syscall thunk */
+#define FLAG_FORWARD   0x1000  /* function is a forwarded name */
+#define FLAG_EXT_LINK  0x2000  /* function links to an external symbol */
+#define FLAG_EXPORT32  0x4000  /* 32-bit export in 16-bit spec file */
 
-#define FLAG_CPU(cpu)  (0x01000 << (cpu))
-#define FLAG_CPU_MASK  (FLAG_CPU(CPU_LAST + 1) - FLAG_CPU(0))
+#define FLAG_CPU(cpu)  (0x10000 << (cpu))
+#define FLAG_CPU_MASK  (FLAG_CPU_WIN32 | FLAG_CPU_WIN64)
 #define FLAG_CPU_WIN64 (FLAG_CPU(CPU_x86_64) | FLAG_CPU(CPU_ARM64))
-#define FLAG_CPU_WIN32 (FLAG_CPU_MASK & ~FLAG_CPU_WIN64)
+#define FLAG_CPU_WIN32 (FLAG_CPU(CPU_i386) | FLAG_CPU(CPU_ARM))
 
 #define MAX_ORDINALS  65535
 
@@ -208,7 +228,8 @@ struct strarray
 #define IMAGE_FILE_UP_SYSTEM_ONLY	   0x4000
 #define IMAGE_FILE_BYTES_REVERSED_HI	   0x8000
 
-#define IMAGE_DLLCHARACTERISTICS_NX_COMPAT 0x0100
+#define IMAGE_DLLCHARACTERISTICS_PREFER_NATIVE 0x0010 /* Wine extension */
+#define IMAGE_DLLCHARACTERISTICS_NX_COMPAT     0x0100
 
 #define	IMAGE_SUBSYSTEM_NATIVE      1
 #define	IMAGE_SUBSYSTEM_WINDOWS_GUI 2
@@ -217,10 +238,6 @@ struct strarray
 
 /* global functions */
 
-#ifndef __GNUC__
-#define __attribute__(X)
-#endif
-
 #ifndef DECLSPEC_NORETURN
 # if defined(_MSC_VER) && (_MSC_VER >= 1200) && !defined(MIDL_PASS)
 #  define DECLSPEC_NORETURN __declspec(noreturn)
@@ -228,17 +245,7 @@ struct strarray
 #  define DECLSPEC_NORETURN __attribute__((noreturn))
 # endif
 #endif
-
-extern void *xmalloc (size_t size);
-extern void *xrealloc (void *ptr, size_t size);
-extern char *xstrdup( const char *str );
 extern char *strupper(char *s);
-extern int strendswith(const char* str, const char* end);
-extern char *strmake(const char* fmt, ...) __attribute__((__format__ (__printf__, 1, 2 )));
-extern struct strarray strarray_fromstring( const char *str, const char *delim );
-extern void strarray_add( struct strarray *array, ... );
-extern void strarray_addv( struct strarray *array, char * const *argv );
-extern void strarray_addall( struct strarray *array, struct strarray args );
 extern DECLSPEC_NORETURN void fatal_error( const char *msg, ... )
    __attribute__ ((__format__ (__printf__, 1, 2)));
 extern DECLSPEC_NORETURN void fatal_perror( const char *msg, ... )
@@ -251,8 +258,11 @@ extern int output( const char *format, ... )
    __attribute__ ((__format__ (__printf__, 1, 2)));
 extern void output_cfi( const char *format, ... )
    __attribute__ ((__format__ (__printf__, 1, 2)));
+extern void output_rva( const char *format, ... )
+   __attribute__ ((__format__ (__printf__, 1, 2)));
 extern void spawn( struct strarray array );
 extern struct strarray find_tool( const char *name, const char * const *names );
+extern struct strarray find_link_tool(void);
 extern struct strarray get_as_command(void);
 extern struct strarray get_ld_command(void);
 extern const char *get_nm_command(void);
@@ -261,6 +271,9 @@ extern char *get_temp_file_name( const char *prefix, const char *suffix );
 extern void output_standard_file_header(void);
 extern FILE *open_input_file( const char *srcdir, const char *name );
 extern void close_input_file( FILE *file );
+extern void open_output_file(void);
+extern void close_output_file(void);
+extern char *open_temp_output_file( const char *suffix );
 extern void dump_bytes( const void *buffer, unsigned int size );
 extern int remove_stdcall_decoration( char *name );
 extern void assemble_file( const char *src_file, const char *obj_file );
@@ -268,39 +281,50 @@ extern DLLSPEC *alloc_dll_spec(void);
 extern void free_dll_spec( DLLSPEC *spec );
 extern char *make_c_identifier( const char *str );
 extern const char *get_stub_name( const ORDDEF *odp, const DLLSPEC *spec );
-extern int get_cpu_from_name( const char *name );
+extern const char *get_link_name( const ORDDEF *odp );
+extern int sort_func_list( ORDDEF **list, int count, int (*compare)(const void *, const void *) );
 extern unsigned int get_alignment(unsigned int align);
 extern unsigned int get_page_size(void);
-extern unsigned int get_ptr_size(void);
 extern unsigned int get_args_size( const ORDDEF *odp );
 extern const char *asm_name( const char *func );
+extern const char *thunk32_name( const char *func );
 extern const char *func_declaration( const char *func );
 extern const char *asm_globl( const char *func );
 extern const char *get_asm_ptr_keyword(void);
+extern const char *get_asm_host_ptr_keyword(void);
 extern const char *get_asm_string_keyword(void);
+extern const char *get_asm_export_section(void);
 extern const char *get_asm_rodata_section(void);
+extern const char *get_asm_rsrc_section(void);
 extern const char *get_asm_string_section(void);
+extern const char *arm64_page( const char *sym );
+extern const char *arm64_pageoff( const char *sym );
 extern void output_function_size( const char *name );
 extern void output_gnu_stack_note(void);
 
 extern void add_import_dll( const char *name, const char *filename );
 extern void add_delayed_import( const char *name );
 extern void add_extra_ld_symbol( const char *name );
-extern void read_undef_symbols( DLLSPEC *spec, char **argv );
+extern void add_spec_extra_ld_symbol( const char *name );
+extern void read_undef_symbols( DLLSPEC *spec, struct strarray files );
 extern void resolve_imports( DLLSPEC *spec );
 extern int is_undefined( const char *name );
 extern int has_imports(void);
 extern void output_get_pc_thunk(void);
 extern void output_module( DLLSPEC *spec );
 extern void output_stubs( DLLSPEC *spec );
+extern void output_syscalls( DLLSPEC *spec );
 extern void output_imports( DLLSPEC *spec );
-extern void output_import_lib( DLLSPEC *spec, char **argv );
+extern void output_static_lib( DLLSPEC *spec, struct strarray files );
 extern void output_exports( DLLSPEC *spec );
 extern int load_res32_file( const char *name, DLLSPEC *spec );
 extern void output_resources( DLLSPEC *spec );
 extern void output_bin_resources( DLLSPEC *spec, unsigned int start_rva );
+extern void output_spec32_file( DLLSPEC *spec );
 extern void output_fake_module( DLLSPEC *spec );
-extern void output_def_file( DLLSPEC *spec, int include_private );
+extern void output_data_module( DLLSPEC *spec );
+extern void output_def_file( DLLSPEC *spec, int import_only );
+extern void output_apiset_lib( DLLSPEC *spec, const struct apiset *apiset );
 extern void load_res16_file( const char *name, DLLSPEC *spec );
 extern void output_res16_data( DLLSPEC *spec );
 extern void output_bin_res16_data( DLLSPEC *spec );
@@ -310,15 +334,13 @@ extern void output_spec16_file( DLLSPEC *spec );
 extern void output_fake_module16( DLLSPEC *spec16 );
 extern void output_res_o_file( DLLSPEC *spec );
 extern void output_asm_relays16(void);
-
-extern void BuildSpec32File( DLLSPEC *spec );
+extern void make_builtin_files( struct strarray files );
+extern void fixup_constructors( struct strarray files );
 
 extern void add_16bit_exports( DLLSPEC *spec32, DLLSPEC *spec16 );
 extern int parse_spec_file( FILE *file, DLLSPEC *spec );
 extern int parse_def_file( FILE *file, DLLSPEC *spec );
 
-extern int sort_func_list( ORDDEF **list, int count, int (*compare)(const void *, const void *) );
-
 /* buffer management */
 
 extern int byte_swapped;
@@ -326,30 +348,12 @@ extern const char *input_buffer_filename;
 extern const unsigned char *input_buffer;
 extern size_t input_buffer_pos;
 extern size_t input_buffer_size;
-extern unsigned char *output_buffer;
-extern size_t output_buffer_pos;
-extern size_t output_buffer_rva;
-extern size_t output_buffer_size;
 
 extern void init_input_buffer( const char *file );
-extern void init_output_buffer(void);
-extern void flush_output_buffer(void);
 extern unsigned char get_byte(void);
 extern unsigned short get_word(void);
 extern unsigned int get_dword(void);
-extern void put_data( const void *data, size_t size );
-extern void put_byte( unsigned char val );
-extern void put_word( unsigned short val );
-extern void put_dword( unsigned int val );
-extern void put_qword( unsigned int val );
 extern void put_pword( unsigned int val );
-extern void put_str( const char *str );
-extern void align_output( unsigned int align );
-extern void align_output_rva( unsigned int file_align, unsigned int rva_align );
-extern size_t label_pos( const char *name );
-extern size_t label_rva( const char *name );
-extern size_t label_rva_align( const char *name );
-extern void put_label( const char *name );
 
 /* global variables */
 
@@ -362,6 +366,11 @@ extern int verbose;
 extern int link_ext_symbols;
 extern int force_pointer_size;
 extern int unwind_tables;
+extern int use_msvcrt;
+extern int unix_lib;
+extern int safe_seh;
+extern int prefer_native;
+extern int data_only;
 
 extern char *input_file_name;
 extern char *spec_file_name;
@@ -369,6 +378,7 @@ extern FILE *output_file;
 extern const char *output_file_name;
 
 extern struct strarray lib_path;
+extern struct strarray tools_path;
 extern struct strarray as_command;
 extern struct strarray cc_command;
 extern struct strarray ld_command;
@@ -379,5 +389,6 @@ extern char *arch_option;
 extern const char *float_abi_option;
 extern int thumb_mode;
 extern int needs_get_pc_thunk;
+extern int needs_invoke32;
 
 #endif  /* __WINE_BUILD_H */
