@@ -23,7 +23,6 @@
  */
 
 #include "config.h"
-#include "wine/port.h"
 
 #include <assert.h>
 #include <ctype.h>
@@ -56,7 +55,6 @@ static const char * const TypeNames[TYPE_NBTYPES] =
     "stdcall",      /* TYPE_STDCALL */
     "cdecl",        /* TYPE_CDECL */
     "varargs",      /* TYPE_VARARGS */
-    "thiscall",     /* TYPE_THISCALL */
     "extern"        /* TYPE_EXTERN */
 };
 
@@ -69,6 +67,10 @@ static const char * const FlagNames[] =
     "register",    /* FLAG_REGISTER */
     "private",     /* FLAG_PRIVATE */
     "ordinal",     /* FLAG_ORDINAL */
+    "thiscall",    /* FLAG_THISCALL */
+    "fastcall",    /* FLAG_FASTCALL */
+    "syscall",     /* FLAG_SYSCALL */
+    "import",      /* FLAG_IMPORT */
     NULL
 };
 
@@ -287,10 +289,44 @@ static int parse_spec_arguments( ORDDEF *odp, DLLSPEC *spec, int optional )
     }
 
     odp->u.func.nb_args = i;
-    if (odp->type == TYPE_THISCALL && (!i || odp->u.func.args[0] != ARG_PTR))
+    if (odp->flags & FLAG_THISCALL)
     {
-        error( "First argument of a thiscall function must be a pointer\n" );
-        return 0;
+        if (odp->type != TYPE_STDCALL)
+        {
+            error( "A thiscall function must use the stdcall convention\n" );
+            return 0;
+        }
+        if (!i || odp->u.func.args[0] != ARG_PTR)
+        {
+            error( "First argument of a thiscall function must be a pointer\n" );
+            return 0;
+        }
+    }
+    if (odp->flags & FLAG_FASTCALL)
+    {
+        if (odp->type != TYPE_STDCALL)
+        {
+            error( "A fastcall function must use the stdcall convention\n" );
+            return 0;
+        }
+        if (!i || (odp->u.func.args[0] != ARG_PTR && odp->u.func.args[0] != ARG_LONG))
+        {
+            error( "First argument of a fastcall function must be a pointer or integer\n" );
+            return 0;
+        }
+        if (i > 1 && odp->u.func.args[1] != ARG_PTR && odp->u.func.args[1] != ARG_LONG)
+        {
+            error( "Second argument of a fastcall function must be a pointer or integer\n" );
+            return 0;
+        }
+    }
+    if (odp->flags & FLAG_SYSCALL)
+    {
+        if (odp->type != TYPE_STDCALL && odp->type != TYPE_CDECL)
+        {
+            error( "A syscall function must use either the stdcall or the cdecl convention\n" );
+            return 0;
+        }
     }
     return 1;
 }
@@ -311,11 +347,6 @@ static int parse_spec_export( ORDDEF *odp, DLLSPEC *spec )
         error( "'stdcall' not supported for Win16\n" );
         return 0;
     }
-    if (!is_win32 && odp->type == TYPE_THISCALL)
-    {
-        error( "'thiscall' not supported for Win16\n" );
-        return 0;
-    }
     if (is_win32 && odp->type == TYPE_PASCAL)
     {
         error( "'pascal' not supported for Win32\n" );
@@ -327,6 +358,9 @@ static int parse_spec_export( ORDDEF *odp, DLLSPEC *spec )
     if (odp->type == TYPE_VARARGS)
         odp->flags |= FLAG_NORELAY;  /* no relay debug possible for varags entry point */
 
+    if (target.cpu != CPU_i386 && target.cpu != CPU_x86_32on64)
+        odp->flags &= ~(FLAG_THISCALL | FLAG_FASTCALL);
+
     if (!(token = GetToken(1)))
     {
         if (!strcmp( odp->name, "@" ))
@@ -349,12 +383,6 @@ static int parse_spec_export( ORDDEF *odp, DLLSPEC *spec )
             odp->flags |= FLAG_FORWARD;
         }
     }
-    if (target_cpu == CPU_x86 && odp->type == TYPE_THISCALL && !(odp->flags & FLAG_FORWARD))
-    {
-        char *link_name = strmake( "__thiscall_%s", odp->link_name );
-        free( odp->link_name );
-        odp->link_name = link_name;
-    }
     return 1;
 }
 
@@ -401,11 +429,6 @@ static int parse_spec_stub( ORDDEF *odp, DLLSPEC *spec )
 {
     odp->u.func.nb_args = -1;
     odp->link_name = xstrdup("");
-    /* don't bother generating stubs for Winelib */
-    if (odp->flags & FLAG_CPU_MASK)
-        odp->flags &= FLAG_CPU(CPU_x86) | FLAG_CPU(CPU_x86_64) | FLAG_CPU(CPU_ARM) | FLAG_CPU(CPU_ARM64);
-    else
-        odp->flags |= FLAG_CPU(CPU_x86) | FLAG_CPU(CPU_x86_64) | FLAG_CPU(CPU_ARM) | FLAG_CPU(CPU_ARM64);
 
     return parse_spec_arguments( odp, spec, 1 );
 }
@@ -448,14 +471,13 @@ static int parse_spec_extern( ORDDEF *odp, DLLSPEC *spec )
  *
  * Parse the optional flags for an entry point in a .spec file.
  */
-static const char *parse_spec_flags( DLLSPEC *spec, ORDDEF *odp )
+static const char *parse_spec_flags( DLLSPEC *spec, ORDDEF *odp, const char *token )
 {
-    unsigned int i;
-    const char *token;
+    unsigned int i, cpu_mask = 0;
 
     do
     {
-        if (!(token = GetToken(0))) break;
+        token++;
         if (!strncmp( token, "arch=", 5))
         {
             char *args = xstrdup( token + 5 );
@@ -473,13 +495,24 @@ static const char *parse_spec_flags( DLLSPEC *spec, ORDDEF *odp )
                     odp->flags |= FLAG_CPU_WIN64;
                 else
                 {
-                    int cpu = get_cpu_from_name( cpu_name );
+                    int cpu = get_cpu_from_name( cpu_name + (cpu_name[0] == '!') );
                     if (cpu == -1)
                     {
                         error( "Unknown architecture '%s'\n", cpu_name );
                         return NULL;
                     }
-                    odp->flags |= FLAG_CPU( cpu );
+                    if (cpu_name[0] == '!')
+                    {
+                        cpu_mask |= FLAG_CPU( cpu );
+                        if (cpu == CPU_i386)
+                            cpu_mask |= FLAG_CPU( CPU_x86_32on64 );
+                    }
+                    else
+                    {
+                        odp->flags |= FLAG_CPU( cpu );
+                        if (cpu == CPU_i386)
+                            odp->flags |= FLAG_CPU( CPU_x86_32on64 );
+                    }
                 }
                 cpu_name = strtok( NULL, "," );
             }
@@ -487,7 +520,7 @@ static const char *parse_spec_flags( DLLSPEC *spec, ORDDEF *odp )
         }
         else if (!strcmp( token, "i386" ))  /* backwards compatibility */
         {
-            odp->flags |= FLAG_CPU(CPU_x86);
+            odp->flags |= FLAG_CPU(CPU_i386)| FLAG_CPU(CPU_x86_32on64);
         }
         else
         {
@@ -506,6 +539,8 @@ static const char *parse_spec_flags( DLLSPEC *spec, ORDDEF *odp )
                     error( "Flag '%s' is not supported in Win32\n", FlagNames[i] );
                 break;
             case FLAG_RET64:
+            case FLAG_THISCALL:
+            case FLAG_FASTCALL:
                 if (spec->type == SPEC_WIN16)
                     error( "Flag '%s' is not supported in Win16\n", FlagNames[i] );
                 break;
@@ -515,28 +550,11 @@ static const char *parse_spec_flags( DLLSPEC *spec, ORDDEF *odp )
         token = GetToken(0);
     } while (token && *token == '-');
 
+    if (cpu_mask) odp->flags |= FLAG_CPU_MASK & ~cpu_mask;
     return token;
 }
 
 
-static int needs_syscall( ORDDEF *odp, DLLSPEC *spec )
-{
-    if (target_cpu != CPU_x86 && target_cpu != CPU_x86_64)
-        return 0;
-    if (odp->flags & (FLAG_FORWARD | FLAG_REGISTER))
-        return 0;
-    if (odp->type != TYPE_STDCALL)
-        return 0;
-    if (!spec->dll_name || strcmp(spec->dll_name, "ntdll"))
-        return 0;
-    if (!odp->name)
-        return 0;
-    if (strncmp(odp->name, "Nt", 2) && strncmp(odp->name, "Zw", 2))
-        return 0;
-    return 1;
-}
-
-
 /*******************************************************************
  *         parse_spec_ordinal
  *
@@ -556,12 +574,20 @@ static int parse_spec_ordinal( int ordinal, DLLSPEC *spec )
 
     if (odp->type >= TYPE_NBTYPES)
     {
-        error( "Expected type after ordinal, found '%s' instead\n", token );
-        goto error;
+        if (!strcmp( token, "thiscall" )) /* for backwards compatibility */
+        {
+            odp->type = TYPE_STDCALL;
+            odp->flags |= FLAG_THISCALL;
+        }
+        else
+        {
+            error( "Expected type after ordinal, found '%s' instead\n", token );
+            goto error;
+        }
     }
 
     if (!(token = GetToken(0))) goto error;
-    if (*token == '-' && !(token = parse_spec_flags( spec, odp ))) goto error;
+    if (*token == '-' && !(token = parse_spec_flags( spec, odp, token ))) goto error;
 
     if (ordinal == -1 && spec->type != SPEC_WIN32 && !(odp->flags & FLAG_EXPORT32))
     {
@@ -589,7 +615,6 @@ static int parse_spec_ordinal( int ordinal, DLLSPEC *spec )
     case TYPE_STDCALL:
     case TYPE_VARARGS:
     case TYPE_CDECL:
-    case TYPE_THISCALL:
         if (!parse_spec_export( odp, spec )) goto error;
         break;
     case TYPE_ABS:
@@ -605,21 +630,19 @@ static int parse_spec_ordinal( int ordinal, DLLSPEC *spec )
         assert( 0 );
     }
 
-    if (needs_syscall( odp, spec ))
-    {
-        char *link_name = strmake( "__syscall_%s", odp->link_name );
-        odp->impl_name = odp->link_name;
-        odp->link_name = link_name;
-        odp->flags |= FLAG_SYSCALL;
-    }
-
-    if ((odp->flags & FLAG_CPU_MASK) && !(odp->flags & FLAG_CPU(target_cpu)))
+    if ((odp->flags & FLAG_CPU_MASK) && !(odp->flags & FLAG_CPU(target.cpu)))
     {
         /* ignore this entry point */
         spec->nb_entry_points--;
         return 1;
     }
 
+    if (data_only && !(odp->flags & FLAG_FORWARD))
+    {
+        error( "Only forwarded entry points are allowed in data-only mode\n" );
+        goto error;
+    }
+
     if (ordinal != -1)
     {
         if (!ordinal)
@@ -689,6 +712,132 @@ error:
 }
 
 
+static unsigned int apiset_hash_len( const char *str )
+{
+    return strrchr( str, '-' ) - str;
+}
+
+static unsigned int apiset_hash( const char *str )
+{
+    unsigned int ret = 0, len = apiset_hash_len( str );
+    while (len--) ret = ret * apiset_hash_factor + *str++;
+    return ret;
+}
+
+static unsigned int apiset_add_str( struct apiset *apiset, const char *str, unsigned int len )
+{
+    char *ret;
+
+    if (!apiset->strings || !(ret = strstr( apiset->strings, str )))
+    {
+        if (apiset->str_pos + len >= apiset->str_size)
+        {
+            apiset->str_size = max( apiset->str_size * 2, 1024 );
+            apiset->strings = xrealloc( apiset->strings, apiset->str_size );
+        }
+        ret = apiset->strings + apiset->str_pos;
+        memcpy( ret, str, len );
+        ret[len] = 0;
+        apiset->str_pos += len;
+    }
+    return ret - apiset->strings;
+}
+
+static void add_apiset( struct apiset *apiset, const char *api )
+{
+    struct apiset_entry *entry;
+
+    if (apiset->count == apiset->size)
+    {
+        apiset->size = max( apiset->size * 2, 64 );
+        apiset->entries = xrealloc( apiset->entries, apiset->size * sizeof(*apiset->entries) );
+    }
+    entry = &apiset->entries[apiset->count++];
+    entry->name_len = strlen( api );
+    entry->name_off = apiset_add_str( apiset, api, entry->name_len );
+    entry->hash = apiset_hash( api );
+    entry->hash_len = apiset_hash_len( api );
+    entry->val_count = 0;
+}
+
+static void add_apiset_value( struct apiset *apiset, const char *value )
+{
+    struct apiset_entry *entry = &apiset->entries[apiset->count - 1];
+
+    if (entry->val_count < ARRAY_SIZE(entry->values) - 1)
+    {
+        struct apiset_value *val = &entry->values[entry->val_count++];
+        char *sep = strchr( value, ':' );
+
+        if (sep)
+        {
+            val->name_len = sep - value;
+            val->name_off = apiset_add_str( apiset, value, val->name_len );
+            val->val_len = strlen( sep + 1 );
+            val->val_off = apiset_add_str( apiset, sep + 1, val->val_len );
+        }
+        else
+        {
+            val->name_len = val->name_off = 0;
+            val->val_len = strlen( value );
+            val->val_off = apiset_add_str( apiset, value, val->val_len );
+        }
+    }
+    else error( "Too many values for api '%.*s'\n", entry->name_len, apiset->strings + entry->name_off );
+}
+
+/*******************************************************************
+ *         parse_spec_apiset
+ */
+static int parse_spec_apiset( DLLSPEC *spec )
+{
+    struct apiset_entry *entry;
+    const char *token;
+    unsigned int i, hash;
+
+    if (!data_only)
+    {
+        error( "Apiset definitions are only allowed in data-only mode\n" );
+        return 0;
+    }
+
+    if (!(token = GetToken(0))) return 0;
+
+    if (!strncmp( token, "api-", 4 ) && !strncmp( token, "ext-", 4 ))
+    {
+        error( "Unrecognized API set name '%s'\n", token );
+        return 0;
+    }
+
+    hash = apiset_hash( token );
+    for (i = 0, entry = spec->apiset.entries; i < spec->apiset.count; i++, entry++)
+    {
+        if (entry->name_len == strlen( token ) &&
+            !strncmp( spec->apiset.strings + entry->name_off, token, entry->name_len ))
+        {
+            error( "Duplicate API set '%s'\n", token );
+            return 0;
+        }
+        if (entry->hash == hash)
+        {
+            error( "Duplicate hash code '%.*s' and '%s'\n",
+                   entry->name_len, spec->apiset.strings + entry->name_off, token );
+            return 0;
+        }
+    }
+    add_apiset( &spec->apiset, token );
+
+    if (!(token = GetToken(0)) || strcmp( token, "=" ))
+    {
+        error( "Syntax error near '%s'\n", token );
+        return 0;
+    }
+
+    while ((token = GetToken(1))) add_apiset_value( &spec->apiset, token );
+    return 1;
+}
+
+
 static int name_compare( const void *ptr1, const void *ptr2 )
 {
     const ORDDEF *odp1 = *(const ORDDEF * const *)ptr1;
@@ -747,6 +896,7 @@ static void assign_names( DLLSPEC *spec )
 
         /* sort the list of names */
         qsort( spec->names, spec->nb_names, sizeof(spec->names[0]), name_compare );
+        for (i = 0; i < spec->nb_names; i++) spec->names[i]->hint = i;
     }
 }
 
@@ -809,37 +959,6 @@ static void assign_ordinals( DLLSPEC *spec )
 }
 
 
-static int link_name_compare( const void *e1, const void *e2 )
-{
-    const ORDDEF *odp1 = *(const ORDDEF * const *)e1;
-    const ORDDEF *odp2 = *(const ORDDEF * const *)e2;
-    return strcmp(odp1->link_name, odp2->link_name);
-}
-
-
-static void assign_syscalls( DLLSPEC *spec )
-{
-    int i;
-
-    spec->syscalls = xmalloc( (spec->limit - spec->base + 1) * sizeof(*spec->syscalls) );
-    spec->nb_syscalls = 0;
-
-    for (i = 0; i <= spec->limit; i++)
-    {
-        ORDDEF *odp = spec->ordinals[i];
-        if (!odp || !(odp->flags & FLAG_SYSCALL)) continue;
-        spec->syscalls[spec->nb_syscalls++] = odp;
-    }
-
-    spec->nb_syscalls = sort_func_list( spec->syscalls, spec->nb_syscalls, link_name_compare );
-    if (!spec->nb_syscalls)
-    {
-        free( spec->syscalls );
-        spec->syscalls = NULL;
-    }
-}
-
-
 /*******************************************************************
  *         add_16bit_exports
  *
@@ -851,11 +970,8 @@ void add_16bit_exports( DLLSPEC *spec32, DLLSPEC *spec16 )
     ORDDEF *odp;
 
     spec32->file_name = xstrdup( spec16->file_name );
-    if (spec16->characteristics & IMAGE_FILE_DLL)
-    {
-        spec32->characteristics = IMAGE_FILE_DLL;
-        spec32->init_func = xstrdup( "__wine_spec_dll_entry" );
-    }
+    spec32->characteristics = IMAGE_FILE_DLL;
+    spec32->init_func = xstrdup( "DllMain" );
 
     /* add an export for the NE module */
 
@@ -917,7 +1033,7 @@ int parse_spec_file( FILE *file, DLLSPEC *spec )
     current_line = 0;
 
     comment_chars = "#;";
-    separator_chars = "()-";
+    separator_chars = "()";
 
     while (get_next_line())
     {
@@ -930,6 +1046,10 @@ int parse_spec_file( FILE *file, DLLSPEC *spec )
         {
             if (!parse_spec_ordinal( atoi(token), spec )) continue;
         }
+        else if (strcmp(token, "apiset") == 0)
+        {
+            if (!parse_spec_apiset( spec )) continue;
+        }
         else
         {
             error( "Expected ordinal declaration, got '%s'\n", token );
@@ -941,8 +1061,6 @@ int parse_spec_file( FILE *file, DLLSPEC *spec )
     current_line = 0;  /* no longer parsing the input file */
     assign_names( spec );
     assign_ordinals( spec );
-    assign_syscalls( spec );
-
     return !nb_errors;
 }
 
