@@ -19,7 +19,6 @@
  */
 
 #include "config.h"
-#include "wine/port.h"
 
 #include <assert.h>
 #include <ctype.h>
@@ -27,49 +26,11 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#ifdef HAVE_UNISTD_H
-# include <unistd.h>
-#endif
-#ifdef HAVE_SYS_STAT_H
-# include <sys/stat.h>
-#endif
-#ifdef HAVE_SYS_MMAN_H
-#include <sys/mman.h>
-#endif
 
-#include "wine/list.h"
 #include "build.h"
 
-#if defined(_WIN32) && !defined(__CYGWIN__)
-# define PATH_SEPARATOR ';'
-#else
-# define PATH_SEPARATOR ':'
-#endif
-
 static struct strarray tmp_files;
-static struct strarray empty_strarray;
-
-static const struct
-{
-    const char *name;
-    enum target_cpu cpu;
-} cpu_names[] =
-{
-    { "i386",    CPU_x86 },
-    { "i486",    CPU_x86 },
-    { "i586",    CPU_x86 },
-    { "i686",    CPU_x86 },
-    { "i786",    CPU_x86 },
-    { "amd64",   CPU_x86_64 },
-    { "x86_64",  CPU_x86_64 },
-    { "powerpc", CPU_POWERPC },
-    { "arm",     CPU_ARM },
-    { "armv5",   CPU_ARM },
-    { "armv6",   CPU_ARM },
-    { "armv7",   CPU_ARM },
-    { "arm64",   CPU_ARM64 },
-    { "aarch64", CPU_ARM64 },
-};
+static const char *output_file_source_name;
 
 /* atexit handler to clean tmp files */
 void cleanup_tmp_files(void)
@@ -79,41 +40,6 @@ void cleanup_tmp_files(void)
 }
 
 
-void *xmalloc (size_t size)
-{
-    void *res;
-
-    res = malloc (size ? size : 1);
-    if (res == NULL)
-    {
-        fprintf (stderr, "Virtual memory exhausted.\n");
-        exit (1);
-    }
-    return res;
-}
-
-void *xrealloc (void *ptr, size_t size)
-{
-    void *res = realloc (ptr, size);
-    if (size && res == NULL)
-    {
-        fprintf (stderr, "Virtual memory exhausted.\n");
-        exit (1);
-    }
-    return res;
-}
-
-char *xstrdup( const char *str )
-{
-    char *res = strdup( str );
-    if (!res)
-    {
-        fprintf (stderr, "Virtual memory exhausted.\n");
-        exit (1);
-    }
-    return res;
-}
-
 char *strupper(char *s)
 {
     char *p;
@@ -121,88 +47,6 @@ char *strupper(char *s)
     return s;
 }
 
-int strendswith(const char* str, const char* end)
-{
-    int l = strlen(str);
-    int m = strlen(end);
-    return l >= m && strcmp(str + l - m, end) == 0;
-}
-
-char *strmake( const char* fmt, ... )
-{
-    int n;
-    size_t size = 100;
-    va_list ap;
-
-    for (;;)
-    {
-        char *p = xmalloc( size );
-        va_start( ap, fmt );
-	n = vsnprintf( p, size, fmt, ap );
-	va_end( ap );
-        if (n == -1) size *= 2;
-        else if ((size_t)n >= size) size = n + 1;
-        else return p;
-        free( p );
-    }
-}
-
-static struct strarray strarray_copy( struct strarray src )
-{
-    struct strarray array;
-    array.count = src.count;
-    array.max = src.max;
-    array.str = xmalloc( array.max * sizeof(*array.str) );
-    memcpy( array.str, src.str, array.count * sizeof(*array.str) );
-    return array;
-}
-
-static void strarray_add_one( struct strarray *array, const char *str )
-{
-    if (array->count == array->max)
-    {
-        array->max *= 2;
-        if (array->max < 16) array->max = 16;
-        array->str = xrealloc( array->str, array->max * sizeof(*array->str) );
-    }
-    array->str[array->count++] = str;
-}
-
-void strarray_add( struct strarray *array, ... )
-{
-    va_list valist;
-    const char *str;
-
-    va_start( valist, array );
-    while ((str = va_arg( valist, const char *))) strarray_add_one( array, str );
-    va_end( valist );
-}
-
-void strarray_addv( struct strarray *array, char * const *argv )
-{
-    while (*argv) strarray_add_one( array, *argv++ );
-}
-
-void strarray_addall( struct strarray *array, struct strarray args )
-{
-    unsigned int i;
-
-    for (i = 0; i < args.count; i++) strarray_add_one( array, args.str[i] );
-}
-
-struct strarray strarray_fromstring( const char *str, const char *delim )
-{
-    const char *tok;
-    struct strarray array = empty_strarray;
-    char *buf = xstrdup( str );
-
-    for (tok = strtok( buf, delim ); tok; tok = strtok( NULL, delim ))
-	strarray_add_one( &array, strdup( tok ));
-
-    free( buf );
-    return array;
-}
-
 void fatal_error( const char *msg, ... )
 {
     va_list valist;
@@ -283,17 +127,62 @@ int output( const char *format, ... )
     return ret;
 }
 
+static struct strarray get_tools_path(void)
+{
+    static int done;
+    static struct strarray dirs;
+
+    if (!done)
+    {
+        strarray_addall( &dirs, tools_path );
+        strarray_addall( &dirs, strarray_frompath( getenv( "PATH" )));
+        done = 1;
+    }
+    return dirs;
+}
+
+/* find a binary in the path */
+static const char *find_binary( const char *prefix, const char *name )
+{
+    struct strarray dirs = get_tools_path();
+    unsigned int i, maxlen = 0;
+    struct stat st;
+    char *p, *file;
+
+    if (strchr( name, '/' )) return name;
+    if (!prefix) prefix = "";
+    for (i = 0; i < dirs.count; i++) maxlen = max( maxlen, strlen(dirs.str[i]) + 2 );
+    file = xmalloc( maxlen + strlen(prefix) + strlen(name) + sizeof(EXEEXT) + 1 );
+
+    for (i = 0; i < dirs.count; i++)
+    {
+        strcpy( file, dirs.str[i] );
+        p = file + strlen(file);
+        if (p == file) *p++ = '.';
+        if (p[-1] != '/') *p++ = '/';
+        if (*prefix)
+        {
+            strcpy( p, prefix );
+            p += strlen(p);
+            *p++ = '-';
+        }
+        strcpy( p, name );
+        strcat( p, EXEEXT );
+        if (!stat( file, &st ) && S_ISREG(st.st_mode) && (st.st_mode & 0111)) return file;
+    }
+    free( file );
+    return NULL;
+}
+
 void spawn( struct strarray args )
 {
-    unsigned int i;
     int status;
+    const char *argv0 = find_binary( NULL, args.str[0] );
 
-    strarray_add_one( &args, NULL );
-    if (verbose)
-        for (i = 0; args.str[i]; i++)
-            fprintf( stderr, "%s%c", args.str[i], args.str[i+1] ? ' ' : '\n' );
+    if (argv0) args.str[0] = argv0;
+    if (verbose) strarray_trace( args );
 
-    if ((status = _spawnvp( _P_WAIT, args.str[0], args.str )))
+    if ((status = strarray_spawn( args )))
     {
 	if (status > 0) fatal_error( "%s failed with status %u\n", args.str[0], status );
 	else fatal_perror( "winebuild" );
@@ -301,38 +190,52 @@ void spawn( struct strarray args )
     }
 }
 
+static const char *find_clang_tool( struct strarray clang, const char *tool )
+{
+    const char *out = get_temp_file_name( "print_tool", ".out" );
+    struct strarray args = empty_strarray;
+    int sout = -1;
+    char *path, *p;
+    struct stat st;
+    size_t cnt;
+
+    strarray_addall( &args, clang );
+    strarray_add( &args, strmake( "-print-prog-name=%s", tool ));
+    if (verbose) strarray_add( &args, "-v" );
+
+    sout = dup( fileno(stdout) );
+    freopen( out, "w", stdout );
+    spawn( args );
+    if (sout >= 0)
+    {
+        dup2( sout, fileno(stdout) );
+        close( sout );
+    }
+
+    if (stat(out, &st) || !st.st_size) return NULL;
+
+    path = xmalloc(st.st_size + 1);
+    sout = open(out, O_RDONLY);
+    if (sout == -1) return NULL;
+    cnt = read(sout, path, st.st_size);
+    close(sout);
+    path[cnt] = 0;
+    if ((p = strchr(path, '\n'))) *p = 0;
+    /* clang returns passed command instead of full path if the tool could not be found */
+    if (!strcmp(path, tool))
+    {
+        free( path );
+        return NULL;
+    }
+    return path;
+}
+
 /* find a build tool in the path, trying the various names */
 struct strarray find_tool( const char *name, const char * const *names )
 {
-    static char **dirs;
-    static unsigned int count, maxlen;
-
-    char *p, *file;
+    struct strarray ret = empty_strarray;
+    const char *file;
     const char *alt_names[2];
-    unsigned int i, len;
-    struct stat st;
-
-    if (!dirs)
-    {
-        char *path;
-
-        /* split the path in directories */
-
-        if (!getenv( "PATH" )) fatal_error( "PATH not set, cannot find required tools\n" );
-        path = xstrdup( getenv( "PATH" ));
-        for (p = path, count = 2; *p; p++) if (*p == PATH_SEPARATOR) count++;
-        dirs = xmalloc( count * sizeof(*dirs) );
-        count = 0;
-        dirs[count++] = p = path;
-        while (*p)
-        {
-            while (*p && *p != PATH_SEPARATOR) p++;
-            if (!*p) break;
-            *p++ = 0;
-            dirs[count++] = p;
-        }
-        for (i = 0; i < count; i++) maxlen = max( maxlen, strlen(dirs[i])+2 );
-    }
 
     if (!names)
     {
@@ -343,92 +246,113 @@ struct strarray find_tool( const char *name, const char * const *names )
 
     while (*names)
     {
-        len = strlen(*names) + sizeof(EXEEXT) + 1;
-        if (target_alias)
-            len += strlen(target_alias) + 1;
-        file = xmalloc( maxlen + len );
-
-        for (i = 0; i < count; i++)
-        {
-            strcpy( file, dirs[i] );
-            p = file + strlen(file);
-            if (p == file) *p++ = '.';
-            if (p[-1] != '/') *p++ = '/';
-            if (target_alias)
-            {
-                strcpy( p, target_alias );
-                p += strlen(p);
-                *p++ = '-';
-            }
-            strcpy( p, *names );
-            strcat( p, EXEEXT );
-
-            if (!stat( file, &st ) && S_ISREG(st.st_mode) && (st.st_mode & 0111))
-            {
-                struct strarray ret = empty_strarray;
-                strarray_add_one( &ret, file );
-                return ret;
-            }
-        }
-        free( file );
+        if ((file = find_binary( target_alias, *names ))) break;
         names++;
     }
-    fatal_error( "cannot find the '%s' tool\n", name );
+
+    if (!file)
+    {
+        if (cc_command.count) file = find_clang_tool( cc_command, name );
+        if (!file && !(file = find_binary( "llvm", name )))
+        {
+            struct strarray clang = empty_strarray;
+            strarray_add( &clang, "clang" );
+            file = find_clang_tool( clang, strmake( "llvm-%s", name ));
+        }
+    }
+    if (!file) fatal_error( "cannot find the '%s' tool\n", name );
+
+    strarray_add( &ret, file );
+    return ret;
+}
+
+/* find a link tool in the path */
+struct strarray find_link_tool(void)
+{
+    struct strarray ret = empty_strarray;
+    const char *file = NULL;
+
+    if (cc_command.count) file = find_clang_tool( cc_command, "lld-link" );
+    if (!file) file = find_binary( NULL, "lld-link" );
+    if (!file)
+    {
+        struct strarray clang = empty_strarray;
+        strarray_add( &clang, "clang" );
+        file = find_clang_tool( clang, "lld-link" );
+    }
+
+    if (!file) fatal_error( "cannot find the 'lld-link' tool\n" );
+    strarray_add( &ret, file );
+    return ret;
 }
 
 struct strarray get_as_command(void)
 {
-    struct strarray args;
+    struct strarray args = empty_strarray;
+    const char *file;
+    unsigned int i;
+    int using_cc = 0;
 
     if (cc_command.count)
     {
-        args = strarray_copy( cc_command );
-        strarray_add( &args, "-xassembler", "-c", NULL );
+        strarray_addall( &args, cc_command );
+        using_cc = 1;
+    }
+    else if (as_command.count)
+    {
+        strarray_addall( &args, as_command );
+    }
+    else if ((file = find_binary( target_alias, "as" )) || (file = find_binary( target_alias, "gas ")))
+    {
+        strarray_add( &args, file );
+    }
+    else if ((file = find_binary( NULL, "clang" )))
+    {
+        strarray_add( &args, file );
+        if (target_alias)
+        {
+            strarray_add( &args, "-target" );
+            strarray_add( &args, target_alias );
+        }
+        using_cc = 1;
+    }
+
+    if (using_cc)
+    {
+        strarray_add( &args, "-xassembler" );
+        strarray_add( &args, "-c" );
         if (force_pointer_size)
-            strarray_add_one( &args, (force_pointer_size == 8) ? "-m64" : "-m32" );
-        if (cpu_option) strarray_add_one( &args, strmake("-mcpu=%s", cpu_option) );
-        if (fpu_option) strarray_add_one( &args, strmake("-mfpu=%s", fpu_option) );
-        if (arch_option) strarray_add_one( &args, strmake("-march=%s", arch_option) );
+            strarray_add( &args, (force_pointer_size == 8) ? "-m64" : "-m32" );
+        if (cpu_option) strarray_add( &args, strmake("-mcpu=%s", cpu_option) );
+        if (fpu_option) strarray_add( &args, strmake("-mfpu=%s", fpu_option) );
+        if (arch_option) strarray_add( &args, strmake("-march=%s", arch_option) );
+        for (i = 0; i < tools_path.count; i++)
+            strarray_add( &args, strmake("-B%s", tools_path.str[i] ));
         return args;
     }
 
-    if (!as_command.count)
-    {
-        static const char * const commands[] = { "gas", "as", NULL };
-        as_command = find_tool( "as", commands );
-    }
-
-    args = strarray_copy( as_command );
-
     if (force_pointer_size)
     {
-        switch (target_platform)
+        switch (target.platform)
         {
         case PLATFORM_APPLE:
-            strarray_add( &args, "-arch", (force_pointer_size == 8) ? "x86_64" : "i386", NULL );
+            strarray_add( &args, "-arch" );
+            strarray_add( &args, (force_pointer_size == 8) ? "x86_64" : "i386" );
             break;
         default:
-            switch(target_cpu)
-            {
-            case CPU_POWERPC:
-                strarray_add_one( &args, (force_pointer_size == 8) ? "-a64" : "-a32" );
-                break;
-            default:
-                strarray_add_one( &args, (force_pointer_size == 8) ? "--64" : "--32" );
-                break;
-            }
+            strarray_add( &args, (force_pointer_size == 8) ? "--64" : "--32" );
             break;
         }
     }
 
-    if (cpu_option) strarray_add_one( &args, strmake("-mcpu=%s", cpu_option) );
-    if (fpu_option) strarray_add_one( &args, strmake("-mfpu=%s", fpu_option) );
+    if (cpu_option) strarray_add( &args, strmake("-mcpu=%s", cpu_option) );
+    if (fpu_option) strarray_add( &args, strmake("-mfpu=%s", fpu_option) );
     return args;
 }
 
 struct strarray get_ld_command(void)
 {
-    struct strarray args;
+    struct strarray args = empty_strarray;
 
     if (!ld_command.count)
     {
@@ -436,31 +360,35 @@ struct strarray get_ld_command(void)
         ld_command = find_tool( "ld", commands );
     }
 
-    args = strarray_copy( ld_command );
+    strarray_addall( &args, ld_command );
 
     if (force_pointer_size)
     {
-        switch (target_platform)
+        switch (target.platform)
         {
         case PLATFORM_APPLE:
-            strarray_add( &args, "-arch", (force_pointer_size == 8) ? "x86_64" : "i386", NULL );
+            strarray_add( &args, "-arch" );
+            strarray_add( &args, (force_pointer_size == 8) ? "x86_64" : "i386" );
             break;
         case PLATFORM_FREEBSD:
-            strarray_add( &args, "-m", (force_pointer_size == 8) ? "elf_x86_64_fbsd" : "elf_i386_fbsd", NULL );
+            strarray_add( &args, "-m" );
+            strarray_add( &args, (force_pointer_size == 8) ? "elf_x86_64_fbsd" : "elf_i386_fbsd" );
+            break;
+        case PLATFORM_MINGW:
+        case PLATFORM_WINDOWS:
+            strarray_add( &args, "-m" );
+            strarray_add( &args, (force_pointer_size == 8) ? "i386pep" : "i386pe" );
             break;
         default:
-            switch(target_cpu)
-            {
-            case CPU_POWERPC:
-                strarray_add( &args, "-m", (force_pointer_size == 8) ? "elf64ppc" : "elf32ppc", NULL );
-                break;
-            default:
-                strarray_add( &args, "-m", (force_pointer_size == 8) ? "elf_x86_64" : "elf_i386", NULL );
-                break;
-            }
+            strarray_add( &args, "-m" );
+            strarray_add( &args, (force_pointer_size == 8) ? "elf_x86_64" : "elf_i386" );
             break;
         }
     }
+
+    if (target.cpu == CPU_ARM && !is_pe())
+        strarray_add( &args, "--no-wchar-size-warning" );
+
     return args;
 }
 
@@ -480,31 +408,12 @@ const char *get_nm_command(void)
 char *get_temp_file_name( const char *prefix, const char *suffix )
 {
     char *name;
-    const char *ext, *basename;
     int fd;
 
-    if (!prefix || !prefix[0]) prefix = "winebuild";
-    if (!suffix) suffix = "";
-    if ((basename = strrchr( prefix, '/' ))) basename++;
-    else basename = prefix;
-    if (!(ext = strchr( basename, '.' ))) ext = prefix + strlen(prefix);
-    name = xmalloc( sizeof("/tmp/") + (ext - prefix) + sizeof(".XXXXXX") + strlen(suffix) );
-    memcpy( name, prefix, ext - prefix );
-    strcpy( name + (ext - prefix), ".XXXXXX" );
-    strcat( name, suffix );
-
-    if ((fd = mkstemps( name, strlen(suffix) )) == -1)
-    {
-        strcpy( name, "/tmp/" );
-        memcpy( name + 5, basename, ext - basename );
-        strcpy( name + 5 + (ext - basename), ".XXXXXX" );
-        strcat( name, suffix );
-        if ((fd = mkstemps( name, strlen(suffix) )) == -1)
-            fatal_error( "could not generate a temp file\n" );
-    }
-
+    if (prefix) prefix = get_basename_noext( prefix );
+    fd = make_temp_file( prefix, suffix, &name );
     close( fd );
-    strarray_add_one( &tmp_files, name );
+    strarray_add( &tmp_files, name );
     return name;
 }
 
@@ -521,136 +430,17 @@ size_t input_buffer_pos;
 size_t input_buffer_size;
 unsigned char *output_buffer;
 size_t output_buffer_pos;
-size_t output_buffer_rva;
 size_t output_buffer_size;
 
-struct label
-{
-    struct list entry;
-    char *name;
-    size_t pos;
-    size_t rva;
-};
-
-static struct list labels = LIST_INIT( labels );
-
-struct label *get_label( const char *name )
-{
-    struct label *label;
-
-    LIST_FOR_EACH_ENTRY( label, &labels, struct label, entry )
-    {
-        if (!strcmp(name, label->name))
-            return label;
-    }
-
-    label = xmalloc( sizeof(*label) );
-    label->name = xstrdup( name );
-    label->pos = 0;
-    label->rva = 0;
-
-    list_add_tail( &labels, &label->entry );
-
-    return label;
-}
-
-size_t label_pos( const char *name )
-{
-    struct label *label = get_label( name );
-    return label->pos;
-}
-
-size_t label_rva( const char *name )
-{
-    struct label *label = get_label( name );
-    return label->rva;
-}
-
-size_t label_rva_align( const char *name )
-{
-    const unsigned int page_size = get_page_size();
-    size_t rva = label_rva( name );
-    size_t size = page_size - (rva % page_size);
-
-    if (size != page_size) rva += size;
-    return rva;
-}
-
-void put_label( const char *name )
-{
-    struct label *label = get_label( name );
-
-    if (label->pos || label->rva)
-    {
-        assert( label->pos == output_buffer_pos );
-        assert( label->rva == output_buffer_rva );
-    }
-
-    label->pos = output_buffer_pos;
-    label->rva = output_buffer_rva;
-}
-
-void free_labels( void )
-{
-    struct label *label, *label2;
-
-    LIST_FOR_EACH_ENTRY_SAFE( label, label2, &labels, struct label, entry )
-    {
-        list_remove( &label->entry );
-        free( label->name );
-        free( label );
-    }
-}
-
-static void check_output_buffer_space( size_t size )
-{
-    if (output_buffer_pos + size >= output_buffer_size)
-    {
-        output_buffer_size = max( output_buffer_size * 2, output_buffer_pos + size );
-        output_buffer = xrealloc( output_buffer, output_buffer_size );
-    }
-}
-
 void init_input_buffer( const char *file )
 {
-    int fd;
-    struct stat st;
-
-    if ((fd = open( file, O_RDONLY | O_BINARY )) == -1) fatal_perror( "Cannot open %s", file );
-    if ((fstat( fd, &st ) == -1)) fatal_perror( "Cannot stat %s", file );
-    if (!st.st_size) fatal_error( "%s is an empty file\n", file );
-#ifdef	HAVE_MMAP
-    if ((input_buffer = mmap( NULL, st.st_size, PROT_READ, MAP_PRIVATE, fd, 0 )) == (void*)-1)
-#endif
-    {
-        unsigned char *buffer = xmalloc( st.st_size );
-        if (read( fd, buffer, st.st_size ) != st.st_size) fatal_error( "Cannot read %s\n", file );
-        input_buffer = buffer;
-    }
-    close( fd );
+    if (!(input_buffer = read_file( file, &input_buffer_size ))) fatal_perror( "Cannot read %s", file );
+    if (!input_buffer_size) fatal_error( "%s is an empty file\n", file );
     input_buffer_filename = xstrdup( file );
-    input_buffer_size = st.st_size;
     input_buffer_pos = 0;
     byte_swapped = 0;
 }
 
-void init_output_buffer(void)
-{
-    output_buffer_size = 1024;
-    output_buffer_pos = 0;
-    output_buffer_rva = 0;
-    output_buffer = xmalloc( output_buffer_size );
-    free_labels();
-}
-
-void flush_output_buffer(void)
-{
-    if (fwrite( output_buffer, 1, output_buffer_pos, output_file ) != output_buffer_pos)
-        fatal_error( "Error writing to %s\n", output_file_name );
-    free( output_buffer );
-    free_labels();
-}
-
 unsigned char get_byte(void)
 {
     if (input_buffer_pos >= input_buffer_size)
@@ -683,48 +473,6 @@ unsigned int get_dword(void)
     return ret;
 }
 
-void put_data( const void *data, size_t size )
-{
-    check_output_buffer_space( size );
-    memcpy( output_buffer + output_buffer_pos, data, size );
-    output_buffer_pos += size;
-    output_buffer_rva += size;
-}
-
-void put_byte( unsigned char val )
-{
-    check_output_buffer_space( 1 );
-    output_buffer[output_buffer_pos++] = val;
-    output_buffer_rva++;
-}
-
-void put_word( unsigned short val )
-{
-    if (byte_swapped) val = (val << 8) | (val >> 8);
-    put_data( &val, sizeof(val) );
-}
-
-void put_dword( unsigned int val )
-{
-    if (byte_swapped)
-        val = ((val << 24) | ((val << 8) & 0x00ff0000) | ((val >> 8) & 0x0000ff00) | (val >> 24));
-    put_data( &val, sizeof(val) );
-}
-
-void put_qword( unsigned int val )
-{
-    if (byte_swapped)
-    {
-        put_dword( 0 );
-        put_dword( val );
-    }
-    else
-    {
-        put_dword( val );
-        put_dword( 0 );
-    }
-}
-
 /* pointer-sized word */
 void put_pword( unsigned int val )
 {
@@ -732,29 +480,6 @@ void put_pword( unsigned int val )
     else put_dword( val );
 }
 
-void put_str( const char *str )
-{
-    put_data( str, strlen(str) + 1 );
-}
-
-void align_output( unsigned int align )
-{
-    size_t size = align - (output_buffer_pos % align);
-
-    if (size == align) return;
-    check_output_buffer_space( size );
-    memset( output_buffer + output_buffer_pos, 0, size );
-    output_buffer_pos += size;
-}
-
-void align_output_rva( unsigned int file_align, unsigned int rva_align )
-{
-    size_t size = rva_align - (output_buffer_rva % rva_align);
-
-    if (size != rva_align) output_buffer_rva += size;
-    align_output( file_align );
-}
-
 /* output a standard header for generated files */
 void output_standard_file_header(void)
 {
@@ -763,6 +488,17 @@ void output_standard_file_header(void)
     else
         output( "/* File generated automatically; do not edit! */\n" );
     output( "/* This file can be copied, modified and distributed without restriction. */\n\n" );
+    if (safe_seh)
+    {
+        output( "\t.def    @feat.00\n\t.scl 3\n\t.type 0\n\t.endef\n" );
+        output( "\t.globl  @feat.00\n" );
+        output( ".set @feat.00, 1\n" );
+    }
+    if (thumb_mode)
+    {
+        output( "\t.syntax unified\n" );
+        output( "\t.thumb\n" );
+    }
 }
 
 /* dump a byte stream into the assembly code */
@@ -820,6 +556,47 @@ void close_input_file( FILE *file )
 }
 
 
+/*******************************************************************
+ *         open_output_file
+ */
+void open_output_file(void)
+{
+    if (output_file_name)
+    {
+        if (strendswith( output_file_name, ".o" ))
+            output_file_source_name = open_temp_output_file( ".s" );
+        else
+            if (!(output_file = fopen( output_file_name, "w" )))
+                fatal_error( "Unable to create output file '%s'\n", output_file_name );
+    }
+    else output_file = stdout;
+}
+
+
+/*******************************************************************
+ *         close_output_file
+ */
+void close_output_file(void)
+{
+    if (!output_file || !output_file_name) return;
+    if (fclose( output_file ) < 0) fatal_perror( "fclose" );
+    if (output_file_source_name) assemble_file( output_file_source_name, output_file_name );
+    output_file = NULL;
+}
+
+
+/*******************************************************************
+ *         open_temp_output_file
+ */
+char *open_temp_output_file( const char *suffix )
+{
+    char *tmp_file = get_temp_file_name( output_file_name, suffix );
+    if (!(output_file = fopen( tmp_file, "w" )))
+        fatal_error( "Unable to create output file '%s'\n", tmp_file );
+    return tmp_file;
+}
+
+
 /*******************************************************************
  *         remove_stdcall_decoration
  *
@@ -830,7 +607,7 @@ int remove_stdcall_decoration( char *name )
 {
     char *p, *end = strrchr( name, '@' );
     if (!end || !end[1] || end == name) return -1;
-    if (target_cpu != CPU_x86) return -1;
+    if (target.cpu != CPU_i386 && target.cpu != CPU_x86_32on64) return -1;
     /* make sure all the rest is digits */
     for (p = end + 1; *p; p++) if (!isdigit(*p)) return -1;
     *end = 0;
@@ -846,7 +623,9 @@ int remove_stdcall_decoration( char *name )
 void assemble_file( const char *src_file, const char *obj_file )
 {
     struct strarray args = get_as_command();
-    strarray_add( &args, "-o", obj_file, src_file, NULL );
+    strarray_add( &args, "-o" );
+    strarray_add( &args, obj_file );
+    strarray_add( &args, src_file );
     spawn( args );
 }
 
@@ -865,9 +644,10 @@ DLLSPEC *alloc_dll_spec(void)
     spec->type               = SPEC_WIN32;
     spec->base               = MAX_ORDINALS;
     spec->characteristics    = IMAGE_FILE_EXECUTABLE_IMAGE;
-    spec->subsystem          = 0;
+    spec->subsystem          = IMAGE_SUBSYSTEM_WINDOWS_CUI;
     spec->subsystem_major    = 4;
     spec->subsystem_minor    = 0;
+    spec->syscall_table      = 0;
     if (get_ptr_size() > 4)
         spec->characteristics |= IMAGE_FILE_LARGE_ADDRESS_AWARE;
     else
@@ -892,7 +672,6 @@ void free_dll_spec( DLLSPEC *spec )
         free( odp->name );
         free( odp->export_name );
         free( odp->link_name );
-        free( odp->impl_name );
     }
     free( spec->file_name );
     free( spec->dll_name );
@@ -902,7 +681,6 @@ void free_dll_spec( DLLSPEC *spec )
     free( spec->names );
     free( spec->ordinals );
     free( spec->resources );
-    free( spec->syscalls );
     free( spec );
 }
 
@@ -949,16 +727,67 @@ const char *get_stub_name( const ORDDEF *odp, const DLLSPEC *spec )
     return buffer;
 }
 
-/* parse a cpu name and return the corresponding value */
-int get_cpu_from_name( const char *name )
+/* return the stdcall-decorated name for an entry point */
+const char *get_link_name( const ORDDEF *odp )
 {
-    unsigned int i;
+    static char *buffer;
+    char *ret;
 
-    for (i = 0; i < sizeof(cpu_names)/sizeof(cpu_names[0]); i++)
-        if (!strcmp( cpu_names[i].name, name )) return cpu_names[i].cpu;
-    return -1;
+    if (target.cpu != CPU_i386) return odp->link_name;
+
+    switch (odp->type)
+    {
+    case TYPE_STDCALL:
+        if (is_pe())
+        {
+            if (odp->flags & FLAG_THISCALL) return odp->link_name;
+            if (odp->flags & FLAG_FASTCALL) ret = strmake( "@%s@%u", odp->link_name, get_args_size( odp ));
+            else if (!kill_at) ret = strmake( "%s@%u", odp->link_name, get_args_size( odp ));
+            else return odp->link_name;
+        }
+        else
+        {
+            if (odp->flags & FLAG_THISCALL) ret = strmake( "__thiscall_%s", odp->link_name );
+            else if (odp->flags & FLAG_FASTCALL) ret = strmake( "__fastcall_%s", odp->link_name );
+            else return odp->link_name;
+        }
+        break;
+
+    case TYPE_PASCAL:
+        if (is_pe() && !kill_at)
+        {
+            int args = get_args_size( odp );
+            if (odp->flags & FLAG_REGISTER) args += get_ptr_size();  /* context argument */
+            ret = strmake( "%s@%u", odp->link_name, args );
+        }
+        else return odp->link_name;
+        break;
+
+    default:
+        return odp->link_name;
+    }
+
+    free( buffer );
+    buffer = ret;
+    return ret;
 }
 
+/*******************************************************************
+ *         sort_func_list
+ *
+ * Sort a list of functions, removing duplicates.
+ */
+int sort_func_list( ORDDEF **list, int count, int (*compare)(const void *, const void *) )
+{
+    int i, j;
+
+    if (!count) return 0;
+    qsort( list, count, sizeof(*list), compare );
+    for (i = j = 0; i < count; i++) if (compare( &list[j], &list[i] )) list[++j] = list[i];
+    return j + 1;
+}
+
+
 /*****************************************************************
  *  Function:    get_alignment
  *
@@ -989,13 +818,13 @@ unsigned int get_alignment(unsigned int align)
 
     assert( !(align & (align - 1)) );
 
-    switch(target_cpu)
+    switch (target.cpu)
     {
-    case CPU_x86:
+    case CPU_i386:
     case CPU_x86_64:
-        if (target_platform != PLATFORM_APPLE) return align;
+    case CPU_x86_32on64:
+        if (target.platform != PLATFORM_APPLE) return align;
         /* fall through */
-    case CPU_POWERPC:
     case CPU_ARM:
     case CPU_ARM64:
         n = 0;
@@ -1010,37 +839,7 @@ unsigned int get_alignment(unsigned int align)
 /* return the page size for the target CPU */
 unsigned int get_page_size(void)
 {
-    switch(target_cpu)
-    {
-    case CPU_x86:
-    case CPU_x86_64:
-    case CPU_POWERPC:
-    case CPU_ARM:
-        return 0x1000;
-    case CPU_ARM64:
-        return 0x10000;
-    }
-    /* unreached */
-    assert(0);
-    return 0;
-}
-
-/* return the size of a pointer on the target CPU */
-unsigned int get_ptr_size(void)
-{
-    switch(target_cpu)
-    {
-    case CPU_x86:
-    case CPU_POWERPC:
-    case CPU_ARM:
-        return 4;
-    case CPU_x86_64:
-    case CPU_ARM64:
-        return 8;
-    }
-    /* unreached */
-    assert(0);
-    return 0;
+    return 0x1000;  /* same on all platforms */
 }
 
 /* return the total size in bytes of the arguments on the stack */
@@ -1054,11 +853,12 @@ unsigned int get_args_size( const ORDDEF *odp )
         {
         case ARG_INT64:
         case ARG_DOUBLE:
+            if (target.cpu == CPU_ARM) size = (size + 7) & ~7;
             size += 8;
             break;
         case ARG_INT128:
             /* int128 is passed as pointer on x86_64 */
-            if (target_cpu != CPU_x86_64)
+            if (target.cpu != CPU_x86_64)
             {
                 size += 16;
                 break;
@@ -1077,10 +877,14 @@ const char *asm_name( const char *sym )
 {
     static char *buffer;
 
-    switch (target_platform)
+    switch (target.platform)
     {
-    case PLATFORM_APPLE:
+    case PLATFORM_MINGW:
     case PLATFORM_WINDOWS:
+        if (target.cpu != CPU_i386) return sym;
+        if (sym[0] == '@') return sym;  /* fastcall */
+        /* fall through */
+    case PLATFORM_APPLE:
         if (sym[0] == '.' && sym[1] == 'L') return sym;
         free( buffer );
         buffer = strmake( "_%s", sym );
@@ -1090,24 +894,39 @@ const char *asm_name( const char *sym )
     }
 }
 
+/* return the 32-bit-to-64-bit thunk name for a C function */
+const char *thunk32_name( const char *func )
+{
+    static const char *thunk_prefix = "wine";
+    static char *buffer;
+    free( buffer );
+    buffer = strmake( "%s_thunk_%s", thunk_prefix, func );
+    return buffer;
+}
+
 /* return an assembly function declaration for a C function name */
 const char *func_declaration( const char *func )
 {
     static char *buffer;
 
-    switch (target_platform)
+    switch (target.platform)
     {
     case PLATFORM_APPLE:
         return "";
+    case PLATFORM_MINGW:
     case PLATFORM_WINDOWS:
         free( buffer );
-        buffer = strmake( ".def _%s; .scl 2; .type 32; .endef", func );
+        buffer = strmake( ".def %s\n\t.scl 2\n\t.type 32\n\t.endef%s", asm_name(func),
+                          thumb_mode ? "\n\t.thumb_func" : "" );
         break;
     default:
         free( buffer );
-        switch(target_cpu)
+        switch (target.cpu)
         {
         case CPU_ARM:
+            buffer = strmake( ".type %s,%%function%s", func,
+                              thumb_mode ? "\n\t.thumb_func" : "" );
+            break;
         case CPU_ARM64:
             buffer = strmake( ".type %s,%%function", func );
             break;
@@ -1123,9 +942,10 @@ const char *func_declaration( const char *func )
 /* output a size declaration for an assembly function */
 void output_function_size( const char *name )
 {
-    switch (target_platform)
+    switch (target.platform)
     {
     case PLATFORM_APPLE:
+    case PLATFORM_MINGW:
     case PLATFORM_WINDOWS:
         break;
     default:
@@ -1147,16 +967,40 @@ void output_cfi( const char *format, ... )
     va_end( valist );
 }
 
+/* output an RVA pointer */
+void output_rva( const char *format, ... )
+{
+    va_list valist;
+
+    va_start( valist, format );
+    switch (target.platform)
+    {
+    case PLATFORM_MINGW:
+    case PLATFORM_WINDOWS:
+        output( "\t.rva " );
+        vfprintf( output_file, format, valist );
+        fputc( '\n', output_file );
+        break;
+    default:
+        output( "\t.long " );
+        vfprintf( output_file, format, valist );
+        output( " - .L__wine_spec_rva_base\n" );
+        break;
+    }
+    va_end( valist );
+}
+
 /* output the GNU note for non-exec stack */
 void output_gnu_stack_note(void)
 {
-    switch (target_platform)
+    switch (target.platform)
     {
+    case PLATFORM_MINGW:
     case PLATFORM_WINDOWS:
     case PLATFORM_APPLE:
         break;
     default:
-        switch(target_cpu)
+        switch (target.cpu)
         {
         case CPU_ARM:
         case CPU_ARM64:
@@ -1176,13 +1020,15 @@ const char *asm_globl( const char *func )
     static char *buffer;
 
     free( buffer );
-    switch (target_platform)
+    switch (target.platform)
     {
     case PLATFORM_APPLE:
         buffer = strmake( "\t.globl _%s\n\t.private_extern _%s\n_%s:", func, func, func );
         break;
+    case PLATFORM_MINGW:
     case PLATFORM_WINDOWS:
-        buffer = strmake( "\t.globl _%s\n_%s:", func, func );
+        buffer = strmake( "\t.globl %s%s\n%s%s:", target.cpu == CPU_i386 ? "_" : "", func,
+                          target.cpu == CPU_i386 ? "_" : "", func );
         break;
     default:
         buffer = strmake( "\t.globl %s\n\t.hidden %s\n%s:", func, func, func );
@@ -1191,9 +1037,9 @@ const char *asm_globl( const char *func )
     return buffer;
 }
 
-const char *get_asm_ptr_keyword(void)
+static const char *get_asm_ptr_keyword_for_size(unsigned int ptr_size)
 {
-    switch(get_ptr_size())
+    switch(ptr_size)
     {
     case 4: return ".long";
     case 8: return ".quad";
@@ -1202,9 +1048,19 @@ const char *get_asm_ptr_keyword(void)
     return NULL;
 }
 
+const char *get_asm_ptr_keyword(void)
+{
+    return get_asm_ptr_keyword_for_size(get_ptr_size());
+}
+
+const char *get_asm_host_ptr_keyword(void)
+{
+    return get_asm_ptr_keyword_for_size(get_host_ptr_size());
+}
+
 const char *get_asm_string_keyword(void)
 {
-    switch (target_platform)
+    switch (target.platform)
     {
     case PLATFORM_APPLE:
         return ".asciz";
@@ -1213,39 +1069,74 @@ const char *get_asm_string_keyword(void)
     }
 }
 
+const char *get_asm_export_section(void)
+{
+    switch (target.platform)
+    {
+    case PLATFORM_APPLE:   return ".data";
+    case PLATFORM_MINGW:
+    case PLATFORM_WINDOWS: return ".section .edata";
+    default:               return ".section .data";
+    }
+}
+
 const char *get_asm_rodata_section(void)
 {
-    switch (target_platform)
+    switch (target.platform)
     {
     case PLATFORM_APPLE: return ".const";
     default:             return ".section .rodata";
     }
 }
 
+const char *get_asm_rsrc_section(void)
+{
+    switch (target.platform)
+    {
+    case PLATFORM_APPLE:   return ".data";
+    case PLATFORM_MINGW:
+    case PLATFORM_WINDOWS: return ".section .rsrc";
+    default:               return ".section .data";
+    }
+}
+
 const char *get_asm_string_section(void)
 {
-    switch (target_platform)
+    switch (target.platform)
     {
     case PLATFORM_APPLE: return ".cstring";
     default:             return ".section .rodata";
     }
 }
 
-/*******************************************************************
- *         sort_func_list
- *
- * Sort a list of functions, removing duplicates.
- */
-int sort_func_list( ORDDEF **list, int count, int (*compare)(const void *, const void *) )
+const char *arm64_page( const char *sym )
 {
-    int i, j;
+    static char *buffer;
 
-    if (!count) return 0;
-    qsort( list, count, sizeof(*list), compare );
-
-    for (i = j = 0; i < count; i++)
+    switch (target.platform)
     {
-        if (compare( &list[j], &list[i] )) list[++j] = list[i];
+    case PLATFORM_APPLE:
+        free( buffer );
+        buffer = strmake( "%s@PAGE", sym );
+        return buffer;
+    default:
+        return sym;
     }
-    return j + 1;
+}
+
+const char *arm64_pageoff( const char *sym )
+{
+    static char *buffer;
+
+    free( buffer );
+    switch (target.platform)
+    {
+    case PLATFORM_APPLE:
+        buffer = strmake( "%s@PAGEOFF", sym );
+        break;
+    default:
+        buffer = strmake( ":lo12:%s", sym );
+        break;
+    }
+    return buffer;
 }
